local HAS_INIT = 'hasInit'
local HAS_BOSS_INIT = 'bossInit'
local DESTROY_BARRICADE_INIT = 'destroyBarricadeInit'

local INTRO_EXPLOSION_TIMER = 'introExplosionTimer'
local INTRO_EXPLOSION_KEY = 'introExplosionKey'

local CHECK_TICK = 3
local CHECK_KEY = 'checkKey'

local UNIT_LUA = 'AliciaBossBattle.lua'

local BOSS_BASE_IDENTIFIER = 'R_AliciaBoss'

local EXPLOSION_Y_POS_KEY = 'explosionYPosKey'
local EXPLOSION_TIMER = 280
local EXPLOSION_TELEGRAPH_ENTITY = 'lineExplosionTelegraphEntity'
local EXPLOSION_ENTITY = 'lineExplosionEntity'

local CURRENT_BOSS_STATE = 'bossStateKey'
local BOSS_STATE_IDLE = 0
local BOSS_STATE_ACTIVE = 1

local WAVE_TIMER_KEY = 'waveTimerKey'
local DELAYED_WAVE = 'hasDelayedWave'
local WAVE_DELAY = 55
local TIMING_BASE_ENTITY = 'BossBattleTimingBase'
local NUM_WAVES = 6
local ASSOCIATED_BOSS = 'associatedBossKey'
local NUM_WAVES_KEY = 'numWavesKey'
local SPAWNED_BOSS_UNIT = 'spawnedBossUnitKey'

AliciaBossBattle_tickTrigger = function(entity)
    local HasInit = objc.get_local_value(entity, HAS_INIT)
    if HasInit == 0 then
        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end

        if is2vE == 1 then
            --0.5 crystal/s in 2vE
            local incomeToSet = 1000
            objc.set_income_generation_frequency_in_milliseconds(1, incomeToSet)
            objc.set_income_generation_frequency_in_milliseconds(3, incomeToSet)
            objc.add_crystal(1, 3)
            objc.add_crystal(3, 3)
        else
            local incomeToSet = 700
            objc.set_income_generation_frequency_in_milliseconds(1, incomeToSet)
            objc.add_crystal(1, 3)
        end

        objc.set_local_value(entity, HAS_INIT, 1)

        local alicia = objc.spawn_at_point(BOSS_BASE_IDENTIFIER, 1, 2, 4600, 0, 0, 0)
        objc.set_local_value(entity, SPAWNED_BOSS_UNIT, alicia[1])

        local backBarricade = objc.spawn_at_point('R_Barricade', 1, 2, 6000, 0, -1, 0)
        objc.add_trigger(backBarricade[1], TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'AliciaBossBattle.lua', 'AliciaBossBattle_barricadeNoDamageTrigger')
    end

    local destroyBarricadeInit = objc.get_local_value(entity, DESTROY_BARRICADE_INIT)
    if destroyBarricadeInit == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'B_Barricade_Ship' then
                objc.set_entity_visibility(unit, false)
                objc.sacrifice_entity(unit)
                objc.set_local_value(entity, DESTROY_BARRICADE_INIT, 1)
            end
        end
    end

    local introTimer = objc.tick_timer(entity, INTRO_EXPLOSION_TIMER, 7)
    if introTimer == 0 and objc.get_local_value(entity, INTRO_EXPLOSION_KEY) == 0 then
        local explosionEntityArray = objc.spawn_at_point(EXPLOSION_ENTITY, 1, 2, 4500, 2100, 0, 0)
        local explosionEntityArray = objc.spawn_at_point(EXPLOSION_ENTITY, 1, 2, 4500, -2100, 0, 0)

        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            -- Kill the enemy barricade.
            if objc.is_barricade(unit) then
                objc.sacrifice_entity(unit)
            end
        end
        objc.set_local_value(entity, INTRO_EXPLOSION_KEY, 1)
    end

    local waveTimer = objc.tick_timer(entity, WAVE_TIMER_KEY, WAVE_DELAY)
    if waveTimer == 0 and objc.get_local_value(entity, DELAYED_WAVE) == 0 then
        local timingEntity = objc.spawn_at_entity(TIMING_BASE_ENTITY, 1, entity)
        objc.set_local_value(timingEntity[1], ASSOCIATED_BOSS, entity)
        objc.set_local_value(timingEntity[1], NUM_WAVES_KEY, NUM_WAVES)
        objc.set_local_value(entity, DELAYED_WAVE, 1)
    end

    local checkTimer = objc.tick_timer(entity, CHECK_KEY, CHECK_TICK)
    if checkTimer == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do

            if objc.get_local_value(entity, HAS_BOSS_INIT) == 0 then
                if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == BOSS_BASE_IDENTIFIER then
                    objc.set_entity_scale_percentage(unit, 250)
                    objc.set_local_value(unit, CURRENT_BOSS_STATE, BOSS_STATE_IDLE)
                    objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_ATTACK_RANGE, 3500)
                    objc.move_entity_to_location_with_velocity(unit, 4500, 0, 40, 70)
                    objc.move_entity_to_planes_of_existence(unit, PLANE_OF_EXISTENCE_GROUND, 0, EASE_NONE, 10)
                    objc.set_local_value(entity, HAS_BOSS_INIT, 1)
                end
            end
        end
    end

    local explosionTimer = objc.tick_timer(entity, 'explosionTimerKey', EXPLOSION_TIMER)
    if explosionTimer == 0 then
        local yPosition = -2100
        local enemies = objc.list_entities(1)
        local topLaneEnemies = 0
        local bottomLaneEnemies = 0
        for _, enemy in pairs(enemies) do
            if objc.is_normal_unit(enemy) then
                local enemyPos = objc.get_entity_position(enemy)
                if enemyPos[2] > 0 then
                    topLaneEnemies = topLaneEnemies + 1
                else
                    bottomLaneEnemies = bottomLaneEnemies + 1
                end
            end
        end

        if topLaneEnemies > bottomLaneEnemies then
            yPosition = 2100
        else
            yPosition = -2100
        end
        objc.spawn_at_point(EXPLOSION_TELEGRAPH_ENTITY, 1, 2, 0, yPosition, -1, 0)
    end
end

AliciaBossBattle_barricadeNoDamageTrigger = function(damage, targetEntity, sourceEntity)
    return 0
end

AliciaBossBattle_Wave_1 = function(entity)
    objc.spawn_at_point('R_Overcharger', 1, 2, 2500, -1200, -1, 0)
    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3000, -1000, -1, 0)
    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3000, -1100, -1, 0)
    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3000, -1200, -1, 0)

    objc.spawn_at_point('R_CheapPlane', 1, 2, 3500, 2500, -1, 0)
    objc.spawn_at_point('R_CheapPlane', 1, 2, 3500, 2700, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_1')
end

AliciaBossBattle_Wave_2 = function (entity)
    objc.spawn_at_point('R_Junker', 1, 2, 4000, -1000, -1, 0)

    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3500, 1000, -1, 0)
    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3500, 1100, -1, 0)
    objc.spawn_at_point('R_ThreeLittleCars', 1, 2, 3500, 1200, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_2')
end

AliciaBossBattle_Wave_3 = function (entity)
    objc.spawn_at_point('R_AliciasTurret', 1, 2, 3500, -2000, -1, 0)
    objc.spawn_at_point('R_Jackal', 1, 2, 4500, 1500, -1, 0)
    objc.spawn_at_point('R_Jackal', 1, 2, 4500, 1500, -1, 0)

    objc.spawn_at_point('R_Raptor', 1, 2, 5000, -1500, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_3')
end

AliciaBossBattle_Wave_4 = function (entity)
    objc.spawn_at_point('R_Steamroller', 1, 2, 3000, -2000, -1, 0)
    objc.spawn_at_point('R_Overcharger', 1, 2, 3000, 2000, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_4')
end

AliciaBossBattle_Wave_5 = function (entity)
    objc.spawn_at_point('R_Raptor', 1, 2, 3500, 2000, -1, 0)
    objc.spawn_at_point('R_Raptor', 1, 2, 3500, -2000, -1, 0)
    objc.spawn_at_point('R_CannonScooter', 1, 2, 3500, -2000, -1, 0)
    objc.spawn_at_point('R_TruckTank', 1, 2, 3500, 2000, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_5')
end

AliciaBossBattle_Wave_6 = function (entity)
    objc.spawn_at_point('R_AliciasTurret', 1, 2, 3500, -2000, -1, 0)
    local alicia = objc.get_local_value(entity, SPAWNED_BOSS_UNIT)
    if alicia ~= 0 then
        objc.set_local_value(alicia, CURRENT_BOSS_STATE, BOSS_STATE_ACTIVE)
    end

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'AliciaBossBattle_Wave_6')
end