local ATTACK_COUNTER_KEY = 'attackCounterKey'
local AOE_LINE_LENGTH = 500
local AOE_LINE_WIDTH = 10000
local BATTLEFIELD_LENGTH = 10000
local BATTLEFIELD_WIDTH = 5000
local AOE_DAMAGE_COEFFICIENT = 1.0
local PROXY_TARGET = 'proxyTargetKey'
local ACTUAL_TARGET = 'actualTargetKey'
local PROXY_IDENTIFIER = 'AnnieWalkerProxy'
local ATTACKING_KEY = 'isAttackingKey'
local HAS_INIT = 'hasInitKey'
local IS_ON_LEFT = 'isOnLeftKey'
local ENEMY_CHECK = 'enemyCheckKey'
local SHOULD_SPAWN = 'shouldSpawnKey'
local SPAWN_TIMER = 'spawnTimerKey'
local OWNER_ENTITY = 'ownerEntityKey'
local SPAWN_DELAY = 2
local CHECK_INTERVAL = 5
local NORMAL_RANGE = 2300
local ANNIE_WALKER_SIZE = 450
local MAX_ENEMY_SIZE = 525
local RANGE_BUFFER = 250
local SIZE_CUTOFF = 5500
local IS_TWO_LANES = 'isTwoLanesKey'
local TWO_LANES_VFX_OFFSET = 2000
local MAP_BOUNDARY = 2500

B_AnnieWalker_spawnTrigger = function(entity)
    local mapEntities = objc.list_entities(0)
    local numObjectives = 0
    for _, mapEntity in pairs(mapEntities) do
        local identifier = objc.get_entity_value_for_key(mapEntity, ENTITY_VALUE_KEY_IDENTIFIER)
        if string.match(identifier, 'CrystalObjectiveDominate') ~= nil then
            numObjectives = numObjectives + 1
        end
    end

    if numObjectives > 1 then
        objc.set_local_value(entity, IS_TWO_LANES, 1)
    end
end

--[[
This attack trigger sets a unique value per attack and assigns it to the target so we can track who the primary
target for each attack is and only apply the visual effect when that target is damaged (or else everything in the splash area
would spawn an effect)
]]--
B_AnnieWalker_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, ATTACKING_KEY, 1)

    local isOnLeft = objc.get_local_value(sourceEntity, IS_ON_LEFT)
    local proxyTarget = objc.get_local_value(sourceEntity, PROXY_TARGET)

    --find out if we have an 'actual' target, and set it accordingly
    local thisTarget
    local actualTarget = objc.get_local_value(sourceEntity, ACTUAL_TARGET)
    if actualTarget == 0 then
        --set range to very high to be able to track proxy wherever it might be
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, 5000)

        --make our proxyTarget targetable
        objc.set_targetable(proxyTarget, true)

        thisTarget = targetEntity

        objc.set_local_value(sourceEntity, ACTUAL_TARGET, targetEntity)
    else
        thisTarget = actualTarget
    end

    local targetPosition = objc.get_entity_position(thisTarget)
    objc.set_targeting(sourceEntity, proxyTarget)
    if isOnLeft == 0 then
        objc.teleport_entity(proxyTarget, targetPosition[1], MAP_BOUNDARY)
        objc.move_entity_to_location_with_velocity(proxyTarget, targetPosition[1], -MAP_BOUNDARY, 4000, 50)
        objc.set_local_value(sourceEntity, IS_ON_LEFT, 1)
    else
        objc.teleport_entity(proxyTarget, targetPosition[1], -MAP_BOUNDARY)
        objc.move_entity_to_location_with_velocity(proxyTarget, targetPosition[1], MAP_BOUNDARY, 4000, 50)
        objc.set_local_value(sourceEntity, IS_ON_LEFT, 0)
    end

    if targetPosition ~= 0 then
        objc.set_local_value(sourceEntity, SHOULD_SPAWN, targetPosition)
    end

    --set values for damage
    local primaryTargetKey = objc.get_entity_value(sourceEntity, 'identityComponent.description')
    local attackCounter = objc.get_local_value(sourceEntity, ATTACK_COUNTER_KEY)

    attackCounter = attackCounter + 1

    objc.set_local_value(targetEntity, primaryTargetKey, attackCounter)
    objc.set_local_value(sourceEntity, ATTACK_COUNTER_KEY, attackCounter)
end

B_AnnieWalker_tickTrigger = function(entity)
    local playerNumber = objc.get_player_number(entity)

    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        --spawn our proxy
        local enemyTeamNumber = objc.get_enemy_team_number(entity)
        local enemyPlayerNumbers = objc.get_player_numbers_for_team_number(enemyTeamNumber)
        local thisPos = objc.get_entity_position(entity)
        local proxyArray = objc.spawn_at_point(PROXY_IDENTIFIER, 1, enemyPlayerNumbers[1], thisPos[1], thisPos[2], 0, 0)
        objc.set_targetable(proxyArray[1], false)
        objc.set_local_value(proxyArray[1], OWNER_ENTITY, entity)
        objc.set_local_value(entity, PROXY_TARGET, proxyArray[1])

        objc.set_local_value(entity, HAS_INIT, 1)
    end

    local actualTarget = objc.get_local_value(entity, ACTUAL_TARGET)
    if actualTarget ~= 0 then
        local actualTargetHealth = objc.get_entity_value_for_key(actualTarget, ENTITY_VALUE_KEY_CURRENT_HEALTH)
        local actualTargetIdentifier = objc.get_entity_value_for_key(actualTarget, ENTITY_VALUE_KEY_IDENTIFIER)
        local actualTargetDistance = math.abs(objc.unit_distance_with_offset(entity, 0, 0, actualTarget, 0, 0))
        local proxyTarget = objc.get_local_value(entity, PROXY_TARGET)
        if actualTargetHealth <= 0 or string.match(actualTargetIdentifier, PROXY_IDENTIFIER)
            or actualTargetDistance > (NORMAL_RANGE + ANNIE_WALKER_SIZE + MAX_ENEMY_SIZE + RANGE_BUFFER) then
            objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, NORMAL_RANGE)
            objc.set_targetable(proxyTarget, false)
            objc.set_local_value(entity, ACTUAL_TARGET, 0)
        end
    end

    local shouldSpawn = objc.get_local_value(entity, SHOULD_SPAWN)
    if shouldSpawn ~= 0 then
        local spawnDelay = objc.tick_timer(entity, SPAWN_TIMER, SPAWN_DELAY)
        if spawnDelay == 0 then
            local yPos = 0
            local isTwoLanes = objc.get_local_value(entity, IS_TWO_LANES)
            if isTwoLanes == 1 then
                if shouldSpawn[2] > 0 then
                    yPos = TWO_LANES_VFX_OFFSET
                elseif shouldSpawn[2] < 0 then
                    yPos = -TWO_LANES_VFX_OFFSET
                end
            end
            objc.spawn_at_point('AnnieWalkerImpactEffect', 1, playerNumber, shouldSpawn[1], yPos, 1, 0)
            objc.set_local_value(entity, SHOULD_SPAWN, 0)
        end
    end
end

B_AnnieWalker_onMovementTrigger = function(entity)
    objc.set_local_value(entity, ATTACKING_KEY, 0)
end

B_AnnieWalker_damageTrigger = function(damage, targetEntity, sourceEntity)
    local entity = objc.get_entity_value(sourceEntity, 'identityComponent.sourceEntity')
    local primaryTargetKey = objc.get_entity_value(entity, 'identityComponent.description')
    local attackCounter = objc.get_local_value(entity, ATTACK_COUNTER_KEY)
    local isCurrentPrimaryTargetValueCheck = objc.get_local_value(targetEntity, primaryTargetKey)


    if attackCounter == isCurrentPrimaryTargetValueCheck then
        local teamNumber = objc.get_team_number(entity)
        local enemyNumber = objc.get_team_number(targetEntity)
        local targetPosition = objc.get_entity_position(targetEntity)
        local entityPosition = objc.get_entity_position(entity)
        local enemyList = objc.list_entities(enemyNumber)
        local directionX = 0

        if entityPosition[1] <= targetPosition[1] then
            directionX = 1
        else
            directionX = -1
        end

        for _, enemy in pairs(enemyList) do
            local enemyPosition = objc.get_entity_position(enemy)
            local distanceFromTargetX = (enemyPosition[1] + BATTLEFIELD_LENGTH) - (targetPosition[1] + BATTLEFIELD_LENGTH)
            distanceFromTargetX = distanceFromTargetX * directionX
            local distanceFromTargetY = math.abs((enemyPosition[2] + BATTLEFIELD_WIDTH) - (targetPosition[2] + BATTLEFIELD_WIDTH))

            local baseEntity = objc.get_base_entity(enemyNumber)
            if distanceFromTargetX > -AOE_LINE_LENGTH 
                and distanceFromTargetX < AOE_LINE_LENGTH 
                and distanceFromTargetY < objc.divide_with_floor(AOE_LINE_WIDTH, 2)
                and enemy ~= targetEntity 
                and objc.is_ground_unit(enemy) then
                    objc.inflict_normal_damage(sourceEntity, enemy, damage)
                    objc.flash_entity_for_damage(enemy, 250)
            end
        end
    end
    return damage
end

B_AnnieWalker_deathTrigger = function(entity)
    local proxyTarget = objc.get_local_value(entity, PROXY_TARGET)
    objc.set_targetable(proxyTarget, false)
    objc.sacrifice_entity(proxyTarget)
end
