local NEXT_ATTACK_IS_BIG_KEY = 'nextAttackIsBig'
local BIG_ATTACK_CHARGE_TIMER_KEY = 'bigAttackChargeDuration'
local IS_CHARGING = 'bigAttackIsChargingKey'
local CHARGED_VFX = 'idle_B_Dreadnaught_ChargingComplete.lvl'
local CHARGING_VFX = 'idle_B_Dreadnaught_Charging.lvl'
local SHOCK_IMPACT_VFX = 'spell_B_stinger_impact.lvl'
local SHOCK_VFX_ENTITY = 'shockEffectEntity'

local BIG_ATTACK_COOLDOWN = 40
local BIG_ATTACK_RADIUS = 1100

local MUZZLE_NODE = 'vfx_muz'
local L_CHARGE_NODE = 'vfx_arc_L_buildup'
local R_CHARGE_NODE = 'vfx_arc_R_buildup'
local BIG_ATTACK_GROUND_ANIMATON_STATE = 'bigAttackAnimationName'
local REGULAR_ATTACK_GROUND_ANIMATON_STATE = 'attackAnimationName'

local CHARGE_DELAY = 5
local CHARGE_DELAY_KEY = 'chargeDelayKey'
local SHOULD_DELAY_CHARGE = 'shouldDelayCharge'

B_Dreadnaught_tickTrigger = function(entity)
   local bigIsCharging = objc.get_local_value(entity, IS_CHARGING)
   local bigIsCharged = objc.get_local_value(entity, NEXT_ATTACK_IS_BIG_KEY)
   if bigIsCharging == 0 and bigIsCharged == 0 then
      local shouldDelayCharge = objc.get_local_value(entity, SHOULD_DELAY_CHARGE)
      if shouldDelayCharge == 1 then
         local chargeDelay = objc.tick_timer(entity, CHARGE_DELAY_KEY, CHARGE_DELAY)
         if chargeDelay == 0 then
            B_Dreadnaught_startCharging(entity)
            objc.set_local_value(entity, SHOULD_DELAY_CHARGE, 0)
         end
      else
         B_Dreadnaught_startCharging(entity)
      end
   elseif bigIsCharging == 1 then
      local bigAttackChargeTimer = objc.get_local_value(entity, BIG_ATTACK_CHARGE_TIMER_KEY)
      if bigAttackChargeTimer == 0 then
         B_Dreadnaught_bigAttackReady(entity)
      end
      objc.set_local_value(entity, BIG_ATTACK_CHARGE_TIMER_KEY, bigAttackChargeTimer - 1)
   end
end

B_Dreadnaught_bigAttackReady = function(entity)
   objc.set_local_value(entity, NEXT_ATTACK_IS_BIG_KEY, 1)
   local bigAttackAnimations = {}
   bigAttackAnimations[ANIMATION_STATE_ATTACK_GROUND] = BIG_ATTACK_GROUND_ANIMATON_STATE
   objc.update_animation_state(entity, bigAttackAnimations)
   objc.play_vfx(entity, CHARGED_VFX, L_CHARGE_NODE)
   objc.play_vfx(entity, CHARGED_VFX, R_CHARGE_NODE)
   objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_SPLASH_RANGE, BIG_ATTACK_RADIUS)
   objc.set_local_value(entity, IS_CHARGING, 0)
end

B_Dreadnaught_damageTrigger = function(damage, targetEntity, sourceEntity)
   local bulletSource = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
   if objc.get_local_value(bulletSource, NEXT_ATTACK_IS_BIG_KEY) == 1 then
      objc.set_local_value(bulletSource, NEXT_ATTACK_IS_BIG_KEY, 0)
      local attackAnims = {}
      attackAnims[ANIMATION_STATE_ATTACK_GROUND] = REGULAR_ATTACK_GROUND_ANIMATON_STATE
      objc.update_animation_state(bulletSource, attackAnims)
      B_Dreadnaught_startCharging(bulletSource)
      objc.set_entity_value_for_key(bulletSource, ENTITY_VALUE_KEY_SPLASH_RANGE, 0)

      local entityPos = objc.get_entity_position(sourceEntity)
      local playerNumber = objc.get_player_number(bulletSource)
      objc.spawn_at_point_with_source_entity(SHOCK_VFX_ENTITY, 1, playerNumber, entityPos[1], entityPos[2], 0, 0, sourceEntity)

      local teamNumber = objc.get_team_number(bulletSource)
      local enemyNumber = objc.get_enemy_team_number(bulletSource)
      local enemiesInRadius = objc.list_entities_in_radius(sourceEntity, BIG_ATTACK_RADIUS, enemyNumber)
      for _, enemy in pairs(enemiesInRadius) do
         if (objc.is_building_unit(enemy) or objc.is_ground_unit(enemy) or objc.is_commander(enemy) or objc.is_base(enemy) or objc.is_barricade(enemy) or (objc.is_boss_unit(enemy) == 1)) and (objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) and not objc.is_dummy_unit(enemy) then
            objc.play_vfx(enemy, SHOCK_IMPACT_VFX, NODE_IMPACT)
            local attackDamage = objc.get_entity_value_for_key(bulletSource, ENTITY_VALUE_KEY_ATTACK_DAMAGE)
            attackDamage = objc.divide_with_floor(attackDamage * 3, 2) 
            objc.inflict_normal_damage(bulletSource, enemy, attackDamage)
            objc.flash_entity_for_damage(enemy, 250)
         end
      end
      return 0
   end
   return damage
end

B_Dreadnaught_onAttackTrigger = function(sourceEntity, targetEntity)
    local bulletSource = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    if objc.get_local_value(bulletSource, NEXT_ATTACK_IS_BIG_KEY) == 1 then   
      objc.stop_vfx(bulletSource, CHARGED_VFX, L_CHARGE_NODE, false)
      objc.stop_vfx(bulletSource, CHARGED_VFX, R_CHARGE_NODE, false)
   end

   local isCharging = objc.get_local_value(bulletSource, IS_CHARGING)
   if isCharging == 1 then
      objc.set_local_value(bulletSource, SHOULD_DELAY_CHARGE, 1)
      B_Dreadnaught_stopCharging(bulletSource)
   end
end

B_Dreadnaught_startCharging = function(entity)
   objc.play_vfx(entity, CHARGING_VFX, L_CHARGE_NODE)
   objc.play_vfx(entity, CHARGING_VFX, R_CHARGE_NODE)
   objc.set_local_value(entity, BIG_ATTACK_CHARGE_TIMER_KEY, BIG_ATTACK_COOLDOWN)
   objc.set_local_value(entity, IS_CHARGING, 1)
end

B_Dreadnaught_stopCharging = function(entity)
   local identifier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_IDENTIFIER)
   objc.stop_vfx(entity, CHARGING_VFX, L_CHARGE_NODE, true)
   objc.stop_vfx(entity, CHARGING_VFX, R_CHARGE_NODE, true)
   objc.set_local_value(entity, IS_CHARGING, 0)
end

B_Dreadnaught_deathTrigger = function(entity)
   local isCharging = objc.get_local_value(entity, IS_CHARGING)
   if isCharging == 1 then
      objc.stop_vfx(entity, CHARGING_VFX, L_CHARGE_NODE, true)
      objc.stop_vfx(entity, CHARGING_VFX, R_CHARGE_NODE, true)
   end
end
