local HEAL_AMOUNT = 'healAmountKey'
local HEALTH_DIVISOR = 20 --0.05 mean we will be getting 5% of health when multiplying
local TICK_ROUNDS = 10
local BASE_CUTOFF = 6000
local HEAL_VFX = 'Status_healing.lvl'
local TICK_TRIGGER_KEY = 'tickTriggerKey'
local BARRICADE_MOVE_OFFSET = 1250

--[[
	THIS SPAWN TRIGGER SETS THE INITAL TARGET FOR FIELD MECHANIC
	IF THE BARRICADE IS ALIVE, TARGET IT. ELSE TARGET THE BASE
]]--

B_FieldMechanic_spawnTrigger = function(entity)
	local teamNumber = objc.get_team_number(entity)
	local playerBase = objc.get_base_entity(teamNumber)
	local xDir = -1
	local entitiesInRadius = objc.list_entities_in_radius(entity, 20000, teamNumber)
	objc.set_targeting(entity, playerBase)

	local closestBarricade = entity
	local closestDist = 9999999
	
	for _, ally in pairs(entitiesInRadius) do
		if objc.is_barricade(ally) then
            local thisDistance = math.abs(objc.unit_distance_with_offset(entity, 0, 0, ally, 0, 0))
            if thisDistance < closestDist then
            	closestDist = thisDistance
            	closestBarricade = ally
            end
   		end
   	end

   	if closestBarricade ~= entity then
   		local barricadePos = objc.get_entity_position(closestBarricade)
		if teamNumber == 2 then
			xDir = 1
		end
		objc.move_entity_to_location_with_velocity(entity, barricadePos[1] + (BARRICADE_MOVE_OFFSET * xDir), barricadePos[2], 100, 400)
		objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, 35)
		objc.set_targeting(entity, closestBarricade)
   	end

    local healAmount = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_REPAIR_AMOUNT)
    objc.set_local_value(entity, HEAL_AMOUNT, healAmount)
end

--[[
	THIS TICK_TRIGGER MAKES FIELD MECHANIC LOSE HEALTH OVER 
	TIME WHEN LIVING TO ALWAYS GIVE IT A FINITE LIFE TIME
]]--

B_FieldMechanic_tickTrigger = function(entity)
	local tickTimer = objc.tick_timer(entity, TICK_TRIGGER_KEY, TICK_ROUNDS)
	if tickTimer == 0 then --Loses 5% health every 1 second
		local currentHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
		local percentLostHealth = objc.divide_with_floor(objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH), HEALTH_DIVISOR)
		objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, currentHealth - percentLostHealth)
	end
end

--[[
	THIS CANTARGETTRIGGER MAKES THE FIELD MECHANIC TARGET THE BARRICADE FIRST 
	ALWAYS, BUT ALSO ALLOWS IT TO CHANGE TARGETS TO THE BASE IF NECESSARY
]]--

B_FieldMechanic_canTargetTrigger = function(entity, targetEntity)
	local teamNumber = objc.get_team_number(entity)
	local base = objc.get_base_entity(teamNumber)
	local barricadeIsDead = true
	local barricade = 'barricade'
   
	local alliesInRadius = objc.list_entities_in_radius(entity, 20000, teamNumber)
	local entityPos = objc.get_entity_position(entity)   
   
	for _, ally in pairs(alliesInRadius) do
		if objc.is_barricade(ally) then
			barricade = ally
			barricadeIsDead = false
		end
	end


	if targetEntity == base 									--if the base is targeted
	  and barricadeIsDead == false 								--and barricade is alive
	  then 
		return NO												--don't target
	elseif targetEntity == base 								--if the base is targeted
	  and barricadeIsDead == true 								--and the barricade is dead
	  and (teamNumber == 1 and entityPos[1] < -BASE_CUTOFF 	--and the unit is in a base
	  or teamNumber == 2 and entityPos[1] > BASE_CUTOFF)
	  then
	  	return NO												--don't target
	end

	return YES
end

--[[
	THIS ONATTACKTRIGGER MAKES THE FIELD MECHANIC HEAL THE
	BARRICADE OR BASE WHILE ALSO TAKING HEALTH AS A RESOURCE TO HEAL
]]--

B_FieldMechanic_onAttackTrigger = function(sourceEntity, targetEntity)   
	local currentTargetHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
	local maxTargetHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
	local currentEntityHealth = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
	local healAmount = objc.get_local_value(sourceEntity, HEAL_AMOUNT)

	if currentTargetHealth < maxTargetHealth and currentTargetHealth + healAmount > maxTargetHealth then
		objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH, maxTargetHealth)
		objc.play_vfx(targetEntity, HEAL_VFX, NODE_STATUS)
	elseif currentTargetHealth < maxTargetHealth then
		objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH, currentTargetHealth + healAmount)
		objc.play_vfx(targetEntity, HEAL_VFX, NODE_STATUS)
	end
end
