--[[
This DamageTrigger gives the B_LaserTank the UnitAbility to damage all units in a given path.
--]]

local DAMAGE_AOE_HALF_WIDTH = 200
local PRIMARY_TARGET_KEY = 'primaryTargetKey'

B_LaserTank_onAttackTrigger = function(sourceEntity, targetEntity)
	objc.set_local_value(sourceEntity, PRIMARY_TARGET_KEY, targetEntity)
end

B_LaserTank_damageTrigger = function(damage, targetEntity, sourceEntity)
	local tank = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
	local tankPosition = objc.get_entity_position(tank)
	local primaryTarget = objc.get_local_value(tank, PRIMARY_TARGET_KEY)
	local primaryPosition = objc.get_entity_position(targetEntity)
	local deltaX = primaryPosition[1] - tankPosition[1]
	local deltaY = primaryPosition[2] - tankPosition[2]
	local theta = math.atan2(deltaY,deltaX)

	if primaryTarget == targetEntity then
		local enemyNumber = objc.get_team_number(targetEntity)
		local enemyList = objc.list_entities(enemyNumber)
		
		for _, enemy in pairs(enemyList) do
			if objc.is_ground_unit(enemy) and objc.is_normal_unit(enemy) and enemy ~= primaryTarget then
				local enemyPosition = objc.get_entity_position(enemy)
				local distanceFromEnemyX = enemyPosition[1] - tankPosition[1]
				local idealYLocationForEnemy = tankPosition[2] + (math.tan(theta)*distanceFromEnemyX)

				if ((idealYLocationForEnemy + DAMAGE_AOE_HALF_WIDTH) > enemyPosition[2] and (idealYLocationForEnemy - DAMAGE_AOE_HALF_WIDTH) < enemyPosition[2]) then
					objc.inflict_normal_damage(tank, enemy, damage)
					objc.flash_entity_for_damage(enemy, 250)
				end
			end
		end
	end
	return damage
end
