
local PISTOL_RANGE_SQUARE = 4368100
local PISTOL_ATTACK_ACTIVE_KEY = 'pistolAttackActiveKey'
local LONGSHOT_INITIAL_COOLDOWN_KEY = 'longshotInitialCooldownKey'
local PISTOL_IDLE_STATE ='gunIdle'
local PISTOL_ATTACK_STATE ='gunAttack'
local PISTOL_ATTACK_AIR_STATE ='gunAttackAir'
local PISTOL_TO_RIFLE_STATE ='gunTransToRifle'
local RIFLE_TO_PISTOL_STATE ='gunTransToRifle'
local RIFLE_IDLE_STATE ='idleAnimationName'
local RIFLE_ATTACK_STATE ='attackAnimationName'
local RIFLE_ATTACK_AIR_STATE ='attackAirAnimationName'
local GUN_SWAP_DURATION = 1000
local CURRENT_TARGET_KEY = 'currentTargetKey'


B_Longshot_spawnTrigger = function(entity)
    local cooldown = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
    objc.set_local_value(entity, LONGSHOT_INITIAL_COOLDOWN_KEY, cooldown)
end

B_Longshot_onTargetTrigger = function(entity, entityToTarget)
    objc.set_local_value(entity, CURRENT_TARGET_KEY, entityToTarget)
    B_Longshot_checkRange(entity, entityToTarget)
end

B_Longshot_tickTrigger = function(entity)
    local pistolActive = objc.get_local_value(entity, PISTOL_ATTACK_ACTIVE_KEY)
    local currentTarget = objc.get_local_value(entity, CURRENT_TARGET_KEY)
    if currentTarget ~= 0 then
        if objc.get_entity_value_for_key(currentTarget, ENTITY_VALUE_KEY_CURRENT_HEALTH) > 0 then
            B_Longshot_checkRange(entity, currentTarget)
        elseif (pistolActive == 1) then
            B_Longshot_swapToRifle(entity, target)
        end
    end
    
end

B_Longshot_damageTrigger = function(damage, targetEntity, sourceEntity)
    local pistolActive = objc.get_local_value(objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY), PISTOL_ATTACK_ACTIVE_KEY)
    if pistolActive == 0 then
        return damage
    else
        local pistolDamage = objc.get_combat_entity_custom_stat(objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY), CUSTOM_ENTITY_STAT_KEY_PISTOL_DAMAGE)
        return pistolDamage
    end
end

B_Longshot_checkRange = function(entity, target)
    local pistolActive = objc.get_local_value(entity, PISTOL_ATTACK_ACTIVE_KEY)
    local distanceSquare = objc.quick_unit_distance_square(entity,target)
    if pistolActive == 0  then
        if distanceSquare <= PISTOL_RANGE_SQUARE and objc.is_ground_unit(target) and objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0 then
            B_Longshot_swapToPistol(entity, target)
        end
    else
        if distanceSquare > PISTOL_RANGE_SQUARE or objc.is_ground_unit(target) == false then
            B_Longshot_swapToRifle(entity, target)
        end        
    end
end

B_Longshot_swapToPistol = function(entity, target)
    objc.set_local_value(entity, PISTOL_ATTACK_ACTIVE_KEY, 1)
    local cooldown = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_PISTOL_COOLDOWN)
    objc.set_entity_value_for_key(entity,ENTITY_VALUE_KEY_ATTACK_RATE, cooldown)
    local pistolAnimations = {}
    pistolAnimations[ANIMATION_STATE_IDLE] = PISTOL_IDLE_STATE
    pistolAnimations[ANIMATION_STATE_ATTACK_AIR] = PISTOL_ATTACK_AIR_STATE
    pistolAnimations[ANIMATION_STATE_ATTACK_GROUND] = PISTOL_ATTACK_STATE
    objc.update_animation_state(entity, pistolAnimations)
    objc.play_override_animation_key(entity, 'gunTransToGun', GUN_SWAP_DURATION)
    objc.idle_entity(entity, GUN_SWAP_DURATION)
end

B_Longshot_swapToRifle = function(entity, target)
    objc.set_local_value(entity, PISTOL_ATTACK_ACTIVE_KEY, 0)
    local cooldown = objc.get_local_value(entity, LONGSHOT_INITIAL_COOLDOWN_KEY)
    objc.set_entity_value_for_key(entity,ENTITY_VALUE_KEY_ATTACK_RATE, cooldown)            
    local rifleAnimations = {}
    rifleAnimations[ANIMATION_STATE_IDLE] = RIFLE_IDLE_STATE
    --This is a workaround to prevent the longshot from aiming at the sky when we switch it back to rifle
    rifleAnimations[ANIMATION_STATE_ATTACK_AIR] = RIFLE_ATTACK_STATE
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'B_Longshot.lua', 'B_Longshot_switchAirAnimBackTrigger')
    rifleAnimations[ANIMATION_STATE_ATTACK_GROUND] = RIFLE_ATTACK_STATE
    objc.update_animation_state(entity, rifleAnimations)
    objc.play_override_animation_key(entity, 'gunTransToRifle', GUN_SWAP_DURATION)
    objc.idle_entity(entity, GUN_SWAP_DURATION)
end

B_Longshot_switchAirAnimBackTrigger = function(entity)
    local cooldown = objc.get_local_value(entity, LONGSHOT_INITIAL_COOLDOWN_KEY)
    local animSwitchTimer = objc.tick_timer(entity, 'switchAnimBackTrigger', objc.divide_with_floor(cooldown, 100))
    if animSwitchTimer == 0 then
        local rifleAnimations = {}
        rifleAnimations[ANIMATION_STATE_ATTACK_AIR] = RIFLE_ATTACK_AIR_STATE
        objc.update_animation_state(entity, rifleAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'B_Longshot_switchAirAnimBackTrigger')
    end
end


