local HAS_INIT = 'hasInitKey'
local IS_SINGLE_TARGET = 'isSingleTargetKey'
local PRIMARY_TARGET = 'primaryTargetKey'
local SPAWN_TIME = 30

local TECH_GIRL_REGULAR_IDLE_ANIMATION_NAME = 'B_TechGirl_Idle_01'
local TECH_GIRL_ULTIMATE_IDLE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local TECH_GIRL_REGULAR_URGENT_IDLE_ANIMATION_NAME = 'B_TechGirl_Idle_01'
local TECH_GIRL_ULTIMATE_URGENT_IDLE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local TECH_GIRL_REGULAR_WAVE_ANIMATION_NAME = 'B_TechGirl_WaveTimer_01'
local TECH_GIRL_ULTIMATE_WAVE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local HEALTH_TICK_KEY = 'healthTickAmountKey'
local DURATION = 250

local DAMAGE_TIMER = 'damageDelayTimerKey'
local HAS_DAMAGED = 'hasDamagedKey'
local DAMAGE_DELAY = 7
local DAMAGE_RADIUS = 1200
local LANDING_DAMAGE = 30

B_Mechazor_spawnTrigger = function(entity)
    local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local healthTickAmount = objc.divide_with_ceiling(maxHealth, DURATION)
    objc.set_local_value(entity, HEALTH_TICK_KEY, healthTickAmount)

    objc.set_local_value(entity, IS_DEBUFF_IMMUNE_KEY, 1)

    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_HOLD_POSITION_COUNT_FOR_SPAWNING, SPAWN_TIME)

    local updatedAnimations = {}
    updatedAnimations[COMMANDER_ANIMATION_STATE_IDLE] = TECH_GIRL_ULTIMATE_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_URGENT_IDLE] = TECH_GIRL_ULTIMATE_URGENT_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_WAVE] = TECH_GIRL_ULTIMATE_WAVE_ANIMATION_NAME
    local teamNumber = objc.get_team_number(entity)
    objc.update_commander_animation_state(teamNumber, updatedAnimations)
end

B_Mechazor_tickTrigger = function(entity)
    local hasDamaged = objc.get_local_value(entity, HAS_DAMAGED)
    if hasDamaged == 0 then
        local damageTimer = objc.tick_timer(entity, DAMAGE_TIMER, DAMAGE_DELAY)
        if damageTimer == 0 then
            local enemyTeamNumber = objc.get_enemy_team_number(entity)
            local enemiesInRadius = objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyTeamNumber)
            for _, enemy in pairs(enemiesInRadius) do
                if objc.is_normal_unit(enemy) then
                    objc.inflict_normal_damage(entity, enemy, LANDING_DAMAGE)
                    objc.flash_entity_for_damage(enemy, 250)
                end
            end
        end
    end
    local healthTick = objc.get_local_value(entity, HEALTH_TICK_KEY)
    objc.inflict_direct_damage(entity, entity, healthTick)
end

B_Mechazor_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, IS_SINGLE_TARGET, 0)
    local targetPlaneOfExistence = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
    if targetPlaneOfExistence == NUM_PLANE_OF_EXISTENCE_AIR then
        objc.set_local_value(sourceEntity, PRIMARY_TARGET, targetEntity)
        objc.set_local_value(sourceEntity, IS_SINGLE_TARGET, 1)
    end
end

B_Mechazor_damageTrigger = function(damage, targetEntity, sourceEntity)
    local mechazor = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local isSingleTarget = objc.get_local_value(mechazor, IS_SINGLE_TARGET)
    if isSingleTarget == 1 then
        local primaryTarget = objc.get_local_value(mechazor, PRIMARY_TARGET)
        if primaryTarget == targetEntity then
            return damage
        else
            return 0
        end
    else
        return damage
    end
end

B_Mechazor_deathTrigger = function(entity)
    local updatedAnimations = {}
    updatedAnimations[COMMANDER_ANIMATION_STATE_IDLE] = TECH_GIRL_REGULAR_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_URGENT_IDLE] = TECH_GIRL_REGULAR_URGENT_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_WAVE] = TECH_GIRL_REGULAR_WAVE_ANIMATION_NAME
    local teamNumber = objc.get_team_number(entity)
    objc.update_commander_animation_state(teamNumber, updatedAnimations)
end
