local IS_PAST_BASE = 'isPastBaseKey'
local HEAL_AMOUNT = 'healAmountKey'
local SCAN_RADIUS = 7000
local CURRENT_TARGET_KEY = 'currentTargetKey'
local COOLDOWN_KEY = 'cooldownKey'
local READY_TO_HEAL_KEY = 'readyToHealKey'



B_MedivacCampaign_spawnTrigger = function(entity)
   local healAmount = 250
   objc.set_local_value(entity, HEAL_AMOUNT, healAmount)
   objc.set_local_value(entity, READY_TO_HEAL_KEY, 1)
end

--[[
This onAttackTrigger causes the B_MedivacCampaign to target the most injured unit within attack range,
if none it finds the closest most injured nearby unit, and then heals it to either the max health
or the current health plus heal amount, whichever is lower.
--]]


B_MedivacCampaign_onTargetTrigger = function(entity, entityToTarget)
   local currentTarget = objc.get_local_value(entity, CURRENT_TARGET_KEY)
   local allyToHeal = B_MedivacCampaign_findHealTarget(entity, currentTarget, SCAN_RADIUS)
   if allyToHeal ~= currentTarget then
      B_MedivacCampaign_setNewTarget(entity, allyToHeal)
      objc.set_local_value(entity, CURRENT_TARGET_KEY, allyToHeal)
   end
end

B_MedivacCampaign_onAttackTrigger = function(sourceEntity, targetEntity)
   local allyToHeal = targetEntity
   local healAmount = objc.get_local_value(sourceEntity, HEAL_AMOUNT)
   local currentHealth = objc.get_entity_value_for_key(allyToHeal, ENTITY_VALUE_KEY_CURRENT_HEALTH)
   local maxHealth = objc.get_entity_value_for_key(allyToHeal, ENTITY_VALUE_KEY_MAX_HEALTH)
   local missingHealth = B_MedivacCampaign_calculateMissingHealth(allyToHeal)

   objc.set_entity_value_for_key(allyToHeal, ENTITY_VALUE_KEY_CURRENT_HEALTH, math.min(currentHealth + healAmount, maxHealth))
   objc.play_vfx(allyToHeal, 'Status_healing.lvl', 'node_status')
   objc.set_local_value(sourceEntity, READY_TO_HEAL_KEY, 0)
   objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'B_MedivacCampaign.lua', 'B_MedivacCampaign_readyToAttackTrigger')
end


B_MedivacCampaign_findHealTarget = function(sourceEntity, currentTarget, range)
   local allyToHeal = currentTarget
   local teamNumber = objc.get_team_number(sourceEntity)
   if allyToHeal == 0 or objc.get_entity_value_for_key(allyToHeal, ENTITY_VALUE_KEY_CURRENT_HEALTH) <= 0 then
      allyToHeal = B_MedivacCampaign_findClosestAllyWithinXY(sourceEntity, SCAN_RADIUS*2, SCAN_RADIUS*2)
   end
   if allyToHeal ~= 0 then
      local currentTargetMissingHealth = B_MedivacCampaign_calculateMissingHealth(allyToHeal)
      local teamNumber = objc.get_team_number(sourceEntity)
      local myLocationX = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_POSITION_X)
      local myLocationY = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_POSITION_Y)
      for _, ally in pairs(objc.list_entities_in_rectangle(myLocationX, myLocationY, SCAN_RADIUS*2, SCAN_RADIUS*2, teamNumber)) do
         if B_MedivacCampaign_isValidTarget(sourceEntity, ally) then
            local allymissingHealth = B_MedivacCampaign_calculateMissingHealth(ally)
            local allyHealth = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_CURRENT_HEALTH)
            if allymissingHealth > currentTargetMissingHealth and allyHealth > 0 then
               allyToHeal = ally
               currentTargetMissingHealth = allymissingHealth
            end
         end
      end
   end
   return allyToHeal
end

B_MedivacCampaign_calculateMissingHealth = function(entityToEvaluate)
   local currentHealth = objc.get_entity_value_for_key(entityToEvaluate, ENTITY_VALUE_KEY_CURRENT_HEALTH)
   local maxHealth = objc.get_entity_value_for_key(entityToEvaluate, ENTITY_VALUE_KEY_MAX_HEALTH)
   local missingHealth = maxHealth - currentHealth
   return missingHealth
end


B_MedivacCampaign_tickTrigger = function(entity)
   --Check targets and change if we need to
   local readyToHeal = objc.get_local_value(entity, READY_TO_HEAL_KEY)
   if readyToHeal == 1 then
      local currentTarget = objc.get_local_value(entity, CURRENT_TARGET_KEY)
      local allyToHeal = B_MedivacCampaign_findHealTarget(entity, currentTarget, SCAN_RADIUS)
      if allyToHeal ~= currentTarget then
        B_MedivacCampaign_setNewTarget(entity, allyToHeal)
        objc.set_local_value(entity, CURRENT_TARGET_KEY, allyToHeal)
      end
   end
end

B_MedivacCampaign_findClosestAllyWithinXY = function(entity, x, y)
    local teamNumber = objc.get_team_number(entity)
    local myLocationX = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
    local myLocationY = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y)
    local allies = objc.list_entities_in_rectangle(myLocationX, myLocationY, x, y, teamNumber)
    local closestEntity = 0
    local closestDist = 99999999
    for _, ally in pairs(allies) do
         if B_MedivacCampaign_isValidTarget(entity, ally) then
            local thisDist = objc.quick_unit_distance_square(entity, ally)
            if thisDist < closestDist and objc.is_targetable(ally) then
                closestDist = thisDist
                closestEntity = ally
            end
        end
    end
    return 0
end

B_MedivacCampaign_isValidTarget = function(entity, ally)
    if (objc.is_normal_unit(ally) or objc.is_commander(ally)) and entity ~= ally then
        return true
    end
    return false
end

B_MedivacCampaign_readyToAttackTrigger = function(entity)
    local cooldown = objc.get_local_value(entity, COOLDOWN_KEY)
    local cooldownTimer = objc.tick_timer(entity, 'readyToAttackTrigger', objc.divide_with_floor(cooldown, 100))
    if cooldownTimer == 0 then
        objc.set_local_value(entity, READY_TO_HEAL_KEY, 1)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'B_MedivacCampaign_readyToAttackTrigger')
    end
end
