local CHECK_RADIUS = 6000
local HAS_INIT = 'hasInit'
local HAS_FOUND_MINE = 'hasFoundMineKey'
local ANIMATION_TIMER_KEY = 'animationTimerKey'
local INCOME_INCREASE_MILLIS = 250

local MOTION_STATE_DICT = 'motionStateDictKey'
local TARGET_MINE = 'targetMineKey'

local SPEED_TO_MOVE = 90
local WORK_ANIM_TIMER_KEY = 'workAnimTimerKey'
local HAS_STARTED_ANIM = 'hasStartedAnimKey'
local NUMBER_OF_FRAMES_IN_WORK_ANIMATION = 90
local WORK_ANIMATION_NAME_KEY = 'B_Builder_Battle_Work_01'

local LIFETIME_ROUNDS = 200
local HEALTH_TICK = 'healthTickAmount'

B_Mule_spawnTrigger = function(entity)
    local playerNumber = objc.get_player_number(entity)
    local currentIncome = objc.get_income_generation_frequency_in_milliseconds(playerNumber)
    objc.set_income_generation_frequency_in_milliseconds(playerNumber, currentIncome + INCOME_INCREASE_MILLIS)

    local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local healthTick = objc.divide_with_floor(maxHealth, LIFETIME_ROUNDS)
    objc.set_local_value(entity, HEALTH_TICK, healthTick)
end

B_Mule_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        B_Mule_findTargetMine(entity)
        objc.set_local_value(entity, HAS_INIT, 1)
    end

    local hasFoundMine = objc.get_local_value(entity, HAS_FOUND_MINE)
    if hasFoundMine == 1 then
        local numRounds = objc.convert_frames_to_rounds(NUMBER_OF_FRAMES_IN_WORK_ANIMATION)
        local animTimer = objc.tick_timer(entity, ANIMATION_TIMER_KEY, numRounds)
        if animTimer == 0 then
            local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_WORK_ANIMATION)
            objc.play_override_animation(entity, WORK_ANIMATION_NAME_KEY, objc.subrounds_to_milliseconds(numberOfSubrounds))
        end
    end

    local healthTick = objc.get_local_value(entity, HEALTH_TICK)
    local currentHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, currentHealth - healthTick)
end

B_Mule_findTargetMine = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local objectives = objc.list_entities(teamNumber)
    local closestMine = entity
    local closestDistance = 99999999
    local entityPosition = objc.get_entity_position(entity)
    for _, objective in pairs(objectives) do
        if objc.is_crystal_mine(objective) then
            local identifier = objc.get_entity_value_for_key(objective, ENTITY_VALUE_KEY_IDENTIFIER)
            local thisDistance = math.abs(objc.unit_distance_with_offset(entity, 0, 0, objective, 0, 0))
            if thisDistance < closestDistance and thisDistance < CHECK_RADIUS then
                closestMine = objective
                closestDistance = thisDistance
            end
        end
    end

    if closestMine ~= entity then
        objc.set_local_value(entity, TARGET_MINE, closestMine)
        B_Mule_moveToMine(entity)
    end
end

B_Mule_moveToMine = function(entity)
    local targetMine = objc.get_local_value(entity, TARGET_MINE)
    if targetMine ~= 0 then
        local velocity = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
        local newMotionState = objc.move_entity_to_entity_with_motion(entity, targetMine, MOTION_STATE_WALK, SPEED_TO_MOVE)
        objc.set_local_value(entity, MOTION_STATE_DICT, newMotionState)
    end
end

B_Mule_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local oldMotionState = objc.get_local_value(sourceEntity, MOTION_STATE_DICT)
    if oldMotionState ~= 0 then
        objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, oldMotionState)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
        objc.set_local_value(sourceEntity, HAS_FOUND_MINE, 1)

        local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_WORK_ANIMATION)
        objc.play_override_animation(sourceEntity, WORK_ANIMATION_NAME_KEY, objc.subrounds_to_milliseconds(numberOfSubrounds))

        objc.set_local_value(sourceEntity, MOTION_STATE_DICT, 0)
    end
end

B_Mule_deathTrigger = function(entity)
    local playerNumber = objc.get_player_number(entity)
    local currentIncome = objc.get_income_generation_frequency_in_milliseconds(playerNumber)
    objc.set_income_generation_frequency_in_milliseconds(playerNumber, currentIncome - INCOME_INCREASE_MILLIS)
end