local SCAN_TIME = 10
local PILOT_BREAKPOINT = 4
local PILOT_COUNTED_KEY = 'pilotCounterKey'
local PILOT_GATHER_ENTITY = 'pilotGatherTargetEntity'
local PILOT_GATHER_TIME = 8
local OFFSET_FROM_BASE = 3000
local IDLE_OFFSET_FROM_BASE = 500
local NUMBER_OF_FRAMES_IN_TRANSFORM_ANIMATION = 30
local TRANSFORM_ANIMATION_NAME = 'B_Wolfpack_Pilot_Battle_Summons'
local MECH_SPAWNING_TIME = 20
local MECH_SPAWN_TIMER_KEY = 'mechSpawnTimerKey'
local DID_MECH_SPAWN_KEY = 'didMechSpawnKey'
local HIGHEST_TIER_KEY = 'highestTierPilotKey'
local IS_BEING_IDLED = 'isBeingIdledKey'
local IS_RETREATING = 'isRetreatingKey'
local IDLE_TICK = 10
local IDLE_Y_OFFSET = 500
local SHOULD_CHECK_KEY = 'shouldCheckForPilotsKey'

B_Pilot_tickTrigger = function(entity)
    --We have to use this pattern to prevent an error from occurring when trying to move a unit on the first round of its existence
    local shouldCheckForPilots = objc.get_local_value(entity, SHOULD_CHECK_KEY)
    if shouldCheckForPilots == 1 then
        local didMechSpawn = objc.get_local_value(entity, DID_MECH_SPAWN_KEY)
        if didMechSpawn == 0 then
            local teamNumber = objc.get_team_number(entity)
            local playerNumber = objc.get_player_number(entity)
            local allyList = objc.list_entities(teamNumber)
            local pilotIdentifier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_IDENTIFIER)
            local scanTimer = objc.tick_timer(entity, 'scanTimer', SCAN_TIME)

            if scanTimer == 0 then
                local pilotCounter = 0
                local pilotArray = {}

                local listNumber = listCount(allyList)

                for _, ally in pairs(allyList) do

                    if B_Pilot_isEntityPilotOfAnyTier(ally) and objc.get_local_value(ally, PILOT_COUNTED_KEY) == 0 then
                        pilotCounter = pilotCounter + 1
                        table.insert(pilotArray, ally)

                        if pilotCounter >= PILOT_BREAKPOINT then
                            for _, pilot in pairs(pilotArray) do
                                local positionX = 0
                                local basePosition = objc.get_base_position(teamNumber)

                                if teamNumber == 1 then
                                    positionX = basePosition[1] + OFFSET_FROM_BASE
                                elseif teamNumber == 2 then
                                    positionX = basePosition[1] - OFFSET_FROM_BASE
                                else
                                    objc.scripting_error('Encountered unknown player number ' .. teamNumber)
                                end

                                local isBeingIdled = objc.get_local_value(pilot, IS_BEING_IDLED)
                                if isBeingIdled == 1 then
                                    objc.set_local_value(pilot, IS_BEING_IDLED, 0)
                                    objc.remove_trigger(pilot, TRIGGER_NAME_TICK_TRIGGER, 'B_Pilot_idleWhileWaiting')
                                    objc.unidle_entity(pilot)
                                end

                                local spawnedEntity = objc.spawn_at_point(PILOT_GATHER_ENTITY, 1, playerNumber, positionX, 0, 0, 0)
                                --set attack range to low to make the pilot get close to the destination
                                objc.set_entity_value_for_key(pilot, ENTITY_VALUE_KEY_ATTACK_RANGE, 200)
                                objc.set_entity_value_for_key(pilot, ENTITY_VALUE_KEY_HOLD_POSITION_COUNT_FOR_SPAWNING, 0)
                                local distance = objc.unit_distance_with_offset(pilot, 0, 0, spawnedEntity[1], 0, 0)
                                local gatherSpeed = math.abs(objc.divide_with_ceiling(distance, PILOT_GATHER_TIME))
                                local isRetreating = objc.get_local_value(pilot, IS_RETREATING)
                                if isRetreating == 1 then
                                    objc.remove_trigger(pilot, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'B_Pilot_retreatingTargetingCompletionTrigger')
                                    objc.set_local_value(pilot, IS_RETREATING, 0)
                                end
                                objc.add_trigger(pilot, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'B_Pilot.lua', 'B_Pilot_gatheringTargetingCompletionTrigger')
                                objc.move_entity_to_entity_with_motion(pilot, spawnedEntity[1], MOTION_STATE_LEAP, gatherSpeed)
                                objc.set_local_value(pilot, IS_RETREATING, 0)
                                pilotCounter = 0
                                objc.set_local_value(pilot, PILOT_COUNTED_KEY, 1)
                            end

                            local spawnEntity = objc.spawn('B_PilotMechSpawnScriptEntity', 1, playerNumber)
                            local highestTier = objc.get_local_value(entity, HIGHEST_TIER_KEY)
                            objc.set_local_value(spawnEntity[1], HIGHEST_TIER_KEY, highestTier)
                        end
                    end
                end
            end
        else
            local mechSpawnTimer = objc.tick_timer(entity, MECH_SPAWN_TIMER_KEY, MECH_SPAWNING_TIME)
            if mechSpawnTimer == 0 then
                objc.sacrifice_entity(entity)
            end
        end
    else
        objc.set_local_value(entity, SHOULD_CHECK_KEY, 1)
    end
end

B_Pilot_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local isBeingIdled = objc.get_local_value(targetEntity, IS_BEING_IDLED)
    if isBeingIdled == 0 then
        local currentHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
        if (currentHealth - damage) <= 0 then
            objc.set_local_value(targetEntity, IS_RETREATING, 1)

            local playerNumber = objc.get_player_number(targetEntity)
            local teamNumber = objc.get_team_number(targetEntity)
            local positionX = 0
            local positionY = objc.roll_number(-IDLE_Y_OFFSET, IDLE_Y_OFFSET)
            local basePosition = objc.get_base_position(teamNumber)

            if teamNumber == 1 then
                positionX = basePosition[1] + IDLE_OFFSET_FROM_BASE
            elseif teamNumber == 2 then
                positionX = basePosition[1] - IDLE_OFFSET_FROM_BASE
            else
                objc.scripting_error('Encountered unknown player number ' .. teamNumber)
            end

            objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, 200)
            local spawnedEntity = objc.spawn_at_point(PILOT_GATHER_ENTITY, 1, playerNumber, positionX, positionY, 0, 0)
            objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_HOLD_POSITION_COUNT_FOR_SPAWNING, 0)
            local distance = objc.unit_distance_with_offset(targetEntity, 0, 0, spawnedEntity[1], 0, 0)
            local gatherSpeed = math.abs(objc.divide_with_ceiling(distance, PILOT_GATHER_TIME))
            objc.add_trigger(targetEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'B_Pilot.lua', 'B_Pilot_retreatingTargetingCompletionTrigger')
            objc.move_entity_to_entity_with_motion(targetEntity, spawnedEntity[1], MOTION_STATE_LEAP, gatherSpeed)
            objc.set_targetable(targetEntity, false)
            return 0
        end
    end

    local isRetreating = objc.get_local_value(targetEntity, IS_RETREATING)
    if isRetreating == 1 then
        return 0
    end

    return damage
end

B_Pilot_isEntityPilotOfAnyTier = function(entity)
    local baseIdentifier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
    local highestTier = objc.get_local_value(entity, HIGHEST_TIER_KEY)
    if highestTier == 0 then
        highestTier = '0'
    end

    local thisTier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
    if baseIdentifier == "B_Pilot" then
        if tonumber(thisTier) > tonumber(highestTier) then
            objc.set_local_value(entity, HIGHEST_TIER_KEY, thisTier)
        end
        return true
    end

    return false
end

B_Pilot_gatheringTargetingCompletionTrigger = function(sourceEntity, targetEntity)
    local teamNumber = objc.get_team_number(sourceEntity)
    local pilotPosition = objc.get_entity_position(sourceEntity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_TRANSFORM_ANIMATION)
    objc.set_targetable(sourceEntity, false)
    local noDeathAnimations = {}
    noDeathAnimations[ANIMATION_STATE_DEATH] = ''
    objc.update_animation_state(sourceEntity, noDeathAnimations)
    objc.idle_entity(sourceEntity, 2000)
    objc.play_override_animation(sourceEntity, TRANSFORM_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
    objc.set_local_value(sourceEntity, DID_MECH_SPAWN_KEY, 1)
end

B_Pilot_retreatingTargetingCompletionTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, IS_RETREATING, 0)
    objc.set_local_value(sourceEntity, IS_BEING_IDLED, 1)
    objc.idle_entity(sourceEntity, IDLE_TICK * 100)
    objc.set_targetable(sourceEntity, true)
    objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'B_Pilot.lua', 'B_Pilot_idleWhileWaiting')
end

B_Pilot_idleWhileWaiting = function(entity)
    local idleTimer = objc.tick_timer(entity, 'idleTickTimer', IDLE_TICK)
    if idleTimer == 0 then
        objc.idle_entity(entity, (IDLE_TICK + 1) * 100)
    end
end
