local SPAWN_ANIM_DUMMY_TIME = 15
local SPAWN_ANIM_TIMER_KEY = 'spawnAnimTimerKey'
local HAS_SPAWNED_DUMMY_KEY = 'hasSpawnedDummyKey'
local SPAWNED_ANIM_DUMMY = 'spawnAnimDummyKey'
local SPAWN_PLACEHOLDER_KEY = 'B_PilotMechSpawn'
local MECH_SPAWN_TIME = 38
local HAS_MECHAZOR_ALREADY_SPAWNED_KEY = 'hasMechazorAlreadySpawned'
local MECH_ID = 'B_PilotMech'
local LANDING_EFFECTS_TIME = 15
local HAS_LANDING_EFFECTS_PLAYED_KEY = 'hasLandingEffectsPlayed'
local LANDING_EFFECT_KEY = 'landingEffect'
local OFFSET_FROM_BASE = 3000
local LIFETIME = 150
local HIGHEST_TIER_KEY = 'highestTierPilotKey'

--[[
This TickTrigger spawns a Mechazor and plays the appropriate spawning VFX where
the Mechazor spawns, and then kills the VFX after a set amount of time.
--]]
B_PilotMechSpawnScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)

   local spawnAnimTimer = objc.tick_timer(entity, SPAWN_ANIM_TIMER_KEY, SPAWN_ANIM_DUMMY_TIME)
   local hasSpawnedDummy = objc.get_local_value(entity, HAS_SPAWNED_DUMMY_KEY)

   local mechSpawnTimer = objc.tick_timer(entity, 'spawnTimer', MECH_SPAWN_TIME)
   local hasMechazorAlreadySpawned = objc.get_local_value(entity, HAS_MECHAZOR_ALREADY_SPAWNED_KEY)
   
   local landingTimer = objc.tick_timer(entity, 'landingTimer', LANDING_EFFECTS_TIME)
   local hasLandingEffectsPlayed = objc.get_local_value(entity, HAS_LANDING_EFFECTS_PLAYED_KEY)

   local killSwitch = objc.tick_timer(entity, 'killSwitch', LIFETIME)
   local killLandingEffectsTimer = objc.tick_timer(entity, 'killLandingEffectsTimer', 80)  -- this needs to be lower than the killSwitch timer
   
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)

   local positionX = 0
   local positionY = 0
   local directionX = 1
   local directionY = 0

   local basePosition = objc.get_base_position(teamNumber)

   if teamNumber == 1 then
      positionX = basePosition[1] + OFFSET_FROM_BASE
      directionX = 1
   elseif teamNumber == 2 then
      positionX = basePosition[1] - OFFSET_FROM_BASE
      directionX = -1
   else
      objc.scripting_error('Encountered unknown player number ' .. teamNumber)
   end

   if spawnAnimTimer == 0 and hasSpawnedDummy == 0 then
      local highestTier = objc.get_local_value(entity, HIGHEST_TIER_KEY)
      if highestTier == 0 then
         highestTier = ''
      end
      local spawnedDummy = objc.spawn_at_point(SPAWN_PLACEHOLDER_KEY .. highestTier, 1, playerNumber, positionX, positionY, directionX, directionY)
      objc.set_local_value(entity, SPAWNED_ANIM_DUMMY, spawnedDummy[1])
      objc.set_local_value(entity, HAS_SPAWNED_DUMMY_KEY, 1)
   end

   if mechSpawnTimer == 0 and hasMechazorAlreadySpawned == 0 then
      local highestTier = objc.get_local_value(entity, HIGHEST_TIER_KEY)
      if highestTier == 0 then
         highestTier = ''
      end
      objc.spawn_at_point(MECH_ID .. highestTier, 1, playerNumber, positionX, positionY, directionX, directionY)
      objc.set_local_value(entity, HAS_MECHAZOR_ALREADY_SPAWNED_KEY, 1)
      local spawnAnimDummy = objc.get_local_value(entity, SPAWNED_ANIM_DUMMY)
      if spawnAnimDummy ~= 0 then
         objc.sacrifice_entity(spawnAnimDummy)
      end
   end
   
   if killSwitch == 0 then
      objc.sacrifice_entity(entity)
   end

   return 0
end
