local SPAWN_ANIM_DURATION_SUBROUNDS = 45 --45
local RESEARCHBOT_START_ANIMATION_NAME = 'B_ResearchLab_Spawn_01'

local HAS_UNFROZEN = 'hasUnFrozenKey'
local CARD_FREEZE_ROUNDS = 70
local ORIGINAL_CARD_DRAW_SPEED = 'originalCardDrawSpeed'
local TECH_IDLE_ANIMATION_STATE = 'techIdle'
local NORMAL_IDLE_ANIMATION_STATE = 'normalIdle'
local TECH_START_ANIMATION_NAME = 'B_ResearchLab_Energy_Up_01'
local FRAMES_IN_TECH_START_ANIMATION = 30
local TECH_FINISH_ANIMATION_NAME = 'B_ResearchLab_Energy_Down_01'
local FRAMES_IN_TECH_FINISH_ANIMATION = 60
local HAS_PLAYED_START_ANIM = 'hasPlayedStartAnimKey'
local SHOULD_CHECK_TECH = 'shouldCheckTechKey'

local BUILDING_STATE_KEY = 'buildingStateKey'

local READY_TO_TECH_VALUE = 0
local CURRENTLY_TECHING_VALUE = 1
local DONE_TECHING_VALUE = 2

local PLAYER_TECH_STATE_NOT_TECHING = 0
local PLAYER_TECH_STATE_TECHING_FROM_BUTTON = 1
local PLAYER_TECH_STATE_TECHING_FROM_LUA = 2
local PLAYER_TECH_STATE_MAX_TECH_STATE = 3

local ROUNDS_COUNT_DOWN_KEY = 'roundsCountdownKey'

local HAS_INIT = 'hasInitKey'
local SPAWN_ANIMATION_STATE = 'spawnAnimName'
local IDLE_ANIMATION_STATE = 'normalIdle'
local SWAP_IDLE_ANIMATION_COUNT = 6

B_ResearchBot_spawnTrigger = function(entity)
   BuildingAbility_setNotSolidIfNextToTwoBoundaries(entity)

   local updatedAnimations = {}
   updatedAnimations[ANIMATION_STATE_IDLE] = SPAWN_ANIMATION_STATE
   objc.update_animation_state(entity, updatedAnimations)
   objc.idle_entity(entity, 999999)

   objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'B_ResearchBot.lua', 'B_ResearchBot_swapIdleTrigger')

   --objc.play_override_animation(entity, RESEARCHBOT_START_ANIMATION_NAME, objc.subrounds_to_milliseconds(SPAWN_ANIM_DURATION_SUBROUNDS))

   local playerNumber = objc.get_player_number(entity)
   local playerTechState = objc.get_tech_up_state(playerNumber)

   if playerTechState == PLAYER_TECH_STATE_MAX_TECH_STATE then 
      objc.set_local_value(entity, BUILDING_STATE_KEY, DONE_TECHING_VALUE)
   end

   if playerTechState == PLAYER_TECH_STATE_TECHING_FROM_BUTTON then
      objc.set_local_value(entity, BUILDING_STATE_KEY, READY_TO_TECH_VALUE)
   end

   if playerTechState == PLAYER_TECH_STATE_TECHING_FROM_LUA then
      objc.set_local_value(entity, BUILDING_STATE_KEY, READY_TO_TECH_VALUE)
   end

   if playerTechState == PLAYER_TECH_STATE_NOT_TECHING then
      -- looks like the player can tech up!
      objc.set_rounds_until_tech_up(playerNumber, CARD_FREEZE_ROUNDS)
      objc.pause_card_draw(playerNumber)
      objc.set_local_value(entity, BUILDING_STATE_KEY, CURRENTLY_TECHING_VALUE)
      objc.set_local_value(entity, ROUNDS_COUNT_DOWN_KEY, CARD_FREEZE_ROUNDS - SWAP_IDLE_ANIMATION_COUNT - 2)
   end
end

B_ResearchBot_tickTrigger = function(entity)
   local shouldCheckTech = objc.get_local_value(entity, SHOULD_CHECK_TECH)
   if shouldCheckTech == 1 then
      local playerNumber = objc.get_player_number(entity)
      local buildingState = objc.get_local_value(entity, BUILDING_STATE_KEY)
      if buildingState == READY_TO_TECH_VALUE then
         local playerTechState = objc.get_tech_up_state(playerNumber)
         if playerTechState == PLAYER_TECH_STATE_MAX_TECH_STATE then 
            objc.set_local_value(entity, BUILDING_STATE_KEY, DONE_TECHING_VALUE)
         end

         if playerTechState == PLAYER_TECH_STATE_NOT_TECHING then
            -- looks like the player can tech up!
            objc.set_rounds_until_tech_up(playerNumber, CARD_FREEZE_ROUNDS)
            objc.pause_card_draw(playerNumber)
            objc.set_local_value(entity, BUILDING_STATE_KEY, CURRENTLY_TECHING_VALUE)
            objc.set_local_value(entity, ROUNDS_COUNT_DOWN_KEY, CARD_FREEZE_ROUNDS - SWAP_IDLE_ANIMATION_COUNT - 2)

            local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TECH_START_ANIMATION)
            objc.play_override_animation(entity, TECH_START_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_IDLE] = TECH_IDLE_ANIMATION_STATE
            objc.update_animation_state(entity, updatedAnimations)
         end
      end

      if buildingState == CURRENTLY_TECHING_VALUE then
         local roundsCountdown = objc.get_local_value(entity, ROUNDS_COUNT_DOWN_KEY)
         local techCountDown = objc.tick_timer(entity, 'techCountDownKey', roundsCountdown)
         if techCountDown <= 0 then
            local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TECH_FINISH_ANIMATION)
            objc.play_override_animation(entity, TECH_FINISH_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_IDLE] = NORMAL_IDLE_ANIMATION_STATE
            objc.update_animation_state(entity, updatedAnimations)
            
            objc.set_local_value(entity, BUILDING_STATE_KEY, DONE_TECHING_VALUE)
            objc.resume_card_draw(playerNumber)
         end
      end
   end
end

B_ResearchBot_swapIdleTrigger = function(entity)
   local swapTimer = objc.tick_timer(entity, 'swapTimerKey', SWAP_IDLE_ANIMATION_COUNT)
   if swapTimer == 0 then
      local updatedAnimations = {}
      updatedAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_STATE
      objc.update_animation_state(entity, updatedAnimations)
      objc.set_local_value(entity, SHOULD_CHECK_TECH, 1)
      objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'B_ResearchBot_swapIdleTrigger')

      local playerNumber = objc.get_player_number(entity)
      local buildingState = objc.get_local_value(entity, BUILDING_STATE_KEY)

      if buildingState == CURRENTLY_TECHING_VALUE then
         local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TECH_START_ANIMATION)
         objc.play_override_animation(entity, TECH_START_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

         local updatedAnimations = {}
         updatedAnimations[ANIMATION_STATE_IDLE] = TECH_IDLE_ANIMATION_STATE
         objc.update_animation_state(entity, updatedAnimations)
      end
   end
end

B_ResearchBot_deathTrigger = function(entity)
    local playerNumber = objc.get_player_number(entity)
    local buildingState = objc.get_local_value(entity, BUILDING_STATE_KEY)
    if buildingState == CURRENTLY_TECHING_VALUE then
      objc.set_rounds_until_tech_up(playerNumber, -1)
      objc.resume_card_draw(playerNumber)
   end
end

B_ResearchBot_reachedOvertimeTrigger = function(entity)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'UnitAbilities.lua', 'UnitAbility_tickDownHealthTrigger')
end