local HAS_BEEN_SCREENED_KEY = 'hasBeenSmokescreened'
local CHECK_TIMER_KEY = 'unitCheckTimer'
local CHECK_TICK = 10
local SCREEN_RADIUS = 800
local CLOAKED_UNITS_LIST_KEY = 'cloakedUnitsListKey'
local UNTARGETABLE_SHADER_NAME = 'skinned_toon_freeze'
local ADD_SHADER_AGAIN_TRIGGER = 'B_Smokescreen_addShaderAgainAfterDelay'
local ADD_SHADER_AGAIN_KEY = 'addShaderAgainTimerKey'
local ADD_SHADER_AGAIN_DELAY = 10

B_Smokescreen_spawnTrigger = function(entity)
    B_Smokescreen_screenNearbyUnits(entity)
end

B_Smokescreen_tickTrigger = function(entity)
    local checkTime = objc.tick_timer(entity, CHECK_TIMER_KEY, CHECK_TICK)
    if checkTime == 0 then
        B_Smokescreen_screenNearbyUnits(entity)
    end
end


B_Smokescreen_deathTrigger = function(entity)
    local cloakedUnitsList = objc.get_local_value(entity, CLOAKED_UNITS_LIST_KEY)
    if cloakedUnitsList ~= 0 then
        for _, cloaked in pairs(cloakedUnitsList) do
            objc.set_targetable(cloaked, true)
            objc.remove_shader(cloaked, UNTARGETABLE_SHADER_NAME)
        end
    end
end

B_Smokescreen_screenNearbyUnits = function(entity)
        local teamNumber = objc.get_team_number(entity)
        local allies = objc.list_entities_in_radius(entity, SCREEN_RADIUS, teamNumber)

        objc.play_vfx(entity, 'B_BattleBard_Pulse.lvl', NODE_ORIGIN)

        local cloakedUnitsList = objc.get_local_value(entity, CLOAKED_UNITS_LIST_KEY)
        if cloakedUnitsList == 0 then
            cloakedUnitsList = {}
        end

        for _, ally in pairs(allies) do
            local plane = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
            local hasBeenScreened = objc.get_local_value(ally, HAS_BEEN_SCREENED_KEY)
            local allyID = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_IDENTIFIER)
            if plane == NUM_PLANE_OF_EXISTENCE_AIR and hasBeenScreened == 0 and allyID ~= 'B_Smokescreen' then
                table.insert(cloakedUnitsList, ally)
                objc.set_targetable(ally, false)
                objc.apply_shader(ally, UNTARGETABLE_SHADER_NAME)
                objc.add_trigger(ally, TRIGGER_NAME_TICK_TRIGGER, 'B_Smokescreen.lua', ADD_SHADER_AGAIN_TRIGGER)
                objc.set_local_value(ally, HAS_BEEN_SCREENED_KEY, 1)
            end
        end

        objc.set_local_value(entity, CLOAKED_UNITS_LIST_KEY, cloakedUnitsList)
end

B_Smokescreen_addShaderAgainAfterDelay = function(entity)
    local addShaderTimer = objc.tick_timer(entity, ADD_SHADER_AGAIN_KEY, ADD_SHADER_AGAIN_DELAY)
    if addShaderTimer == 0 then
        objc.apply_shader(entity, UNTARGETABLE_SHADER_NAME)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, ADD_SHADER_AGAIN_TRIGGER)
    end
end
