local SPAWN_IDENTIFIER = 'B_Rifleman'
local X_OFFSET = 300
local Y_OFFSET = 300

local SPAWN_ANIMATION_OVERRIDE_KEY = 'spawnTroopCarrier'
local FRAMES_IN_SPAWN_ANIMATION = 30
local ANIMATION_TIMER_KEY = 'overrideAnimTimerKey'
local IS_PLAYING_ANIMATION = 'isPlayingAnimation'
local SPAWN_OVERRIDE_TRIGGER = 'B_TroopTransporter_spawnOverrideTickTrigger'


B_TroopTransporter_deathTrigger = function(entity)
    local playerNumber = objc.get_player_number(entity)
    local xDir = 1
    if playerNumber == 2 then
        xDir = -1
    end
    local tier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
    if tier == 0 then
        tier = ''
    end
    local entityPos = objc.get_entity_position(entity)
	local spawnedUnits = {}
    local topLeftSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] + X_OFFSET, entityPos[2] + Y_OFFSET, xDir, 0)
    local topRightSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] - X_OFFSET, entityPos[2] + Y_OFFSET, xDir, 0)
    local bottomLeftSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] + X_OFFSET, entityPos[2] - Y_OFFSET, xDir, 0)
    local bottomRightSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] - X_OFFSET, entityPos[2] - Y_OFFSET, xDir, 0)
    local topSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1], entityPos[2] + Y_OFFSET, xDir, 0)
    local bottomSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1], entityPos[2] - Y_OFFSET, xDir, 0)
    local leftSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] - X_OFFSET, entityPos[2], xDir, 0)
    local rightSpawn = objc.spawn_at_point(SPAWN_IDENTIFIER .. tier, 1, playerNumber, entityPos[1] + X_OFFSET, entityPos[2], xDir, 0)

    table.insert(spawnedUnits, topLeftSpawn[1])
    table.insert(spawnedUnits, topRightSpawn[1])
    table.insert(spawnedUnits, bottomLeftSpawn[1])
    table.insert(spawnedUnits, bottomRightSpawn[1])
    table.insert(spawnedUnits, topSpawn[1])
    table.insert(spawnedUnits, bottomSpawn[1])
    table.insert(spawnedUnits, leftSpawn[1])
    table.insert(spawnedUnits, rightSpawn[1])

	for _,unit in pairs(spawnedUnits) do
        objc.set_targetable(unit, false)
        objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'B_TroopTransporter.lua', SPAWN_OVERRIDE_TRIGGER)
		objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_HOLD_POSITION_COUNT_FOR_SPAWNING, 0)
	end
end

B_TroopTransporter_spawnOverrideTickTrigger = function(entity)
    local isPlayingAnimation = objc.get_local_value(entity, IS_PLAYING_ANIMATION)
    if isPlayingAnimation == 0 then
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_SPAWN_ANIMATION)
        objc.idle_entity(entity, numberOfSubrounds)
        objc.play_override_animation_key(entity, SPAWN_ANIMATION_OVERRIDE_KEY, objc.subrounds_to_milliseconds(numberOfSubrounds))
        objc.set_local_value(entity, IS_PLAYING_ANIMATION, 1)
    end

    local numberOfRounds = objc.convert_frames_to_rounds(FRAMES_IN_SPAWN_ANIMATION)
    local overrideAnimTimer = objc.tick_timer(entity, ANIMATION_TIMER_KEY, numberOfRounds)
    if overrideAnimTimer == 0 then
        objc.set_targetable(entity, true)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, SPAWN_OVERRIDE_TRIGGER)
    end
end