#define SUPPORT_GLSL300
#include "prefix.sh"

precision highp float;

uniform sampler2D s_cc3Texture2Ds[1];

VARYING_IN vec2 v_texCoord[1];

uniform vec2 Exposure_texelSize;

#define BLUR_FUNC blur13

// Adapted from https://github.com/Jam3/glsl-fast-gaussian-blur
// See also http://rastergrid.com/blog/2010/09/efficient-gaussian-blur-with-linear-sampling

vec4 blur5(sampler2D image, vec2 uv, vec2 texelSize, vec2 direction)
{
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.3333333333333333) * direction;
    color += texture2D(image, uv) * 0.29411764705882354;
    color += texture2D(image, uv + (off1 * texelSize)) * 0.35294117647058826;
    color += texture2D(image, uv - (off1 * texelSize)) * 0.35294117647058826;
    return color;
}

vec4 blur9(sampler2D image, vec2 uv, vec2 texelSize, vec2 direction)
{
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.3846153846) * direction;
    vec2 off2 = vec2(3.2307692308) * direction;
    color += texture2D(image, uv) * 0.2270270270;
    color += texture2D(image, uv + (off1 * texelSize)) * 0.3162162162;
    color += texture2D(image, uv - (off1 * texelSize)) * 0.3162162162;
    color += texture2D(image, uv + (off2 * texelSize)) * 0.0702702703;
    color += texture2D(image, uv - (off2 * texelSize)) * 0.0702702703;
    return color;
}

vec4 blur13(sampler2D image, vec2 uv, vec2 texelSize, vec2 direction)
{
    vec4 color = vec4(0.0);
    vec2 off1 = vec2(1.411764705882353) * direction;
    vec2 off2 = vec2(3.2941176470588234) * direction;
    vec2 off3 = vec2(5.176470588235294) * direction;
    color += texture2D(image, uv) * 0.1964825501511404;
    color += texture2D(image, uv + (off1 * texelSize)) * 0.2969069646728344;
    color += texture2D(image, uv - (off1 * texelSize)) * 0.2969069646728344;
    color += texture2D(image, uv + (off2 * texelSize)) * 0.09447039785044732;
    color += texture2D(image, uv - (off2 * texelSize)) * 0.09447039785044732;
    color += texture2D(image, uv + (off3 * texelSize)) * 0.010381362401148057;
    color += texture2D(image, uv - (off3 * texelSize)) * 0.010381362401148057;
    return color;
}

void main()
{
    #if defined(BLUR_HORIZONTAL)
        gl_FragColor = BLUR_FUNC(s_cc3Texture2Ds[0], v_texCoord[0], Exposure_texelSize, vec2(1, 0));
    #else
        gl_FragColor = BLUR_FUNC(s_cc3Texture2Ds[0], v_texCoord[0], Exposure_texelSize, vec2(0, 1));
    #endif
}
