local ALLY_CHECK_INTERVAL = 2
local BUFF_RADIUS = 2000
local HAS_BEEN_NASH_BUFFED = 'hasBeenNashBuffed'
local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local BUFF_AMOUNT = 'buffAmountKey'
local BUFF_VFX = 'C_Ai_Buff.lvl'
local HAS_INIT = 'hasInitKey'
local X_SHARK_OFFSET = -800
local X_GRUNT_OFFSET = 200
local Y_SHARK_OFFSET = 600
local Y_GRUNT_OFFSET = 200
local GROUND_UNIT_IDENTIFIER = 'B_Rifleman'
local AIR_UNIT_IDENTIFIER = 'B_SkyShark'
local IDLE_ANIMATION_NAME = 'combatIdle'
local FRAMES_IN_ABILITY_ANIMATION = 30
local IDLE_BUFFER = 1
local SKYSHARK_TIER_ARRAY = {"", "", "", 1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 4}

local ANIMATION_SWITCH_TIMER = 'animationSwitchTimerKey'
local ANIMATION_SWITCH_DELAY = 9

local SPAWN_VFX = 'B_Nash_spawnReinforcements.lvl'

C_Ai_spawnTrigger = function(entity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ABILITY_ANIMATION)
    objc.idle_entity(entity, numberOfSubrounds + IDLE_BUFFER)
    objc.set_targetable(entity, false)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Ai.lua', 'C_Ai_switchIdleAnimationAfterDelay')

    local tierOfNash = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
    local tierOfSkyshark = SKYSHARK_TIER_ARRAY[tierOfNash + 1]
    if tierOfNash == 0 
        then tierOfNash = ''
    end

    local teamNumber = objc.get_team_number(entity)
    local directionToFace = 0
    if teamNumber == 1
        then directionToFace = 1
    else
        directionToFace = -1
    end

    riflemanOfCorrectTier = GROUND_UNIT_IDENTIFIER..tierOfNash
    skysharkOfCorrectTier = AIR_UNIT_IDENTIFIER..tierOfSkyshark

    nashX = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
    nashY = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y)

    local playerNumber = objc.get_player_number(entity)
    local leftRifle = objc.spawn_at_point(riflemanOfCorrectTier, 1, playerNumber, nashX + X_GRUNT_OFFSET * directionToFace, nashY + Y_GRUNT_OFFSET, directionToFace, 0)
    objc.play_vfx(leftRifle[1], SPAWN_VFX, NODE_ORIGIN)
    local leftRifle2 = objc.spawn_at_point(riflemanOfCorrectTier, 1, playerNumber, nashX + X_GRUNT_OFFSET * directionToFace, nashY + (Y_GRUNT_OFFSET * 2), directionToFace, 0)
    objc.play_vfx(leftRifle2[1], SPAWN_VFX, NODE_ORIGIN)
    local leftShark = objc.spawn_at_point(skysharkOfCorrectTier, 1, playerNumber, nashX + X_SHARK_OFFSET * directionToFace, nashY + Y_SHARK_OFFSET, directionToFace, 0)
    objc.play_vfx(leftShark[1], SPAWN_VFX, NODE_ORIGIN)
    local rightRifle = objc.spawn_at_point(riflemanOfCorrectTier, 1, playerNumber, nashX + X_GRUNT_OFFSET * directionToFace, nashY - Y_GRUNT_OFFSET, directionToFace, 0)
    objc.play_vfx(rightRifle[1], SPAWN_VFX, NODE_ORIGIN)
    local rightRifle2 = objc.spawn_at_point(riflemanOfCorrectTier, 1, playerNumber, nashX + X_GRUNT_OFFSET * directionToFace, nashY - (Y_GRUNT_OFFSET * 2), directionToFace, 0)
    objc.play_vfx(rightRifle2[1], SPAWN_VFX, NODE_ORIGIN)
    local rightShark = objc.spawn_at_point(skysharkOfCorrectTier, 1, playerNumber, nashX + X_SHARK_OFFSET * directionToFace, nashY - Y_SHARK_OFFSET, directionToFace, 0)
    objc.play_vfx(rightShark[1], SPAWN_VFX, NODE_ORIGIN)

    objc.set_local_value(entity, BUFF_AMOUNT, objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_AURA_DAMAGE_BONUS))
end

C_Ai_tickTrigger = function(entity)
    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_INTERVAL)
    if allyCheckTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
        if buffedUnits == 0 then
            buffedUnits = {}
        end

        for _, buffedUnit in pairs(buffedUnits) do
            C_Ai_removeBuff(entity, buffedUnit)
        end

        buffedUnits = {}

        local allies = objc.list_entities_in_radius(entity, BUFF_RADIUS, teamNumber)
        for _, ally in pairs(allies) do
            local hasBeenBuffed = objc.get_local_value(ally, HAS_BEEN_NASH_BUFFED)
            if objc.is_normal_unit(ally) and hasBeenBuffed == 0 then
                C_Ai_buffUnit(entity, ally)
                table.insert(buffedUnits, ally)
            end
        end

        objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
    end
end

C_Ai_switchIdleAnimationAfterDelay = function(entity)
    local animationSwitchTimer = objc.tick_timer(entity, ANIMATION_SWITCH_TIMER, ANIMATION_SWITCH_DELAY)
    if animationSwitchTimer == 0 then
        local idleAnimations = {}
        idleAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_NAME
        objc.update_animation_state(entity, idleAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Ai_switchIdleAnimationAfterDelay')
        objc.set_targetable(entity, true)
    end
end

C_Ai_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits ~= 0 then
        for _, buffedUnit in pairs(buffedUnits) do
            C_Ai_removeBuff(entity, buffedUnit)
        end
    end
end

C_Ai_buffUnit = function(entity, targetEntity)
    local damageBuff = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    local buffToGive = objc.get_local_value(entity, BUFF_AMOUNT)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF, damageBuff + buffToGive)
    objc.play_vfx(targetEntity, BUFF_VFX, NODE_STATUS)
    objc.set_local_value(targetEntity, HAS_BEEN_NASH_BUFFED, 1)
end

C_Ai_removeBuff = function(entity, targetEntity)
    local damageBuff = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    local buffToGive = objc.get_local_value(entity, BUFF_AMOUNT)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF, damageBuff - buffToGive)
    objc.stop_vfx(targetEntity, BUFF_VFX, NODE_STATUS, false)
    objc.set_local_value(targetEntity, HAS_BEEN_NASH_BUFFED, 0)
end