local HAS_INIT = 'hasInit'
local HALLUCINATION_ENTITY_IDENTIFIER = 'HallucinationDummyUnit'
local HALLUCINATION_TAUNT_ENTITY_IDENTIFIER = 'hallucinationTauntUnit'
local CURRENT_HALLUCINATION_ENTITY = 'currentHallucinationEntityKey'
local HALLUCINATION_DUMMY_UNIT_KEY = 'hallucinationDummyUnitKey'
local ORIGINAL_RANGE_KEY = 'originalRangeKey'
local RANGE_INCREASED_KEY = 'hasRangeIncreasedKey'
local OLD_PARAMS_KEY = 'oldParamsDictKey'
local HAS_BEEN_PUSHED_KEY = 'hasBeenPushed'

local SPAWN_OFFSET_X = 3000
local KNOCKBACK_SPEED = 10000
local KNOCKBACK_DESTINATION_X = 4500
local KNOCKBACK_DESTINATION_INCREMENT = 600
local KNOCKBACK_DISTANCE = 2700
local KNOCKBACK_BLOCK_HEIGTH = 1500
local KNOCKBACK_BLOCK_WIDTH = 800
local KNOCKBACK_START = 3
local KNOCKBACK_END = 7
local KNOCKBACK_TIMER_KEY = 'knockbackTimerKey'
local MAX_KNOCKBACK_TIME = 5

local ABILITY_ANIMATION_NAME = 'W_Kuro_Battle_Combat_Ability_01'
local FRAMES_IN_ABILITY_ANIMATION = 30
local ABILITY_ANIMATION_OFFSET = 1000
local ABILITY_ANIMATION_DURATION_KEY = 'abilityAnimationDurationKey'

local IDLE_ANIMATION_NAME = 'combatIdle'
local FRAMES_IN_ABILITY_ANIMATION = 30
local IDLE_BUFFER = 1

local ANIMATION_SWITCH_TIMER = 'animationSwitchTimerKey'
local ANIMATION_SWITCH_DELAY = 9

C_Ascension2_spawnTrigger = function(entity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ABILITY_ANIMATION)
    objc.set_local_value(entity, ABILITY_ANIMATION_DURATION_KEY, objc.convert_frames_to_rounds(FRAMES_IN_ABILITY_ANIMATION))
    objc.idle_entity(entity, numberOfSubrounds)
    objc.play_override_animation(entity, ABILITY_ANIMATION_NAME, numberOfSubrounds)
    objc.set_targetable(entity, false)
end

C_Ascension2_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local knockbackTimer = objc.get_local_value(entity, KNOCKBACK_TIMER_KEY)
        objc.set_local_value(entity, KNOCKBACK_TIMER_KEY, knockbackTimer + 1)
        if knockbackTimer >= KNOCKBACK_START and knockbackTimer <= KNOCKBACK_END then
            local teamNumber = objc.get_team_number(entity)
            local playerNumber = objc.get_player_number(entity)
            local entityPosition = objc.get_entity_position(entity)
            local spawnX = entityPosition[1]
            local xDir = 1
            if teamNumber == 2 then
                spawnX = spawnX - SPAWN_OFFSET_X
                xDir = -1
            else
                spawnX = spawnX + SPAWN_OFFSET_X
            end
            if knockbackTimer == KNOCKBACK_START then
                local tauntOfRightTier = HALLUCINATION_TAUNT_ENTITY_IDENTIFIER
                if objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER) > 0 then
                    tauntOfRightTier = HALLUCINATION_TAUNT_ENTITY_IDENTIFIER..objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
                end
                local hallucinationEntityArray = objc.spawn_at_point(tauntOfRightTier, 1, playerNumber, spawnX, entityPosition[2], xDir, 0)

                local hallucinationOfRightTier = HALLUCINATION_ENTITY_IDENTIFIER
                if objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER) > 0 then
                    hallucinationOfRightTier = HALLUCINATION_ENTITY_IDENTIFIER..objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
                end
                local dummyUnitArray = objc.spawn_at_point(hallucinationOfRightTier, 1, playerNumber, spawnX, entityPosition[2], xDir, 0)
                objc.set_local_value(hallucinationEntityArray[1], HALLUCINATION_DUMMY_UNIT_KEY, dummyUnitArray[1])
                objc.set_local_value(entity, CURRENT_HALLUCINATION_ENTITY, hallucinationEntityArray[1])
            end




            local kuroPosition = objc.get_entity_position(entity)
            local enemyTeamNumber = objc.get_enemy_team_number(entity)
            local enemiesInRadius = objc.list_entities_in_rectangle(kuroPosition[1] + ((knockbackTimer - KNOCKBACK_START + 1) * KNOCKBACK_BLOCK_WIDTH * xDir), kuroPosition[2], KNOCKBACK_BLOCK_WIDTH, KNOCKBACK_BLOCK_HEIGTH, enemyTeamNumber)

            for _, enemy in pairs(enemiesInRadius) do
                if (objc.is_normal_unit(enemy) or objc.is_commander(enemy) or (objc.is_boss_unit(enemy) == 1)) and objc.get_local_value(enemy, HAS_BEEN_PUSHED_KEY) == 0 then
                    local enemyPos = objc.get_entity_position(enemy)
                    local oldRange = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE)
                    local targetDestinationX = KNOCKBACK_DESTINATION_X + KNOCKBACK_DESTINATION_INCREMENT * (knockbackTimer - KNOCKBACK_START)
                    local targetDestinationY = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_POSITION_Y)
                    local craterOffsetModifier = 1


                    if teamNumber == 2 then
                        targetDestinationX = targetDestinationX * -1
                        craterOffsetModifier = -1
                    end

                    targetDestinationX = targetDestinationX + kuroPosition[1]

                    --local targetArray = objc.spawn_at_point('SumoTargetEffectEntity', 1, teamNumber, targetDestinationX, targetDestinationY, 0, 0)

                    objc.set_local_value(enemy, ORIGINAL_RANGE_KEY, oldRange)
                    objc.set_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE, 0)
                    --objc.add_trigger(enemy, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'C_Ascension2.lua', 'C_Ascension2_endRangeTrigger')
                    --objc.add_trigger(enemy, TRIGGER_NAME_TICK_TRIGGER, 'C_Ascension2.lua', 'C_Ascension2_maxKnockbackTimeTrigger')
                    --objc.set_entity_should_reverse_facing_direction(enemy, true)
                    objc.set_solid(enemy, false)
                    objc.set_local_value(enemy, HAS_BEEN_PUSHED_KEY, 1)

                    --THIS IS A HACK TO MAKE THE COMBAT MINER WORK FOR 1.80, PLEASE REMOVE WHEN WE CREATE AN onInterruptTrigger
                    local targetBaseIdentifier = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
                    if string.match(targetBaseIdentifier, 'R_CombatMiner') ~= nil then
                        local isIdle = objc.is_entity_idle(enemy)
                        if isIdle == true then
                            objc.unidle_entity(enemy)
                            objc.set_local_value(enemy, 'isMiningKey', 0)
                        end
                    end

                    -- Is subtracted off in code, so use a positive value.
                    local velocityDamping = 150
                    local startingVelocity = 1400
                    -- Set maxVelocity and maxAcceleration to be the starting velocity so that
                    -- We immediately begin motion at the startingVelocity, and decay down using the velocityDamping
                    local maxVelocity = startingVelocity
                    local maxAcceleration = startingVelocity
                    local timeoutRounds = MAX_KNOCKBACK_TIME
                    local oldParams = objc.smoothly_move_entity_to_location_with_trigger(enemy, enemy, entity, targetDestinationX,
                                                                                         targetDestinationY, MOTION_STATE_LEAP,
                                                                                         startingVelocity, maxVelocity, maxAcceleration,
                                                                                         100, velocityDamping,
                                                                                         'C_Ascension2.lua', 'C_Ascension2_endRangeTrigger',
                                                                                         timeoutRounds)
                    objc.set_local_value(enemy, OLD_PARAMS_KEY, oldParams)

                    --Set the offset on the EffectEntity for where to spawn the impact effect
                    --objc.set_local_value(targetArray[1], 'craterSpawnPosition', targetDestinationX * craterOffsetModifier * 2)
                end
            end
        end
        if knockbackTimer >= objc.get_local_value(entity, ABILITY_ANIMATION_DURATION_KEY) then
            objc.set_local_value(entity, HAS_INIT, 1)
            objc.set_targetable(entity, true)
        end        
    end
end

C_Ascension2_maxKnockbackTimeTrigger = function(entity)
    local knockbackTime = objc.tick_timer(entity, 'knockbackLengthKey', MAX_KNOCKBACK_TIME)
    if knockbackTime == 0 then
        objc.idle_entity(entity, 100)
        C_Ascension2_endRangeTrigger(entity, entity)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Ascension2_maxKnockbackTimeTrigger')
    end
end

C_Ascension2_endRangeTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local oldParams = objc.get_local_value(sourceEntity, OLD_PARAMS_KEY)
    objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, oldParams)
    objc.set_solid(sourceEntity, true)


    local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
    objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)

    --objc.set_entity_should_reverse_facing_direction(sourceEntity, false)
    objc.remove_trigger(sourceEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'C_Ascension2_endRangeTrigger')
end
