local ALLY_CHECK_INTERVAL = 2
local BUFF_RADIUS = 3500
local HAS_BEEN_ROXIE_BUFFED = 'hasBeenRoxieBuffed'
local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local BUFF_AMOUNT = 'buffAmountKey'
local BUFF_VFX = 'CommanderPowerUpText.lvl'
local UNIT_CHECK_TIME = 1
local PRIMARY_RAPTOR_KEY = 'primaryRaptorKey'
local UNIT_CHECK_KEY = 'unitCheckTimerKey'
local HAS_BEEN_RAPTOR_BUFFED = 'hasBeenRaptorBuffedKey'
local BUFFED_UNITS_ARRAY = 'buffedUnitsArrayKey'
local SELF_VFX_TIMER = 20
local SELF_VFX_TICK_KEY = 'selfVFXTickKey'
local SELF_VFX_IDENTIFIER = 'idle_R_Raptor.lvl'
local BUFFED_UNITS_VFX_TIMER = 20
local BUFFED_VFX_TICK_KEY = 'buffedUnitsVFXTickKey'
local BUFFED_VFX_IDENTIFIER = 'CommanderPowerUp.lvl'
local HAS_INIT = 'hasInitKey'
local TIER_KEY = 'tierKey'

local CLEAVE_RANGE_BUFFER = 600
local CLEAVE_ATTACK_RATE = 500
local NEXT_TARGET = 'nextTargetKey'
local ORIGINAL_RANGE_KEY = 'originalAttackRangeKey'
local ORIGINAL_ATTACK_KEY = 'originalAttackRateKey'
local MAX_CLEAVE_RANGE_KEY = 'maxCleaveRange'
local NUM_ATTACKS = 'numThisAttacksKey'
local MAX_ATTACKS = 8

local ATTACK_ANIMATION_BASE_NAME = 'R_Roxy_Battle_Combat_Uzi_Attack_01'
local ATTACK_SEQUENCE_NUMBER = 'attackSequenceNumberKey'
local FRAMES_IN_ATTACK_ANIMATION = 15

local IDLE_ANIMATION_NAME = 'idleAnimationName'
local FRAMES_IN_ABILITY_ANIMATION = 30
local IDLE_BUFFER = 1

local ANIMATION_SWITCH_TIMER = 'animationSwitchTimerKey'
local ANIMATION_SWITCH_DELAY = 9

C_Duchess_spawnTrigger = function(entity)
     objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Duchess.lua', 'C_Duchess_setTargetable')
     objc.set_targetable(entity, false)
     objc.idle_entity(entity, ANIMATION_SWITCH_DELAY * 100)

    local originalAttack = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
    objc.set_local_value(entity, ORIGINAL_ATTACK_KEY, originalAttack)

    local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)

    local maxCleaveRange = originalRange + CLEAVE_RANGE_BUFFER
    objc.set_local_value(entity, MAX_CLEAVE_RANGE_KEY, maxCleaveRange)

    objc.set_local_value(entity, BUFF_AMOUNT, objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_AURA_SPEED_INCREASE))
end

C_Duchess_setTargetable = function(entity)
    local animationSwitchTimer = objc.tick_timer(entity, ANIMATION_SWITCH_TIMER, ANIMATION_SWITCH_DELAY)
    if animationSwitchTimer == 0 then
        local idleAnimations = {}
        idleAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_NAME
        objc.update_animation_state(entity, idleAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Duchess_setTargetable')
        objc.set_targetable(entity, true)
    end
end

C_Duchess_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local playerNumber = objc.get_player_number(entity)
        local thisPos = objc.get_entity_position(entity)
        local arrayOfOneEntity = objc.spawn_at_point('fireArrowsTargetedScriptEntity', 1, playerNumber, thisPos[1], thisPos[2], 0, 0)
        objc.set_local_value(entity, HAS_INIT, 1)
        local tierOfRoxie = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
        objc.set_local_value(arrayOfOneEntity[1], TIER_KEY, tierOfRoxie)
    end

    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_INTERVAL)
    if allyCheckTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
        if buffedUnits == 0 then
            buffedUnits = {}
        end

        for _, buffedUnit in pairs(buffedUnits) do
            C_Duchess_removeBuff(entity, buffedUnit)
        end

        buffedUnits = {}

        local allies = objc.list_entities_in_radius(entity, BUFF_RADIUS, teamNumber)
        for _, ally in pairs(allies) do
            local hasBeenBuffed = objc.get_local_value(ally, HAS_BEEN_ROXIE_BUFFED)
            if objc.is_normal_unit(ally) and hasBeenBuffed == 0 then
                C_Duchess_buffUnit(entity, ally)
                table.insert(buffedUnits, ally)
            end
        end

        objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
    end


    local nextTarget = objc.get_local_value(entity, NEXT_TARGET)
    if nextTarget ~= 0 then
        objc.set_targeting(entity, nextTarget)
        objc.set_local_value(entity, NEXT_TARGET, 0)
    end
end

C_Duchess_onAttackTrigger = function(sourceEntity, targetEntity)    
    local maxCleaveRange = objc.get_local_value(sourceEntity, MAX_CLEAVE_RANGE_KEY)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ATTACK_ANIMATION)
    local attackSequenceNumber = objc.get_local_value(sourceEntity, ATTACK_SEQUENCE_NUMBER)
    local attackSequenceLetter = C_Duchess_convertNumberToLetter(attackSequenceNumber)
    objc.play_passive_animation(sourceEntity, ATTACK_ANIMATION_BASE_NAME .. attackSequenceLetter, 0, numberOfSubrounds)
    if attackSequenceNumber < 7 then
        objc.set_local_value(sourceEntity, ATTACK_SEQUENCE_NUMBER, attackSequenceNumber + 1)
    else
        objc.set_local_value(sourceEntity, ATTACK_SEQUENCE_NUMBER, 0)
    end

    local numAttacks = objc.get_local_value(sourceEntity, NUM_ATTACKS)
    if numAttacks + 1 < MAX_ATTACKS then
        local enemyNumber = objc.get_enemy_team_number(sourceEntity)
        local enemiesInRange = objc.list_entities_in_radius(sourceEntity, maxCleaveRange, enemyNumber)
        local highestEnemyScore = -100
        local highestEntity = -1
        local truckDamage = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_DAMAGE)

        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RATE, CLEAVE_ATTACK_RATE)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, maxCleaveRange)

        for _,enemy in pairs(enemiesInRange) do
            local randomScore = objc.roll_number(1, 100)
            if randomScore > highestEnemyScore then
                highestEntity = enemy
                highestEnemyScore = randomScore
            end
        end

        if highestEntity ~= -1 then
            objc.set_local_value(sourceEntity, NEXT_TARGET, highestEntity)
        end

        objc.set_local_value(sourceEntity, NUM_ATTACKS, numAttacks + 1)
    else
        local originalAttack = objc.get_local_value(sourceEntity, ORIGINAL_ATTACK_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RATE, originalAttack)
        
        local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
        objc.set_local_value(sourceEntity, NUM_ATTACKS, 1)
    end
end

C_Duchess_convertNumberToLetter = function(number)
    if number == 0 then
        return 'a'
    elseif number == 1 then
        return 'b'
    elseif number == 2 then
        return 'c'
    elseif number == 3 then
        return 'd'
    elseif number == 4 then
        return 'e'
    elseif number == 5 then
        return 'f'
    elseif number == 6 then
        return 'g'
    elseif number == 7 then
        return 'h'
    else
        return 'a'
    end
end

C_Duchess_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits ~= 0 then
        for _, buffedUnit in pairs(buffedUnits) do
            C_Duchess_removeBuff(entity, buffedUnit)
        end
    end
end

C_Duchess_buffUnit = function(entity, targetEntity)
    local thisSpeed = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    local buffToGive = objc.divide_with_floor(objc.get_local_value(entity, BUFF_AMOUNT) * thisSpeed, 100)
    --print('ROXIE ADDING thisSpeed: ' .. thisSpeed .. ' buffToGive: ' .. buffToGive)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY, thisSpeed + buffToGive)
    --print("NEW SPEED: " .. objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY))
    objc.play_vfx(targetEntity, BUFF_VFX, NODE_STATUS)
    objc.set_local_value(targetEntity, HAS_BEEN_ROXIE_BUFFED, 1)
end

C_Duchess_removeBuff = function(entity, targetEntity)
    local thisSpeed = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    local buffRemovalMultiplier = 100 - objc.divide_with_ceiling(10000, objc.get_local_value(entity, BUFF_AMOUNT) + 100)
    local buffToRemove = objc.divide_with_ceiling(buffRemovalMultiplier * thisSpeed, 100)
    --print('ROXIE REMOVING thisSpeed: ' .. thisSpeed .. ' buffToRemove: ' .. buffToRemove .. ' buffRemovalMultiplier: ' .. buffRemovalMultiplier)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY, thisSpeed - buffToRemove)
    --print("NEW SPEED: " .. objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_VELOCITY))
    objc.stop_vfx(targetEntity, BUFF_VFX, NODE_STATUS, false)
    objc.set_local_value(targetEntity, HAS_BEEN_ROXIE_BUFFED, 0)
end