local DASH_SPEED = 500
local STUN_DURATION = 5000
local DASH_DAMAGE = 300
local HAS_INIT = 'hasInitKey'
local HAS_BEEN_DASHED_TO = 'hasBeenDashedToKey'
local IS_DASHING = 'isDashingKey'
local IS_PAUSING = 'isPausingKey'
local ORIGINAL_MOTION_DICT = 'originalMotionDictionaryKey'
local DASH_TARGET = 'dashTargetKey'

local DASH_OFFSET = 500
local DASH_PAUSE_DURATION = 3

local ALLY_CUTOFF = 3
local DAMAGE_BUFF = 40
local ALLY_CHECK_TIMER = 2

C_FemmeFatale_spawnTrigger = function(entity)
    objc.set_targetable(entity, false)
    objc.set_local_value(entity, IS_DASHING, 1)

    local closestEnemy = C_FemmeFatale_searchForEnemies(entity)
    if closestEnemy ~= 0 then
        C_FemmeFatale_dashToTarget(entity, closestEnemy)
    else
        objc.set_local_value(entity, IS_DASHING, 0)
        objc.set_targetable(entity, true)
    end
end

C_FemmeFatale_tickTrigger = function(entity)
    local isPausing = objc.get_local_value(entity, IS_PAUSING)
    if isPausing == 1 then
        local dashPauseTimer = objc.tick_timer(entity, 'dashPauseTimer', DASH_PAUSE_DURATION)
        if dashPauseTimer == 0 then
            objc.set_local_value(entity, IS_PAUSING, 0)
            
            local closestEnemy = C_FemmeFatale_searchForEnemies(entity)
            if closestEnemy ~= 0 then
                C_FemmeFatale_dashToTarget(entity, closestEnemy)
            else                objc.set_local_value(entity, IS_DASHING, 0)
                objc.set_targetable(entity, true)
            end
        end
    end

    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_TIMER)
    if allyCheckTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local allies = objc.list_entities(teamNumber)
        local numAllies = 0
        for _, ally in pairs(allies) do
            if objc.is_normal_unit(ally) and ally ~= entity then
                numAllies = numAllies + 1
            end
        end

        local numUnderCutoff = ALLY_CUTOFF - numAllies
        if numUnderCutoff < 0 then
            numUnderCutoff = 0
        end
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF, (DAMAGE_BUFF * numUnderCutoff))
    end
end

C_FemmeFatale_damageTrigger = function(damage, targetEntity, sourceEntity)
    local commander = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local isDashing = objc.get_local_value(commander, IS_DASHING)
    if isDashing == 1 then
        objc.set_local_value(commander, IS_PAUSING, 1)
        objc.play_vfx(commander, 'ptl_Asc_blink_teleport.lvl', NODE_ORIGIN)
        objc.idle_entity(commander, DASH_PAUSE_DURATION * 100)
    end

    return damage
end

C_FemmeFatale_searchForEnemies = function(entity)
    local enemyTeamNumber = objc.get_enemy_team_number(entity)
    local enemies = objc.list_entities(enemyTeamNumber)
    local closestEnemy = entity
    local closestDistance = 99999999
    for _, enemy in pairs(enemies) do
        local hasBeenDashed = objc.get_local_value(enemy, HAS_BEEN_DASHED_TO)
        local identifier = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IDENTIFIER)
        if (objc.is_normal_unit(enemy) or objc.is_commander(enemy)) and hasBeenDashed == 0 then
            local enemyPos = objc.get_entity_position(enemy)
            local xDir = 1
            if enemyTeamNumber == 2 then
                xDir = -1
            end
            if (enemyPos[1] * xDir) > 0 then
                local thisDistance = objc.quick_unit_distance_square(entity, enemy)
                if thisDistance < closestDistance then
                    closestDistance = thisDistance
                    closestEnemy = enemy
                end
            end
        end
    end

    if closestEnemy ~= entity then
        return closestEnemy
    end

    return 0
end

C_FemmeFatale_dashToTarget = function(entity, closestEnemy)
    local teamNumber = objc.get_team_number(entity)
    local xDir = 1
    if teamNumber == 1 then
        xDir = -1
    end

    local targetPos = objc.get_entity_position(closestEnemy)
    objc.teleport_entity(entity, targetPos[1] + (DASH_OFFSET * xDir), targetPos[2])
    objc.inflict_normal_damage(entity, closestEnemy, DASH_DAMAGE)
    objc.flash_entity_for_damage(closestEnemy, 250)
end