local POWER_CHORD_DISTANCE_X = 2500
local POWER_CHORD_Y_FACTOR = 2
local POWER_CHORD_DAMAGE = 500

local ALLY_CHECK_TIME = 2
local IS_HEAL_STANCE = 'isHealingStanceKey'
local STANCE_TIMER = 'stanceTimerKey'
local STANCE_LENGTH = 20
local VFX_TIME = 2

local BUFF_RADIUS = 1000
local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local HEAL_AMOUNT = 5
local MOVE_SPEED_BONUS = 15
local DAMAGE_INCREASE = 30
local BUFF_VFX = 'CommanderPowerUpText.lvl'

C_Guitarist_spawnTrigger = function(entity)
    objc.idle_entity(entity, 250)
    objc.play_vfx(entity, 'C_Guitarist_Damage.lvl', NODE_ORIGIN)
    objc.play_vfx(entity, 'R_Guitarist_Pulse.lvl', NODE_ORIGIN)

    local guitaristPosition = objc.get_entity_position(entity)
    local teamNumber = objc.get_team_number(entity)
    local enemyNumber = objc.get_enemy_team_number(entity)
    local xDir = 1
    if teamNumber == 2 then
        xDir = -1
    end

    local enemies = objc.list_entities_in_radius(entity, POWER_CHORD_DISTANCE_X, enemyNumber)
    for _,enemy in pairs(enemies) do
        local enemyPos = objc.get_entity_position(enemy)
        local xDiff = enemyPos[1] - guitaristPosition[1]
        local yDiff = math.abs(enemyPos[2] - guitaristPosition[2])
        local yCutoff = objc.divide_with_ceiling(xDiff, POWER_CHORD_Y_FACTOR)

        if xDir == 1 then
            if xDiff > 0 and yDiff <= yCutoff then
                objc.inflict_normal_damage(entity, enemy, POWER_CHORD_DAMAGE)
                objc.flash_entity_for_damage(enemy, 250)
            end
        else
            if xDiff < 0 and yDiff <= yCutoff then
                objc.inflict_normal_damage(entity, enemy, POWER_CHORD_DAMAGE)
                objc.flash_entity_for_damage(enemy, 250)
            end
        end
    end
end

C_Guitarist_tickTrigger = function(entity)
    local allyCheck = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_TIME)
    if allyCheck == 0 then
        local isHealing = objc.get_local_value(entity, IS_HEAL_STANCE)
        if isHealing == 0 then
            local stanceTimer = objc.tick_timer(entity, STANCE_TIMER, STANCE_LENGTH)
            if stanceTimer == 0 then
                objc.set_local_value(entity, IS_HEAL_STANCE, 1)
                objc.stop_vfx(entity, 'R_Guitarist_Pulse.lvl', NODE_ORIGIN, false)
                objc.play_vfx(entity, 'B_BattleBard_Pulse.lvl', NODE_ORIGIN)
            else
                C_Guitarist_buffNearbyUnits(entity)
            end
        else
            local stanceTimer = objc.tick_timer(entity, STANCE_TIMER, STANCE_LENGTH)
            if stanceTimer == 0 then
                objc.set_local_value(entity, IS_HEAL_STANCE, 0)
                objc.play_vfx(entity, 'R_Guitarist_Pulse.lvl', NODE_ORIGIN)
            else
                local vfxTimer = objc.tick_timer(entity, 'vfxTimerKey', VFX_TIME)
                if vfxTimer == 0 then
                    objc.play_vfx(entity, 'B_BattleBard_Pulse.lvl', NODE_ORIGIN)
                end
                C_Guitarist_healNearbyUnits(entity)
            end
        end
    end
end

C_Guitarist_buffNearbyUnits = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits == 0 then
        buffedUnits = {}
    end

    for _, buffedUnit in pairs(buffedUnits) do
        C_Guitarist_removeBuff(buffedUnit)
    end

    buffedUnits = {}

    local allies = objc.list_entities_in_radius(entity, BUFF_RADIUS, teamNumber)
    for _, ally in pairs(allies) do
        if objc.is_normal_unit(ally) then
            C_Guitarist_buffUnit(ally)
            table.insert(buffedUnits, ally)
        end
    end

    objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
end

C_Guitarist_healNearbyUnits = function(entity)
    local allies = objc.list_entities_in_radius(entity, BUFF_RADIUS, teamNumber)
    for _, ally in pairs(allies) do
        if objc.is_normal_unit(ally) then
            local currentHealth = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_CURRENT_HEALTH)
            local maxHealth = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_MAX_HEALTH)
            local targetHealth = currentHealth + HEAL_AMOUNT
            if targetHealth > maxHealth then
                targetHealth = maxHealth
            end

            objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_CURRENT_HEALTH, targetHealth)
        end
    end
end

C_Guitarist_buffUnit = function(entity)
    local maxVelocity = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, maxVelocity + MOVE_SPEED_BONUS)

    local bonusDamage = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF, bonusDamage + DAMAGE_INCREASE)
    objc.play_vfx(entity, BUFF_VFX, NODE_STATUS)
end

C_Guitarist_removeBuff = function(entity)
    local maxVelocity = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, maxVelocity - MOVE_SPEED_BONUS)

    local bonusDamage = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF, bonusDamage - DAMAGE_INCREASE)

    objc.stop_vfx(entity, BUFF_VFX, NODE_STATUS, false)
end