local HAS_INIT = 'hasInit'

local OLD_PARAMS_KEY = 'oldParamsDictKey'
local CRATER_OFFSET = 350
local ORIGINAL_RANGE_KEY = 'originalRangeKey'
local MELEE_RANGE = 550
local KNOCKBACK_SPEED = 350
local KNOCKBACK_DISTANCE = 1200
local INITIAL_BOUNCE_RADIUS = 1050
local INITIAL_BOUNCE_OFFSET = 1900

local STUN_DURATION = 1000

local ABILITY_ANIMATION_NAME = 'W_SumoCommander_Combat_Ability_01'
local FRAMES_IN_ABILITY_ANIMATION = 30
local FRAMES_UNTIL_KNOCKBACK = 16

local INTRO_TIMER = 'introTimerKey'
local HAS_FINISHED_INTRO = 'hasFinishedIntroKey'

C_KnockbackSumo_spawnTrigger = function(entity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ABILITY_ANIMATION)
    objc.idle_entity(entity, numberOfSubrounds)
    objc.set_targetable(entity, false)
    objc.play_override_animation(entity, ABILITY_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
    local damageTimer = objc.tick_timer(entity, INTRO_TIMER, objc.convert_frames_to_rounds(FRAMES_IN_ABILITY_ANIMATION))
    objc.set_local_value(entity, HAS_FINISHED_INTRO, 0) 
end

C_KnockbackSumo_tickTrigger = function(entity)
    local hasFinishedIntro = objc.get_local_value(entity, HAS_FINISHED_INTRO)
    if hasFinishedIntro == 0 then
        local abilityAnimationRounds = objc.convert_frames_to_rounds(FRAMES_IN_ABILITY_ANIMATION)
        local knockbackRounds = objc.convert_frames_to_rounds(FRAMES_UNTIL_KNOCKBACK)
        local damageTimer = objc.tick_timer(entity, INTRO_TIMER, abilityAnimationRounds)
        if damageTimer == knockbackRounds * 100 then
            local playerNumber = objc.get_player_number(entity)
            local teamNumber = objc.get_team_number(entity)
            local enemyNumber = objc.get_enemy_team_number(entity)
            local sumoPos = objc.get_entity_position(entity)
            local xDir = 1
            if teamNumber == 2 then
                xDir = -1
            end
            local enemies = objc.list_enemies_in_radius_of_point(sumoPos[1] + (INITIAL_BOUNCE_OFFSET * xDir), sumoPos[2], INITIAL_BOUNCE_RADIUS, teamNumber)
            local landingDamage = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_IMPACT_DAMAGE)
            for _, enemy in pairs(enemies) do
                if (objc.is_building_unit(enemy) or objc.is_normal_unit(enemy) or objc.is_commander(enemy) or objc.is_base(enemy) or objc.is_barricade(enemy) or (objc.is_boss_unit(enemy) == 1)) and (objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) then
                    objc.inflict_normal_damage(entity, enemy, landingDamage)
                    objc.flash_entity_for_damage(enemy, 250)
                   if not objc.is_building_unit(enemy) then
                        local targetDestinationX = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_POSITION_X)
                        local targetDestinationY = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_POSITION_Y)
                        local craterOffsetModifier = 1

                        if sumoPos[1] < targetDestinationX then
                            targetDestinationX = targetDestinationX + KNOCKBACK_DISTANCE
                        else
                            targetDestinationX = targetDestinationX - KNOCKBACK_DISTANCE
                            craterOffsetModifier = -1
                        end

                        --local targetArray = objc.spawn_at_point('SumoTargetEffectEntity', 1, playerNumber, targetDestinationX, targetDestinationY, 0, 0)
                        local oldRange = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE)
                        objc.set_local_value(enemy, ORIGINAL_RANGE_KEY, oldRange)
                        objc.set_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE, MELEE_RANGE)
                        --objc.add_trigger(enemy, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'C_KnockbackSumo.lua', 'C_KnockbackSumo_endRangeTrigger')

                        if not objc.is_building_unit(enemy) then
                            objc.set_entity_should_reverse_facing_direction(enemy, true)
                        end

                        --THIS IS A HACK TO MAKE THE COMBAT MINER WORK FOR 1.80, PLEASE REMOVE WHEN WE CREATE AN onInterruptTrigger
                        local targetBaseIdentifier = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
                        if string.match(targetBaseIdentifier, 'R_CombatMiner') ~= nil then
                            local isIdle = objc.is_entity_idle(enemy)
                            if isIdle == true then
                                objc.unidle_entity(enemy)
                                objc.set_local_value(enemy, 'isMiningKey', 0)
                            end
                        end

                        -- Is subtracted off in code, so use a positive value.
                        local velocityDamping = 100
                        local startingVelocity = 450
                        -- Set maxVelocity and maxAcceleration to be the starting velocity so that
                        -- We immediately begin motion at the startingVelocity, and decay down using the deceleration
                        local maxVelocity = startingVelocity
                        local maxAcceleration = startingVelocity
                        local timeoutRounds = 10
                        local oldParams = objc.smoothly_move_entity_to_location_with_trigger(enemy, enemy, entity, targetDestinationX,
                                                                                             targetDestinationY, MOTION_STATE_LEAP,
                                                                                             startingVelocity, maxVelocity, maxAcceleration,
                                                                                             100, velocityDamping,
                                                                                             'C_KnockbackSumo.lua', 'C_KnockbackSumo_endRangeTrigger',
                                                                                             timeoutRounds)
                        objc.set_local_value(enemy, OLD_PARAMS_KEY, oldParams)

                        --Set the offset on the EffectEntity for where to spawn the impact effect
                        --objc.set_local_value(targetArray[1], 'craterSpawnPosition', targetDestinationX - CRATER_OFFSET * craterOffsetModifier * 2)
                   end
                end
            end
        elseif damageTimer == 0 then
            objc.set_local_value(entity, HAS_FINISHED_INTRO, 1) 
            objc.set_targetable(entity, true)
        end
    end
end

C_KnockbackSumo_damageTrigger = function(damage, targetEntity, sourceEntity)
    local entity = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local hasFinishedIntro = objc.get_local_value(entity, HAS_FINISHED_INTRO)
    if hasFinishedIntro == 1 then
        objc.stun_entity(targetEntity, STUN_DURATION)
        objc.play_vfx(targetEntity, 'Status_stunned.lvl', NODE_STATUS)
    end
    return damage
end

C_KnockbackSumo_endRangeTrigger = function(sourceEntity, targetEntity)
    local oldParams = objc.get_local_value(sourceEntity, OLD_PARAMS_KEY)
    local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
    objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, oldParams)
    objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
    objc.set_entity_should_reverse_facing_direction(sourceEntity, false)
    objc.remove_trigger(sourceEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'C_KnockbackSumo_endRangeTrigger')
end
