local TURRET_IDENTIFIER = 'R_ValsTurret'
local TURRET_X_OFFSET = 0
local TURRET_Y_OFFSET = 1000
local BASE_CUTOFF = 6000

local ALLY_CHECK_TIMER = 'allyCheckTimerKey'
local ALLY_CHECK_INTERVAL = 5
local BUFF_RADIUS = 2000
local ATTACK_SPEED_BUFF_NUMERATOR = 3
local ATTACK_SPEED_BUFF_DENOMINATOR = 4
local BUFFED_UNITS_ARRAY = 'buffedUnitsArrayKey'
local HAS_BEEN_SPEED_BUFFED = 'hasBeenSpeedBuffedKey'
local PREBUFF_ATTACK_SPEED = 'prebuffAttackSpeedKey'

local TURRET_DELAY = 0 --Currently set to 0 as the beat for when val's turret drops in is in flux
local TURRET_DELAY_TIMER = 'turretDelayTickTimer'
local TURRET_DELAY_TRIGGER = 'C_Mechanist_turretDelayTrigger'
local IDLE_ANIMATION_NAME = 'idleAnim'
local ANIMATION_SWITCH_DELAY = 10
local ANIMATION_SWITCH_TIMER = 'animationSwitchTimerKey'

local AOE_RADIUS = 900
local PRIMARY_TARGET_KEY = 'primaryTargetKey'
local ON_KILL_ENTITY_GROUND = 'valKillEntityGround'
local ON_KILL_ENTITY_AIR = 'valKillEntityAir'
local AOE_VFX = 'imp_R_Val.lvl'

C_Mechanist_spawnTrigger = function(entity)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Mechanist.lua', TURRET_DELAY_TRIGGER)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Mechanist.lua', 'C_Mechanist_switchAnimationTrigger')
    objc.set_targetable(entity, false)
    objc.idle_entity(entity, TURRET_DELAY * 100)
    objc.set_local_value(entity, PRIMARY_TARGET_KEY, 0)
end

C_Mechanist_switchAnimationTrigger = function(entity)
    local animationSwitchTimer = objc.tick_timer(entity, ANIMATION_SWITCH_TIMER, ANIMATION_SWITCH_DELAY)
    if animationSwitchTimer == 0 then
        local idleAnimations = {}
        idleAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_NAME
        objc.update_animation_state(entity, idleAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'C_Mechanist_switchAnimationTrigger')
        objc.set_targetable(entity, true)
    end
end

C_Mechanist_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, PRIMARY_TARGET_KEY, 0)
end

C_Mechanist_damageTrigger = function(damage, targetEntity, sourceEntity)
    local teamNumber = objc.get_team_number(sourceEntity)
    local enemyNumber = objc.get_enemy_team_number(sourceEntity)
    local val = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)

    local targetPlane = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
    local primaryTarget = objc.get_local_value(val, PRIMARY_TARGET_KEY)

    if primaryTarget == 0 then
        objc.set_local_value(val, PRIMARY_TARGET_KEY, targetEntity)
        
        if targetPlane == NUM_PLANE_OF_EXISTENCE_AIR then
            objc.play_vfx(targetEntity, AOE_VFX, NODE_IMPACT)
        else
            objc.play_vfx(targetEntity, AOE_VFX, NODE_ORIGIN)
        end

        local enemies = objc.list_entities_in_radius(targetEntity, AOE_RADIUS, enemyNumber)
        for _,enemy in pairs(enemies) do
            local enemyPlane = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
            if enemyPlane == targetPlane and enemy ~= targetEntity then
                objc.inflict_normal_damage(sourceEntity, enemy, damage)
                objc.flash_entity_for_damage(enemy, 250)
            end
        end
    end
    return damage
end

C_Mechanist_onKillTrigger = function(killedEntity, killerEntity)
    local primaryTarget = objc.get_local_value(killerEntity, PRIMARY_TARGET_KEY)
    if primaryTarget == killedEntity then
        local enemyPosX = objc.get_entity_value_for_key(killedEntity, ENTITY_VALUE_KEY_POSITION_X)
        local enemyPosY = objc.get_entity_value_for_key(killedEntity, ENTITY_VALUE_KEY_POSITION_Y)
        local playerNumber = objc.get_player_number(killerEntity)
        local enemyPlane = objc.get_entity_value_for_key(killedEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
        if enemyPlane == NUM_PLANE_OF_EXISTENCE_GROUND then
            objc.spawn_at_point(ON_KILL_ENTITY_GROUND, 1, playerNumber, enemyPosX, enemyPosY, 0, 0)
        else
            objc.spawn_at_point(ON_KILL_ENTITY_AIR, 1, playerNumber, enemyPosX, enemyPosY, 0, 0)
        end
    end
end

C_Mechanist_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_UNITS_ARRAY)
    if buffedUnits ~= 0 then
        for _, unit in pairs(buffedUnits) do
            local originalAttackRate = objc.get_local_value(unit, PREBUFF_ATTACK_SPEED)
            if originalAttackRate ~= 0 then
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_ATTACK_RATE, originalAttackRate)
            end
        end
    end
end

C_Mechanist_turretDelayTrigger = function(entity)
    local turretDelayTimer = objc.tick_timer(entity, TURRET_DELAY_TIMER, TURRET_DELAY)
    if turretDelayTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local entityPos = objc.get_entity_position(entity)
        local xDir = 1
        if teamNumber == 2 then
            xDir = -1
        end

        local tierOfVal = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
        if tierOfVal == 0 then
            tierOfVal = ''
        end

        local turretXPos = entityPos[1] + (TURRET_X_OFFSET * xDir)
        local turretYPos = entityPos[2] + (TURRET_Y_OFFSET * -xDir)
        if turretXPos > BASE_CUTOFF then
            turretXPos = BASE_CUTOFF
        elseif turretXPos < -BASE_CUTOFF then
            turretXPos = -BASE_CUTOFF
        end

        local idleAnimations = {}
        idleAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_NAME
        objc.update_animation_state(entity, idleAnimations)

        objc.spawn_at_point(TURRET_IDENTIFIER..tierOfVal, 1, teamNumber, turretXPos, turretYPos, xDir, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, TURRET_DELAY_TRIGGER)
    end
end