local HAS_INIT = 'hasInitKey'
local IS_SINGLE_TARGET = 'isSingleTargetKey'
local PRIMARY_TARGET = 'primaryTargetKey'
local SPAWN_TIME = 30

local TECH_GIRL_REGULAR_IDLE_ANIMATION_NAME = 'B_TechGirl_Idle_01'
local TECH_GIRL_ULTIMATE_IDLE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local TECH_GIRL_REGULAR_URGENT_IDLE_ANIMATION_NAME = 'B_TechGirl_Idle_01'
local TECH_GIRL_ULTIMATE_URGENT_IDLE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local TECH_GIRL_REGULAR_WAVE_ANIMATION_NAME = 'B_TechGirl_WaveTimer_01'
local TECH_GIRL_ULTIMATE_WAVE_ANIMATION_NAME = 'B_TechGirl_Ultimate_Idle_01'

local ABILITY_ANIMATION_NAME = 'B_Mechrazor_Battle_Spawn_02'
local FRAMES_IN_ABILITY_ANIMATION = 65

local HEALTH_TICK_KEY = 'healthTickAmountKey'
local DURATION = 150

local DAMAGE_TIMER = 'damageDelayTimerKey'
local HAS_DAMAGED = 'hasDamagedKey'
local DAMAGE_DELAY = 7
local DAMAGE_RADIUS = 1200

local SPLASH_DISTANCE = 2950
local SPLASH_ANGLE = 55

local SHIELD_VFX = 'imp_B_ShieldLarge.lvl'

C_Nerd_spawnTrigger = function(entity)
    objc.set_local_value(entity, IS_DEBUFF_IMMUNE_KEY, 1)
    objc.set_solid(entity, false)
    objc.set_targetable(entity, false)

    local updatedAnimations = {}
    updatedAnimations[COMMANDER_ANIMATION_STATE_IDLE] = TECH_GIRL_ULTIMATE_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_URGENT_IDLE] = TECH_GIRL_ULTIMATE_URGENT_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_WAVE] = TECH_GIRL_ULTIMATE_WAVE_ANIMATION_NAME
    local teamNumber = objc.get_team_number(entity)
    objc.update_commander_animation_state(teamNumber, updatedAnimations)


    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ABILITY_ANIMATION)
    objc.idle_entity(entity, numberOfSubrounds)
    objc.play_override_animation(entity, ABILITY_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
end

C_Nerd_tickTrigger = function(entity)
    local hasDamaged = objc.get_local_value(entity, HAS_DAMAGED)
    if hasDamaged == 0 then
        local damageTimer = objc.tick_timer(entity, DAMAGE_TIMER, DAMAGE_DELAY)
        if damageTimer == 0 then
            objc.set_local_value(entity, HAS_DAMAGED, 1) 
            local enemyTeamNumber = objc.get_enemy_team_number(entity)
            local enemiesInRadius = objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyTeamNumber)
            local landingDamage = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_IMPACT_DAMAGE)
            objc.set_solid(entity, true)
            objc.set_targetable(entity, true)      
            for _, enemy in pairs(enemiesInRadius) do
                if (objc.is_building_unit(enemy) or objc.is_normal_unit(enemy) or objc.is_commander(enemy) or objc.is_base(enemy) or objc.is_barricade(enemy) or (objc.is_boss_unit(enemy) == 1)) and (objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) then
                    objc.inflict_normal_damage(entity, enemy, landingDamage)
                    objc.flash_entity_for_damage(enemy, 250)
                end
            end
        end
    end
end

C_Nerd_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, IS_SINGLE_TARGET, 0)
    objc.set_local_value(sourceEntity, PRIMARY_TARGET, targetEntity)
    local targetPlaneOfExistence = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
    if targetPlaneOfExistence == NUM_PLANE_OF_EXISTENCE_AIR then
        objc.set_local_value(sourceEntity, IS_SINGLE_TARGET, 1)
    end
end

C_Nerd_damageTrigger = function(damage, targetEntity, sourceEntity)
    local mechazor = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local isSingleTarget = objc.get_local_value(mechazor, IS_SINGLE_TARGET)
    if isSingleTarget == 0 then
        local primaryTarget = objc.get_local_value(mechazor, PRIMARY_TARGET)
        if primaryTarget == targetEntity then
            local mechazorPosition = objc.get_entity_position(mechazor)
            local targetPosition = objc.get_entity_position(targetEntity)
            local trianglePoints = objc.get_triangle_points(mechazorPosition[1], mechazorPosition[2], targetPosition[1] - mechazorPosition[1], targetPosition[2] - mechazorPosition[2], SPLASH_DISTANCE, SPLASH_ANGLE)

            local enemyTeamNumber = objc.get_team_number(targetEntity)
            local enemiesInTriangle = objc.list_entities_in_triangle(mechazorPosition[1], mechazorPosition[2], trianglePoints[1], trianglePoints[2], trianglePoints[3], trianglePoints[4], enemyTeamNumber)
            local enemiesInRadius = objc.list_entities_in_radius(mechazor, SPLASH_DISTANCE, enemyTeamNumber)
            for _, triangleEnemy in pairs(enemiesInTriangle) do
                local isInCircle = false
                for _, circleEnemy in pairs(enemiesInRadius) do
                    if triangleEnemy == circleEnemy and objc.is_normal_unit(circleEnemy) and not objc.is_air_unit(circleEnemy) then
                        isInCircle = true
                    end
                end

                if isInCircle == true and triangleEnemy ~= targetEntity then
                    objc.inflict_normal_damage(sourceEntity, triangleEnemy, damage)
                    objc.flash_entity_for_damage(triangleEnemy, 250)
                end
            end
        end
    else
        local singleTargetDamage = objc.divide_with_floor(damage * 13, 10)
        return singleTargetDamage
    end

    return damage
end

C_Nerd_deathTrigger = function(entity)
    local updatedAnimations = {}
    updatedAnimations[COMMANDER_ANIMATION_STATE_IDLE] = TECH_GIRL_REGULAR_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_URGENT_IDLE] = TECH_GIRL_REGULAR_URGENT_IDLE_ANIMATION_NAME
    updatedAnimations[COMMANDER_ANIMATION_STATE_WAVE] = TECH_GIRL_REGULAR_WAVE_ANIMATION_NAME
    local teamNumber = objc.get_team_number(entity)
    objc.update_commander_animation_state(teamNumber, updatedAnimations)
end

C_Nerd_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local armorAmount = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ARMOR)
    local flashCutoff = objc.divide_with_floor(armorAmount * 10, 4)
    local originalDamage = damage + armorAmount
    if originalDamage <= flashCutoff then
        objc.play_vfx(targetEntity, SHIELD_VFX, NODE_IMPACT)
    end
    return damage
end