--[[
The Charge ability takes all of the player's current units, buffs their attack,
speeds up their max velocity, and plays VFX on them.
--]]
Charge = function(sourceEntity)
   local teamNumber = objc.get_team_number(sourceEntity)
   for _, ally in pairs(objc.list_entities(teamNumber)) do
      if objc.is_normal_unit(ally) then
         -- Buffing the unit's damage
         if objc.get_entity_value(ally, 'aiComponent.damageComponent.attackDamage') > 0 then
            local damageBonus = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF)
            objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF, damageBonus + 5)
         end

         -- Buffing the unit's speed
         local speed = objc.get_entity_value(ally, 'velocityComponent.primaryMaxVelocity')
         local newSpeed = math.floor(speed * 1.5)
         objc.set_entity_value(ally, 'velocityComponent.primaryMaxVelocity', newSpeed)

         -- Blink
         local parameterDictionary = { red = 255,
                                       green = 0, 
                                       blue = 0, 
                                       alpha = 128,
                                       initialBlinkFrequency = 0,
                                       initialBlinkIntensity = 0,
                                       finalBlinkFrequency = 10, 
                                       finalBlinkIntensity = 10,
                                       animationDurationInDeciseconds = 10,
                                       animationAcceleration = 3,
                                       easingType = EASE_IN }

         objc.start_blinking(ally, parameterDictionary)

         -- Playing VFX
         objc.play_vfx(ally, 'CommanderPowerUpText.lvl', NODE_STATUS)
         objc.play_vfx(ally, 'CommanderPowerUp.lvl', NODE_IMPACT)
      end
   end
end
