#define SUPPORT_GLSL300
#include "prefix.sh"
#include "system_uniforms.sh"
#include "texture_utility.sh"
#include "projection_utility.sh"

uniform sampler2D s_cc3Texture2Ds[2];

uniform int CinematicDoFCoC_planarDepthProjection;
uniform int CinematicDoFCoC_maxCoCRadiusPixels;
uniform float CinematicDoFCoC_aperture;
uniform float CinematicDoFCoC_sensorHeight;
uniform float CinematicDoFCoC_focalDistance;
uniform float CinematicDoFCoC_focalLength;
uniform mat4 CinematicDoFCoC_projectionMatrixInv;
uniform vec4 CinematicDoFCoC_frustumCorners[4];

VARYING_IN vec2 v_texCoord[1];

vec3 CalculateWorldPosition(vec3 viewFrustumVector, float linear0To1Depth, vec3 viewPosition)
{
    return viewPosition + viewFrustumVector * linear0To1Depth;
}

vec3 CalculateEyePos_FrustumCorners(float rawDepth)
{
    float x = (gl_FragCoord.x - getViewport().x) / getViewport().z;
    float y = (gl_FragCoord.y - getViewport().y) / getViewport().w;
    
    float topLeftWeight      = (1.f - x) * y;
    float bottomLeftWeight   = (1.f - x) * (1.f - y);
    float topRightWeight     = x * y;
    float bottomRightWeight  = x * (1.f - y);
    
    vec3 viewFrustumVector =
          topLeftWeight     * CinematicDoFCoC_frustumCorners[0].xyz
        + bottomLeftWeight  * CinematicDoFCoC_frustumCorners[1].xyz
        + topRightWeight    * CinematicDoFCoC_frustumCorners[2].xyz
        + bottomRightWeight * CinematicDoFCoC_frustumCorners[3].xyz
        ;
    
    float worldSpaceDepth = depthSampleToWorldSpaceDepth(rawDepth);;
    return CalculateWorldPosition(viewFrustumVector, worldSpaceDepth / getCameraFrustumDepth().x, vec3(0.0f, 0.0f, 0.0f));
}

void main()
{
    float rawDepth = sampleLevelZero(s_cc3Texture2Ds[1], v_texCoord[0]).r;
    
    float dist;
    if (CinematicDoFCoC_planarDepthProjection != 0) {
        // Calculate pixel depth by linearizing depth buffer value
        dist = depthSampleToWorldSpaceDepth(rawDepth);
    } else {
        // Calculate pixel depth using view space position
        vec3 viewPos = CalculateEyePos_FrustumCorners(rawDepth);
        // Alternative math using inverse projection matrix
        // vec4 viewPos = viewSpacePositionFromDepthSample(rawDepth, CinematicDoFCoC_projectionMatrixInv);
        dist = length(viewPos.xyz);
    }
    
    float signedCoCWorldUnits = CinematicDoFCoC_aperture 
                                    * (CinematicDoFCoC_focalLength * (CinematicDoFCoC_focalDistance - dist)) 
                                    / (dist * (CinematicDoFCoC_focalDistance - CinematicDoFCoC_focalLength));
    
    // Calculate coc size in terms of percentage on film / camera sensor
    float signedCoCPercentage = signedCoCWorldUnits / CinematicDoFCoC_sensorHeight;
    
    // Convert to pixels, assuming fixed sensor pixel resolution. Node that this shouldn't be tied to viewport size. 
    const float sensorPixels = 2048.0;
    float signedCoCPixels = signedCoCPercentage * sensorPixels;
    
    gl_FragColor.rgb = sampleLevelZero(s_cc3Texture2Ds[0], v_texCoord[0]).rgb;
    gl_FragColor.a = clamp(signedCoCPixels * 0.5 + 0.5, 0.0, 1.0);
}
