local HAS_INIT = 'hasInitKey'
local STARTS_OWNED = 'startsOwnedKey'
 
local CAPTURING_RADIUS_SIZE = 1300
 
local CAPTURING_TIME_IN_SECONDS = 4
 
local OBJECTIVE_STATE_KEY = "ObjectiveStateKey"
local OBJECTIVE_STATE_UNCAPTURED = 0
local OBJECTIVE_STATE_CAPTURING = 1
local OBJECTIVE_STATE_CAPTURED = 2
 
local OBJECTIVE_CAPTURED_STATE_KEY = "ObjectiveCapturedStateKey"
local OBJECTIVE_CATPURED_STATE_NEVER_CAPTURED = 1
local OBJECTIVE_CAPTURED_STATE_HAS_BEEN_CAPTURED = 2
 
local CAPTURING_UNIT_KEY = "CapturingUnit"
local CAPTURING_START_TIME_KEY = "CapturingStartTime"
local CAPTURING_TEAM_KEY = "CapturingTeam"

--If true, bonus crystals added to team's income. If false, bonus crystals added discretely.
local CONTINUOUS_INCOME_TOGGLE = true

--Crystal rate for continuous income
local CRYSTAL_PER_SECOND_GRANTED = 0.06

--Variables for adding crystals discretely
local TICKS_BETWEEN_CRYSTAL_GRANTS = 'ticksBetweenCrystalGrantsKey'
local INITIAL_TICKS_BETWEEN_CRYSTAL_GRANTS = 100
local crystalPerCrystalGrant = 1

local OWNERSHIP_INDICATOR_KEY = 'OWNERSHIP_INDICATOR_KEY'
 
local NUMBER_OF_MILLISECONDS_IN_ONE_SECOND = 1000
 
 
 
--[[
The Objective_StrategicPoint functionality works as follows (section below ---- line is not yet implemented):
- The objective starts out uncaptured
- Capturing units who come within CAPTURING_RADIUS_SIZE of the objective can capture it if they wait at least CAPTURING_TIME_IN_SECONDS
- If the capturing unit gets attacked while capturing, the stop capturing and return to attacking. Capturing timer gets reset.
----------------------------
- There is some kind of progress bar that shows the progress of the capturing
--]]
 
 
CrystalObjectiveDominate_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local startsOwned = objc.get_local_value(entity, STARTS_OWNED)
        if startsOwned ~= 0 then
            CrystalObjectiveDominate_changeOwnershipToTeam(entity, startsOwned)
        end
 
        objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_UNCAPTURED)
        objc.set_local_value(entity, OBJECTIVE_CAPTURED_STATE_KEY, OBJECTIVE_CATPURED_STATE_NEVER_CAPTURED)
        objc.set_local_value(entity, TICKS_BETWEEN_CRYSTAL_GRANTS, INITIAL_TICKS_BETWEEN_CRYSTAL_GRANTS)
        objc.set_local_value(entity, HAS_INIT, 1)
    end
 
 
    local captureState = objc.get_local_value(entity, OBJECTIVE_STATE_KEY)

    local nearbyUnit = CrystalObjectiveDominate_checkForAnyNearbyUnits(entity)
    local Team1Captor = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(entity, 1)
    local Team2Captor = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(entity, 2)

    if captureState == OBJECTIVE_STATE_UNCAPTURED then 
        if (Team1Captor ~= 0 and Team2Captor == 0) or (Team1Captor == 0 and Team2Captor ~= 0) then
            CrystalObjectiveDominate_startCapturing(entity, nearbyUnit)
        end
    elseif captureState == OBJECTIVE_STATE_CAPTURING then
            CrystalObjectiveDominate_attemptCapture(entity)
    elseif captureState == OBJECTIVE_STATE_CAPTURED then
        local capturingTeamNumber = objc.get_local_value(entity, CAPTURING_TEAM_KEY)
        local capturingUnit = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(entity, capturingTeamNumber)
        local opponentTeamNumber = objc.get_enemy_team_number_for_team_number(capturingTeamNumber)
        local opponentCaptor = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(entity, opponentTeamNumber)
        if capturingUnit == 0 and opponentCaptor ~= 0 then
            CrystalObjectiveDominate_startCapturing(entity, opponentCaptor)
        elseif not CONTINUOUS_INCOME_TOGGLE then
            CrystalObjectiveDominate_tickCrystalRewards(entity, capturingTeamNumber)
        end
    end
end
 
CrystalObjectiveDominate_checkForAnyNearbyUnits = function(objectiveEntity)
    local nearbyTeam1Unit = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(objectiveEntity, 1)
    if nearbyTeam1Unit ~= 0 then
        return nearbyTeam1Unit
    end
 
    local nearbyTeam2Unit = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(objectiveEntity, 2)
    if nearbyTeam2Unit ~= 0 then
        return nearbyTeam2Unit
    end
 
    return 0
end
 
CrystalObjectiveDominate_checkForNearbyUnitsOfTeam = function(objectiveEntity, teamNumber)
    local nearbyUnit = objc.list_entities_in_radius(objectiveEntity, CAPTURING_RADIUS_SIZE, teamNumber)
    for _, unit in pairs(nearbyUnit) do
        if objc.is_normal_unit(unit) then
            return unit
        end
    end
 
   return 0
end
 
CrystalObjectiveDominate_startCapturing = function(objectiveEntity, nearbyUnit)
 
    objc.set_local_value(objectiveEntity, CAPTURING_UNIT_KEY, nearbyUnit)
    local elapsedGameTimeInMilliseconds = objc.get_elapsed_game_time_in_milliseconds()
    objc.set_local_value(objectiveEntity, CAPTURING_START_TIME_KEY, elapsedGameTimeInMilliseconds)
    objc.set_local_value(objectiveEntity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURING)
end
 
CrystalObjectiveDominate_attemptCapture = function(objectiveEntity)
    local capturingUnit = objc.get_local_value(objectiveEntity, CAPTURING_UNIT_KEY) 
    local capturingTeamNumber = objc.get_team_number(capturingUnit)
    local opponentTeamNumber = objc.get_enemy_team_number_for_team_number(capturingTeamNumber)
    local opponentCaptor = CrystalObjectiveDominate_checkForNearbyUnitsOfTeam(objectiveEntity, opponentTeamNumber)
    if opponentCaptor == 0 then
        local elapsedGameTimeInMilliseconds = objc.get_elapsed_game_time_in_milliseconds()
        local captureStartTimeInMilliseconds = objc.get_local_value(objectiveEntity, CAPTURING_START_TIME_KEY)
        local elapsedCaptureTimeInMilliseconds = elapsedGameTimeInMilliseconds - captureStartTimeInMilliseconds
        if elapsedCaptureTimeInMilliseconds > (CAPTURING_TIME_IN_SECONDS * NUMBER_OF_MILLISECONDS_IN_ONE_SECOND) then
           
            CrystalObjectiveDominate_changeOwnership(objectiveEntity, capturingUnit)
          
            local objectiveMaxHealth = objc.get_entity_value_for_key(objectiveEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
            objc.set_entity_value_for_key(objectiveEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH, objectiveMaxHealth)
        else
            -- HACK: We're going to use the objective's health bar to show the progress of the capture
            local objectiveMaxHealth = objc.get_entity_value_for_key(objectiveEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
            local captureProgress = elapsedCaptureTimeInMilliseconds * objectiveMaxHealth
            captureProgress = objc.divide_with_floor(captureProgress, (CAPTURING_TIME_IN_SECONDS * NUMBER_OF_MILLISECONDS_IN_ONE_SECOND))
            
            if captureProgress < 1 then
                captureProgress = 1
            end
            objc.set_entity_value_for_key(objectiveEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH, captureProgress)
        end
    end
end
 
 
CrystalObjectiveDominate_changeOwnership = function(objectiveEntity, nearbyUnit)
    local nearbyUnitTeam = objc.get_team_number(nearbyUnit)
    objc.set_local_value(objectiveEntity, CAPTURING_TEAM_KEY, nearbyUnitTeam)
    objc.set_local_value(objectiveEntity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURED)
 
    local enemyUnitTeam = objc.get_enemy_team_number(nearbyUnit)
    
    local ownershipIndicator = objc.get_local_value(objectiveEntity, OWNERSHIP_INDICATOR_KEY)
 
    if ownershipIndicator ~= 0 then
        objc.kill_entity(ownershipIndicator)
        if CONTINUOUS_INCOME_TOGGLE then
            --Don't punish the other team's income if the objective was never captured to begin with.
            if objc.get_local_value(objectiveEntity, OBJECTIVE_CAPTURED_STATE_KEY) ~= OBJECTIVE_CATPURED_STATE_NEVER_CAPTURED then
                --Remove the previous owner's bonus crystal income. Doesn't properly account for having over 6 sources of bonus income.
                local prevEnemyIncome = objc.get_income_generation_frequency_in_milliseconds(enemyUnitTeam)
                local newEnemyIncome
                if prevEnemyIncome == 1300 then
                    newEnemyIncome = 1400
                elseif prevEnemyIncome == 1400 then
                    newEnemyIncome = 1500
                elseif prevEnemyIncome == 1500 then
                    newEnemyIncome = 1700
                elseif prevEnemyIncome == 1700 then
                    newEnemyIncome = 1900
                elseif prevEnemyIncome == 1900 then
                    newEnemyIncome = 2100
                elseif prevEnemyIncome == 2100 then
                    newEnemyIncome = 2400
                elseif prevEnemyIncome == 2400 then
                    newEnemyIncome = 2700
                else
                    newEnemyIncome = 3300
                end
                objc.set_income_generation_frequency_in_milliseconds(enemyUnitTeam, newEnemyIncome)
            end
        end
    end

    objc.set_local_value(objectiveEntity, OBJECTIVE_CAPTURED_STATE_KEY, OBJECTIVE_CAPTURED_STATE_HAS_BEEN_CAPTURED)
 
    local ownershipIndicatorSpawnIdentifier = 'blueRegionCapture'
    if nearbyUnitTeam == 2 then
        ownershipIndicatorSpawnIdentifier = 'redRegionCapture'
    end
 
    local entityPosition = objc.get_entity_position(objectiveEntity)
    local newOwnershipIndicatorArray = objc.spawn_at_point(ownershipIndicatorSpawnIdentifier, 1, 0, entityPosition[1], entityPosition[2] - 2500, 0, 0)  
    objc.set_local_value(objectiveEntity, OWNERSHIP_INDICATOR_KEY, newOwnershipIndicatorArray[1])
    
    if CONTINUOUS_INCOME_TOGGLE then
        --Grant the new owner bonus crystal income. Only works for up to 6 sources of bonus income.
        local prevOwnerIncome = objc.get_income_generation_frequency_in_milliseconds(nearbyUnitTeam)
        local newOwnerIncome
        if prevOwnerIncome == 3300 then
            newOwnerIncome = 2700
        elseif prevOwnerIncome == 2700 then
            newOwnerIncome = 2400
        elseif prevOwnerIncome == 2400 then
            newOwnerIncome = 2100
        elseif prevOwnerIncome == 2100 then
            newOwnerIncome = 1900
        elseif prevOwnerIncome == 1900 then
            newOwnerIncome = 1700
        elseif prevOwnerIncome == 1700 then
            newOwnerIncome = 1500
        elseif prevOwnerIncome == 1500 then
            newOwnerIncome = 1400
        else
            newOwnerIncome = 1300
        end
        objc.set_income_generation_frequency_in_milliseconds(nearbyUnitTeam, newOwnerIncome)
    end
end

 
CrystalObjectiveDominate_changeOwnershipToTeam = function(objectiveEntity, nearbyUnit)
    objc.set_local_value(objectiveEntity, CAPTURING_TEAM_KEY, nearbyUnit)
    objc.set_local_value(objectiveEntity, OBJECTIVE_CAPTURED_STATE_KEY, OBJECTIVE_CAPTURED_STATE_HAS_BEEN_CAPTURED)
    local ownershipIndicator = objc.get_local_value(objectiveEntity, OWNERSHIP_INDICATOR_KEY)
 
    if ownershipIndicator ~= 0 then
        objc.kill_entity(ownershipIndicator)
        --print('Killed the previous indicator')
    end
 
    local ownershipIndicatorSpawnIdentifier = 'blueRegionCapture'
    if nearbyUnitTeam == 2 then
        ownershipIndicatorSpawnIdentifier = 'redRegionCapture'
    end
 
    local entityPosition = objc.get_entity_position(objectiveEntity)
    local newOwnershipIndicatorArray = objc.spawn_at_point(ownershipIndicatorSpawnIdentifier, 1, 0, entityPosition[1], entityPosition[2] - 2500, 0, 0)  
    objc.set_local_value(objectiveEntity, OWNERSHIP_INDICATOR_KEY, newOwnershipIndicatorArray[1])
 
 
 
end
 
 
CrystalObjectiveDominate_resetObjective = function(objectiveEntity)
    objc.set_local_value(objectiveEntity, CAPTURING_UNIT_KEY, 0)
    objc.set_local_value(objectiveEntity, CAPTURING_START_TIME_KEY, 0)
 
    local capturedState = objc.get_local_value(objectiveEntity, OBJECTIVE_CAPTURED_STATE_KEY)
    if capturedState == OBJECTIVE_CAPTURED_STATE_HAS_BEEN_CAPTURED then
        objc.set_local_value(objectiveEntity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURED)
    else
        objc.set_local_value(objectiveEntity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_UNCAPTURED)
    end
end
 
--Only called if CONTINUOUS_INCOME_TOGGLE == false
CrystalObjectiveDominate_tickCrystalRewards = function(entity, ownerTeam)
    local crystalGrantTicks = objc.get_local_value(entity, TICKS_BETWEEN_CRYSTAL_GRANTS)
    local crystalTimer = objc.tick_timer(entity, 'crystalTimer', crystalGrantTicks)
 
    if crystalTimer == 0 then
        objc.add_crystal(ownerTeam, crystalPerCrystalGrant)
        local entityPosition = objc.get_entity_position(entity)
        --REMOVING WORKER BOT UNTIL I CAN ACTUALLY MAKE THE STRUCTURAL CHANGE
        --objc.spawn_at_point(WORKER_BOT_IDENTIFIER, 1, ownerTeam, entityPosition[1] - 100, entityPosition[2] + 1000, 0, 0)
        objc.spawn_at_point('crystalGainEffectEntity', 1, 0, entityPosition[1] - 100, entityPosition[2] + 1000, 0, 0) 
    end
end
 
