local CAPTURING_RADIUS_SIZE = 1400

local OBJECTIVE_STATE_KEY = "ObjectiveStateKey"
local OBJECTIVE_STATE_UNCAPTURED = 0
local OBJECTIVE_STATE_CAPTURING = 1
local OBJECTIVE_STATE_CAPTURED = 2
local OBJECTIVE_STATE_CONTESTED = 3

local IS_TWO_LANE_MAP_GLOBAL_KEY = "isTwoLaneMapGlobalKey"
-- is 0 if on single lane map, 1 if on two lane map and is on the top lane, and 2 if on two lane map and is on the bottom lane
local LANE_INDEX_KEY = "laneIndexKey"

local CAPTURE_TIME_IN_ROUNDS = 40
local CURRENT_CAPTURE_PROGRESS = 'CurrentCaptureProgressKey'
local CAPTURING_TEAM_KEY = 'CapturingTeam'
local OWNER_TEAM_KEY = 'OwnerTeamKey'

local CRYSTAL_PER_SECOND_GRANTED = 0.1

local CAPTURE_ENTITY = 'firstTimeObjectiveCaptureEntity'
local HAS_BEEN_CAPTURED = 'hasBeenCapturedKey'
local UNIT_FLARE_VFX = 'fx_unit_flare.lvl'
local HAS_PLAYED_UNIT_FLARE = 'hasPlayedUnitFlareKey'
local IDLE_VFX = 'env_mineral_low.lvl'
local IS_PLAYING_IDLE_VFX = 'isPlayingIdleVFXKey'
local CRYSTAL_NODE_LEFT = 'VFX_Left'
local CRYSTAL_NODE_RIGHT = 'VFX_Right'

local SPAWNED_POWER_PLANTS = 'spawnedPowerPlantKey'

local POWERPLANT_OBJECTIVE_INDEX_KEY = 'powerplantObjectiveIndex'

---These need to be explicitly named to be picked up as dependencies
local RED_POWER_PLANT = 'R_PowerPlant_Objective'
local BLUE_POWER_PLANT = 'B_PowerPlant_Objective'
local WHITE_POWER_PLANT = 'W_PowerPlant_Objective'

CrystalObjectiveDominate_spawnTrigger = function(entity)
    objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_UNCAPTURED)
    local isTwoLanes = objc.get_global_value(IS_TWO_LANE_MAP_GLOBAL_KEY)

    if isTwoLanes == 1 then
        if objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y) < 0 then
            objc.set_local_value(entity, LANE_INDEX_KEY, 1)
        else
            objc.set_local_value(entity, LANE_INDEX_KEY, 2)
        end

        objc.play_vfx(entity, IDLE_VFX, CRYSTAL_NODE_LEFT)
    else
        objc.set_local_value(entity, LANE_INDEX_KEY, 1)

        objc.play_vfx(entity, IDLE_VFX, CRYSTAL_NODE_LEFT)
        objc.play_vfx(entity, IDLE_VFX, CRYSTAL_NODE_RIGHT)
    end
    objc.set_local_value(entity, IS_PLAYING_IDLE_VFX, 1)
end

CrystalObjectiveDominate_tickTrigger = function(entity)
    local numTeam1 = CrystalObjectiveDominate_numNearbyUnitsOfTeamRECTANGLE(entity, 1)
    local numTeam2 = CrystalObjectiveDominate_numNearbyUnitsOfTeamRECTANGLE(entity, 2)
    local tag = objc.get_local_value(entity, LUA_KEY_TAG)
    local capturedKey = tag .. "CapturedByTeam"
    local ownedState = objc.get_global_value(capturedKey)

    local captureDirection = 0
    local contested = 0

    if (numTeam1 > 0 and numTeam2 == 0) then
        captureDirection = 1
    elseif (numTeam1 == 0 and numTeam2 > 0) then
        captureDirection = -1
    elseif (numTeam1 > 0 and numTeam2 > 0) then
        contested = 1
    end

    local currentCaptureProgress = objc.get_local_value(entity, CURRENT_CAPTURE_PROGRESS)

    if (captureDirection == 0 and ownedState == 0 and contested == 0) then --lerp back towards unowned if there are no capturing units
        if currentCaptureProgress > 0 then
            captureDirection = -1
        elseif currentCaptureProgress < 0 then
            captureDirection = 1
        end
    end

    if (captureDirection == 0 and ownedState == 1 and contested == 0) then --lerp back to previous owner if there are no capturing units
        captureDirection = 1
    end

    if (captureDirection == 0 and ownedState == 2 and contested == 0) then
        captureDirection = -1
    end

    currentCaptureProgress = currentCaptureProgress + captureDirection

    if currentCaptureProgress > CAPTURE_TIME_IN_ROUNDS then --Meter is moving in Team 1's direction
        currentCaptureProgress = CAPTURE_TIME_IN_ROUNDS
        local hasBeenCaptured = objc.get_local_value(entity, HAS_BEEN_CAPTURED)
        if hasBeenCaptured == 0 then --Only run this section once
            local objectivePos = objc.get_entity_position(entity)
            objc.spawn_at_point(CAPTURE_ENTITY, 1, 1, objectivePos[1], objectivePos[2], 0, 0) --play VFX
            objc.set_local_value(entity, HAS_BEEN_CAPTURED, 1)
            objc.set_global_value(capturedKey, 1) --TEAM 1 HAS CAPTURED
            objc.set_global_value("placementHasChanged", 1) --Set flag to update placement area

            local capturingFaction = CrystalObjectiveDominate_getFactionFromBase(1)
            CrystalObjectiveDominate_spawnPowerPlantOfFactionForPlayer(entity, capturingFaction, 1)

            CrystalObjectiveDominate_killIdleVFX(entity)
        end
        objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURED) --Set income
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_OVERRIDE_PLAYER_NUMBER, 1) --Set which player owns the objective
        objc.set_local_value(entity, CAPTURING_TEAM_KEY, 1)
    elseif currentCaptureProgress < -CAPTURE_TIME_IN_ROUNDS then
        currentCaptureProgress = -CAPTURE_TIME_IN_ROUNDS
        local hasBeenCaptured = objc.get_local_value(entity, HAS_BEEN_CAPTURED)
        if hasBeenCaptured == 0 then
            local objectivePos = objc.get_entity_position(entity)
            objc.spawn_at_point(CAPTURE_ENTITY, 1, 1, objectivePos[1], objectivePos[2], 0, 0)
            objc.set_local_value(entity, HAS_BEEN_CAPTURED, 1)
            objc.set_global_value(capturedKey, 2)
            objc.set_global_value("placementHasChanged", 1)

            local capturingFaction = CrystalObjectiveDominate_getFactionFromBase(2)
            CrystalObjectiveDominate_spawnPowerPlantOfFactionForPlayer(entity, capturingFaction, 2)

            CrystalObjectiveDominate_killIdleVFX(entity)
        end
        objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURED)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_OVERRIDE_PLAYER_NUMBER, 2)
        objc.set_local_value(entity, CAPTURING_TEAM_KEY, 2)
    elseif captureDirection > 0 and currentCaptureProgress == 0 then --when the progress is back to 0, and Team 1 is capturing
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_OVERRIDE_PLAYER_NUMBER, 0)
        objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURING)
        objc.set_local_value(entity, CAPTURING_TEAM_KEY, 1)
        objc.set_local_value(entity, HAS_BEEN_CAPTURED, 0)
        objc.set_global_value(capturedKey, 0)
        objc.set_global_value("placementHasChanged", 1)
        CrystalObjectiveDominate_killPowerPlants(entity)
    elseif captureDirection < 0 and currentCaptureProgress == 0 then --when the progress is back to 0, and Team 2 is capturing
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_OVERRIDE_PLAYER_NUMBER, 0)
        objc.set_local_value(entity, OBJECTIVE_STATE_KEY, OBJECTIVE_STATE_CAPTURING)
        objc.set_local_value(entity, CAPTURING_TEAM_KEY, 2)
        objc.set_local_value(entity, HAS_BEEN_CAPTURED, 0)
        objc.set_global_value(capturedKey, 0)
        objc.set_global_value("placementHasChanged", 1)
        CrystalObjectiveDominate_killPowerPlants(entity)
    elseif captureDirection > 0 then
        CrystalObjectiveDominate_playFlareOnCapturingUnits(entity, 1)
    elseif captureDirection < 0 then
        CrystalObjectiveDominate_playFlareOnCapturingUnits(entity, 2)
    end


    objc.set_local_value(entity, CURRENT_CAPTURE_PROGRESS, currentCaptureProgress)


    CrystalObjectiveDominate_updateVisual(entity, captureDirection)
end

CrystalObjectiveDominate_playFlareOnCapturingUnits = function(entity, teamNumber)
    local hasPlayedUnitFlare = objc.get_local_value(entity, HAS_PLAYED_UNIT_FLARE)
    if hasPlayedUnitFlare == 0 then
        local capturingUnits = CrystalObjectiveDominate_nearbyUnitsOfTeamRECTANGLE(entity, teamNumber)
        for _, capturingUnit in pairs(capturingUnits) do
            if objc.is_normal_unit(capturingUnit) or objc.is_commander(capturingUnit) then
                objc.play_vfx(capturingUnit, UNIT_FLARE_VFX, NODE_ORIGIN)
            end
        end

        objc.set_local_value(entity, HAS_PLAYED_UNIT_FLARE, 1)
    end
end

CrystalObjectiveDominate_updateVisual = function(entity, captureDirection)
    local currentCaptureProgress = objc.get_local_value(entity, CURRENT_CAPTURE_PROGRESS)
    local capturePercentage = math.floor(currentCaptureProgress / CAPTURE_TIME_IN_ROUNDS * 100)
    objc.set_capture_percentage(entity, capturePercentage)
end

CrystalObjectiveDominate_numNearbyUnitsOfTeam = function(objectiveEntity, teamNumber)
    local nearbyUnits = objc.list_entities_in_radius(objectiveEntity, CAPTURING_RADIUS_SIZE, teamNumber)
    local numUnits = 0
    for _, unit in pairs(nearbyUnits) do
        if objc.is_normal_unit(unit) or objc.is_commander(unit) then
            numUnits = numUnits + 1
        end
    end

    return numUnits
end

-- To use the rectangular math, use something like this for a one lane map
CrystalObjectiveDominate_nearbyUnitsOfTeamRECTANGLE = function(objectiveEntity, teamNumber)
    local leftOffset = 1202
    local rightOffset = 10000
    local width = 6600
    local position = objc.get_entity_position(objectiveEntity)
    if teamNumber == 2 then
        leftOffset = leftOffset * -1
        rightOffset = rightOffset * -1
    end
    local x = (-leftOffset + rightOffset) / 2
    local y = position[2]
    if y > 100 or y < -100 then
        width = 3000
    end

    local nearbyUnits = objc.list_entities_in_rectangle(x, y, math.abs(leftOffset + rightOffset), width, teamNumber)
    return nearbyUnits
end

CrystalObjectiveDominate_numNearbyUnitsOfTeamRECTANGLE = function(objectiveEntity, teamNumber)
    local nearbyUnits = CrystalObjectiveDominate_nearbyUnitsOfTeamRECTANGLE(objectiveEntity, teamNumber)
    local numUnits = 0
    for _, unit in pairs(nearbyUnits) do
        if objc.is_normal_unit(unit) or objc.is_commander(unit) or objc.is_building_unit(unit) then
            numUnits = numUnits + 1
        end
    end
    return numUnits
end

-- Get base of playerNumber, then get faction of the player that owns that ship to get a TEAM's faction (relevant in 2v2) instead of a PLAYER's faction
CrystalObjectiveDominate_getFactionFromBase = function(playerNumber)
    local baseEntityPlayerNumber = objc.get_player_number(objc.get_base_entity(playerNumber))
    return objc.get_commander_factions(baseEntityPlayerNumber)[1]
end

CrystalObjectiveDominate_spawnPowerPlantOfFactionForPlayer = function(entity, capturingFaction, playerNumber)
    local identifier = ''
    local prefix = ''
    if capturingFaction == 'blue' then
        identifier = WHITE_POWER_PLANT
        prefix = 'W_'
    elseif capturingFaction == 'red' then
        identifier = RED_POWER_PLANT
        prefix = 'R_'
    elseif capturingFaction == 'green' then
        identifier = BLUE_POWER_PLANT
        prefix = 'B_'
    end

    if identifier ~= '' then
        local laneIndex = objc.get_local_value(entity, LANE_INDEX_KEY)
        local powerplantMarkers = objc.location_marker_node_names_with_prefix("Center_PowerPlant_" .. laneIndex)
        local spawnedArray = {}
        local index = 1
        for _, marker in pairs(powerplantMarkers) do
            local markerX, markerY = objc.location_of_marker_node_named(marker)
            local spawned = objc.spawn_at_point(identifier, 1, playerNumber, markerX, markerY, 0, 1)
            objc.set_local_value(spawned[1], POWERPLANT_OBJECTIVE_INDEX_KEY, index)
            objc.set_local_value(spawned[1], LANE_INDEX_KEY, laneIndex)
            index = index + 1
            table.insert(spawnedArray, spawned[1])
        end

        -- Play the Objective Power Plant's spawn animation
        for _, powerplant in pairs(spawnedArray) do
            objc.play_override_animation(powerplant, prefix .. 'PowerPlant_Objective_Spawn_01', objc.convert_frames_to_subrounds(70))
        end

        objc.set_local_value(entity, SPAWNED_POWER_PLANTS, spawnedArray)
    else
        objc.scripting_error('Unable to spawn power plant for faction: ' .. capturingFaction)
    end
end

CrystalObjectiveDominate_killPowerPlants = function(entity)
    local spawnedPowerPlant = objc.get_local_value(entity, SPAWNED_POWER_PLANTS)
    if spawnedPowerPlant ~= 0 then
        for _, powerplant in pairs(spawnedPowerPlant) do
            if powerplant ~= 0 then
                objc.sacrifice_entity(powerplant)
            end
        end
        objc.set_local_value(entity, SPAWNED_POWER_PLANTS, 0)
    end
end

CrystalObjectiveDominate_killIdleVFX = function(entity)
    local isPlayingIdleVFX = objc.get_local_value(entity, IS_PLAYING_IDLE_VFX)
    if isPlayingIdleVFX == 1 then
        local isTwoLanes = objc.get_global_value(IS_TWO_LANE_MAP_GLOBAL_KEY)
        if isTwoLanes == 1 then
            objc.stop_vfx(entity, IDLE_VFX, CRYSTAL_NODE_LEFT, false)
        else
            objc.stop_vfx(entity, IDLE_VFX, CRYSTAL_NODE_LEFT, false)
            objc.stop_vfx(entity, IDLE_VFX, CRYSTAL_NODE_RIGHT, false)
        end
        objc.set_local_value(entity, IS_PLAYING_IDLE_VFX, 0)
    end
end
