local CHECK_RADIUS = 6000
local CHECK_INTERVAL = 4

local MOTION_STATE_DICT = 'motionStateDictKey'
local IS_MOVING_TO_BASE = 'isMovingToBaseKey'
local SHOULD_MOVE = 'shouldDoMoveCommand'
local TARGET_MINE = 'targetMineKey'

local MINE_MAX_MINERS_KEY = 'maxMinersKey'
local NUM_MINERS_KEY = 'numMinersKey'
local CURRENT_CRYSTALS_KEY = 'currentCrystalsKey'
local MINER_LIST = 'minerListKey'
local MINER_WAYPOINT = 'MinerWaypoint'

--Set to 1 so other units can identify this unit as a miner.
local IS_MINER_KEY = "isMinerKey"
local HAS_INIT = 'hasInitKey'
local INITIAL_TIME = 'initialTimeTaken'

local SPEED_TO_MOVE = 90
local CRYSTAL_PAUSE_DURATION = 30
local BASE_PAUSE_DURATION = 10
local PAUSE_TIMER_KEY = 'pauseTimerKey'
local HAS_STARTED_ANIM = 'hasStartedAnimKey'
local NUMBER_OF_FRAMES_IN_WORK_ANIMATION = 90
local WORK_ANIMATION_NAME_KEY = 'attackAnimationNameKey'
local CRYSTAL_DOWN_WORK_ANIMATION_NAME_KEY = 'crystalDownWorkAnimKey'
local CRYSTAL_DOWN_MOVE_ANIM_NAME_KEY = 'crystalDownMoveAnimKey'
local MOVE_ANIM_NAME_KEY = 'moveAnimKey'
local SPAWN_ANIM_NAME = 'spawnAnimNameKey'

local PLAY_SPAWN_DELAY = 35
local SPAWN_DELAY_TRIGGER_NAME = 'CrystalWorker_playSpawnAfterDelayTrigger'

--Kill the miner after Fight It Out starts.
local GAME_TIME_UNTIL_DEATH = 180
local GAME_2V2_TIME_UNTIL_DEATH = 240

--Make the miner invisible if spawned at the very beginning of the game
local GAME_TIME_UNTIL_VISIBLE = 2

--If true, bonus crystals added to team's income. If false, bonus crystals added discretely.
local CONTINUOUS_INCOME_TOGGLE = true
local BASE_OFFSET = 3000

local CRYSTAL_CARRY_AMOUNT = 1

local PLATFORM_ID_KEY = 'platformEntityKey'
local ASSOCIATED_WORKER = 'associatedWorkerKey'

local MINE_POINT_X = 'minePointXKey'
local MINE_POINT_Y = 'minePointYKey'

local IS_PRE_PLACED = 'isPrePlacedKey'

CrystalWorker_spawnTrigger = function(entity)
    local entityPosition = objc.get_entity_position(entity)
    local teamNumber = objc.get_team_number(entity)
    local allies = objc.list_entities(teamNumber)
    for _, ally in pairs(allies) do
        if objc.is_barricade(ally) then
            objc.set_targetable(entity, false)
        end
    end

    objc.set_local_value(entity, IS_MINER_KEY, 1)
end

CrystalWorker_findMinerWaypoint = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local objectives = objc.list_entities(teamNumber)
    local baseEntity = entity
    for _, objective in pairs(objectives) do
        local thisID = objc.get_entity_value_for_key(objective, ENTITY_VALUE_KEY_IDENTIFIER)
        if string.match(thisID, MINER_WAYPOINT) ~= nil then
            baseEntity = objective
        end
    end

    objc.set_local_value(entity, MINER_WAYPOINT, baseEntity)
end


CrystalWorker_findTargetMine = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local playerNumber = objc.get_player_number(entity)
    local objectives = objc.list_entities(teamNumber)
    local closestMine = entity
    local closestDistance = 99999999
    local entityPosition = objc.get_entity_position(entity)
    local minePointX = objc.get_local_value(entity, MINE_POINT_X)
    local minePointY = objc.get_local_value(entity, MINE_POINT_Y)
    for _, objective in pairs(objectives) do
        if objc.is_crystal_mine(objective) ~= 0 then
            local numMiners = objc.get_local_value(objective, NUM_MINERS_KEY)
            if numMiners < objc.get_local_value(objective, MINE_MAX_MINERS_KEY) then
                local thisDistance = math.abs(objc.unit_distance_with_offset(entity, 0, 0, objective, 0, 0))
                --Check if we're a pre-placed miner
                if minePointX ~= 0 or minePointY ~= 0 then
                    local thisPosition = objc.get_entity_position(objective)
                    if minePointX == thisPosition[1] and minePointY == thisPosition[2] then
                        thisDistance = 0
                        closestMine = objective
                        break
                    end
                end

                if thisDistance < closestDistance and thisDistance < CHECK_RADIUS then
                    closestMine = objective
                    closestDistance = thisDistance
                end
            end
        end
    end

    if closestMine ~= entity then
        objc.set_local_value(entity, TARGET_MINE, closestMine)
        local targetMineList = objc.get_local_value(closestMine, MINER_LIST)
        if targetMineList == 0 then
            targetMineList = {}
        end
        table.insert(targetMineList, entity)
        objc.set_local_value(closestMine, MINER_LIST, targetMineList)

        local numMiners = objc.get_local_value(closestMine, NUM_MINERS_KEY)
        objc.set_local_value(closestMine, NUM_MINERS_KEY, numMiners + 1)

        local platformId = objc.get_local_value(entity, PLATFORM_ID_KEY)
        if platformId ~= 0 then
            local minePos = objc.get_entity_position(closestMine)
            local playerNumber = objc.get_player_number(entity)
            local spawnedPlatform = objc.spawn_at_point(platformId, 1, playerNumber, minePos[1], minePos[2], 0, 0)

            -- Play the PowerPlant's spawn animation
            local faction = objc.get_commander_factions(objc.get_player_number(spawnedPlatform[1]))
            local factionPrefix = ''
            if faction[1] == 'red' then
                factionPrefix = 'R_'
            elseif faction[1] == 'blue' then
                factionPrefix = 'W_'
            elseif faction[1] == 'green' then
                factionPrefix = 'B_'
            end
            if factionPrefix ~= '' then
                objc.play_override_animation(spawnedPlatform[1], factionPrefix .. 'PowerPlant_Spawn_01', objc.convert_frames_to_subrounds(60))
            end

            local isPrePlaced = objc.get_local_value(entity, IS_PRE_PLACED)
            objc.set_local_value(spawnedPlatform[1], IS_PRE_PLACED, isPrePlaced)

            objc.set_solid(spawnedPlatform[1], false)
            objc.set_local_value(spawnedPlatform[1], ASSOCIATED_WORKER, entity)
        end
    else
        objc.sacrifice_entity(entity)
    end
end

CrystalWorker_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local elapsedGameTime = objc.get_elapsed_game_time_in_seconds()
        if elapsedGameTime < GAME_TIME_UNTIL_VISIBLE then
            --delay for one round to allow us to correctly play override animations
            --idle for a bit to show them spawning from the doggie door on base if it's a beginning one
            objc.set_entity_visibility(entity, false)
            objc.idle_entity(entity, PLAY_SPAWN_DELAY * 100)
            objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'CrystalWorker.lua', SPAWN_DELAY_TRIGGER_NAME)
        end

        CrystalWorker_findMinerWaypoint(entity)
        CrystalWorker_findTargetMine(entity)

        objc.set_local_value(entity, HAS_INIT, 1)
    elseif hasInit == 1 then
        CrystalWorker_moveToMine(entity)
        objc.set_local_value(entity, HAS_INIT, 2)
    end

    local shouldMove = objc.get_local_value(entity, SHOULD_MOVE)
    local isMovingToBase = objc.get_local_value(entity, IS_MOVING_TO_BASE)
    if shouldMove == 1 then
        if isMovingToBase == 1 then
            CrystalWorker_moveToBase(entity)
        else
            CrystalWorker_moveToMine(entity)
        end

        objc.set_local_value(entity, SHOULD_MOVE, 0)
    end


    local is2vE = 0
    local playerNumbers = objc.get_player_numbers_for_team_number(1)
    for _, number in pairs(playerNumbers) do
        if number > 2 then
            is2vE = 1
        end
    end

    if is2vE == 0 then
        if objc.get_elapsed_game_time_in_seconds() > GAME_TIME_UNTIL_DEATH then
            objc.sacrifice_entity(entity)
        end
    else
        if objc.get_elapsed_game_time_in_seconds() > GAME_2V2_TIME_UNTIL_DEATH then
            objc.sacrifice_entity(entity)
        end
    end
end

CrystalWorker_moveToMine = function(entity)
    local targetMine = objc.get_local_value(entity, TARGET_MINE)
    if targetMine ~= 0 then
        local velocity = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
        local newMotionState = objc.move_entity_to_entity_with_motion(entity, targetMine, MOTION_STATE_WALK, SPEED_TO_MOVE)
        objc.set_local_value(entity, MOTION_STATE_DICT, newMotionState)
    end
end

CrystalWorker_moveToBase = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local baseEntity = objc.get_local_value(entity, MINER_WAYPOINT)
    local velocity = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    local newMotionState = objc.move_entity_to_entity_with_motion(entity, baseEntity, MOTION_STATE_WALK, SPEED_TO_MOVE)
    objc.set_local_value(entity, MOTION_STATE_DICT, newMotionState)
end

CrystalWorker_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local didMoveToBase = objc.get_local_value(sourceEntity, IS_MOVING_TO_BASE)
    local teamNumber = objc.get_team_number(sourceEntity)
    local oldMotionState = objc.get_local_value(sourceEntity, MOTION_STATE_DICT)
    if oldMotionState ~= 0 then
        objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, oldMotionState)
        objc.set_local_value(sourceEntity, MOTION_STATE_DICT, 0)
    end

    if didMoveToBase == 0 then
        local targetMine = objc.get_local_value(sourceEntity, TARGET_MINE)
        if targetMine ~= 0 then
            local numCrystals = objc.get_local_value(targetMine, CURRENT_CRYSTALS_KEY)
            objc.set_local_value(targetMine, CURRENT_CRYSTALS_KEY, numCrystals - 1)
        end
        objc.set_local_value(sourceEntity, IS_MOVING_TO_BASE, 1)
    else
        objc.set_local_value(sourceEntity, IS_MOVING_TO_BASE, 0)
    end
    objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'CrystalWorker.lua', 'CrystalWorker_waitForAnimationTrigger')

end

CrystalWorker_waitForAnimationTrigger = function(entity)
    local isMovingToBase = objc.get_local_value(entity, IS_MOVING_TO_BASE)

    local hasStartedAnim = objc.get_local_value(entity, HAS_STARTED_ANIM)
    if hasStartedAnim == 0 then
        if isMovingToBase == 1 then
            local crystalDownWorkAnimationName = objc.get_local_value(entity, CRYSTAL_DOWN_WORK_ANIMATION_NAME_KEY)
            if crystalDownWorkAnimationName ~= 0 then
                local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_WORK_ANIMATION)
                objc.play_override_animation(entity, crystalDownWorkAnimationName, objc.subrounds_to_milliseconds(numberOfSubrounds))
            end

            local crystalUpMoveAnimName = objc.get_local_value(entity, MOVE_ANIM_NAME_KEY)
            if crystalUpMoveAnimName ~= 0 then
                local crystalUpAnims = {}
                crystalUpAnims[ANIMATION_STATE_MOVE] = crystalUpMoveAnimName
                objc.update_animation_state(entity, crystalUpAnims)
            end
        else
            local crystalUpWorkAnimationName = objc.get_local_value(entity, WORK_ANIMATION_NAME_KEY)
            if crystalUpWorkAnimationName ~= 0 then
                local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_WORK_ANIMATION)
                objc.play_override_animation(entity, crystalUpWorkAnimationName, objc.subrounds_to_milliseconds(numberOfSubrounds))
            end

            local crystalDownMoveAnimName = objc.get_local_value(entity, CRYSTAL_DOWN_MOVE_ANIM_NAME_KEY)
            if crystalDownMoveAnimName ~= 0 then
                local crystalDownAnims = {}
                crystalDownAnims[ANIMATION_STATE_MOVE] = crystalDownMoveAnimName
                objc.update_animation_state(entity, crystalDownAnims)
            end
        end
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
        objc.set_local_value(entity, HAS_STARTED_ANIM, 1)
    end

    local waitTimer
    if isMovingToBase == 0 then
        waitTimer = objc.tick_timer(entity, PAUSE_TIMER_KEY, BASE_PAUSE_DURATION)
    else
        waitTimer = objc.tick_timer(entity, PAUSE_TIMER_KEY, CRYSTAL_PAUSE_DURATION)
    end

    if waitTimer == 0 then
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, SPEED_TO_MOVE)
        objc.set_local_value(entity, SHOULD_MOVE, 1)
        objc.set_local_value(entity, HAS_STARTED_ANIM, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'CrystalWorker_waitForAnimationTrigger')
    end
end

CrystalWorker_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    --The miners themselves should not be damaged ever. Instead, the mining rings should be damaged
    --This gives us some protections around lua-based damage and splash damage.
    return 0
end

CrystalWorker_deathTrigger = function(entity)
    local targetMine = objc.get_local_value(entity, TARGET_MINE)
    if targetMine ~= 0 then
        local numMiners = objc.get_local_value(targetMine, NUM_MINERS_KEY)
        local minerList = objc.get_local_value(targetMine, MINER_LIST)
        if minerList ~= 0 then
            local thisIndex = -1
            for i = 1, listCount(minerList) do
                if minerList[i] == entity then
                    thisIndex = i
                end
            end

            if thisIndex ~= -1 then
                table.remove(minerList, thisIndex)
            end
            objc.set_local_value(targetMine, MINER_LIST, minerList)
        end
        objc.set_local_value(targetMine, NUM_MINERS_KEY, numMiners - 1)
    end
end

CrystalWorker_playSpawnAfterDelayTrigger = function(entity)
    local playSpawnTrigger = objc.tick_timer(entity, 'playSpawnDelayKey', PLAY_SPAWN_DELAY)
    if playSpawnTrigger == 0 then
        local isPrePlaced = objc.get_local_value(entity, IS_PRE_PLACED)
        if isPrePlaced == 0 then
            objc.set_entity_visibility(entity, true)
        end

        local spawnAnimationName = objc.get_local_value(entity, SPAWN_ANIM_NAME)
        if spawnAnimationName ~= 0 then
            objc.play_override_animation(entity, spawnAnimationName, 1000)
        end
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, SPAWN_DELAY_TRIGGER_NAME)
    end
end
