local NUM_BOUNCES = 'numBouncesKey'
local STUN_LENGTH = 'stunLengthKey'
local BOUNCE_RADIUS = 'bounceRadiusKey'
local HAS_INIT = 'hasInitKey'
local INIT_TARGET = 'initTargetKey'

EternalWarriorShieldEntity_tickTrigger = function(entity)
	local hasInit = objc.get_local_value(entity, HAS_INIT)
	if hasInit == 0 then
		objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
		objc.set_local_value(entity, HAS_INIT, 1)
	end
	local initTarget = objc.get_local_value(entity, INIT_TARGET)
	if initTarget ~= 0 then
		objc.move_entity_to_entity_with_motion(entity, initTarget, MOTION_STATE_WALK, 600)
		objc.set_local_value(entity, INIT_TARGET, 0)
	end
end

EternalWarriorShieldEntity_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
	local numBounces = objc.get_local_value(sourceEntity, NUM_BOUNCES)
	local stunLength = objc.get_local_value(sourceEntity, STUN_LENGTH)
	local bounceRadius = objc.get_local_value(sourceEntity, BOUNCE_RADIUS)
	local enemyNumber = objc.get_enemy_team_number(sourceEntity)

	if numBounces >= 0 then
		objc.set_local_value(sourceEntity, NUM_BOUNCES, numBounces - 1)
		objc.stun_entity(targetEntity, stunLength)
		local enemies = objc.list_entities_in_radius(targetEntity, bounceRadius, enemyNumber)
		local closestDistance = 9999999
		local closestEntity = targetEntity
		for _,enemy in pairs(enemies) do
			if objc.is_ground_unit(enemy) and enemy ~= targetEntity then
				local thisDistance = objc.quick_unit_distance_square(enemy, targetEntity)
				if thisDistance < closestDistance then
					closestEntity = enemy
					closestDistance = thisDistance
				end
			end
		end

		if closestEntity ~= targetEntity then
			objc.set_local_value(sourceEntity, INIT_TARGET, closestEntity)
		else
			objc.kill_entity(sourceEntity)
		end
	else
		objc.kill_entity(sourceEntity)
	end
end



