#define SUPPORT_GLSL300
#include "prefix.sh"
#include "hdr_utility.sh"
#include "texture_utility.sh"

precision highp float;

uniform sampler2D s_cc3Texture2Ds[6];

VARYING_IN vec2 v_texCoord[1];

uniform vec4 Exposure_bloomDecay;
uniform vec3 Exposure_whitePoint;
uniform vec3 Exposure_sunLightColor;
uniform float Exposure_sunGlowIntensity;
uniform float Exposure_bloomIntensity;
uniform int Exposure_sunGlowEnabled;
uniform int Exposure_bloomEnabled;
uniform int Exposure_lightShaftEnabled;
uniform int Exposure_processInHDR;

void main()
{
    vec4 inColor = texture2D(s_cc3Texture2Ds[0], v_texCoord[0]);
    vec3 hdrColor = inColor.rgb;
    
    // apply bloom
    if (Exposure_bloomEnabled == 1) {
        hdrColor += texture2D(s_cc3Texture2Ds[2], v_texCoord[0]).rgb * Exposure_bloomDecay.x * Exposure_bloomIntensity;
        hdrColor += texture2D(s_cc3Texture2Ds[3], v_texCoord[0]).rgb * Exposure_bloomDecay.y * Exposure_bloomIntensity;
        hdrColor += texture2D(s_cc3Texture2Ds[4], v_texCoord[0]).rgb * Exposure_bloomDecay.z * Exposure_bloomIntensity;
        hdrColor += texture2D(s_cc3Texture2Ds[5], v_texCoord[0]).rgb * Exposure_bloomDecay.w * Exposure_bloomIntensity;
    }
    
    // apply sun glow (prototype)
    if (Exposure_sunGlowEnabled == 1) {
        // fake visibility occlusion of sun with bloom mask, 2nd in bloom chain looks best
        vec3 bloomMask = texture2D(s_cc3Texture2Ds[3], v_texCoord[0]).rgb * Exposure_bloomDecay.y;

        // tentatively store sun glow strength in alpha channel of scene color by mesh shader
        float sunGlowMask = inColor.a;

        hdrColor += sunGlowMask * luminance(bloomMask) * Exposure_sunLightColor * Exposure_sunGlowIntensity;
    }
    
    // apply volumetric
    if (Exposure_lightShaftEnabled == 1) {
        hdrColor += texture2D(s_cc3Texture2Ds[1], v_texCoord[0]).rgb;
    }
    
    vec3 outColor;
    if (Exposure_processInHDR == 1) {
        // tone mapping
        outColor = Uncharted2Tonemap(hdrColor);
        
        // fake over exposure
        outColor = mix(hdrColor, outColor, 0.97);
        
        // white balance
        outColor = outColor / Uncharted2Tonemap(Exposure_whitePoint);
        
        // gamma correction
        outColor = LinearToSRGB(outColor);
    } else {
        outColor = hdrColor;
    }

    gl_FragColor.rgb = clamp(outColor, 0.0, 1.0);
    gl_FragColor.a = 1.0;
}
