local HAS_INIT = 'hasInit'
local HAS_BOSS_INIT = 'bossInit'
local DESTROY_BARRICADE_INIT = 'destroyBarricadeInit'

local UNIT_LUA = 'KyrieBossBattle.lua'

local UNIT_HAS_SPAWNED = 'unitHasSpawned'
local IDLE_TIMER_KEY = 'idleTimerKey'

local CHECK_TICK = 3
local CHECK_KEY = 'checkKey'
local HAS_BEEN_SCALED = 'hasBeenScaledKey'
local SCALE_LEVEL = 'scaleLevelKey'

local BOSS_BASE_IDENTIFIER = 'W_KyrieBoss'
local BOSS_HEALTH = 10000
local BOSS_DAMAGE = 110

local CURRENT_BOSS_STATE = 'bossStateKey'
local BOSS_STATE_IDLE = 0
local BOSS_STATE_ACTIVE = 1

local WAVE_CONTROLLER_ENTITY = 'waveControllerEntity'
local WAVE_DELAY_AFTER_RETREAT = 7

local LIFE_INDICATOR_ID = 'W_KyrieLifeIndicator'
local LIFE_INDICATORS = 3

KyrieBossBattle_tickTrigger = function(entity)
    local HasInit = objc.get_local_value(entity, HAS_INIT)
    if HasInit == 0 then
        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end

        if is2vE == 1 then
            --0.5 crystal/s in 2vE
            local incomeToSet = 1000
            objc.set_income_generation_frequency_in_milliseconds(1, incomeToSet)
            objc.set_income_generation_frequency_in_milliseconds(3, incomeToSet)
            objc.add_crystal(1, 3)
            objc.add_crystal(3, 3)
        else
            local incomeToSet = 700
            objc.set_income_generation_frequency_in_milliseconds(1, incomeToSet)
            objc.add_crystal(1, 3)
        end

        objc.set_local_value(entity, HAS_INIT, 1)

        local bossUnit = objc.spawn_at_point(BOSS_BASE_IDENTIFIER, 1, 2, 6250, 0, 0, 0)
        objc.idle_entity(bossUnit[1], 4500)
        objc.set_local_value(bossUnit[1], WAVE_CONTROLLER_ENTITY, entity)

        local base = objc.get_base_entity(2)
        objc.add_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, UNIT_LUA, 'base_invulnerable')
        objc.set_targetable(base, false)
    end

    local destroyBarricadeInit = objc.get_local_value(entity, DESTROY_BARRICADE_INIT)
    if destroyBarricadeInit == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.is_barricade(unit) then
                local bossBase = objc.get_base_entity(2)
                objc.kill_entity(unit)
                objc.set_local_value(entity, DESTROY_BARRICADE_INIT, 1)
            end
        end
    elseif destroyBarricadeInit == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'B_Barricade_Ship' then
                objc.set_entity_visibility(unit, false)
                objc.sacrifice_entity(unit)
                objc.set_local_value(entity, DESTROY_BARRICADE_INIT, 2)
            end
        end
    end

    local checkTimer = objc.tick_timer(entity, CHECK_KEY, CHECK_TICK)
    if checkTimer == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_local_value(entity, HAS_BOSS_INIT) == 0 then
                if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == BOSS_BASE_IDENTIFIER then
                    objc.set_entity_scale_percentage(unit, 125)
                    objc.set_local_value(unit, CURRENT_BOSS_STATE, BOSS_STATE_ACTIVE)
                    objc.set_local_value(entity, HAS_BOSS_INIT, 1)
                end
            end
        end
    end
end

KyrieBossBattle_BossWave_1 = function(entity)
    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_BossWave_1')
end

KyrieBossBattle_BossWave_2 = function (entity)
    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_BossWave_2')
end

KyrieBossBattle_BossWave_3 = function (entity)
    objc.spawn_at_point('W_Eagle', 1, 2, 4400, 0, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_BossWave_3')
end

KyrieBossBattle_BossWave_4 = function (entity)
    objc.spawn_at_point('W_StunTank', 1, 2, 4400, 1200, -1, 0)
    objc.spawn_at_point('W_StunTank', 1, 2, 4400, -1200, -1, 0)

    objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_BossWave_4')
end

KyrieBossBattle_Wave_1 = function(entity)
    local waveDelay = objc.tick_timer(entity, 'waveDelayTimer', WAVE_DELAY_AFTER_RETREAT)
    if waveDelay == 0 then
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 1000, -1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 1000, 1500, -1, 0)

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_Wave_1')
    end
end

KyrieBossBattle_Wave_2 = function (entity)
    local waveDelay = objc.tick_timer(entity, 'waveDelayTimer', WAVE_DELAY_AFTER_RETREAT)
    if waveDelay == 0 then
        --11 crystal
        objc.spawn_at_point('W_StunTank', 1, 2, 3750, 1000, -1, 0)
        objc.spawn_at_point('W_StunTank', 1, 2, 3750, -1000, -1, 0)
        objc.spawn_at_point('W_Spearbot', 1, 2, 2250, 0, -1, 0)
        objc.spawn_at_point('W_Spearbot', 1, 2, 2500, 250, -1, 0)
        objc.spawn_at_point('W_Spearbot', 1, 2, 2500, -250, -1, 0)

        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, -1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, 1500, -1, 0)

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_Wave_2')
    end
end

KyrieBossBattle_Wave_3 = function (entity)
    local waveDelay = objc.tick_timer(entity, 'waveDelayTimer', WAVE_DELAY_AFTER_RETREAT)
    if waveDelay == 0 then
        --11 crystal
        objc.spawn_at_point('W_Swordsman', 1, 2, 2750, 0, -1, 0)
        objc.spawn_at_point('W_ScoutFlier', 1, 2, 3500, 0, -1, 0)

        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, -1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, 1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, 0, -1, 0)

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_Wave_3')
    end
end

KyrieBossBattle_Wave_4 = function (entity)
    local waveDelay = objc.tick_timer(entity, 'waveDelayTimer', WAVE_DELAY_AFTER_RETREAT)
    if waveDelay == 0 then
        --13 crystal
        objc.spawn_at_point('W_Fighter', 1, 2, 3500, 250, -1, 0)
        objc.spawn_at_point('W_ShieldedUnit', 1, 2, 3000, 0, -1, 0)

        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, -1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, 1500, -1, 0)
        objc.spawn_at_point('W_KyrieCultist', 1, 2, 3000, 0, -1, 0)

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'KyrieBossBattle_Wave_4')
    end
end

base_invulnerable = function(entity)
    return 0
end