#define SUPPORT_GLSL300
#include "prefix.sh"
#include "texture_utility.sh"

// Based on work of Evan Wallace (https://github.com/evanw/webgl-filter/) and 
// Colin Barré-Brisebois (https://colinbarrebrisebois.com/2017/04/18/hexagonal-bokeh-blur-revisited/)

#if defined(BLUR_BYPASS)
    uniform sampler2D s_cc3Texture2Ds[1];
#elif BLUR_PASS == 0
    uniform sampler2D s_cc3Texture2Ds[1];
#elif BLUR_PASS == 1
    uniform sampler2D s_cc3Texture2Ds[2];
#elif BLUR_PASS == 2
    uniform sampler2D s_cc3Texture2Ds[3];
#endif

uniform int LensBlur_samples;
uniform float LensBlur_radius;
uniform vec2 LensBlur_directionVertical;
uniform vec2 LensBlur_directionDiagonal;
uniform vec2 LensBlur_directionReflection;

VARYING_IN vec2 v_texCoord[1];

float random(vec2 scale, float seed)
{
    return fract(sin(dot(gl_FragCoord.xy + seed, scale)) * 43758.5453 + seed);
}

float random(vec3 scale, float seed)
{
    return fract(sin(dot(gl_FragCoord.xyz + seed, scale)) * 43758.5453 + seed);
}

vec3 directionalBlur(sampler2D tex, vec2 delta)
{
    float offset = random(delta, 0.0);
    vec3 color = vec3(0.0);
    float total = 0.0;
    
    for (float t = 0.0; t <= float(LensBlur_samples); t++) {
        float percent = (t + offset) / float(LensBlur_samples);
        color += sampleLevelZero(tex, v_texCoord[0] + delta * percent * LensBlur_radius).rgb;
        total += 1.0;        
    }
    
    return color / total;
}

void main()
{
    #if defined(BLUR_BYPASS)
        gl_FragColor.rgb = sampleLevelZero(s_cc3Texture2Ds[0], v_texCoord[0]).rgb;
    #elif BLUR_PASS == 0
        gl_FragColor.rgb = directionalBlur(s_cc3Texture2Ds[0], LensBlur_directionVertical);
    #elif BLUR_PASS == 1
        gl_FragColor.rgb = (directionalBlur(s_cc3Texture2Ds[0], LensBlur_directionDiagonal) + 
                            sampleLevelZero(s_cc3Texture2Ds[1], v_texCoord[0]).rgb) * 0.5;
    #elif BLUR_PASS == 2
        gl_FragColor.rgb = (directionalBlur(s_cc3Texture2Ds[1], LensBlur_directionDiagonal) + 
                            directionalBlur(s_cc3Texture2Ds[2], LensBlur_directionReflection) * 2.0) / 3.0; 
    #endif
    
    gl_FragColor.a = 1.0;
}
