#define SUPPORT_GLSL300
#include "prefix.sh"

precision highp float;

uniform sampler2D s_cc3Texture2Ds[1];

VARYING_IN vec2 v_texCoord[1];

uniform vec2 Exposure_screenLightPosition;
uniform float Exposure_lightAlignment;
uniform float Exposure_lightShaftIntensity;
uniform float Exposure_lightShaftDistance;

#define NUM_SAMPLES 10.0

void main()
{
    vec3 color = texture2D(s_cc3Texture2Ds[0], v_texCoord[0]).rgb;

    const float maxDistance = 2.0;
    vec2 delta = clamp(v_texCoord[0] - Exposure_screenLightPosition, vec2(-maxDistance), vec2(maxDistance));
    delta *= 1.0 / NUM_SAMPLES * Exposure_lightShaftDistance;

    float decay = Exposure_lightShaftIntensity * Exposure_lightAlignment;
    float weight = 1.0 / NUM_SAMPLES;
    vec2 texCoord = v_texCoord[0];

    for (int i = 0; i < int(NUM_SAMPLES); i++) {
        texCoord -= delta;
        color += texture2D(s_cc3Texture2Ds[0], texCoord).rgb * decay * weight;
        decay *= exp((float(-i) + 2.0) / 10.0);
    }

    gl_FragColor.rgb = color;
    gl_FragColor.a = 1.0;
}
