//
//  MetalFragmentFramework.metal
//  TestBedOmegaOSX
//
//  Created by Ken Domke on 3/13/18.
//

#if !defined(FRAMEWORK_METAL_FRAG)
#define FRAMEWORK_METAL_FRAG


#define FRAGMENT_FUNCTION_BEGIN float4 frag_out = float4(1.0);
#define FRAGMENT_FUNCTION_END return frag_out;

////////////////////////////////////////////////////////////////////////////////////////////////////

#define gl_FragColor                frag_out
#define discard                     discard_fragment()

////////////////////////////////////////////////////////////////////////////////////////////////////

float4 texture2D(texture2d<float> input_texture, float2 texcoord)
{
    /* is_null_texture is only available in Metal shading language 2.0 */
    #if 0
    /* KenD -- Sample if texture is non-null, but do not sample if texture is null!
     * Sampling a null texture can result in GPU hangs.
     */
    {
        if (is_null_texture(input_texture)) {
            return float4(0.0, 0.0, 1.0, 1.0);
        }
    }
    #endif

    constexpr sampler textureSampler(mip_filter::linear,
                                     mag_filter::linear,
                                     min_filter::linear,
                                     address::repeat);
    return input_texture.sample(textureSampler, texcoord);
}


////////////////////////////////////////////////////////////////////////////////////////////////////
//      I N P U T   A T T R I B U T E S   (   V A R Y I N G S   )

/* See general.frag */

////////////////////////////////////////////////////////////////////////////////////////////////////

/* KenD -- Metal TODO -- region and vertex shader should use v_texCoord0 and v_color0; so should omegaData (but omegaData hasn't been refactored yet to remove array inputs)
 */

#define v_texCoord0             omegaData_inout.v_texCoord
#define v_color                 omegaData_inout.v_color
#define v_position_world        omegaData_inout.v_position_world
#define v_N                     omegaData_inout.v_N
#define v_T                     omegaData_inout.v_T
#define v_B                     omegaData_inout.v_B
/* KenD -- Metal hack -- disable shadow */
//#define v_shadow                omegaData_inout.v_shadow

////////////////////////////////////////////////////////////////////////////////////////////////////

#define gl_FragCoord            omegaData_inout.clipSpacePosition /* [[position]] attribute */

#endif // FRAMEWORK_METAL_FRAG
