//
//  MetalPrefix.metal
//  TestBedOmega
//
//  Created by Ken Domke on 3/12/18.
//

using namespace metal;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Standard type changes from GLSL to Metal Shading Language
////////////////////////////////////////////////////////////////////////////////////////////////////

#define mat2 float2x2
#define mat4 float4x4
#define mat3 float3x3
#define vec2 float2
#define vec3 float3
#define vec4 float4
#define ivec2 int2

////////////////////////////////////////////////////////////////////////////////////////////////////

#define METAL_INSTEAD_OF_OPENGL

#define inout_VSOUT     thread VSOUT&

#define out_float       thread float&
#define out_float2      thread float2&
#define out_float3      thread float3&
#define out_float4      thread float4&
#define inout_float     thread float&
#define inout_float2    thread float2&
#define inout_float3    thread float3&
#define inout_float4    thread float4&

#define sampler2D       texture2d<float>
#define samplerCube     texturecube<float>

#define dFdx            dfdx
#define dFdy            dfdy

float mod(int a, int b)
{
    return a % b;
}

////////////////////////////////////////////////////////////////////////////////////////////////////
// Similar to uniform structs
////////////////////////////////////////////////////////////////////////////////////////////////////

#define uniform typedef

////////////////////////////////////////////////////////////////////////////////////////////////////

/*

 generic vertex function uses ATTRIBUTE and VARYING_OUT.
 generic fragment function would use VARYING_IN.

 specific vertex function, vsMain, uses:
 VSIN as input structure
 VSOUT as output structure

 The generic vertex function populates VSIN from ATTRIBUTES and uses VSOUT to populate VARYING_OUT elements

 In Metal, the input to the generic vertex function will be OmegaVertex, which has all the ATTRIBUTES.
 The VARYING_OUT elements will go into an OmegaData structure.

 In Metal, there is a structure that goes between vertex and fragment shader.
 This structure will have to contain the VARYING_OUT and VARYING_IN elements.

 */

////////////////////////////////////////////////////////////////////////////////////////////////////

#define ATTRIBUTE
#define VARYING_OUT
#define VARYING_IN

////////////////////////////////////////////////////////////////////////////////////////////////////

/* Consider bringing program scope variables into constant address space */

////////////////////////////////////////////////////////////////////////////////////////////////////
