//
//  MetalVertexFramework.metal
//  TestBedOmega
//
//  Created by Ken Domke on 3/12/18.
//

#if !defined(FRAMEWORK_METAL_VERT)
#define FRAMEWORK_METAL_VERT

#include "../../framework/framework.sh"
#include "../../framework/skinning.sh"

////////////////////////////////////////////////////////////////////////////////////////////////////
    //      C O N F I G U R A T I O N

/* KenD -- Metal TODO -- disable SUPPORT_ARRAY_INPUTS
 * (see later note - arrays are not supported in stage_in structs)
 */

////////////////////////////////////////////////////////////////////////////////////////////////////
    //      I N P U T   A T T R I B U T E S

typedef struct
{
#if VS_IN_S_POSITION
    ATTRIBUTE vec4 a_cc3Position [[attribute(0)]];
#endif

#if VS_IN_S_TEXCOORD > 0
    #if defined(SUPPORT_ARRAY_INPUTS)
        ATTRIBUTE vec2 a_cc3TexCoord[VS_IN_S_TEXCOORD] [[attribute(1)]];;
    #else
        ATTRIBUTE vec2 a_cc3TexCoord [[attribute(1)]];;
    #endif
#endif

#if VS_IN_S_COLOR > 0
    #if defined(SUPPORT_ARRAY_INPUTS)
        ATTRIBUTE vec4 a_cc3Color[VS_IN_S_COLOR] [[attribute(2)]];;
    #else
        ATTRIBUTE vec4 a_cc3Color [[attribute(2)]];;
    #endif
#endif

#if VS_IN_S_BONE_WEIGHTS
    ATTRIBUTE vec4 a_cc3BoneWeights [[attribute(3)]];;
    ATTRIBUTE vec4 a_cc3BoneIndices [[attribute(4)]];;
#endif

#if VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
    ATTRIBUTE vec3 a_cc3Normal [[attribute(5)]];;
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_FULL
    ATTRIBUTE vec3 a_cc3Normal [[attribute(5)]];
    ATTRIBUTE vec3 a_cc3Tangent [[attribute(6)]];
    ATTRIBUTE vec3 a_cc3Bitangent [[attribute(7)]];
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
    ATTRIBUTE vec3 a_cc3Normal [[attribute(5)]];
    ATTRIBUTE vec4 a_cc3Tangent [[attribute(6)]];
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_TB
    ATTRIBUTE vec4 a_cc3Tangent [[attribute(5)]];
    ATTRIBUTE vec3 a_cc3Bitangent [[attribute(6)]];
#endif
} OmegaVertex;

/* KenD -- arrays of scalars are not supported as members of the struct declared with the stage_in attribute.
 * (From Metal Shading Language Specification, there's a VertexOutput structure with texcoord[4],
 * but it isn't used with a fragment function.  In the fragment function example, that texcoord member is not an array.
 * this is a bit of a limitation.)
 */


////////////////////////////////////////////////////////////////////////////////////////////////////
//      O U T P U T   A T T R I B U T E S   (   V A R Y I N G S   )

typedef struct
{
    float4 clipSpacePosition [[position]];

#if VS_OUT_S_TEXCOORD
#if defined(SUPPORT_ARRAY_INPUTS)
    VARYING_OUT vec2 v_texCoord[VS_OUT_S_TEXCOORD];
#else
    VARYING_OUT vec2 v_texCoord;
#endif
#endif

#if VS_OUT_S_COLOR
#if defined(SUPPORT_ARRAY_INPUTS)
    VARYING_OUT vec4 v_color[VS_OUT_S_COLOR];
#else
    VARYING_OUT vec4 v_color;
#endif
#endif

#if VS_OUT_S_WORLD_SPACE_POSITION
    VARYING_OUT vec3 v_position_world;
#endif

#if VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
    VARYING_OUT vec3 v_N;
#elif VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL
    VARYING_OUT vec3 v_N;
    VARYING_OUT vec3 v_T;
    VARYING_OUT vec3 v_B;
#endif

    /* KenD -- Metal hack -- disable shadow */
#if VS_OUT_S_SHADOW_SAMPLE && 0
    VARYING_OUT vec4 v_shadow;
#endif
} OmegaData;


#if defined(TC_TRANSFORM)
    uniform vec4 u_cc3TC0Transform;
#endif


////////////////////////////////////////////////////////////////////////////////////////////////////


#define VERTEX_FUNCTION_BEGIN   OmegaData omegaData_inout;
#define VERTEX_FUNCTION_END     return omegaData_inout;

#define gl_Position             omegaData_inout.clipSpacePosition

/* Convert from GLSL vertex attributes to accessors of the OmegaVertex structure in Metal */
#define a_cc3Position           v_in.a_cc3Position
#define a_cc3TexCoord           v_in.a_cc3TexCoord
#define a_cc3Color              v_in.a_cc3Color
#define a_cc3BoneWeights        v_in.a_cc3BoneWeights
#define a_cc3BoneIndices        v_in.a_cc3BoneIndices
#define a_cc3Normal             v_in.a_cc3Normal
#define a_cc3Tangent            v_in.a_cc3Tangent
#define a_cc3Bitangent          v_in.a_cc3Bitangent

/* Convert from GLSL varyings to accessors of the OmegaData structure in Metal */
#define v_texCoord              omegaData_inout.v_texCoord
#define v_color                 omegaData_inout.v_color
#define v_position_world        omegaData_inout.v_position_world
#define v_N                     omegaData_inout.v_N
#define v_T                     omegaData_inout.v_T
#define v_B                     omegaData_inout.v_B
/* KenD -- Metal hack -- disable shadow */
//#define v_shadow                omegaData_inout.v_shadow


/* KenD -- because Metal can't support arrays as members of structures used with the [[stage_in]] attribute
 * (such structures include OmegaVertex, containing vertex attributes,
 * and OmegaData, containing information passed from vertex function to fragment function [similar to GLSL varyings]),
 * we have to expand these arrays to a series of individual elements.
 *
 * This should be done for a_cc3Color, a_cc3TexCoord, v_color, and v_texCoord,
 * changing them to a_cc3Color0 and v_texCoord0 and so on.
 * That format would match `BuildSemanticHash` that we use in Magnesium and XLE.
 *
 * -- Metal TODO --
 * This refactor would involve SUPPORT_ARRAY_INPUTS, SIMPLE_VERTEX_COLOR, and others.
 */


////////////////////////////////////////////////////////////////////////////////////////////////////


void
nonSpecificVertexShader(thread OmegaData& omegaData_inout,
                        thread const OmegaVertex& v_in,
                        NON_SPECIFIC_FUNCTION_PARAMETERS)
{
    VSIN vsin;
    #if VS_IN_S_POSITION
        vsin._position = a_cc3Position;
    #endif

    #if VS_IN_S_TEXCOORD > 0
        #if defined(SUPPORT_ARRAY_INPUTS)
            for (int c=0; c<VS_OUT_S_TEXCOORD; ++c) {
                vsin._texCoord[c] = a_cc3TexCoord[c];
            }
        #else
            vsin._texCoord[0] = a_cc3TexCoord;
        #endif
        #if defined(TC_TRANSFORM)
            vsin._texCoord[0] = vsin._texCoord[0] * u_cc3TC0Transform.xy + u_cc3TC0Transform.zw;
        #else
            vsin._texCoord[0] = vsin._texCoord[0];
        #endif
    #endif

    #if VS_IN_S_COLOR > 0
        #if defined(SUPPORT_ARRAY_INPUTS)
            for (int c=0; c<VS_OUT_S_COLOR; ++c) {
#if SIMPLE_VERTEX_COLOR > 0
                /* KenD -- Metal TODO -- remove this when removing array inputs for Metal.
                 * Note, this is setting the output color early. */
                v_color = a_cc3Color[c];
#else
                vsin._color[c] = a_cc3Color[c];
#endif
            }
        #else
#if SIMPLE_VERTEX_COLOR > 0
    /* KenD -- this is setting the output color early.
     * Hack is to have one color varying for Metal.  todo: add v_color0, v_color1, v_color2. */
                //v_color[c] = a_cc3Color[c];
                v_color = a_cc3Color;
#else
                vsin._color[0] = a_cc3Color;
#endif
        #endif
    #endif

    #if VS_IN_S_BONE_WEIGHTS
        vsin._boneWeights = a_cc3BoneWeights;
        vsin._boneIndices = a_cc3BoneIndices;
    #endif

    #if VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = vec3(0,0,0);
        vsin._tangentFrame._handiness = 0.0;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_FULL
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = a_cc3Tangent;
        vsin._tangentFrame._handiness = dot(cross(a_cc3Normal, a_cc3Tangent), a_cc3Bitangent) >= 0.0 ? 1.0 : -1.0;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = a_cc3Tangent.xyz;
        vsin._tangentFrame._handiness = a_cc3Tangent.w;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
        vsin._tangentFrame._axesA = a_cc3Tangent.xyz;
        vsin._tangentFrame._axesB = a_cc3Bitangent;
        vsin._tangentFrame._handiness = a_cc3Tangent.w;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #endif

    VSOUT vsout;

    SPECIFIC_MAIN_FUNCTION_CALL

    gl_Position = vsout._position;

    /* KenD -- Metal TODO -- remove SUPPORT_ARRAY_INPUTS; for now, just having one texCoord and one color varying for Metal. */
    #if VS_OUT_S_TEXCOORD
        v_texCoord = vsout._texCoord[0];
    #endif

    #if VS_OUT_S_COLOR && SIMPLE_VERTEX_COLOR == 0
        v_color = vsout._color[0];
    #endif

    #if VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
        v_N = vsout._tangentFrame._normal;
    #elif VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL
        v_N = vsout._tangentFrame._normal;
        v_T = vsout._tangentFrame._tangent;
        v_B = vsout._tangentFrame._bitangent;
    #endif

    #if VS_OUT_S_WORLD_SPACE_POSITION
        v_position_world = vsout._worldSpacePosition;
    #endif

    #if VS_OUT_S_SHADOW_SAMPLE
    /* KenD -- Metal TODO -- shadowing
        v_shadow = vec4(0,0,0,0);
     */
        v_shadow = vsout._shadow;
    #endif
}

#endif // FRAMEWORK_METAL_VERT
