local CHECK_RADIUS = 8000
local IS_PRE_PLACED = 'isPrePlacedKey'

local TARGET_MINE = 'targetMineKey'
local MINER_LIST = 'minerListKey'

local MINE_MAX_MINERS_KEY = 'maxMinersKey'
local NUM_MINERS_KEY = 'numMinersKey'
local MINE_INDEX_KEY = 'mineIndexKey'

local HARVEST_ANIMATION_INTERVAL = 40
local HARVEST_ANIMATION_TIMER = 'harvestAnimationTimer'

local NUM_FRAMES_IN_SPAWN_ANIM = 70

local HAS_SPAWNED_CABLE = 'hasSpawnedCable'
local SPAWN_CABLE_KEY = 'cableSpawnID'
local SPAWNED_CABLE = 'spawnedCableKey'
local SHOULD_START_PULSE_COUNTDOWN = 'shouldStartPulseCountdown'
local SHOULD_KILL_CABLE = 'shouldKillCableKey'

PowerPlant_spawnTrigger = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local objectives = objc.list_entities(teamNumber)
    local closestDistance = 99999999
    local closestMine

    local playerNumbers = objc.get_player_numbers_for_team_number(teamNumber) -- if this is a 2v2 or a 2ve, set income on mine to half
    local count = 0
    for _ in pairs(playerNumbers) do
        count = count + 1
    end
    if count > 1 then
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_INCOME_TIER, 3)
    end

    for _, objective in pairs(objectives) do
        if objc.is_crystal_mine(objective) ~= 0 then
            local numMiners = objc.get_local_value(objective, NUM_MINERS_KEY)
            if numMiners < objc.get_local_value(objective, MINE_MAX_MINERS_KEY) then
                local thisDistance = math.abs(objc.unit_distance_with_offset(entity, 0, 0, objective, 0, 0))
                --Check if we're a pre-placed miner
                if minePointX ~= 0 or minePointY ~= 0 then
                    local thisPosition = objc.get_entity_position(objective)
                    if minePointX == thisPosition[1] and minePointY == thisPosition[2] then
                        thisDistance = 0
                        closestMine = objective
                        break
                    end
                end

                if thisDistance < closestDistance and thisDistance < CHECK_RADIUS then
                    closestMine = objective
                    closestDistance = thisDistance
                end
            end
        end
    end

    if not closestMine then
        -- we couldn't find a closest mine, so we destroy this and refund the player
        objc.sacrifice_entity(entity)
        local playerNumber = objc.get_player_number(entity)
        local refundAmount = objc.get_previous_powerplant_tier_cost(playerNumber)
        objc.add_crystal(playerNumber, refundAmount)
    elseif closestMine ~= entity then
        objc.set_local_value(entity, TARGET_MINE, closestMine)
        local targetMineList = objc.get_local_value(closestMine, MINER_LIST)
        if targetMineList == 0 then
            targetMineList = {}
        end
        table.insert(targetMineList, entity)
        objc.set_local_value(closestMine, MINER_LIST, targetMineList)

        local numMiners = objc.get_local_value(closestMine, NUM_MINERS_KEY)
        objc.set_local_value(closestMine, NUM_MINERS_KEY, numMiners + 1)

        -- in case someone gets the crystal before you but you still dragged it there,
        -- we teleport the powerplant to the crystal
        local targetPosition = objc.get_entity_position(closestMine)
        objc.teleport_entity(entity, targetPosition[1], targetPosition[2])
    else
        objc.sacrifice_entity(entity)
    end


    for _, objective in pairs(objectives) do
        if objc.is_barricade(objective) then
            local barricadePos = objc.get_entity_position(objective)
            local thisPosition = objc.get_entity_position(entity)
            local sameSign = false
            if thisPosition[2] > 0 and barricadePos[2] > 0 then
                sameSign = true
            elseif thisPosition[2] < 0 and barricadePos[2] < 0 then
                sameSign = true
            end
            if barricadePos[2] == 0 or sameSign == true then
                objc.set_targetable(entity, false)
            end
            break
        end
    end
end

PowerPlant_tickTrigger = function(entity)
    local hasSpawnedCable = objc.get_local_value(entity, HAS_SPAWNED_CABLE)
    if hasSpawnedCable == 0 then
        local playerNumber = objc.get_player_number(entity)
        local teamNumber = objc.get_team_number(entity)

        local baseEntity = objc.get_base_entity(teamNumber)
        local basePlayerNumber = objc.get_player_number(baseEntity)
        local baseFaction = objc.get_commander_factions(basePlayerNumber)[1]
        local entityFaction = objc.get_commander_factions(playerNumber)[1]

        if baseFaction == entityFaction then
            local cableID = objc.get_local_value(entity, SPAWN_CABLE_KEY)
            if cableID ~= 0 then
                local targetMine = objc.get_local_value(entity, TARGET_MINE)
                if targetMine ~= 0 then
                    local targetIndex = objc.get_local_value(targetMine, MINE_INDEX_KEY)
                    local xDir = 1
                    if objc.get_global_value(IS_TWO_LANE_MAP_GLOBAL_KEY) == 1 then
                        cableID = cableID .. '_TL'
                    else
                        cableID = cableID .. '_SL'
                    end
                    local toSpawnId = cableID .. '_0' .. targetIndex
                    if teamNumber == 2 then
                        xDir = -1
                    end

                    local spawnedCable = objc.spawn_at_point('PowerPlantCable', 1, playerNumber, 0, 0, 0, xDir)
                    objc.update_asset_name(spawnedCable[1], toSpawnId .. '.lvl')
                    objc.set_entity_visibility(spawnedCable[1], false)
                    objc.set_targetable(spawnedCable[1], false)
                    objc.set_solid(spawnedCable[1], false)
                    objc.set_local_value(entity, SPAWNED_CABLE, spawnedCable[1])
                end
            end
        end
        objc.set_local_value(entity, HAS_SPAWNED_CABLE, 1)
    end

    local harvestTimer = objc.tick_timer(entity, HARVEST_ANIMATION_TIMER, HARVEST_ANIMATION_INTERVAL)
    if harvestTimer == 0 then
        objc.play_override_animation_key(entity, 'harvestAnimationName', objc.convert_frames_to_subrounds(30))
        local spawnedCable = objc.get_local_value(entity, SPAWNED_CABLE)
        if spawnedCable ~= 0 then
            objc.set_local_value(spawnedCable, SHOULD_START_PULSE_COUNTDOWN, 1)
        end
    end
end

PowerPlant_deathTrigger = function(entity)
    local targetMine = objc.get_local_value(entity, TARGET_MINE)
    if targetMine ~= 0 then
        local numMiners = objc.get_local_value(targetMine, NUM_MINERS_KEY)
        local minerList = objc.get_local_value(targetMine, MINER_LIST)
        if minerList ~= 0 then
            local thisIndex = -1
            for i = 1, listCount(minerList) do
                if minerList[i] == entity then
                    thisIndex = i
                end
            end

            if thisIndex ~= -1 then
                table.remove(minerList, thisIndex)
            end
            objc.set_local_value(targetMine, MINER_LIST, minerList)
        end
        objc.set_local_value(targetMine, NUM_MINERS_KEY, numMiners - 1)
    end

    local spawnedCable = objc.get_local_value(entity, SPAWNED_CABLE)
    if spawnedCable ~= 0 then
        objc.set_local_value(spawnedCable, SHOULD_KILL_CABLE, 1)
    end
end

PowerPlant_reachedOvertimeTrigger = function(entity)
    objc.sacrifice_entity(entity)
end
