local HAS_SWITCHED_ANIM = 'hasSwitchedAnimKey'
local SPAWN_ANIM_TIME = 27
local IS_PRE_PLACED = 'isPrePlacedKey'

local HARVEST_ANIMATION_INTERVAL = 40
local HARVEST_ANIMATION_TIMER = 'harvestAnimationTimer'

local HAS_SPAWNED_CABLE = 'hasSpawnedCable'
local SPAWN_CABLE_KEY = 'cableSpawnID'
local SPAWNED_CABLE = 'spawnedCableKey'
local SHOULD_START_PULSE_COUNTDOWN = 'shouldStartPulseCountdown'
local SHOULD_KILL_CABLE = 'shouldKillCableKey'

-- is 0 if on single lane map, 1 if on two lane map and is on the top lane, and 2 if on two lane map and is on the bottom lane
local LANE_INDEX_KEY = "laneIndexKey"
local POWERPLANT_OBJECTIVE_INDEX_KEY = 'powerplantObjectiveIndex'

local IS_TWO_LANE_MAP_GLOBAL_KEY = "isTwoLaneMapGlobalKey"

PowerPlant_Objective_spawnTrigger = function(entity)
    objc.set_entity_visibility(entity, false)
    objc.set_targetable(entity, false)
end

PowerPlant_Objective_tickTrigger = function(entity)
    local hasSpawnedCable = objc.get_local_value(entity, HAS_SPAWNED_CABLE)
    if hasSpawnedCable == 0 then
        local cableID = objc.get_local_value(entity, SPAWN_CABLE_KEY)
        if cableID ~= 0 then
            -- laneIndex is 1 for the top lane, and 2 for the bottom lane of two-lane maps
            local laneIndex = objc.get_local_value(entity, LANE_INDEX_KEY)
            -- If on single lane map, ignore the lane index
            if objc.get_global_value(IS_TWO_LANE_MAP_GLOBAL_KEY) == 0 then
                cableID = cableID .. '_SL'
                local powerplantIndex = objc.get_local_value(entity, POWERPLANT_OBJECTIVE_INDEX_KEY)
                if powerplantIndex > 1 then
                    cableID = cableID .. '_0' .. powerplantIndex
                end
            elseif laneIndex == 1 then
                cableID = cableID .. '_TL'
            else
                cableID = cableID .. '_TL_0' .. laneIndex
            end

            local teamNumber = objc.get_team_number(entity)
            if teamNumber == 2 then
                cableID = cableID .. '_P2'
            end

            local playerNumber = objc.get_player_number(entity)
            local spawnedCable = objc.spawn_at_point('PowerPlantCable', 1, playerNumber, 0, 0, 0, 1)
            objc.update_asset_name(spawnedCable[1], cableID .. '.lvl')
            objc.set_entity_visibility(spawnedCable[1], false)
            objc.set_targetable(spawnedCable[1], false)
            objc.set_solid(spawnedCable[1], false)
            objc.set_local_value(entity, SPAWNED_CABLE, spawnedCable[1])
        end
        objc.set_local_value(entity, HAS_SPAWNED_CABLE, 1)
    end

    local isPrePlaced = objc.get_local_value(entity, IS_PRE_PLACED)
    if isPrePlaced == 0 then
        objc.set_entity_visibility(entity, true)
    else
        objc.set_entity_visibility(entity, false)
    end

    local harvestTimer = objc.tick_timer(entity, HARVEST_ANIMATION_TIMER, HARVEST_ANIMATION_INTERVAL)
    if harvestTimer == 0 then
        local faction = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_FACTION)
        local factionPrefix = ''
        if faction == 'red' then
            factionPrefix = 'R_'
        elseif faction == 'blue' then
            factionPrefix = 'W_'
        elseif faction == 'green' then
            factionPrefix = 'B_'
        end
        if factionPrefix ~= '' then
            objc.play_override_animation(entity, factionPrefix .. 'PowerPlant_Objective_Attack_01', objc.convert_frames_to_subrounds(30))
            local spawnedCable = objc.get_local_value(entity, SPAWNED_CABLE)
            if spawnedCable ~= 0 then
                objc.set_local_value(spawnedCable, SHOULD_START_PULSE_COUNTDOWN, 1)
            end
        end
    end
end

PowerPlant_Objective_deathTrigger = function(entity)
    local spawnedCable = objc.get_local_value(entity, SPAWNED_CABLE)
    if spawnedCable ~= 0 then
        objc.set_local_value(spawnedCable, SHOULD_KILL_CABLE, 1)
    end
end

PowerPlant_Objective_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    return 0
end

PowerPlant_Objective_reachedOvertimeTrigger = function(entity)
    objc.sacrifice_entity(entity)
end
