local ASSOCIATED_BOSS_KEY = 'associatedBossKey'

local INITIAL_IDLE_TIMER = 20

local CURRENT_BOSS_STATE = 'bossStateKey'
local BOSS_STATE_IDLE = 0
local BOSS_STATE_ACTIVE = 1

local ORIGINAL_SPEED = 'originalSpeedKey'

local EXPLOSION_ATTACK_ID = 'aliciaAttackExplosionEntity'
local EXPLOSION_CADENCE = 5
local NUM_ATTACKS_KEY = 'numAttacksKey'
local PREVIOUS_TARGET = 'previousTargetKey'

local TARGET_ENTITY = 'targetEntityKey'

R_AliciaBoss_spawnTrigger = function(entity)
    local originalSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    objc.set_local_value(entity, ORIGINAL_SPEED, originalSpeed)
end

R_AliciaBoss_tickTrigger = function(entity)
	local bossState = objc.get_local_value(entity, CURRENT_BOSS_STATE)
	if bossState ~= 0 then
        local originalSpeed = objc.get_local_value(entity, ORIGINAL_SPEED)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, originalSpeed)
	else
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
	end
end

R_AliciaBoss_onAttackTrigger = function(sourceEntity, targetEntity)
    local teamNumber = objc.get_team_number(sourceEntity)
    local previousTarget = objc.get_local_value(sourceEntity, PREVIOUS_TARGET)
    if previousTarget == targetEntity then
        local numAttacks = objc.get_local_value(sourceEntity, NUM_ATTACKS_KEY)
        numAttacks = numAttacks + 1
        if numAttacks % EXPLOSION_CADENCE == 0 then
            local enemyPos = objc.get_entity_position(targetEntity)
            local explosion = objc.spawn_at_point(EXPLOSION_ATTACK_ID, 1, teamNumber, enemyPos[1], enemyPos[2], 0, 0)
            objc.set_local_value(explosion[1], TARGET_ENTITY, targetEntity)
            objc.set_local_value(explosion[1], ASSOCIATED_BOSS_KEY, sourceEntity)
        end
        objc.set_local_value(sourceEntity, NUM_ATTACKS_KEY, numAttacks)
    else
        objc.set_local_value(sourceEntity, NUM_ATTACKS_KEY, 1)
    end
    objc.set_local_value(sourceEntity, PREVIOUS_TARGET, targetEntity)
end

R_AliciaBoss_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local teamNumber = objc.get_team_number(targetEntity)
    local aliciaBase = objc.get_base_entity(teamNumber)
    local aliciaMaxHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local aliciaCurrentHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
    objc.set_entity_value_for_key(aliciaBase, ENTITY_VALUE_KEY_MAX_HEALTH, aliciaMaxHealth)
    objc.set_entity_value_for_key(aliciaBase, ENTITY_VALUE_KEY_CURRENT_HEALTH, aliciaCurrentHealth - damage)
    return damage
end


R_AliciaBoss_deathTrigger = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local baseEntity = objc.get_base_entity(teamNumber)
    objc.kill_entity(baseEntity)
end
