--[[
This onDamageTrigger gives the R_AntBall the ability to break into new units when damaged.
--]]

local SPAWN_CHECK_KEY = 'spawnCheckTimerKey'
local SPAWN_TIMING = 70


local SPAWN_ANTS_TRIGGER_NAME = 'R_AntBall_spawnAntsTrigger'

local ANT_SPAWN_ANIMATION = 'R_AntBall_Battle_Spawner_Attack_01'
local FRAMES_IN_SPAWNER_ANIMATION = 45

local SPAWN_QUANITITY = 3
local ANT_X_OFFSET = 1000
local INITIAL_SPAWN_DELAY = 3
local ANT_SPAWN_DELAY = 2
local ANTS_SPAWNED_KEY = 'antsSpawnedKey'

local HAS_FIRST_SPAWN = 'hasFirstSpawned'
local FIRST_SPAWN_TIMING = 2

local SPAWN_ENTITY = 'R_Ant'

local SPAWN_ANIMATION_NAME_KEY = 'R_Ant_Battle_Spawn_01'

R_AntBall_tickTrigger = function(entity)
	local spawnTimer = SPAWN_TIMING
	local firstSpawn = objc.get_local_value(entity, HAS_FIRST_SPAWN)
	if firstSpawn == 0 then
		objc.idle_entity(entity, FIRST_SPAWN_TIMING * 100)
		spawnTimer = FIRST_SPAWN_TIMING
	end

	local spawnAnts = objc.tick_timer(entity, SPAWN_CHECK_KEY, spawnTimer)
	if spawnAnts == 0 then
		objc.set_local_value(entity, HAS_FIRST_SPAWN, 1)

		local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_SPAWNER_ANIMATION)
	    objc.idle_entity(entity, numberOfSubrounds)
		objc.play_override_animation(entity, ANT_SPAWN_ANIMATION, objc.subrounds_to_milliseconds(numberOfSubrounds))
		objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_AntBall.lua', SPAWN_ANTS_TRIGGER_NAME)
	end
end

R_AntBall_spawnAntsTrigger = function(entity)
	local antsSpawned = objc.get_local_value(entity, ANTS_SPAWNED_KEY)
	if antsSpawned < SPAWN_QUANITITY then
		local spawnDelay
		if antsSpawned == 0 then
			spawnDelay = objc.tick_timer(entity, 'antSpawnDelay', INITIAL_SPAWN_DELAY)
		else
			spawnDelay = objc.tick_timer(entity, 'antSpawnDelay', ANT_SPAWN_DELAY)
		end

		if spawnDelay == 0 then
			local teamNumber = objc.get_team_number(entity)
			local entityPosition = objc.get_entity_position(entity)
			if teamNumber == 1 then
				directionX = 1
			else
				directionX = -1
			end

		    local playerNumber = objc.get_player_number(entity)
		   	local tier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
	        if tier == 0 then
	            tier = ''
	        end
			local ant = objc.spawn_at_point(SPAWN_ENTITY .. tier, 1, playerNumber, entityPosition[1] + (ANT_X_OFFSET * directionX), entityPosition[2], directionX, 0)
			objc.idle_entity(ant[1], 400)
			objc.play_override_animation(ant[1], SPAWN_ANIMATION_NAME_KEY, 400)
			objc.set_local_value(entity, ANTS_SPAWNED_KEY, antsSpawned + 1)

		end
	else
		objc.set_local_value(entity, ANTS_SPAWNED_KEY, 0)
		objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, SPAWN_ANTS_TRIGGER_NAME)
	end
end
