local SPAWN_CHECK_KEY = 'spawnCheckTimerKey'
local SPAWN_TIMING = 75

local SPAWN_QUANITITY = 1

local HAS_FIRST_SPAWN = 'hasFirstSpawned'
local FIRST_SPAWN_TIMING = 20
local SPAWN_ENTITY = 'R_FactoryAnt'

local BUILD_ANIM_NAME = 'R_BotFactory_Attack_01'
local ANIMATION_SPAWN_TIMING = 5
local FRAMES_IN_BUILD_ANIM = 45

local HAS_INIT = 'hasInitKey'
local SPAWN_ANIMATION_STATE = 'spawnAnimationName'
local IDLE_ANIMATION_STATE = 'idleAnimationName'
local SWAP_IDLE_ANIMATION_COUNT = 10

R_AntFactory_spawnTrigger = function(entity)
    BuildingAbility_setNotSolidIfNextToTwoBoundaries(entity)

    local updatedAnimations = {}
    updatedAnimations[ANIMATION_STATE_IDLE] = SPAWN_ANIMATION_STATE
    objc.update_animation_state(entity, updatedAnimations)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_AntFactory.lua', 'R_AntFactory_swapIdleTrigger')
    objc.idle_entity(entity, 999999)
end

R_AntFactory_tickTrigger = function(entity)
	local spawnTimer = SPAWN_TIMING
	local playerNumber = objc.get_player_number(entity)
	local firstSpawn = objc.get_local_value(entity, HAS_FIRST_SPAWN)
	if firstSpawn == 0 then
		spawnTimer = FIRST_SPAWN_TIMING
	end

	local spawnAnts = objc.tick_timer(entity, SPAWN_CHECK_KEY, spawnTimer)
    if spawnAnts == 0 then
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_BUILD_ANIM)
        objc.play_override_animation(entity, BUILD_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
		objc.set_local_value(entity, HAS_FIRST_SPAWN, 1)
        objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_AntFactory.lua', 'R_AntFactory_spawnAntTrigger')
	end
end

R_AntFactory_spawnAntTrigger = function(entity)
    local animationTimer = objc.tick_timer(entity, 'animationTimerKey', ANIMATION_SPAWN_TIMING)
    if animationTimer == 0 then
        local tier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
        if tier == 0 then
            tier = ''
        end
        objc.spawn_at_entity(SPAWN_ENTITY .. tier, SPAWN_QUANITITY, entity)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_AntFactory_spawnAntTrigger')
    end
end

R_AntFactory_swapIdleTrigger = function(entity)
    local swapTimer = objc.tick_timer(entity, 'swapTimerKey', SWAP_IDLE_ANIMATION_COUNT)
    if swapTimer == 0 then
        local updatedAnimations = {}
        updatedAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_STATE
        objc.update_animation_state(entity, updatedAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_AntFactory_swapIdleTrigger')
    end
end

R_AntFactory_reachedOvertimeTrigger = function(entity)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'UnitAbilities.lua', 'UnitAbility_tickDownHealthTrigger')
end
