local TRIGGER_DISTANCE = 0
local DEATH_DELAY_KEY = 'deathTimerDelayKey'
local KILL_DELAY_IN_MILLISECONDS = 1500
local TO_KILL_ENTITY = 'toKillEntityKey'
local PICKUP_TRIGGER_NAME = 'R_BlackWidow_pickupTrigger'
local HAS_INIT = 'hasInitKey'
local ATTACK_ANIMATION_NAME = 'R_BlackWidow_Attack_01'
local FRAMES_IN_ATTACK_ANIMATION = 45
local HEIGHT_DIFFERENCE_FOR_MAGNET_AT_END_OF_ANIMATION = 420
local FRAMES_IN_DEATH_ANIMATION_UNTIL_EXPLOSION = 6
local TARGET_KILL_COUNTDOWN_TRIGGER_NAME = 'R_BlackWidow_toKillEntityCountdownTrigger'
local KILL_COUNTDOWN_KEY = 'R_BlackWidow_killCountdownKey'
local BASE_CUTOFF = 6000
local ORIGINAL_ATTACK_DAMAGE = 'originalAttackDamageKey'
local DAMAGE_SOURCE_ENTITY = 'damageSourceEntityKey'
local IS_NORMAL_ATTACK_DAMAGE = 'isNormalAttackDamageKey'

R_BlackWidow_spawnTrigger = function(entity)
	local originalAttackDamage = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_DAMAGE)
	objc.set_local_value(entity, ORIGINAL_ATTACK_DAMAGE, originalAttackDamage)
	objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF, -originalAttackDamage)
end

--[[
This OnAttackTrigger kills the Black Widow and the unit it attacks
--]]
R_BlackWidow_onAttackTrigger = function(sourceEntity, targetEntity)
	objc.set_targetable(sourceEntity, false)

	local numberOfSubroundsForIdle = objc.milliseconds_to_subrounds(KILL_DELAY_IN_MILLISECONDS)
	
	-- Since the kill countdown starts on the next round, add in one round of idle
	local sourceEntityIdleSubrounds = numberOfSubroundsForIdle + NUMBER_OF_SUBROUNDS_IN_ROUND
	objc.idle_entity(sourceEntity, sourceEntityIdleSubrounds)

	local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ATTACK_ANIMATION)
	objc.play_override_animation(sourceEntity, ATTACK_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

	if not (objc.is_barricade(targetEntity) or objc.is_base(targetEntity) or (objc.is_boss_unit(targetEntity) == 1)) then
		objc.set_targetable(targetEntity, false)

		-- Since the kill countdown starts on the next round, and the BlackWidow's attack trigger occurred after the target's tick trigger, add in two rounds of idle
		local targetEntityIdleSubrounds = numberOfSubroundsForIdle + NUMBER_OF_SUBROUNDS_IN_ROUND * 2
		objc.idle_entity(targetEntity, targetEntityIdleSubrounds)

		local airHeightVariance = objc.get_entity_air_height_variance(sourceEntity)
		objc.set_entity_air_height_variance(targetEntity, airHeightVariance + HEIGHT_DIFFERENCE_FOR_MAGNET_AT_END_OF_ANIMATION)
		objc.set_entity_air_height_alignment(targetEntity, AIR_HEIGHT_ALIGN_TOP)
		objc.move_entity_to_planes_of_existence(targetEntity, PLANE_OF_EXISTENCE_AIR, 700, EASE_IN, 50)

		local sourcePosition = objc.get_entity_position(sourceEntity)
		objc.teleport_entity(targetEntity, sourcePosition[1], sourcePosition[2])
	end

	objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'R_BlackWidow.lua', PICKUP_TRIGGER_NAME)
	objc.set_local_value(sourceEntity, TO_KILL_ENTITY, targetEntity)
end

R_BlackWidow_tickTrigger = function(entity)
	local hasInit = objc.get_local_value(entity, HAS_INIT)
	if hasInit == 0 then
		objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, TRIGGER_DISTANCE)
		objc.set_local_value(entity, HAS_INIT, 1)
	end
end

R_BlackWidow_pickupTrigger = function(entity)
	local killDelayInRounds = objc.milliseconds_to_rounds(KILL_DELAY_IN_MILLISECONDS)

	local deathTime = objc.tick_timer(entity, DEATH_DELAY_KEY, killDelayInRounds)
	if deathTime == 0 then
		objc.sacrifice_entity(entity)
	end
end

R_BlackWidow_deathTrigger = function (entity)
	local toKillEntity = objc.get_local_value(entity, TO_KILL_ENTITY)
	if toKillEntity ~= 0 then
		local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_DEATH_ANIMATION_UNTIL_EXPLOSION)
		objc.set_local_value(toKillEntity, DAMAGE_SOURCE_ENTITY, entity)
		objc.idle_entity(toKillEntity, numberOfSubrounds)
		objc.add_trigger(toKillEntity, TRIGGER_NAME_TICK_TRIGGER, 'R_BlackWidow.lua', TARGET_KILL_COUNTDOWN_TRIGGER_NAME)		
	end
end

R_BlackWidow_canTargetTrigger = function(entity, targetEntity)
	if objc.is_miner(targetEntity) then
		return NO
	end
	return YES
end

R_BlackWidow_toKillEntityCountdownTrigger = function(entity)
	local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_DEATH_ANIMATION_UNTIL_EXPLOSION)
	local countdownRounds = numberOfSubrounds / NUMBER_OF_SUBROUNDS_IN_ROUND 

	local countdownTime = objc.tick_timer(entity, KILL_COUNTDOWN_KEY, countdownRounds)
	if countdownTime == 0 then
		if objc.is_barricade(entity) or objc.is_base(entity) or (objc.is_boss_unit(entity) == 1) then
			local damageSource = objc.get_local_value(entity, DAMAGE_SOURCE_ENTITY)
			local damageToDo = objc.get_local_value(damageSource, ORIGINAL_ATTACK_DAMAGE)
			objc.inflict_normal_damage(damageSource, entity, damageToDo)
			objc.flash_entity_for_damage(entity, 250)
		else
			objc.set_entity_visibility(entity, false)
			objc.kill_entity(entity)
		end	

		objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, TARGET_KILL_COUNTDOWN_TRIGGER_NAME)
	end
end