local TELEPORT_DISTANCE = 3950
local KNOCKBACK_TARGET_ENTITY = 'cannonScooterTargetEntity'
local ORIGINAL_RANGE_KEY = 'originalRangeKey'
local MOTION_STATE_DICTIONARY = 'motionStateDictKey'
local IS_KNOCKING_SELF_BACK = 'knockingSelfBackKey'
local KNOCKBACK_BUFFER = 100
local KNOCKBACK_SPEED = 600
local KNOCKBACK_DELAY_TRIGGER = 'R_CannonScooter_waitForKnockbackDelayTrigger'
local FRAMES_IN_ATTACK_ANIMATION = 51
local ATTACK_ANIMATION_NAME = 'R_CannonScooter_Battle_Attack_01'
local KNOCKBACK_DELAY_TIME = 0
local KNOCKBACK_TIMER_KEY = 'knockbackDelayKey'
local KNOCKBACK_TARGET = 'knockbackTargetKey'
local KNOCKBACK_MAX_TIMER = 'knockbackMaxTimerKey'
local KNOCKBACK_MAX_COUNT = 35
local KNOCKBACK_MAX_TRIGGER_NAME = 'R_CannonScooter_timeOutKnockbackTrigger'
local REVERSE_TIMEOUT_TRIGGER_NAME = 'R_CannonScooter_reverseTimeoutTrigger'
local REVERSE_TIMEOUT_TIMER = 'reverseTimeoutTimerKey'
local REVERSE_TIMEOUT_COUNT = 20
local ATTACK_SPEED_BUFFER = 1000

R_CannonScooter_spawnTrigger = function(entity)
    local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)
end

R_CannonScooter_actionSystemLoopStartTrigger = function(sourceEntity, targetEntity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_ATTACK_ANIMATION)
    objc.play_override_animation(sourceEntity, ATTACK_ANIMATION_NAME, numberOfSubrounds)
end

R_CannonScooter_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, KNOCKBACK_TARGET, targetEntity)
    objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'R_CannonScooter.lua', KNOCKBACK_DELAY_TRIGGER)
end

R_CannonScooter_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local isKnockingSelfBack = objc.get_local_value(sourceEntity, IS_KNOCKING_SELF_BACK)
    if isKnockingSelfBack ~= 0 then
        local motionDict = objc.get_local_value(sourceEntity, MOTION_STATE_DICTIONARY)
        local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
        objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, motionDict)
        objc.set_local_value(sourceEntity, IS_KNOCKING_SELF_BACK, 0)

        local attackRate = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RATE)
        objc.idle_entity(sourceEntity, attackRate - ATTACK_SPEED_BUFFER)

        objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'R_CannonScooter.lua', REVERSE_TIMEOUT_TRIGGER_NAME)
        objc.remove_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, KNOCKBACK_MAX_TRIGGER_NAME)
    end
end

R_CannonScooter_waitForKnockbackDelayTrigger = function(entity)
    local knockbackTime = objc.tick_timer(entity, KNOCKBACK_TIMER_KEY, KNOCKBACK_DELAY_TIME)
    if knockbackTime == 0 then
        local teamNumber = objc.get_team_number(entity)
        local posX = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
        local posY = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y)
        local targetEntity = objc.get_local_value(entity, KNOCKBACK_TARGET)
        if targetEntity ~= 0 then
            local targetPosX = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_X)

            if targetPosX > posX then
                local knockbackEntity = objc.spawn_at_point(KNOCKBACK_TARGET_ENTITY, 1, teamNumber, posX - TELEPORT_DISTANCE, posY, 0, 0)
                --objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, KNOCKBACK_BUFFER)
                objc.set_entity_should_reverse_facing_direction(entity, true)
                local motionDict = objc.move_entity_to_entity_with_motion(entity, knockbackEntity[1], MOTION_STATE_WALK, KNOCKBACK_SPEED)
                objc.set_local_value(entity, MOTION_STATE_DICTIONARY, motionDict)
                objc.set_local_value(entity, IS_KNOCKING_SELF_BACK, 1)
            else
                local knockbackEntity = objc.spawn_at_point(KNOCKBACK_TARGET_ENTITY, 1, teamNumber, posX + TELEPORT_DISTANCE, posY, 0, 0)
                --objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, KNOCKBACK_BUFFER)                
                objc.set_entity_should_reverse_facing_direction(entity, true)
                local motionDict = objc.move_entity_to_entity_with_motion(entity, knockbackEntity[1], MOTION_STATE_WALK, KNOCKBACK_SPEED)
                objc.set_local_value(entity, MOTION_STATE_DICTIONARY, motionDict)
                objc.set_local_value(entity, IS_KNOCKING_SELF_BACK, 1)
            end

            objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_CannonScooter.lua', KNOCKBACK_MAX_TRIGGER_NAME)
        end

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, KNOCKBACK_DELAY_TRIGGER)
    end
end

R_CannonScooter_timeOutKnockbackTrigger = function(entity)
    local knockbackMaxTimer = objc.tick_timer(entity, KNOCKBACK_MAX_TIMER, KNOCKBACK_MAX_COUNT)
    if knockbackMaxTimer == 0 then
        objc.idle_entity(entity, 100)
        objc.set_entity_should_reverse_facing_direction(entity, false)

        local motionDict = objc.get_local_value(entity, MOTION_STATE_DICTIONARY)
        objc.swap_out_entity_motion_state_with_dictionary(entity, motionDict)

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, KNOCKBACK_MAX_TRIGGER_NAME)
    end
end

R_CannonScooter_reverseTimeoutTrigger = function(entity)
    local reverseTimeoutTimer = objc.tick_timer(entity, REVERSE_TIMEOUT_TIMER, REVERSE_TIMEOUT_COUNT)
    if reverseTimeoutTimer == 0 then
        objc.set_entity_should_reverse_facing_direction(entity, false)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, REVERSE_TIMEOUT_TRIGGER_NAME)
    end
end
