local CLEAVE_RANGE_BUFFER = 600
local CLEAVE_ATTACK_RATE = 500
local NEXT_TARGET = 'nextTargetKey'
local ORIGINAL_RANGE_KEY = 'originalAttackRangeKey'
local ORIGINAL_ATTACK_KEY = 'originalAttackRateKey'
local MAX_CLEAVE_RANGE_KEY = 'maxCleaveRange'
local NUM_ATTACKS = 'numThisAttacksKey'
local MAX_ATTACKS = 8

R_Dragonfly_spawnTrigger = function(entity)
    local originalAttack = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
    objc.set_local_value(entity, ORIGINAL_ATTACK_KEY, originalAttack)

    local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)

    local maxCleaveRange = originalRange + CLEAVE_RANGE_BUFFER
    objc.set_local_value(entity, MAX_CLEAVE_RANGE_KEY, maxCleaveRange)
end

R_Dragonfly_tickTrigger = function(entity)
    local nextTarget = objc.get_local_value(entity, NEXT_TARGET)
    if nextTarget ~= 0 then
        objc.set_targeting(entity, nextTarget)
        objc.set_local_value(entity, NEXT_TARGET, 0)
    end
end

R_Dragonfly_onAttackTrigger = function(sourceEntity, targetEntity)
    local maxCleaveRange = objc.get_local_value(sourceEntity, MAX_CLEAVE_RANGE_KEY)
    local numAttacks = objc.get_local_value(sourceEntity, NUM_ATTACKS)
    if numAttacks + 1 < MAX_ATTACKS then
        local enemyNumber = objc.get_enemy_team_number(sourceEntity)
        local enemiesInRange = objc.list_entities_in_radius(sourceEntity, maxCleaveRange, enemyNumber)
        local highestEnemyScore = -100
        local highestEntity = -1
        local truckDamage = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_DAMAGE)

        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RATE, CLEAVE_ATTACK_RATE)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, maxCleaveRange)

        for _,enemy in pairs(enemiesInRange) do
            if objc.is_normal_unit(enemy) or objc.is_commander(enemy) then
                local randomScore = math.floor(objc.roll_number(1, 100))
                if randomScore > highestEnemyScore then
                    highestEntity = enemy
                    highestEnemyScore = randomScore
                end
            end
        end

        if highestEntity ~= -1 then
            objc.set_local_value(sourceEntity, NEXT_TARGET, highestEntity)
        end

        objc.set_local_value(sourceEntity, NUM_ATTACKS, numAttacks + 1)
    else
        local originalAttack = objc.get_local_value(sourceEntity, ORIGINAL_ATTACK_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RATE, originalAttack)
        
        local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
        objc.set_local_value(sourceEntity, NUM_ATTACKS, 1)
    end
end
