local NEXT_TARGET_RANGE = 4300
local NEW_TARGET_KEY = 'newTargetKey'
local HAS_TARGET_KEY = 'hasTarget'
local FLAME_ENTITY_IDENTIFIER = 'flameScriptEntity'
local OWNER_ENTITY = 'ownerEntityKey'
local ORIGINAL_MOVEMENT = 'originalMovementKey'
local IS_FLAMED_KEY = 'isFlamedKey'
local FLAME_PARTICLE = 'test_fire.lvl'
local MOVE_SPEED = 60
local DOT_TICK_TRIGGER_NAME = 'R_Flamer_dotTickTrigger'
local DOT_LENGTH = 200
local DOT_LENGTH_KEY = 'dotLengthTimerKey'
local DAMAGE_TICK_SPEED = 10
local DOT_TICK_KEY = 'dotDamageTickKey'
local DAMAGE_PER_TICK = 30
local SOURCE_FLAMER_KEY = 'sourceFlamerUnit'
local IS_FIRE_DAMAGE_KEY = 'isFireDamageKey'
local ATTACK_RATE_INCREASE = 500

R_Flamer_spawnTrigger = function(entity)
    objc.set_local_value(entity, IS_FIRE_DAMAGE_KEY, 1)
end

R_Flamer_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, HAS_TARGET_KEY, 0)
end

R_Flamer_damageTrigger = function(damage, targetEntity, sourceEntity)
    local isThisFlamed = objc.get_local_value(targetEntity, IS_FLAMED_KEY)
    local targetNumber = objc.get_team_number(targetEntity)
    objc.set_local_value(sourceEntity, IS_FIRE_DAMAGE_KEY, 1)

    if isThisFlamed == 0 then
        objc.set_local_value(targetEntity, IS_FLAMED_KEY, 1)
        objc.set_local_value(targetEntity, SOURCE_FLAMER_KEY, sourceEntity)
        objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'R_Flamer.lua', DOT_TICK_TRIGGER_NAME)
        local currentAttackRate = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RATE)
        objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RATE, currentAttackRate + ATTACK_RATE_INCREASE)
        objc.play_vfx(targetEntity, FLAME_PARTICLE, NODE_STATUS)
    end

    local parent = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local enemies = objc.list_entities_in_radius(parent, NEXT_TARGET_RANGE, targetNumber)
    local closestEntity = -1
    local closestDistance = 99999999
    for _,enemy in pairs(enemies) do
        local thisDist = objc.quick_unit_distance_square(parent, enemy)
        local isFlamed = objc.get_local_value(enemy, IS_FLAMED_KEY)
        if thisDist < closestDistance and enemy ~= targetEntity and isFlamed == 0 and objc.is_normal_unit(enemy) then
            closestDistance = thisDist
            closestEntity = enemy
        end
    end

    if closestEntity ~= -1 then
        local originalMovement = objc.move_entity_to_entity_with_motion(parent, closestEntity, MOTION_STATE_WALK, MOVE_SPEED)
        objc.set_local_value(parent, ORIGINAL_MOVEMENT, originalMovement)
    end

    return damage
end

R_Flamer_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local originalDict = objc.get_local_value(sourceEntity, ORIGINAL_MOVEMENT)
    objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, originalDict)
    objc.set_targeting(sourceEntity, targetEntity)
end

R_Flamer_dotTickTrigger = function(entity)
    local lengthTimer = objc.tick_timer(entity, DOT_LENGTH_KEY, DOT_LENGTH)
    local damageTimer = objc.tick_timer(entity, DOT_TICK_KEY, DAMAGE_TICK_SPEED)
    if damageTimer == 0 then
        local sourceFlamerUnit = objc.get_local_value(entity, SOURCE_FLAMER_KEY)
        objc.inflict_normal_damage(sourceFlamerUnit, entity, DAMAGE_PER_TICK)
        objc.flash_entity_for_damage(entity, 250)
    end

    if lengthTimer == 0 then
        objc.stop_vfx(entity, FLAME_PARTICLE, NODE_STATUS, false)
        local currentAttackRate = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE, currentAttackRate - ATTACK_RATE_INCREASE)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, DOT_TICK_TRIGGER_NAME)
    end
end
