local ATTACK_SPEED_DIVISOR = 2
local MOVE_SPEED_BONUS = 50
local HAS_BEED_ENRAGED_KEY = 'hasBeenEnragedKey'
local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local BUFF_VFX = 'CommanderPowerUpText.lvl'
local rage_duration = 0

local ALLY_CHECK_INTERVAL = 2


local COMBAT_IDLE_ANIM_NAME = 'attackAnimationName'

--Only used by campaign scripts to set Rage's duration to infinite
local CAMPAIGN_INFINITE_DURATION_KEY = 'campaignInfiniteDurationKey'

R_Rage_spawnTrigger = function(entity)
    objc.set_targetable(entity, false)

    local updatedAnimations = {}
    updatedAnimations[ANIMATION_STATE_IDLE] = COMBAT_IDLE_ANIM_NAME
    objc.update_animation_state(entity, updatedAnimations)
    rage_duration = objc.divide_with_floor(objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_DURATION), 100)

    objc.set_solid(entity, false)
    objc.idle_entity(entity, 10000)
end

R_Rage_tickTrigger = function(entity)
    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_INTERVAL)
    if allyCheckTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
        if buffedUnits == 0 then
            buffedUnits = {}
        end

        for _, buffedUnit in pairs(buffedUnits) do
            R_Rage_removeBuff(buffedUnit)
        end

        local buff_radius = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_SPELL_RADIUS)
        buffedUnits = {}

        local allies = objc.list_entities_in_radius(entity, buff_radius, teamNumber)
        for _, ally in pairs(allies) do
            local hasBeenEnraged = objc.get_local_value(ally, HAS_BEED_ENRAGED_KEY)
            if objc.is_normal_unit(ally) and hasBeenEnraged == 0 then
                R_Rage_buffUnit(ally)
                table.insert(buffedUnits, ally)
            end
        end

        objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
    end

    local durationTimer
    if objc.get_local_value(entity, CAMPAIGN_INFINITE_DURATION_KEY) == 1 then
        durationTimer = objc.tick_timer(entity, 'rageDurationTimer', 999999)
    else
        durationTimer = objc.tick_timer(entity, 'rageDurationTimer', rage_duration)
    end

    if durationTimer == 0 then
        objc.sacrifice_entity(entity)
    end
end

R_Rage_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits ~= 0 then
        for _, buffedUnit in pairs(buffedUnits) do
            R_Rage_removeBuff(buffedUnit)
        end
    end
end


--Prevents Rage from taking splash damage and dying
R_Rage_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    return 0
end

R_Rage_buffUnit = function(entity)
    local thisSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    local buffToGive = objc.divide_with_floor(MOVE_SPEED_BONUS * thisSpeed, 100)
    --print('RAGE ADDING thisSpeed: ' .. thisSpeed .. ' buffToGive: ' .. buffToGive)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, thisSpeed + buffToGive)
    --print("NEW SPEED: " .. objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY))

    local attackSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
    local newAttackRate = objc.divide_with_floor(attackSpeed, ATTACK_SPEED_DIVISOR)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE, newAttackRate)
    objc.play_vfx(entity, BUFF_VFX, NODE_STATUS)

    objc.set_local_value(entity, HAS_BEED_ENRAGED_KEY, 1)
end

R_Rage_reachedOvertimeTrigger = function(entity)
    objc.kill_entity(entity)
end

R_Rage_removeBuff = function(entity)
    local thisSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
    local buffRemovalMultiplier = 100 - objc.divide_with_ceiling(10000, MOVE_SPEED_BONUS + 100)
    local buffToRemove = objc.divide_with_ceiling(buffRemovalMultiplier * thisSpeed, 100)


    --print('RAGE REMOVING thisSpeed: ' .. thisSpeed .. ' buffToRemove: ' .. buffToRemove)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, thisSpeed - buffToRemove)
    --print("NEW SPEED: " .. objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY))

    local attackSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
    local newAttackRate = math.floor(attackSpeed * ATTACK_SPEED_DIVISOR)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE, newAttackRate)
    --print('RAGE DEBUFFED ATK SPEED: '..attackSpeed)
    --print('RAGE RESTORED ATK SPEED: '..newAttackRate)    

    objc.stop_vfx(entity, BUFF_VFX, NODE_STATUS, false)

    objc.set_local_value(entity, HAS_BEED_ENRAGED_KEY, 0)
end
