local SEEK_RADIUS = 1200
local NUM_MISSILES = 3
local HAS_BEEN_TARGETED = 'hasBeenTargetedKey'
local DAMAGE_TARGET = 'damageTargetKey'
local DAMAGE_AMOUNT = 'damageAmountKey'
local SOURCE_SPELL_TIER = 'sourceSpellTierKey'
local HAS_INIT = 'hasInitKey'

local MOVE_TO_TARGET_TIME_ROUNDS = 2
local AIR_MOVE_ANIMATION_NAME = 'normalMoveAnimationName'
local GROUND_MOVE_ANIMATION_NAME = 'groundMoveAnimationName'

local SPAWN_ANIM_NAME = 'spawnAnimationName'
local HAS_SET_TO_MOVE = 'hasSetToMoveKey'
local FRAMES_IN_SPAWN_ANIM = 30
local WAIT_FOR_ANIMATION_TIME = 1
local WAIT_FOR_SPAWN_TIME = 1

local EARLY_DETONATION_DISTANCE = 4000000
local INITIAL_AIR_HEIGHT = 1500

R_SingleSeekerMissile_spawnTrigger = function(entity)
   objc.set_entity_visibility(entity, false)
   objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, 0)
   objc.set_targetable(entity, false)
   objc.set_solid(entity, false)
end

R_SingleSeekerMissile_tickTrigger = function(entity)
   local waitForSpawnTimer = objc.tick_timer(entity, 'waitForSpawnTimerKey', WAIT_FOR_SPAWN_TIME)
   if waitForSpawnTimer == 0 then
      local hasInit = objc.get_local_value(entity, HAS_INIT)
      if hasInit == 0 then
         objc.move_entity_to_vertical_height(entity, INITIAL_AIR_HEIGHT, 0, EASE_NONE, 20)
         objc.set_entity_air_height_variance(entity, INITIAL_AIR_HEIGHT)
         objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_SingleSeekerMissile.lua', 'R_SingleSeekerMissile_waitForMoveUpTrigger')
         objc.set_local_value(entity, HAS_INIT, 1)
      end
   end

   --this used to be the special case for power plants but I'm using it for everything now because it's more reliable than the targeting completion trigger
   if targetEntity ~= nil and objc.quick_unit_distance_square(entity, targetEntity) < EARLY_DETONATION_DISTANCE then
      R_SingleSeekerMissile_onOverrideTargetingCompletionTrigger(entity,targetEntity)
   end
end

R_SingleSeekerMissile_waitForMoveUpTrigger = function(entity)
   local waitForMoveTime = objc.tick_timer(entity, 'waitForAnimationKey', WAIT_FOR_ANIMATION_TIME)
   if waitForMoveTime == 0 then
      objc.set_entity_visibility(entity, true)
      local targetEntity = objc.get_local_value(entity, DAMAGE_TARGET)
      if targetEntity ~= 0 then
         local targetDistance = objc.unit_distance_with_offset(entity, 0, 0, targetEntity, 0, 0)
         local movementVelocity = objc.divide_with_ceiling(targetDistance, MOVE_TO_TARGET_TIME_ROUNDS)
   
         local targetPlaneofExistence = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
         local updatedAnimations = {}
         updatedAnimations[ANIMATION_STATE_IDLE] = ''
         if targetPlaneofExistence == NUM_PLANE_OF_EXISTENCE_AIR then
            local enemyHeightVariance = objc.get_entity_air_height_variance(targetEntity)
            updatedAnimations[ANIMATION_STATE_MOVE] = AIR_MOVE_ANIMATION_NAME
            objc.move_entity_to_vertical_height(entity, 800 + enemyHeightVariance, MOVE_TO_TARGET_TIME_ROUNDS * 100, EASE_NONE, 20)
         elseif targetPlaneofExistence == NUM_PLANE_OF_EXISTENCE_GROUND then
            updatedAnimations[ANIMATION_STATE_MOVE] = AIR_MOVE_ANIMATION_NAME
            objc.move_entity_to_vertical_height(entity, 0, MOVE_TO_TARGET_TIME_ROUNDS * 100, EASE_NONE, 20)
         end
         objc.update_animation_state(entity, updatedAnimations)
         objc.move_entity_to_entity_with_motion(entity, targetEntity, MOTION_STATE_WALK, movementVelocity)
         objc.set_local_value(entity, HAS_SET_TO_MOVE, 1)
      else
         objc.sacrifice_entity(entity)
      end

      objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_SingleSeekerMissile_waitForMoveUpTrigger')
   end
end

R_SingleSeekerMissile_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
   if targetEntity ~= nil then
      objc.set_local_value(targetEntity, HAS_BEEN_TARGETED, 0)
      objc.stop_vfx(targetEntity, 'idle_target.lvl', NODE_STATUS, true)
   end

   local targetPlaneofExistence = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
   if targetPlaneofExistence == NUM_PLANE_OF_EXISTENCE_AIR then
      objc.spawn_at_entity_with_source_entity('seekerMissileImpactAirEntity', 1, targetEntity, sourceEntity)
   else
      objc.spawn_at_entity_with_source_entity('seekerMissileImpactEntity', 1, targetEntity, sourceEntity)
   end

   local damageAmount = objc.get_local_value(sourceEntity, DAMAGE_AMOUNT)
   objc.inflict_normal_damage(sourceEntity, targetEntity, damageAmount)
   objc.flash_entity_for_damage(targetEntity, 250)

   objc.set_entity_visibility(sourceEntity, false)
   objc.sacrifice_entity(sourceEntity)
end
