local DAMAGE_TIMER_KEY = 'damageTimerKey'
local DAMAGE_TICK_TIME = 3
local DAMAGE_PER_TICK = 9999
local DAMAGE_RADIUS = 600
local STOP_TIME = 10
local ROLLOVER_ANIMATION_NAME = 'R_Pancake_Attack_01'
local ROLLOVER_ANIMATION_FRAMES = 45
local IS_ROLLOVER_PLAYING_KEY = 'isRolloverPlayingKey'
local ROLLOVER_TIMER_KEY = 'rolloverTimerKey'
local EMPTY_DEATH_ANIM = ''
local IS_MOVING_KEY = 'isMovingKey'
local SHIELD_VFX = 'imp_R_ShieldLarge.lvl'

R_Steamroller_tickTrigger = function(entity)
    local damageTick = objc.tick_timer(entity, DAMAGE_TIMER_KEY, DAMAGE_TICK_TIME)
    local isMoving = objc.get_local_value(entity, IS_MOVING_KEY)
    if damageTick == 0 and isMoving == 1 then
        local enemyNumber = objc.get_enemy_team_number(entity)
        local enemies = objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyNumber)
        R_Steamroller_damageEntities(entity, enemies, DAMAGE_PER_TICK)
    end

    --Check if the rollover animation is playing. If so, count down to when it's done, kill the squashed
    --units, then clear the list of units to kill
    local isRolloverPlaying = objc.get_local_value(entity, IS_ROLLOVER_PLAYING_KEY)
    if isRolloverPlaying == 1 then
        local rolloverRounds = objc.convert_frames_to_rounds(ROLLOVER_ANIMATION_FRAMES)
        local rolloverTimer = objc.tick_timer(entity, ROLLOVER_TIMER_KEY, rolloverRounds)
        if rolloverTimer == 0 then
            objc.set_local_value(entity, IS_ROLLOVER_PLAYING_KEY, 0)
        end
    end
end


R_Steamroller_damageEntities = function(steamroller, entityList, damage)
    local myHealth = objc.get_entity_value_for_key(steamroller, ENTITY_VALUE_KEY_MAX_HEALTH)

    --make sure we're not immobilzied, and not targeting things
    local damageTypes = {'building', 'base'}
    objc.set_entity_value_for_key(steamroller, ENTITY_VALUE_KEY_DAMAGE_TYPE, damageTypes)

    for _,target in pairs(entityList) do
        local thisHealth = objc.get_entity_value_for_key(target, ENTITY_VALUE_KEY_MAX_HEALTH)
        if objc.is_ground_unit(target) and objc.is_normal_unit(target) then
            if myHealth > thisHealth then
                local rolloverSubrounds = objc.convert_frames_to_subrounds(ROLLOVER_ANIMATION_FRAMES)
                objc.kill_entity(target)

                --Start the rollover animation
                local isRolloverPlaying = objc.get_local_value(steamroller, IS_ROLLOVER_PLAYING_KEY)
                if isRolloverPlaying == 0 then
                    objc.play_passive_animation(steamroller, ROLLOVER_ANIMATION_NAME, 0, rolloverSubrounds)
                    objc.set_local_value(steamroller, IS_ROLLOVER_PLAYING_KEY, 1)
                end
            --else
                --if we should be immobilized and targeting things, do so
                --local damageTypes = {'building', 'base', 'ground'}
                --objc.set_entity_value_for_key(steamroller, ENTITY_VALUE_KEY_DAMAGE_TYPE, damageTypes)
                --objc.set_targeting(steamroller, target)
            end
        end
    end
end

R_Steamroller_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local armorAmount = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ARMOR)
    local flashCutoff = objc.divide_with_floor(armorAmount * 10, 4)
    local originalDamage = damage + armorAmount
    if originalDamage <= flashCutoff then
        objc.play_vfx(targetEntity, SHIELD_VFX, NODE_IMPACT)
    end
    return damage
end

R_Steamroller_actionSystemLoopStartTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, IS_MOVING_KEY, 0)
end

R_Steamroller_onMovementTrigger = function(entity)
    objc.set_local_value(entity, IS_MOVING_KEY, 1)
end
