local PRIMARY_TARGET_KEY = 'primaryTargetKey'
local DAMAGE_PROPORTION = 1
local SPLASH_DISTANCE = 2950
--these determine the size of the triangle, derived from the splash distance
local X_DIFFERENCE = 2600
local Y_DIFFERENCE = 1400
local IMPACT_VFX_ENTITY = 'truckTankImpactEntity'

R_TruckTank_spawnTrigger = function(entity)
    objc.set_local_value(entity, PRIMARY_TARGET_KEY, 0)
end

R_TruckTank_onAttackTrigger = function(sourceEntity, targetEntity)
    objc.set_local_value(sourceEntity, PRIMARY_TARGET_KEY, 0)
end

R_TruckTank_damageTrigger = function(damage, targetEntity, sourceEntity)
    local pyro = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local primaryTarget = objc.get_local_value(pyro, PRIMARY_TARGET_KEY)
    local pyroPosition = objc.get_entity_position(pyro)
    pyroPosition[1] = pyroPosition[1] + 10000
    local targetPosition = objc.get_entity_position(targetEntity)
    targetPosition[1] = targetPosition[1] + 10000
    local attackDamage = objc.get_entity_value_for_key(pyro, ENTITY_VALUE_KEY_ATTACK_DAMAGE)
    local deltaX = targetPosition[1] - pyroPosition[1]
    local deltaY = targetPosition[2] - pyroPosition[2]
    local theta = math.atan(deltaY, deltaX)
    --pointA is pyroPosition, for now
    local pointB = R_TruckTank_rotatePoint(X_DIFFERENCE, Y_DIFFERENCE, theta, pyroPosition[1], pyroPosition[2])
    local pointC = R_TruckTank_rotatePoint(X_DIFFERENCE, -Y_DIFFERENCE, theta, pyroPosition[1], pyroPosition[2])
    
    if primaryTarget == 0 then
        objc.set_local_value(pyro, PRIMARY_TARGET_KEY, targetEntity)
        local targetNumber = objc.get_team_number(targetEntity)
        local enemies = objc.list_entities_in_radius(pyro, SPLASH_DISTANCE, targetNumber)

        for _,enemy in pairs(enemies) do
            local enemyPos = objc.get_entity_position(enemy)

            local isInsideTrangle = R_TruckTank_checkIfPointIsInTriangle(pyroPosition[1], pyroPosition[2], pointB[1], pointB[2], pointC[1], pointC[2], enemyPos[1], enemyPos[2])
            if isInsideTrangle and enemy ~= targetEntity and objc.is_ground_unit(enemy) then
                objc.spawn_at_entity(IMPACT_VFX_ENTITY, 1, enemy)
                objc.inflict_normal_damage(sourceEntity, enemy, math.ceil(attackDamage * DAMAGE_PROPORTION))
            end
        end
    end
    return damage
end

R_TruckTank_rotatePoint = function(pointX, pointY, angle, originalX, originalY)
    local s = math.sin(angle)
    local c = math.cos(angle)

    local tempX = pointX - originalX
    local tempY = pointY - originalY

    local rotateX = pointX * c - pointY * s
    local rotateY = pointX * s + pointY * c

    local newX = rotateX + originalX
    local newY = rotateY + originalY

    return {newX, newY}
end

R_TruckTank_checkIfPointIsInTriangle = function(x1, y1, x2, y2, x3, y3, x, y)
    x = objc.divide_with_floor(math.floor(x), 100)
    x1 = objc.divide_with_floor(math.floor(x1), 100)
    x2 = objc.divide_with_floor(math.floor(x2), 100)
    x3 = objc.divide_with_floor(math.floor(x3), 100)
    y = objc.divide_with_floor(math.floor(y), 100)
    y1 = objc.divide_with_floor(math.floor(y1), 100)
    y2 = objc.divide_with_floor(math.floor(y2), 100)
    y3 = objc.divide_with_floor(math.floor(y3), 100)
    local denominator = ((y2 - y3)*(x1 - x3) + (x3 - x2)*(y1 - y3));
    local a = objc.divide_with_floor((y2 - y3)*(x - x3) + (x3 - x2)*(y - y3) * 100, denominator);
    local b = objc.divide_with_floor((y3 - y1)*(x - x3) + (x1 - x3)*(y - y3) * 100, denominator);
    local c = 100 - a - b;

    if -100 <= a and a <= 100 and 0 <= b and b <= 100 and 0 <= c and c <= 100 then
        return true
    end

    return false
end
