local DURATION = 300 --in game rounds
local HEALTH_TICK_INTERVAL = 10 --in game rounds
local HEALTH_INTERVAL_TIMER = 'healthIntervalTimerKey'
local STAGE_UPDATED_KEY = 'stageUpdatedKey'
local ANIMATION_SWITCH_DELAY = 5
local ANIMATION_SWITCH_TIMER = 'animationSwitchTimerKey'
local IDLE_ANIMATION_NAME = 'idleStage1'
local SHOULD_TICK_HEALTH = 'shouldTickHealthKey'

R_ValsTurret_spawnTrigger = function(entity)
    objc.immobilize_entity(entity, 1000000)
    objc.set_targetable(entity, false)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_ValsTurret.lua', 'R_ValsTurret_switchAnimationTrigger')
end

R_ValsTurret_switchAnimationTrigger = function(entity)
    local animationSwitchTimer = objc.tick_timer(entity, ANIMATION_SWITCH_TIMER, ANIMATION_SWITCH_DELAY)
    if animationSwitchTimer == 0 then
        local idleAnimations = {}
        idleAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIMATION_NAME
        objc.update_animation_state(entity, idleAnimations)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'R_ValsTurret_switchAnimationTrigger')
        objc.set_targetable(entity, true)
        objc.set_local_value(entity, SHOULD_TICK_HEALTH, 1)
    end
end

R_ValsTurret_tickTrigger = function(entity)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
    local shouldTickHealth = objc.get_local_value(entity, SHOULD_TICK_HEALTH)
    if shouldTickHealth == 1 then
        local healthTick = objc.tick_timer(entity, HEALTH_INTERVAL_TIMER, HEALTH_TICK_INTERVAL)
        if healthTick == 0 then
            local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
            local durationProp = objc.divide_with_ceiling(DURATION, HEALTH_TICK_INTERVAL)
            local healthToLose = objc.divide_with_ceiling(maxHealth, durationProp)
            objc.inflict_direct_damage(entity, entity, healthToLose)

            local currentHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
            local healthProp = objc.divide_with_ceiling(currentHealth * 5, maxHealth)
            local stageUpdated = objc.get_local_value(entity, STAGE_UPDATED_KEY)

            if healthProp > 4 and stageUpdated < 1 then
                local updatedAnimations = {}
                updatedAnimations[ANIMATION_STATE_IDLE] = 'idleStage1'
                updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = 'fireStage1'
                updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = 'fireStage1'
                objc.update_animation_state(entity, updatedAnimations)
                objc.set_local_value(entity, STAGE_UPDATED_KEY, 1)
            elseif healthProp < 4 and healthProp > 2 and stageUpdated < 2 then
                local updatedAnimations = {}
                updatedAnimations[ANIMATION_STATE_IDLE] = 'idleStage2'
                updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = 'fireStage3'
                updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = 'fireStage3'
                objc.update_animation_state(entity, updatedAnimations)
                objc.set_local_value(entity, STAGE_UPDATED_KEY, 2)
            elseif healthProp < 2 and stageUpdated < 3 then
                local updatedAnimations = {}
                updatedAnimations[ANIMATION_STATE_IDLE] = 'idleStage3'
                updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = 'fireStage3'
                updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = 'fireStage3'
                objc.update_animation_state(entity, updatedAnimations)
                objc.set_local_value(entity, STAGE_UPDATED_KEY, 3)
            end
        end
    end
end

R_ValsTurret_reachedOvertimeTrigger = function(entity)
    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'UnitAbilities.lua', 'UnitAbility_tickDownHealthTrigger')
end
