#define SUPPORT_GLSL300

#include "prefix.sh"
#include "texture_utility.sh"
#include "SimpleDoF.sh"

uniform highp sampler2D s_cc3Texture2Ds[3];

VARYING_IN vec2 v_texCoord[1];

vec3 sampleHighRes(vec2 offset) {
    return sampleLevelZero(s_cc3Texture2Ds[0], v_texCoord[0] + offset).rgb;
}

float sampleDepthHighRes(vec2 offset) {
    return sampleLevelZero(s_cc3Texture2Ds[1], v_texCoord[0] + offset).x;
}

vec4 sampleLowRes(vec2 offset) {
    return sampleLevelZero(s_cc3Texture2Ds[2], v_texCoord[0] + offset);
}

vec2 ReadDOFSamplingPattern(int idx) {
    return texelFetch(DOFSamplingPattern, ivec2(idx, 0), 0).xy;
}

int StipplePattern(ivec2 pixelCoords)
{
#if __VERSION__ >= 300
    const int ditherArray[] = int[](
        4, 12,  0,  8,
        10,  2, 14,  6,
        15,  7, 11,  3,
        1,  9,  5, 13
    );
    ivec2 t = pixelCoords.xy & 0x3;
    return ditherArray[t.x+t.y*4];
#else
    return 0;
#endif
}

void main() {
    float centerDepthSample = sampleDepthHighRes(vec2(0.0, 0.0));
    // gl_FragColor.rgb = vec3(depthValueToCircleOfConfusion(centerDepthSample));
    // gl_FragColor.a = 1.0;
    // return;

    vec4 centerTap = vec4(
        sampleHighRes(vec2(0.0, 0.0)),
        depthValueToCircleOfConfusion(centerDepthSample));
    float centerCoc = centerTap.a;
    float cocRadius = centerCoc * DOFUniforms._dofParams.z;

    // Use a 2x2 pattern for jittering the sample pattern. This introduces a little
    // grainness, but can help reduce the number of taps required per sample.
    // bool xJitter = fract(gl_FragCoord.x * 0.5) < 0.5;
    // bool yJitter = fract(gl_FragCoord.y * 0.5) < 0.5;
    ivec2 t = ivec2(gl_FragCoord.xy) & 0x3;
    int stipple = t.y * 4 + t.x;

    // jumble up 4x4 blocks a little bit... (this seems to reduce patterns in the stippling quite a bit)
    int a = StipplePattern(ivec2(gl_FragCoord.xy)/4);
    stipple = (stipple + a) % 16;

    const int tapsPerPixel = 4;

    vec4 result = vec4(centerTap.rgb * centerTap.a, centerTap.a);
    for (int c=0; c<tapsPerPixel; ++c) {
        vec2 stochasticOffset = ReadDOFSamplingPattern(c + stipple * tapsPerPixel) * cocRadius;
        vec4 tapHigh = vec4(
            sampleHighRes(stochasticOffset).rgb,
            depthValueToCircleOfConfusion(sampleDepthHighRes(stochasticOffset)));
        vec4 tapLow = sampleLowRes(stochasticOffset * 0.25);    // (assuming 16:1 downsample for low res image)

        float biasedA = tapHigh.a;
        biasedA = 1.0 - biasedA;
        biasedA *= biasedA;
        biasedA = 1.0 - biasedA;

        // as per Scheuermann04, mix low and high res taps
        vec4 mixedTap = mix(tapHigh, tapLow, biasedA);

        mixedTap.a = (mixedTap.a >= centerTap.a) ? 1.0 : mixedTap.a;

        result.rgb += mixedTap.rgb * mixedTap.a;
        result.a += mixedTap.a;
    }

    gl_FragColor.rgb = result.rgb / result.a;
    gl_FragColor.a = centerTap.a;    // write out an approximation of the effect that depth of field has had on this pixel
}


