#define SUPPORT_GLSL300

#include "prefix.sh"
#include "texture_utility.sh"
#include "SimpleDoF.sh"

uniform highp sampler2D s_cc3Texture2Ds[2];

VARYING_IN vec2 v_texCoord[1];

const vec2 halfPixelOffset = vec2(-2.0, -2.0); // vec2(0.5, 0.5);

vec3 sampleColor(vec2 offset) {
    return sampleLevelZero(s_cc3Texture2Ds[0], v_texCoord[0] + (offset + halfPixelOffset) * DOFUniforms._reciprocalInputSize.xy).rgb;
}

float sampleDepth(vec2 offset) {
    return sampleLevelZero(s_cc3Texture2Ds[1], v_texCoord[0] + (offset + halfPixelOffset) * DOFUniforms._reciprocalInputSize.xy).x;
}

void main() {

    // See reference from Scheuermann05 (https://developer.amd.com/wordpress/media/2012/10/Scheuermann_DepthOfField.pdf)
    // and Infinity Ward (http://http.developer.nvidia.com/GPUGems3/gpugems3_ch28.html)
    //
    // We want to build a pre-blurred version of the image with a "blurriness" value written into the alpha
    // channel of the texture.

    // First, take 4 taps of the color texture with bilinear turned on (to find the average of 16 pixels)
    vec3 colorPart =
          sampleColor(vec2(1.0, 1.0))
        + sampleColor(vec2(3.0, 3.0))
        + sampleColor(vec2(1.0, 3.0))
        + sampleColor(vec2(3.0, 3.0))
        ;
    colorPart /= 4.0;

    // Second, take 16 taps of the depth texture (with bilinear off), and for each sample calculate the
    // appropriate circle of confusion.
    // If we can assume that the circle of confusion only gets bigger as depthValue increases, we can
    // do that calculation once on the max depth.
    float maxcoc = 0.0, maxDepth = 0.0;
    for (int y=0; y<4; ++y) {
        for (int x=0; x<4; ++x) {
            float depthValue = sampleDepth(vec2(float(x), float(y)));
            maxDepth = max(maxDepth, depthValue);
        }
    }
    maxcoc = depthValueToCircleOfConfusion(maxDepth);

    gl_FragColor = vec4(colorPart.rgb, maxcoc);
}
