--[[
To use this template:
1. Copy+paste the file, and rename it to your units name. (e.g. B_Medivac.lua)
2. Rename the functions below to your unit's name (e.g. Faction_Name_damageTrigger -> B_Medivac_damageTrigger)
3. Uncomment the functions you want to use.
--]]


--[[
damageTrigger:

This is invoked when the sourceEntity is damaging the targetEntity.
If a unit has multiple damageTriggers, the highest value is used.
Note: The sourceEntity is typically a bullet

- damage: damage to be done
- targetEntity: entity to be damaged
- sourceEntity: source of the damage
- Returns: the amount of damage to be done
--]]
-- Faction_Name_damageTrigger = function(damage, targetEntity, sourceEntity)
--    return damage
-- end


--[[
onDamageTrigger:

This is invoked when the targetEntity is being damaged by the sourceEntity.
If a unit has multiple onDamageTriggers, the lowest value is used.
Note: The sourceEntity is typically a bullet

- damage damage to be done
- targetEntity: entity to be damaged
- sourceEntity: source of the damage
- Returns: the amount of damage to be done
--]]
-- Faction_Name_onDamageTrigger = function(damage, targetEntity, sourceEntity)
--    return damage
-- end


--[[
tickTrigger:

Note: This function triggers every game round.  There are 10 game rounds per second

- entity: the entity that is ticking
--]]
-- Faction_Name_tickTrigger = function(entity)
-- end


--[[
spawnTrigger:

Note: This triggers when a unit is spawned by a unit factory but NOT when it is spawned from a function

- entity: the entity that was just spawned
--]]
-- Faction_Name_spawnTrigger = function(entity)
-- end


--[[
deathTrigger:

- entity: the entity that just died
--]]
-- Faction_Name_deathTrigger = function(entity)
-- end


--[[
canTargetTrigger:

This method is called to determine if a unit should target another unit.
Use case: Use this with the Medivac to have it filter out units with full health it shouldn't try to target.

If a unit has multiple canTargetTriggers, if any of the canTargetTriggers return NO, this unit will not
target said unit.

NOTE: The only logic in this method should be related to determing if this unit can target the other unit.
This trigger is used for both determing which unit to move towards, as well as which unit to attack.

- entity: The entity that is targeting
- targetEntity: The entity that is being check it if should be targeted
- Returns: YES if it should target the entity. NO otherwise.
--]]
-- Faction_Name_canTargetTrigger = function(entity, targetEntity)
--    return YES
-- end


--[[
checkTargetScoreTrigger:

This method is called on all units that this unit can target. The unit with the highest checkTargetScoreTrigger score will be targeted.
If a unit has multiple checkTargetScoreTriggers, the highest value returned is used.

NOTE: The only logic in this method should be related to determing if the targeting score for the target unit.
This trigger is used for both determing which unit to move towards, as well as which unit to attack.

- entity: The entity that is targeting
- entityToTarget: The entity that is being checked to see if it should be targeted
- Returns: the score for the entity being checked, where the unit with the highest score is targeted.
--]]
-- Faction_Name_checkTargetScoreTrigger = function(entity, entityToTarget)
--    local targetScore = 0
--    return targetScore
-- end


--[[
onTargetTrigger:

Note: This triggers when a unit is evaluating targets to attack.  This generally happens as soon as there is at least one valid target in range or the previous target is dead or missing.
This trigger is used only when targeting a unit to attack.

- entity: The entity that is targeting
- entityToTarget: The entity that is being checked to see if it should be targeted
--]]
-- Faction_Name_onTargetTrigger = function(entity, entityToTarget)
-- end


--[[
onMovementTrigger:

Note: This triggers when a unit begins moving

- entity: the entity that owns the script and is moving
--]]
-- Faction_Name_onMovementTrigger = function(entity)
-- end


--[[
onOverrideTargetingCompletionTrigger:

Note: This trigger happens after calling move_entity_to_entity_with_motion and the entity gets close enough to deal damage.

- sourceEntity: the entity that was moved
- targetEntity: the target entity
--]]
-- Faction_Name_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
-- end


--[[
actionSystemLoopStartTrigger:

Note: This trigger happens when the unit is about to start an attack loop (wind up, attack until killed unit, wind down) on a unit it has targeted.

- sourceEntity: the entity that is about to start an attack loop
- targetEntity: the entity that is about to get attacked
--]]
-- Faction_Name_actionSystemLoopStartTrigger = function(sourceEntity, targetEntity)
-- end


--[[
actionSystemLoopEndTrigger:

Note: This trigger happens when the unit hass finished an attack loop (wind up, attack until killed unit, wind down) on a unit it has targeted.

- entity: the entity that finished an attack loop
--]]
-- Faction_Name_actionSystemLoopEndTrigger = function(entity)
-- end


--[[
onAttackTrigger:

Note: This triggers as a unit begins its attack

- sourceEntity: the entity that performs the attack
- targetEntity: the entity that is being targeted by the attack
--]]
-- Faction_Name_onAttackTrigger = function(sourceEntity, targetEntity)
-- end


--[[
onKillTrigger:

Note: We currently do not support adding additional on kill triggers to a unit.
Note: This triggers when a unit kills an entity

- killedEntity: the entity that got killed
- killerEntity: the entity that killed the killedEntity
--]]
-- Faction_Name_allyOnKillTrigger = function(killedEntity, killerEntity)
-- end
