local HAS_INIT = 'hasInit'

local CHECK_TICK = 10
local CHECK_KEY = 'checkKey'
local UNIT_HAS_SPAWNED = 'unitHasSpawned'
local IDLE_TIMER_KEY = "idleTimerKey"
local HAS_ADDED_TRIGGER = 'hasAddedTrigger'
local IDLE_TIME = 20000000

local CAMPAIGN_WEAKER_EXPLOSION_KEY = "campaignWeakerExplosionKey"

local UNIT_LUA = 'W1CampaignAliciaWeak.lua'

local SEND_BOMBS_DELAY = 100
local MOVE_TIMER_KEY = 'moveTimerKey'
local ADJUST_TIMER_KEY = 'adjustTimerKey'
local SHOULD_ADJUST_KEY = 'shouldAdjustKey'
local STOP_TIMER_KEY = "stopTimerKey"
local SHOULD_STOP_KEY = 'shouldStopKey'
local BOMBS_TIMER_KEY = 'bombsTimerKey'
local SHOULD_SEND_BOMB_KEY = 'shouldSendBombKey'

local PREV_LOCATION_KEY = 'prevLocationKey'

W1CampaignAliciaWeak_tickTrigger = function(entity)
    local HasInit = objc.get_local_value(entity, HAS_INIT)
    if HasInit == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_SniperRoadster6' then
                objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'Alicia_IdleTrigger')
                objc.add_trigger(unit, TRIGGER_NAME_DAMAGE_TRIGGER, UNIT_LUA, 'Alicia_WeakerAttackTrigger')
                R_SniperRoadster_removeBuff(unit)
                objc.set_entity_scale_percentage(unit, 115)
                objc.set_entity_display_name(unit, 'Alicia')
                objc.set_entity_unit_description(unit, 'The boss roadster')
                objc.set_local_value(entity, HAS_INIT, 1)
            end
        end
    end
end

Alicia_WeakerAttackTrigger = function (damage, targetEntity, sourceEntity)
    return objc.divide_with_ceiling(damage, 2)
end

Alicia_IdleTrigger = function(entity)
    local hasSpawned = objc.get_local_value(entity, UNIT_HAS_SPAWNED)
    local idleTimer = objc.tick_timer(entity, IDLE_TIMER_KEY, 5)
    if idleTimer == 0 and hasSpawned == 0 then
        objc.idle_entity(entity, IDLE_TIME)
        objc.set_local_value(entity, UNIT_HAS_SPAWNED, 1)
    end

    local hasAddedTrigger = objc.get_local_value(entity, HAS_ADDED_TRIGGER)
    local checkTimer = objc.tick_timer(entity, CHECK_KEY, CHECK_TICK)
    if objc.is_entity_idle(entity) and hasAddedTrigger == 0 and checkTimer == 0 then
        local barricadeDestroyed = true
        local team2Entities = objc.list_entities(2)
        for _, enemy in pairs(team2Entities) do
            if objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'R_OneLaneBarricade' then
                barricadeDestroyed = false
            end
        end
        if barricadeDestroyed == true then
            objc.set_local_value(entity, HAS_ADDED_TRIGGER, 1)
            local thisPos = objc.get_entity_position(entity)
            objc.spawn_at_point('fireArrowsTargetedScriptEntity', 1, 2, thisPos[1], thisPos[2], 0, 0)
            objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'Alicia_ActivatePart1Trigger')
        end
    end
end

Alicia_ActivatePart1Trigger = function(entity)
    local bombsTimer = objc.tick_timer(entity, BOMBS_TIMER_KEY, SEND_BOMBS_DELAY)
    if bombsTimer == 0 then
        local thisPos = objc.get_entity_position(entity)
        objc.spawn_at_point('fireArrowsTargetedScriptEntity', 1, 2, thisPos[1], thisPos[2], 0, 0)
    end

    local team2Entities = objc.list_entities(2)
    for _,unit in pairs(team2Entities) do
        if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'roxiAbility3EffectEntityPlayer2' then
            if objc.get_local_value(unit, CAMPAIGN_WEAKER_EXPLOSION_KEY) == 0 then
                objc.set_local_value(unit, CAMPAIGN_WEAKER_EXPLOSION_KEY, 1)
            end
        end
    end

    if objc.is_entity_idle(entity) then
        objc.unidle_entity(entity)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'Alicia_IdleTrigger')
    end
end