local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local BUFF_VFX = 'C_Ai_Buff.lvl'

local ALLY_CHECK_INTERVAL = 2
--local PULSE_VFX = 'B_tactic_rallyPoint_Aura.lvl'

local SPAWN_ANIMATION_KEY = 'spawnAnimationName'
local NUMBER_OF_FRAMES_FOR_SPAWN_ANIM = 30

local HAS_BEEN_RALLY_POINTED = 'hasBeenRallyPointedKey'

local DAMAGE_BUFF = 50

W1CampaignBuffBeacon_spawnTrigger = function(entity)
    objc.set_targetable(entity, true)

    --objc.play_vfx(entity, PULSE_VFX, NODE_ORIGIN)
    objc.idle_entity_indefinitely(entity)

    local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_FOR_SPAWN_ANIM)
    objc.play_override_animation_key(entity, SPAWN_ANIMATION_KEY, numberOfSubrounds)
end

W1CampaignBuffBeacon_tickTrigger = function(entity)
    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_INTERVAL)
    if allyCheckTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
        if buffedUnits == 0 then
            buffedUnits = {}
        end

        for _, buffedUnit in pairs(buffedUnits) do
            W1CampaignBuffBeacon_removeBuff(entity, buffedUnit)
        end

        buffedUnits = {}
        --local buff_radius = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_SPELL_RADIUS)
        local buff_radius = 9000
        local allies = objc.list_entities_in_radius(entity, buff_radius, teamNumber)
        for _, ally in pairs(allies) do
            local hasBeenBuffed = objc.get_local_value(ally, HAS_BEEN_RALLY_POINTED)
            if objc.is_normal_unit(ally) and hasBeenBuffed == 0 and ally ~= entity then
                W1CampaignBuffBeacon_buffUnit(entity, ally)
                table.insert(buffedUnits, ally)
            end
        end

        objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
    end
end

W1CampaignBuffBeacon_canTargetTrigger = function(entity, targetEntity)
    return NO
end

W1CampaignBuffBeacon_reachedOvertimeTrigger = function(entity)
    objc.kill_entity(entity)
end

W1CampaignBuffBeacon_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits ~= 0 then
        for _, buffedUnit in pairs(buffedUnits) do
            W1CampaignBuffBeacon_removeBuff(entity, buffedUnit)
        end
    end
end

W1CampaignBuffBeacon_buffUnit = function(entity, targetEntity)
    local damageBuff = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    --local buffToGive = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_AURA_DAMAGE_BONUS)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF, damageBuff + DAMAGE_BUFF)
    objc.play_vfx(targetEntity, BUFF_VFX, NODE_STATUS)

    objc.set_local_value(targetEntity, HAS_BEEN_RALLY_POINTED, 1)
end

W1CampaignBuffBeacon_removeBuff = function(entity, targetEntity)
    local damageBuff = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
    --local buffToGive = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_AURA_DAMAGE_BONUS)
    objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_DAMAGE_BUFF, damageBuff - DAMAGE_BUFF)
    objc.stop_vfx(targetEntity, BUFF_VFX, NODE_STATUS, false)

    objc.set_local_value(targetEntity, HAS_BEEN_RALLY_POINTED, 0)
end
