local GAME_HAS_INIT = 'gameHasInit'
local BOSS_INIT = 'bossInit'

local BUFF_BEACON_INIT = 'buffBeaconInit'
local BUFF_BEACON_COUNT_KEY = 'buffBeaconCountKey'
local BUFF_BEACON_HEALTH_INCREASE = 200

local BUFF_BEACON_DESTROYED_KEY = 'BuffBeaconDestroyed'

local LUA_FILE = 'W1CampaignReaper.lua'

W1CampaignReaper_tickTrigger = function(entity)
    local HasInit = objc.get_local_value(entity, GAME_HAS_INIT)
    if HasInit == 0 then
        local base = objc.get_base_entity(2)
        objc.set_targetable(base, false)
        objc.add_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, LUA_FILE, 'base_invulnerable')
        objc.set_local_value(entity, GAME_HAS_INIT, 1)
    end

    local bossInit = objc.get_local_value(entity, BOSS_INIT)
    if bossInit == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'B_AnnieWalkerCampaign' then
                objc.set_entity_scale_percentage(unit, 140)
                objc.set_entity_facing_direction(unit, -1, 0)
                objc.set_flow_field_enabled_for_entity(unit, false)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
                local newMaxHP = 4000
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_HEALTH, newMaxHP)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_CURRENT_HEALTH, newMaxHP)

                local damageDebuff = objc.divide_with_ceiling(2 * objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_ATTACK_DAMAGE), 3)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_DAMAGE_BUFF, -1 * damageDebuff)

                objc.add_trigger(unit, TRIGGER_NAME_ON_DAMAGE_TRIGGER, LUA_FILE, 'W1CampaignReaper_spellResistant')

                objc.set_local_value(entity, BOSS_INIT, 1)
                break
            end
        end
    end

    local checkTimer = objc.tick_timer(entity, 'checkTimer', 5)
    if checkTimer == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'W1CampaignBuffBeacon' then
                if objc.get_local_value(unit, BUFF_BEACON_INIT) == 0 then
                    -- Each subsequent beacon gets more and more health
                    local beaconCount = objc.get_local_value(entity, BUFF_BEACON_COUNT_KEY)
                    local prevMaxHealth = objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_CURRENT_HEALTH)
                    local healthBoost = beaconCount * BUFF_BEACON_HEALTH_INCREASE
                    local newMaxHealth = prevMaxHealth + healthBoost
                    objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_HEALTH, newMaxHealth)
                    objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_CURRENT_HEALTH, newMaxHealth)

                    objc.set_local_value(unit, BUFF_BEACON_INIT, 1)
                    objc.set_local_value(entity, BUFF_BEACON_COUNT_KEY, beaconCount + 1)
                end
                break
            end
        end
    end
end

-- OnDamageTrigger
W1CampaignReaper_spellResistant = function(damage, targetEntity, sourceEntity)
    local source = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local sourceIdentifier = objc.get_entity_value_for_key(source, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
    if sourceIdentifier == 'W_Blast' or sourceIdentifier == 'W_Fog' then
        return objc.divide_with_ceiling(damage, 3)
    end
    return damage
end

base_invulnerable = function(entity)
    return 0
end