local GAME_HAS_INIT = 'gameHasInit'
local UNIT_HAS_SPAWMED = 'unitHasSpawned'

local CHECK_TICK = 3
local CHECK_RADIUS = 900
local CHECK_RADIUS2 = 900
local CHECK_KEY = "checkKey"
local IDLE_TIME = 20000000

local NEGATE_LIMIT = 99999
local MECH_ARRIVED = 'mechHasArrived'

local UNIT_LUA = 'W2CampaignFortressMech.lua'


W2CampaignFortressMech_tickTrigger = function(entity)
    local HasInit = objc.get_local_value(entity, GAME_HAS_INIT)
    if HasInit == 0 then
        local team1Entities = objc.list_entities(1)
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team1Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'B_FortressMech4' then
                objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'W2CampaignFortressMech_IdleTrigger')
                objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'W2CampaignFortressMech_ActivateTrigger')
            end
        end
        for _,unit in pairs(team2Entities) do
            if objc.is_barricade(unit) then
                objc.add_trigger(unit, TRIGGER_NAME_ON_DAMAGE_TRIGGER, UNIT_LUA, 'barricade_invincibleTrigger')
            end
        end
        objc.set_local_value(entity, GAME_HAS_INIT, 1)
    end
end

W2CampaignFortressMech_IdleTrigger = function(entity)
    local hasSpawned = objc.get_local_value(entity, UNIT_HAS_SPAWMED)
    if hasSpawned == 0 then
        objc.idle_entity(entity, IDLE_TIME)
        objc.set_local_value(entity, UNIT_HAS_SPAWMED, 1)
    end
end

W2CampaignFortressMech_ActivateTrigger = function(entity)
    local checkTimer = objc.tick_timer(entity, CHECK_KEY, CHECK_TICK)
    if checkTimer == 0 then
        local teamNumber = objc.get_team_number(entity)
        local enemyTeamNumber = objc.get_enemy_team_number(entity)

        local allies = objc.list_entities_in_radius(entity, CHECK_RADIUS, teamNumber)
        local numAllies = 0
        for _, ally in pairs(allies) do
            if objc.is_normal_unit(ally) or objc.is_commander(ally) then
                numAllies = numAllies + 1
            end
        end

        local enemies = objc.list_entities_in_radius(entity, CHECK_RADIUS, enemyTeamNumber)
        local numEnemies = 0
        for _, enemy in pairs(enemies) do
            if objc.is_normal_unit(enemy) or objc.is_commander(enemy) then
                numEnemies = numEnemies + 1
            end
        end

        if numEnemies == 0 and numAllies > 0 then
            objc.unidle_entity(entity)
        else objc.idle_entity(entity, IDLE_TIME)
        end
    end
end

barricade_invincibleTrigger = function(damage, targetEntity, sourceEntity)
    local checkTimer = objc.tick_timer(targetEntity, CHECK_KEY, CHECK_TICK)
    if checkTimer == 0 then
        local enemyTeamNumber = objc.get_enemy_team_number(targetEntity)
        local enemies = objc.list_entities_in_radius(targetEntity, CHECK_RADIUS2, enemyTeamNumber)
        for _, enemy in pairs(enemies) do
            if objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IDENTIFIER) == 'B_FortressMech4'  then
                return damage
            else
                return UnitAbility_negate(targetEntity, damage, NEGATE_LIMIT)
            end
        end
    end
end

