local BLINK_DISTANCE = 2700
local BLINK_SPEED = 550
local BLINK_ANIMATION_NAME = 'W_HummingBird_Battle_Knockback'
local FRAMES_IN_BLINK_ANIMATION_NAME = 45
local IS_BLINKING = 'isBlinkingKey'
local KNOCKBACK_TARGET_ENTITY = 'cannonScooterTargetEntity'
local MOTION_STATE_DICTIONARY = 'motionStateDictKey'
local ORIGINAL_RANGE_KEY = 'originalRangeKey'

W_BackBlinker_spawnTrigger = function(entity)
    local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)
end

W_BackBlinker_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local isBlinking = objc.get_local_value(targetEntity, IS_BLINKING)
    if isBlinking == 0 then
        local thisX = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_X)
        local thisY = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_Y)
        local teamNumber = objc.get_team_number(targetEntity)

        local xOffset = BLINK_DISTANCE
        if teamNumber == 1 then
            xOffset = -xOffset
        end

        local basePos = objc.get_base_position(teamNumber)
        local newXPos = thisX + xOffset

        if teamNumber == 1 then
            if newXPos < basePos[1] then
                newXPos = basePos[1]
            end
        else
            if newXPos > basePos[1] then
                newXPos = basePos[1]
            end
        end

        local knockbackEntity = objc.spawn_at_point(KNOCKBACK_TARGET_ENTITY, 1, teamNumber, thisX + xOffset, thisY, 0, 0)            
        objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, 100)
        objc.set_entity_should_reverse_facing_direction(targetEntity, true)
        local motionDict = objc.move_entity_to_entity_with_motion(targetEntity, knockbackEntity[1], MOTION_STATE_WALK, BLINK_SPEED)
        objc.set_local_value(targetEntity, MOTION_STATE_DICTIONARY, motionDict)
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_BLINK_ANIMATION_NAME)
        objc.play_override_animation(targetEntity, BLINK_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
        objc.set_local_value(targetEntity, IS_BLINKING, 1)
    end

    return damage
end

W_BackBlinker_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local isBlinking = objc.get_local_value(sourceEntity, IS_BLINKING)
    if isBlinking ~= 0 then
        local motionDict = objc.get_local_value(sourceEntity, MOTION_STATE_DICTIONARY)
        local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
        objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, motionDict)
        objc.set_entity_should_reverse_facing_direction(sourceEntity, false)
        objc.set_local_value(sourceEntity, IS_BLINKING, 0)
    end
end
