--keys
local CHANNELING_TICKS = 'numChannelingTicks'
local OLD_MAX_VELOCITY = 'oldMaxVelKey'
local PROGRESS_BAR = 'progressBarKey'
local SHOULD_CHANNEL = 'shouldBeCheckingKey'
local VFX_PLAYING = 'playVfxKey'
local HAS_INIT_CHANNEL = 'hasInitChannelActivity'
local DRAGON_SUMMON_SPAWN = 'shouldSummonSpawn'
local DRAGON_IDENTIFIER = 'W_Dragon'

--Constants
local CHANNEL_ROUNDS = 120
local DRAGON_SPAWN_OFFSET = 500

--Animation Values
local CASTING_ANIMATION = 'W_Cultist_Attack_01'
local CASTING_ANIMATION_2 = 'W_Cultist_Attack_02'
local CASTING_FRAMES = 350

W_Cultist_tickTrigger = function(entity)
   local shouldChannel = objc.get_local_value(entity, SHOULD_CHANNEL)
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)

   --If I should not already be channeling, then check my position to
   --see if I should
   if shouldChannel == 0 then
      --If I'm past the midpoint, I should be channeling
      local position = objc.get_entity_position(entity)
      if teamNumber == 1 then
         if position[1] > 0 then
            objc.set_local_value(entity, SHOULD_CHANNEL, 1)
         end
      else
         if position[1] < 0 then
            objc.set_local_value(entity, SHOULD_CHANNEL, 1)
         end
      end
   else
      --Set up the necessary things to channel
      local hasInitChannel = objc.get_local_value(entity, HAS_INIT_CHANNEL)
      if hasInitChannel == 0 then
         --Stop myself and play the summoning animation
         local numberOfSubrounds = objc.convert_frames_to_subrounds(CASTING_FRAMES)
         objc.idle_entity(entity, numberOfSubrounds)
         local castAnim = CASTING_ANIMATION
         if teamNumber == 2 then
            castAnim = CASTING_ANIMATION_2
         end
         objc.play_override_animation(entity, castAnim, objc.subrounds_to_milliseconds(numberOfSubrounds))

         objc.set_local_value(entity, HAS_INIT_CHANNEL, 1)
      end

      --Tick down my channeling time
      local channelTimer = objc.tick_timer(entity, CHANNELING_TICKS, CHANNEL_ROUNDS)
      if channelTimer == 0 then
         --Once I've channeled, summon a dragon slightly offset from myself
         --so it looks nice
         local thisPos = objc.get_entity_position(entity)
         local direction = 1
         if teamNumber == 2 then
            direction = -1
         end
         local offset = DRAGON_SPAWN_OFFSET * direction

         local tier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
         if tier == 0 then
            tier = ''
         end
         local dragonArray = objc.spawn_at_point(DRAGON_IDENTIFIER .. tier, 1, playerNumber, thisPos[1] + offset, thisPos[2], direction, 0)

         --Make sure the dragon uses the 'summoned' spawn animation, so it doesn't conflict with the VFX
         objc.set_local_value(dragonArray[1], DRAGON_SUMMON_SPAWN, 1)

         --Sacrifice myself to spawn the dragon
         objc.sacrifice_entity(entity)
      end
   end
end
