local ATTACK_COUNTER_KEY = 'attackCounterKey'
local AOE_LINE_LENGTH = 5500
local AOE_LINE_WIDTH = 800
local BATTLEFIELD_LENGTH = 10000
local BATTLEFIELD_WIDTH = 4000
local AOE_DAMAGE_COEFFICIENT = 0.1
local SLOW_AMOUNT = 1 --as a percent, 1 = 100%
local SLOW_DURATION = 50
local IS_VFX_ACTIVE_KEY = 'isVFXActive'

--[[
This attack trigger sets a unique value per attack and assigns it to the target so we can track who the primary
target for each attack is and only apply the visual effect when that target is damaged (or else everything in the splash area
would spawn an effect)
]]--
W_DragonSpawner_onAttackTrigger = function(sourceEntity, targetEntity)
	local primaryTargetKey = objc.get_entity_value(sourceEntity, 'identityComponent.description')
	local attackCounter = objc.get_local_value(sourceEntity, ATTACK_COUNTER_KEY)

	attackCounter = attackCounter + 1

	objc.set_local_value(targetEntity, primaryTargetKey, attackCounter)
	objc.set_local_value(sourceEntity, ATTACK_COUNTER_KEY, attackCounter)	

	objc.play_vfx(sourceEntity, 'head_W_Dragon.lvl', 'node_head')
	objc.set_local_value(sourceEntity, IS_VFX_ACTIVE_KEY, 1)
end

W_DragonSpawner_tickTrigger = function(entity)
local isVFXActive = objc.get_local_value(entity, IS_VFX_ACTIVE_KEY)

	if isVFXActive == 1 then
		local killVFXTimer = objc.tick_timer(entity, 'killVFXTimer', 15) -- THIS TIMER HAS TO BE SHORTER THAN THE ATTACK COOLDOWN OR ELSE VFX WILL PILE UP
		if killVFXTimer == 0 then
			objc.stop_vfx(entity, 'head_W_Dragon.lvl', 'node_head', false)
			objc.set_local_value(entity, IS_VFX_ACTIVE_KEY, 0)
		end
	end


end

--[[
This DamageTrigger gives W_Dragon the Slow UnitAbility.
--]]
W_DragonSpawner_damageTrigger = function(damage, targetEntity, sourceEntity)
	local dragon = objc.get_entity_value(sourceEntity, 'identityComponent.sourceEntity')
	local primaryTargetKey = objc.get_entity_value(dragon, 'identityComponent.description')
	local attackCounter = objc.get_local_value(dragon, ATTACK_COUNTER_KEY)
	local isCurrentPrimaryTargetValueCheck = objc.get_local_value(targetEntity, primaryTargetKey)


	if attackCounter == isCurrentPrimaryTargetValueCheck then
		local playerNumber = objc.get_player_number(dragon)
		local enemyNumber = objc.get_team_number(targetEntity)
		local targetPosition = objc.get_entity_position(targetEntity)
		local dragonPosition = objc.get_entity_position(dragon)
		local enemyList = objc.list_entities(enemyNumber)
		local directionX = 0

		if objc.get_local_value(enemy, IS_DEBUFF_IMMUNE_KEY) == 0 then
			objc.apply_buff(dragon, 'immobilize', SLOW_DURATION, targetEntity)
		end

		if dragonPosition[1] <= targetPosition[1] then
			directionX = 1
		else
			directionX = -1
		end

		for _, enemy in pairs(enemyList) do
			local enemyPosition = objc.get_entity_position(enemy)
			local distanceFromTargetX = (enemyPosition[1] + BATTLEFIELD_LENGTH) - (targetPosition[1] + BATTLEFIELD_LENGTH)
			distanceFromTargetX = distanceFromTargetX * directionX
			local distanceFromTargetY = math.abs((enemyPosition[2] + BATTLEFIELD_WIDTH) - (targetPosition[2] + BATTLEFIELD_WIDTH))

			if distanceFromTargetX > 0 and distanceFromTargetX < AOE_LINE_LENGTH and distanceFromTargetY < objc.divide_with_floor(AOE_LINE_WIDTH, 2) and enemy ~= targetEntity and objc.is_ground_unit(enemy) and math.abs(enemyPosition[1]) < 9700 then
				--objc.set_combat_text(enemy, '!!!')
				objc.inflict_normal_damage(sourceEntity, enemy, math.ceil(damage * AOE_DAMAGE_COEFFICIENT))
				objc.flash_entity_for_damage(enemy, 250)
				if objc.get_local_value(enemy, IS_DEBUFF_IMMUNE_KEY) == 0 then
				objc.apply_buff(dragon, 'immobilize', SLOW_DURATION, enemy)
				end
			end
		end

		objc.spawn_at_point('W_DragonTrailEffect', 1, playerNumber, targetPosition[1], targetPosition[2], directionX, 0)


		--local endPoint = targetPosition[1]
		--endPoint = endPoint + 5000
		--objc.spawn_at_point('W_Swordsman_TeleportEffect', 1, playerNumber, endPoint, targetPosition[2], directionX, 0)
	end
	return damage
end

W_DragonSpawner_immobilize = function(entity, component)
	objc.buff_property(entity, component, 'velocityComponent.primaryMaxVelocity', nil, -100, 1)
end
