local ENEMY_CHECK_INTERVAL = 2
local ENEMY_CHECK_TIMER_KEY = 'enemyCheckTimerKey'
local ENEMY_CHECK_RADIUS = 5000

local PREVIOUS_ENEMIES_KEY = 'previousEnemiesKey'

local CRYSTAL_GAIN = 1
local GAIN_EFFECT = 'crystalGainEffect.lvl'

local IDLE_ANIMATION_KEY = 'idleAnimation'
local IDLE_ANIMATION_NAME = 'W_Harvester_Idle_01'
local FRAMES_IN_IDLE_ANIMATION = 30
local BASE_ATTACK_ANIMATION_NAME = 'baseAttackAnimation'
local BASE_ATTACK_ANIMATION_START_NAME = 'baseAttackStartAnimation'

local MELEE_RANGE = 550
local ORIGINAL_RANGE_KEY = 'originalRangeKey'

local IS_ATTACKING_KEY = 'isAttackingKey'

local CAST_START_LENGTH = 'castStartLengthKey'
local CAST_START_TIMER_KEY = 'castStartTimerKey'
local LOOP_TIME_KEY = 'loopTimeKey'
local LOOP_START_TIME = 5
local LOOP_TIME_PER_DEATH = 10
local LOOP_TRIGGER_NAME = 'W_Harvester_loopWhileHarvestingTickTrigger'

local SOUL_ENTITY_ID = 'HarvesterSoulEntity'
local SOURCE_HARVESTER_KEY = 'sourceHarvesterKey'

W_Harvester_spawnTrigger = function(entity)
    local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)


    --[[local idleAttackAnimations = {}
    idleAttackAnimations[ANIMATION_STATE_ATTACK_GROUND] = IDLE_ANIMATION_KEY
    idleAttackAnimations[ANIMATION_STATE_ATTACK_GROUND_START] = ''
    idleAttackAnimations[ANIMATION_STATE_ATTACK_AIR] = IDLE_ANIMATION_KEY
    idleAttackAnimations[ANIMATION_STATE_ATTACK_AIR_START] = ''
    objc.update_animation_state(entity, idleAttackAnimations)]]
end

W_Harvester_actionSystemLoopStartTrigger = function(sourceEntity, targetEntity)
    local targetId = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_IDENTIFIER)
    if string.match(targetId, 'Barricade') ~= nil or string.match(targetId, 'base') ~= nil then
        local baseAttackAnimations = {}
        baseAttackAnimations[ANIMATION_STATE_ATTACK_GROUND] = BASE_ATTACK_ANIMATION_NAME
        baseAttackAnimations[ANIMATION_STATE_ATTACK_GROUND_START] = BASE_ATTACK_ANIMATION_START_NAME
        baseAttackAnimations[ANIMATION_STATE_ATTACK_AIR] = BASE_ATTACK_ANIMATION_NAME
        baseAttackAnimations[ANIMATION_STATE_ATTACK_AIR_START] = BASE_ATTACK_ANIMATION_START_NAME
        objc.update_animation_state(sourceEntity, baseAttackAnimations)
    end
end

W_Harvester_tickTrigger = function(entity)
    local enemyCheck = objc.tick_timer(entity, ENEMY_CHECK_TIMER_KEY, ENEMY_CHECK_INTERVAL)
    if enemyCheck == 0 then
        local enemyTeamNumber = objc.get_enemy_team_number(entity)
        local enemies = objc.list_entities_in_radius(entity, ENEMY_CHECK_RADIUS, enemyTeamNumber)
        local trackedEnemies = objc.get_local_value(entity, PREVIOUS_ENEMIES_KEY)

        if trackedEnemies == 0 then
            trackedEnemies = {}
        end

        for _, tracked in pairs(trackedEnemies) do
            objc.set_local_value(tracked, SOURCE_HARVESTER_KEY, 0)
            objc.remove_trigger(tracked, TRIGGER_NAME_DEATH_TRIGGER, 'W_Harvester_incomeOnDeathTrigger')
        end

        trackedEnemies = {}

        for _, enemy in pairs(enemies) do
            local sourceHarvester = objc.get_local_value(enemy, SOURCE_HARVESTER_KEY)
            if sourceHarvester == 0 and objc.is_normal_unit(enemy) then
                objc.set_local_value(enemy, SOURCE_HARVESTER_KEY, entity)
                objc.add_trigger(enemy, TRIGGER_NAME_DEATH_TRIGGER, 'W_Harvester.lua', 'W_Harvester_incomeOnDeathTrigger')
                table.insert(trackedEnemies, enemy)
            end
        end

        objc.set_local_value(entity, PREVIOUS_ENEMIES_KEY, trackedEnemies)
    end
end

W_Harvester_incomeOnDeathTrigger = function(entity)
    local sourceHarvester = objc.get_local_value(entity, SOURCE_HARVESTER_KEY)
    if sourceHarvester ~= 0 then
        W_Harvester_generateIncome(sourceHarvester, entity)
    end
end

--[[
This function gives the user a set amount of currency after a set amount of
ticks.
--]]
W_Harvester_generateIncome = function(entity, oldEnemy)
	local playerNumber = objc.get_player_number(entity)

    local oldEnemyPos = objc.get_entity_position(oldEnemy)
    local soulEntity = objc.spawn_at_point(SOUL_ENTITY_ID, 1, playerNumber, oldEnemyPos[1], oldEnemyPos[2], 0, 0)
    objc.set_local_value(soulEntity[1], SOURCE_HARVESTER_KEY, entity)
    --objc.set_entity_value_for_key(soulEntity[1], ENTITY_VALUE_KEY_HOLD_POSITION_COUNT_FOR_SPAWNING, 0)
    objc.set_entity_value_for_key(soulEntity[1], ENTITY_VALUE_KEY_ATTACK_RANGE, 1)
    
    objc.add_crystal(playerNumber, CRYSTAL_GAIN)

    local currentLoopTime = objc.get_local_value(entity, LOOP_TIME_KEY)
    if currentLoopTime == 0 then
        objc.set_local_value(entity, LOOP_TIME_KEY, LOOP_START_TIME + LOOP_TIME_PER_DEATH)

	    objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Harvester.lua', LOOP_TRIGGER_NAME)
    else
        objc.set_local_value(entity, LOOP_TIME_KEY, currentLoopTime + LOOP_TIME_PER_DEATH)
    end
end

W_Harvester_loopWhileHarvestingTickTrigger = function(entity)
    local loopTime = objc.get_local_value(entity, LOOP_TIME_KEY)
    if loopTime == -1 then
        objc.set_local_value(entity, LOOP_TIME_KEY, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, LOOP_TRIGGER_NAME)
    else
        local initialCastTime = objc.get_local_value(entity, CAST_START_LENGTH)
        local initalCastTickTimer = objc.tick_timer(entity, CAST_START_TIMER_KEY, initialCastTime)
        if initalCastTickTimer == 0 then
            local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_IDLE_ANIMATION)
            local numberOfRounds = objc.convert_frames_to_rounds(FRAMES_IN_IDLE_ANIMATION)
            objc.play_override_animation(entity, IDLE_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
            objc.idle_entity(entity, numberOfSubrounds)
            objc.set_local_value(entity, CAST_START_LENGTH, numberOfRounds)
        end
        objc.set_local_value(entity, LOOP_TIME_KEY, loopTime - 1)
    end
end
