local ATTACK_SPEED_DIVISOR = 2
local MOVE_SPEED_BONUS = 50
local HAS_BEED_ENRAGED_KEY = 'hasBeenProtectedKey'
local BUFFED_ALLIES_ARRAY = 'buffedAlliesArrayKey'
local ORIGINAL_ATTACK_SPEED = 'originalAttackSpeedKey'
local BUFF_VFX = 'HealingPlus.lvl'
local HEAL_DURATION = 50
local HEAL_RADIUS = 'healRadiusKey'
local HEAL_AMOUNT = 'healAmountKey'

local ALLY_CHECK_INTERVAL = 1

local ARMOR_AMOUNT = 99999

local IDLE_ANIM_NAME = 'W_Heal_Activate_01'
local FRAMES_IN_IDLE_ANIM_NAME = 150

--Only used by campaign scripts to set Rage's duration to infinite
local CAMPAIGN_INFINITE_DURATION_KEY = 'campaignInfiniteDurationKey'

local HAS_INIT = 'hasInitKey'
local HAS_VFX_ADDED = 'hasVFXAddedKey'

local HOLOGRAM_SHADER = 'skinned_toon_hologram'
local DISSOLVE_SHADER_COLOR_DEFAULT = {red = 0, green = 255, blue = 200, alpha = 127}
local UNIFORM_NAME_DISSOLVE_COLOR = 'DissolveColor'

W_Heal_spawnTrigger = function(entity)
    local heal_radius = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_SPELL_RADIUS)
    objc.set_local_value(entity, HEAL_RADIUS, heal_radius)
    local healAmount = objc.get_combat_entity_custom_stat(entity, CUSTOM_ENTITY_STAT_KEY_HEAL)
    objc.set_local_value(entity, HEAL_AMOUNT, healAmount)

    objc.set_targetable(entity, false)

    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_IDLE_ANIM_NAME)
    objc.play_override_animation(entity, IDLE_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

    objc.set_solid(entity, false)
    objc.idle_entity(entity, 10000)
end

W_Heal_tickTrigger = function(entity)
    local allyCheckTimer = objc.tick_timer(entity, 'allyCheckTimerKey', ALLY_CHECK_INTERVAL)
    if allyCheckTimer == 0 then
        local heal_radius = objc.get_local_value(entity, HEAL_RADIUS)
        local healAmount = objc.get_local_value(entity, HEAL_AMOUNT)
        local teamNumber = objc.get_team_number(entity)
        local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
        if buffedUnits == 0 then
            buffedUnits = {}
        end

        for _, buffedUnit in pairs(buffedUnits) do
            W_Heal_removeBuff(buffedUnit)
        end

        buffedUnits = {}

        local allies = objc.list_entities_in_radius(entity, heal_radius, teamNumber)
        for _, ally in pairs(allies) do
            local hasBeenEnraged = objc.get_local_value(ally, HAS_BEED_ENRAGED_KEY)
            if (objc.is_normal_unit(ally) or objc.is_commander(ally)) and hasBeenEnraged == 0 then
                W_Heal_buffUnit(ally, healAmount)
                table.insert(buffedUnits, ally)
            end
        end

        objc.set_local_value(entity, BUFFED_ALLIES_ARRAY, buffedUnits)
    end
end

W_Heal_deathTrigger = function(entity)
    local buffedUnits = objc.get_local_value(entity, BUFFED_ALLIES_ARRAY)
    if buffedUnits ~= 0 then
        for _, buffedUnit in pairs(buffedUnits) do
            W_Heal_removeBuff(buffedUnit)
        end
    end
end


--Prevents Rage from taking splash damage and dying
W_Heal_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    return 0
end

W_Heal_buffUnit = function(entity, healAmount)
    local currentHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
    local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local newHealth = currentHealth + healAmount
    if newHealth > maxHealth then
        newHealth = maxHealth
    end
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, newHealth)
    local hasVFXAdded = objc.get_local_value(entity, HAS_VFX_ADDED)
    if hasVFXAdded == 0 then
        objc.play_vfx(entity, BUFF_VFX, NODE_STATUS)
        objc.set_local_value(entity, HAS_VFX_ADDED, 1)
        objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Heal.lua', 'W_Heal_stopVFXTrigger')
    end

    objc.add_trigger(entity, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'W_Heal.lua', 'W_Heal_invulnTrigger')

    objc.apply_shader(entity, HOLOGRAM_SHADER)
    objc.update_shader_uniform_color(entity, UNIFORM_NAME_DISSOLVE_COLOR, DISSOLVE_SHADER_COLOR_DEFAULT)

    objc.set_local_value(entity, HAS_BEED_ENRAGED_KEY, 1)
end

W_Heal_removeBuff = function(entity)
    objc.remove_trigger(entity, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'W_Heal_invulnTrigger')

    objc.remove_shader(entity, HOLOGRAM_SHADER)

    objc.set_local_value(entity, HAS_BEED_ENRAGED_KEY, 0)

end

W_Heal_invulnTrigger = function(damage, targetEntity, sourceEntity)
    return 0
end

W_Heal_stopVFXTrigger = function(entity)
    local stopEffectTimer = objc.tick_timer(entity, 'stopEffectTimerKey', 10)
    if stopEffectTimer == 0 then
        objc.stop_vfx(entity, BUFF_VFX, NODE_STATUS, false)
        objc.set_local_value(entity, HAS_VFX_ADDED, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Heal_stopVFXTrigger')
    end
end
