local TRIGGER_ENTITY = 'bossBattleTriggerEntity'
local CURRENT_TRIGGER = 'currentTriggerKey'
local CURRENT_BOSS_STATE = 'bossStateKey'
local BOSS_STATE_IDLE = 0
local BOSS_STATE_ACTIVE = 1
local BUILDING_DAMAGE_BONUS = 2

local OLD_PARAMS_KEY = 'oldParamsKey'
local KNOCKBACK_SPEED = 5000
local KNOCKBACK_DISTANCE = 2700

local STARTING_HEALTH_NUMERATOR = 7
local STARTING_HEALTH_DENOMINATOR = 10

local HAS_MOVED_BACK = 'hasMovedBackKey'
local NUM_RETREATS = 'numRetreatsKey'
local MAX_NUM_RETREATS = 4
local HEALTH_ADD_TIME = 50
local HEALTH_PROP_NUM = 3
local HEALTH_PROP_DEN = 4
local IS_HEALING = 'isHealingKey'
local HEALTH_CUTOFF = 100

local MOVEMENT_VELOCITY = 250
local HAS_INIT = 'hasInitKey'

local WAVE_CONTROLLER_ENTITY = 'waveControllerEntity'

local POSITION_ENTITY_IDENTIFIER = 'AnnieWalkerProxy'
local OWNER_ENTITY = 'ownerEntityKey'
local ORIGINAL_POSITION_ENTITY = 'originalPositionEntity'
local ORIGINAL_POSITION_BUFFER = -2000

local RETREAT_TIMER = 300
local FIRST_RETREAT_TIMER = 90
local RETREAT_COUNTER_KEY = 'retreatCounterKey'

local FIRST_WAVE_DELAY = 52
local FIRST_WAVE_TRIGGER = 'W_KyrieBoss_firstWaveTrigger'
local FIRST_FREEZE_DELAY = 10
local SPAWN_WAVE_DELAY = 50
local SPAWN_WAVE_TRIGGER = 'W_KyrieBoss_spawnWaveTrigger'
local RETREAT_DELAY_TRIGGER = 'W_KyrieBoss_retreatDelayTrigger'
local RETREAT_DELAY = 40
local FIRST_FREEZE_TRIGGER = 'W_KyrieBoss_firstFreezeTrigger'
local AIR_HEIGHT_DELAY = 5
local AIR_HEIGHT_TRIGGER = 'W_KyrieBoss_setAirHeightTrigger'
local HEALTH_TRIGGER = 'W_KyrieBoss_addHealthTrigger'
local HEALTH_ADD_DELAY = 50
local IMMUNITY_TRIGGER = 'W_KyrieBoss_immunityTrigger'
local IS_IMMUNE = 'isImmuneToDamageKey'
local IMMUNITY_TIME = 100
local IDLE_TRIGGER = 'W_KyrieBoss_idleAnimTrigger'
local IDLE_ANIM = 'idleAnimationName'
local FIRST_IDLE_ANIM = 'firstIdleAnimationName'
local HAS_LANDED = 'hasLandedKey'

local ORIGINAL_RANGE = 'originalRangeKey'
local ORIGINAL_SPEED_KEY = 'originalMoveSpeedKey'

local FREEZE_IDENTIFIER = 'W_KyrieTrailEffect'
local AOE_LINE_LENGTH = 6000
local BATTLEFIELD_LENGTH = 10000
local BATTLEFIELD_WIDTH = 4000
local ICE_DAMAGE_AMOUNT = 2500
local SLOW_DURATION = 50
local SLOW_TIMER_KEY = 'slowTimerKey'
local DRAGON_SLOW_TRIGGER_NAME = 'W_KyrieBoss_immobilizeTickTrigger'
local HAS_INITIALIZED_SLOW_KEY = 'hasInitializedSlowKey'
local SLOW_SHADER_NAME = 'skinned_toon_freeze'

local ATTACK_COUNTER_KEY = 'attackCounterKey'
local ATTACK_AOE_LINE_LENGTH = 4000
local ATTACK_AOE_LINE_WIDTH = 750
local DAMAGE_ENTITY_IDENTIFIER = 'W_DragonDamageScriptEntity'
local DAMAGE_ENTITY_DAMAGE_KEY = 'toDealDamageKey'
local DAMAGE_ENTITY_TARGET_KEY = 'damageEntityTargetKey'
local VFX_BASE_OFFSET = 2000

local RETREAT_ANIM_NAME = 'W_Dragon_Boss_Retreat_01'
local FRAMES_IN_RETREAT_ANIM = 120

local LANDING_ANIM_NAME = 'W_Dragon_Boss_Landing_01'
local FRAMES_IN_LANDING_ANIM = 45

local TAKEOFF_ANIM_NAME = 'W_Dragon_Boss_Takeoff_01'
local FRAMES_IN_TAKEOFF_ANIM = 30

local INTRO_ANIM_NAME = 'W_Dragon_Boss_Intro_01'
local FRAMES_IN_INTRO_ANIM = 135

local IDLE_ANIM_NAME = 'W_Dragon_Boss_Idle_01'

local LEVEL_ANIM_PREFIX = 'W_Dragon_Boss_Level_'
local FRAMES_IN_LEVEL_ANIMS = 1

local DRAGON_IDENTIFIER = 'W_Dragon'

local BARRICADE_X_OFFSET = 500

W_KyrieBoss_spawnTrigger = function(entity)
    objc.set_local_value(entity, CURRENT_BOSS_STATE, BOSS_STATE_ACTIVE)

    local range = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
    objc.set_local_value(entity, ORIGINAL_RANGE, range)

    local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local healthToSet = objc.divide_with_ceiling(maxHealth * STARTING_HEALTH_NUMERATOR, STARTING_HEALTH_DENOMINATOR)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, healthToSet)

    --Keep base's damage updated to make the health bar work
    local teamNumber = objc.get_team_number(entity)
    local kyrieBase = objc.get_base_entity(teamNumber)
    objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_MAX_HEALTH, maxHealth)
    objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_CURRENT_HEALTH, healthToSet)
    objc.set_targetable(kyrieBase, false)
end

W_KyrieBoss_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local thisPos = objc.get_entity_position(entity)
        local teamNumber = objc.get_team_number(entity)
        local positionEntity = objc.spawn_at_point(POSITION_ENTITY_IDENTIFIER, 1, teamNumber, thisPos[1] + ORIGINAL_POSITION_BUFFER, thisPos[2], 0, 0)
        objc.set_local_value(entity, ORIGINAL_POSITION_ENTITY, positionEntity[1])
        objc.set_local_value(positionEntity[1], OWNER_ENTITY, entity)
        objc.set_entity_air_height_alignment(entity, AIR_HEIGHT_ALIGN_BOTTOM)
        objc.set_local_value(entity, HAS_INIT, 1)

        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_INTRO_ANIM)
        objc.play_override_animation(entity, INTRO_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
        objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', FIRST_WAVE_TRIGGER)
    end

    local currentTrigger = objc.get_local_value(entity, CURRENT_TRIGGER)
    if currentTrigger ~= 0 then
        objc.sacrifice_entity(currentTrigger)
        objc.set_local_value(entity, CURRENT_TRIGGER, 0)
    end

    local bossState = objc.get_local_value(entity, CURRENT_BOSS_STATE)
    local hasMovedBack = objc.get_local_value(entity, HAS_MOVED_BACK)
    if bossState == BOSS_STATE_IDLE then
        local numRetreats = objc.get_local_value(entity, NUM_RETREATS)
        local retreatTimer = RETREAT_TIMER
        if numRetreats == 1 then
            retreatTimer = FIRST_RETREAT_TIMER
        end

        local retreatTimer = objc.tick_timer(entity, RETREAT_COUNTER_KEY .. numRetreats, retreatTimer)
        if retreatTimer == 0 then
            objc.set_local_value(entity, CURRENT_BOSS_STATE, BOSS_STATE_ACTIVE)
            local teamNumber = objc.get_team_number(entity)
            local spawnedTrigger = objc.spawn_at_point(TRIGGER_ENTITY, 1, teamNumber, 0, 0, 0, 0)
            objc.set_local_value(entity, CURRENT_TRIGGER, spawnedTrigger[1])
        end
    else
        if hasMovedBack == 1 then
            local numRetreats = objc.get_local_value(entity, NUM_RETREATS)
            if numRetreats <= MAX_NUM_RETREATS then
                W_KyrieBoss_spawnBossWaveNumber(entity, numRetreats)
            end

            objc.set_entity_air_height_variance(entity, 0)
            local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TAKEOFF_ANIM)
            objc.play_override_animation(entity, TAKEOFF_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
            objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', AIR_HEIGHT_TRIGGER)
            objc.unidle_entity(entity)
            objc.set_local_value(entity, HAS_MOVED_BACK, 0)
            objc.set_local_value(entity, HAS_LANDED, 0)
        end
    end
end

W_KyrieBoss_freezeMap = function(entity, posX, posY)
    local teamNumber = objc.get_team_number(entity)
    local enemyNumber = objc.get_enemy_team_number(entity)
    objc.spawn_at_point(FREEZE_IDENTIFIER, 1, teamNumber, 0, 0, 1, 0)

    local enemyList = objc.list_entities(enemyNumber)
    for _, enemy in pairs(enemyList) do
        if objc.is_normal_unit(enemy) or objc.is_commander(enemy) then
            W_KyrieBoss_knockbackEnemy(entity, enemy)
            local tickTriggers = objc.lua_functions_for_trigger(enemy, TRIGGER_NAME_TICK_TRIGGER)
            local hasImmobilizeTrigger = false
            for _, trigger in pairs(tickTriggers) do
                if string.match(trigger, 'W_KyrieBoss_immobilizeTickTrigger') then
                    hasImmobilizeTrigger = true
                end
            end

            objc.set_local_value(enemy, SLOW_TIMER_KEY, SLOW_DURATION)
            if hasImmobilizeTrigger == false then
                objc.add_trigger(enemy, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', DRAGON_SLOW_TRIGGER_NAME)
            end

            local damageEntity = objc.spawn(DAMAGE_ENTITY_IDENTIFIER, 1, teamNumber)
            objc.set_local_value(damageEntity[1], DAMAGE_ENTITY_DAMAGE_KEY, ICE_DAMAGE_AMOUNT)
            objc.set_local_value(damageEntity[1], DAMAGE_ENTITY_TARGET_KEY, enemy)
        end
    end
end

W_KyrieBoss_immobilizeTickTrigger = function(entity)
    local hasInitSlow = objc.get_local_value(entity, HAS_INITIALIZED_SLOW_KEY)
    if hasInitSlow == 0 then
        local originalMaxVel = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
        objc.apply_shader(entity, SLOW_SHADER_NAME)
        objc.set_local_value(entity, ORIGINAL_SPEED_KEY, originalMaxVel)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 3)
        objc.set_local_value(entity, HAS_INITIALIZED_SLOW_KEY, 1)
    end

    local slowTimerValue = objc.get_local_value(entity, SLOW_TIMER_KEY)
    if slowTimerValue <= 0 then
        local originalMaxVel = objc.get_local_value(entity, ORIGINAL_SPEED_KEY)
        objc.set_local_value(entity, HAS_INITIALIZED_SLOW_KEY, 0)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, originalMaxVel)
        objc.remove_shader(entity, SLOW_SHADER_NAME)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss_immobilizeTickTrigger')
        objc.idle_entity(entity, 1)
    else
        objc.set_local_value(entity, SLOW_TIMER_KEY, slowTimerValue - 1)
    end
end

W_KyrieBoss_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local targetPos = objc.get_entity_position(targetEntity)
    local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE)
    objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)

    local entityPos = objc.get_entity_position(sourceEntity)
    objc.move_entity_to_location_with_velocity(sourceEntity, entityPos[1] - 50, entityPos[2], 100, 25)

    objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', IDLE_TRIGGER)
    objc.set_local_value(sourceEntity, HAS_MOVED_BACK, 1)
end

W_KyrieBoss_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local isImmune = objc.get_local_value(targetEntity, IS_IMMUNE)
    if isImmune == 1 then
        return 0
    end

    local currentState = objc.get_local_value(targetEntity, CURRENT_BOSS_STATE)
    local hasMovedBack = objc.get_local_value(targetEntity, HAS_MOVED_BACK)
    if currentState == BOSS_STATE_IDLE and hasMovedBack == 1 then
        local teamNumber = objc.get_team_number(targetEntity)
        local spawnedTrigger = objc.spawn_at_point(TRIGGER_ENTITY, 1, teamNumber, 0, 0, 0, 0)
        objc.set_local_value(targetEntity, CURRENT_TRIGGER, spawnedTrigger[1])
        objc.set_local_value(targetEntity, CURRENT_BOSS_STATE, BOSS_STATE_ACTIVE)
    end

    --Should I retreat?
    local teamNumber = objc.get_team_number(targetEntity)
    local maxHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local currentHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
    local numRetreats = objc.get_local_value(targetEntity, NUM_RETREATS) + 1
    if currentHealth - damage <= HEALTH_CUTOFF and numRetreats <= MAX_NUM_RETREATS then
        local isHealing = objc.get_local_value(targetEntity, IS_HEALING)
        if isHealing == 0 then
            objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', RETREAT_DELAY_TRIGGER)
            objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', HEALTH_TRIGGER)
            objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', FIRST_FREEZE_TRIGGER)
            objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', SPAWN_WAVE_TRIGGER)
            objc.set_local_value(targetEntity, IS_IMMUNE, 1)
            objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss.lua', IMMUNITY_TRIGGER)
            objc.set_local_value(targetEntity, CURRENT_BOSS_STATE, BOSS_STATE_IDLE)
            objc.set_local_value(targetEntity, NUM_RETREATS, numRetreats)
            objc.set_local_value(targetEntity, IS_HEALING, 1)

            --[[local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_IDLE] = IDLE_ANIM_NAME
            objc.update_animation_state(targetEntity, updatedAnimations)]]

            local spawnedTrigger = objc.spawn_at_point(TRIGGER_ENTITY, 1, teamNumber, 0, 0, 0, 0)
            objc.set_local_value(targetEntity, CURRENT_TRIGGER, spawnedTrigger[1])

            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_MOVE_START] = ''
            objc.update_animation_state(targetEntity, updatedAnimations)

            local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_RETREAT_ANIM)
            objc.idle_entity(targetEntity, numberOfSubrounds)
            objc.play_override_animation(targetEntity, RETREAT_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
        end

        --For the case of large amounts of damage, to make the health bar make sense
        objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH, HEALTH_CUTOFF)
        local kyrieBase = objc.get_base_entity(teamNumber)
        objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_MAX_HEALTH, maxHealth)
        objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_CURRENT_HEALTH, HEALTH_CUTOFF)
        return 0
    end

    --Keep base's damage updated to make the health bar work, with a relief valve for accidental AoE
    local kyrieBase = objc.get_base_entity(teamNumber)
    objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_MAX_HEALTH, maxHealth)
    if currentHealth - damage > 0 then
        objc.set_entity_value_for_key(kyrieBase, ENTITY_VALUE_KEY_CURRENT_HEALTH, currentHealth - damage)
    end
    return damage
end

W_KyrieBoss_damageTrigger = function(damage, targetEntity, sourceEntity)
    local dragon = objc.get_entity_value(sourceEntity, 'identityComponent.sourceEntity')
    local primaryTargetKey = objc.get_entity_value(dragon, 'identityComponent.description')
    local attackCounter = objc.get_local_value(dragon, ATTACK_COUNTER_KEY)
    local isCurrentPrimaryTargetValueCheck = objc.get_local_value(targetEntity, primaryTargetKey)


    if attackCounter == isCurrentPrimaryTargetValueCheck then
        local playerNumber = objc.get_player_number(dragon)
        local enemyNumber = objc.get_team_number(targetEntity)
        local targetPosition = objc.get_entity_position(targetEntity)
        local dragonPosition = objc.get_entity_position(dragon)
        local enemyList = objc.list_entities(enemyNumber)
        local directionX = 0

        if dragonPosition[1] <= targetPosition[1] then
            directionX = 1
        else
            directionX = -1
        end

        for _, enemy in pairs(enemyList) do
            local enemyPosition = objc.get_entity_position(enemy)
            local distanceFromTargetX = (enemyPosition[1] + BATTLEFIELD_LENGTH) - (targetPosition[1] + BATTLEFIELD_LENGTH)
            distanceFromTargetX = distanceFromTargetX * directionX
            local distanceFromTargetY = math.abs((enemyPosition[2] + BATTLEFIELD_WIDTH) - (targetPosition[2] + BATTLEFIELD_WIDTH))

            if distanceFromTargetX > -100
            and distanceFromTargetX < ATTACK_AOE_LINE_LENGTH
            and distanceFromTargetY < objc.divide_with_floor(ATTACK_AOE_LINE_WIDTH, 2) then

                --if it's a ground unit, damage it
                if objc.is_ground_unit(enemy) then
                    local damageEntity = objc.spawn(DAMAGE_ENTITY_IDENTIFIER, 1, playerNumber)
                    if objc.is_barricade(enemy) or objc.is_base(enemy) then
                        damage = damage * BUILDING_DAMAGE_BONUS
                    end
                    objc.set_local_value(damageEntity[1], DAMAGE_ENTITY_DAMAGE_KEY, damage)
                    objc.set_local_value(damageEntity[1], DAMAGE_ENTITY_TARGET_KEY, enemy)
                end

            end
        end

        local enemyBaseEntity = objc.get_base_entity(enemyNumber)
        local baseOffset = 0
        if targetEntity == enemyBaseEntity then
            baseOffset = VFX_BASE_OFFSET * directionX
        end
        objc.spawn_at_point('W_DragonTrailEffect', 1, playerNumber, targetPosition[1] - baseOffset, targetPosition[2], directionX, 0)

    end
    return 0
end

W_KyrieBoss_knockbackEnemy = function(entity, enemy)
    local teamNumber = objc.get_team_number(entity)
    local enemyPos = objc.get_entity_position(enemy)
    local targetDestinationX = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_POSITION_X)
    local targetDestinationY = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_POSITION_Y)
    local entityPos = objc.get_entity_position(entity)
    local oldRange = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE)

    targetDestinationX = targetDestinationX - KNOCKBACK_DISTANCE - oldRange

    local targetArray = objc.spawn_at_point('SumoTargetEffectEntity', 1, teamNumber, targetDestinationX, targetDestinationY, 0, 0)
    --Commenting this out as a workaround until the way this interacts with move_entity_to_entity_with_motion is fixed
    --[[objc.set_local_value(enemy, ORIGINAL_RANGE_KEY, oldRange)
    objc.set_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE, MELEE_RANGE)]]
    objc.add_trigger(enemy, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'W_KyrieBoss.lua', 'W_KyrieBoss_endRangeTrigger')
    local planeOfExistence = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)

    if not objc.is_building_unit(enemy) then
        objc.set_entity_should_reverse_facing_direction(enemy, true)
    end
    

    local oldParams = objc.move_entity_to_entity_with_motion(enemy, targetArray[1], MOTION_STATE_WALK, KNOCKBACK_SPEED)
    objc.set_local_value(enemy, OLD_PARAMS_KEY, oldParams)

    --Set the offset on the EffectEntity for where to spawn the impact effect
    objc.set_local_value(targetArray[1], 'craterSpawnPosition', targetDestinationX)
end

W_KyrieBoss_endRangeTrigger = function(sourceEntity, targetEntity)
    local oldParams = objc.get_local_value(sourceEntity, OLD_PARAMS_KEY)
    objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, oldParams)

    --local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
    --objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)

    objc.set_entity_should_reverse_facing_direction(sourceEntity, false)
    objc.remove_trigger(sourceEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'C_Ascension2_endRangeTrigger')
end

W_KyrieBoss_spawnWaveNumber = function(entity, waveNumber)
    local waveController = objc.get_local_value(entity, WAVE_CONTROLLER_ENTITY)
    if waveController ~= 0 then
        local bossID = objc.get_entity_value_for_key(waveController, ENTITY_VALUE_KEY_IDENTIFIER)
        objc.add_trigger(waveController, TRIGGER_NAME_TICK_TRIGGER, bossID .. '.lua', bossID .. '_Wave_' .. waveNumber)
    end
end

W_KyrieBoss_spawnBossWaveNumber = function(entity, waveNumber)
    local waveController = objc.get_local_value(entity, WAVE_CONTROLLER_ENTITY)
    if waveController ~= 0 then
        local bossID = objc.get_entity_value_for_key(waveController, ENTITY_VALUE_KEY_IDENTIFIER)
        objc.add_trigger(waveController, TRIGGER_NAME_TICK_TRIGGER, bossID .. '.lua', bossID .. '_BossWave_' .. waveNumber)
    end
end

W_KyrieBoss_firstWaveTrigger = function(entity)
    local firstWaveTimer = objc.tick_timer(entity, 'firstWaveTimerKey', FIRST_WAVE_DELAY)
    if firstWaveTimer == 5000 then
        local numRetreats = objc.get_local_value(entity, NUM_RETREATS) + 1
        objc.set_local_value(entity, NUM_RETREATS, numRetreats)
        local originalPositionEntity = objc.get_local_value(entity, ORIGINAL_POSITION_ENTITY)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, 0)
        objc.move_entity_to_entity_with_motion(entity, originalPositionEntity, MOTION_STATE_WALK, MOVEMENT_VELOCITY)
        objc.set_local_value(entity, HAS_MOVED_BACK, 1)
        objc.set_local_value(entity, CURRENT_BOSS_STATE, BOSS_STATE_IDLE)
    elseif firstWaveTimer == 0 then
        local numRetreats = objc.get_local_value(entity, NUM_RETREATS)
        W_KyrieBoss_spawnWaveNumber(entity, numRetreats)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, FIRST_WAVE_TRIGGER)
    end
end

W_KyrieBoss_immobilizeTickTrigger = function(entity)
    local hasInitSlow = objc.get_local_value(entity, HAS_INITIALIZED_SLOW_KEY)
    if hasInitSlow == 0 then
        local originalMaxVel = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
        objc.apply_shader(entity, SLOW_SHADER_NAME)
        objc.set_local_value(entity, ORIGINAL_SPEED_KEY, originalMaxVel)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 3)
        objc.set_local_value(entity, HAS_INITIALIZED_SLOW_KEY, 1)
    end

    local slowTimerValue = objc.get_local_value(entity, SLOW_TIMER_KEY)
    if slowTimerValue <= 0 then
        local originalMaxVel = objc.get_local_value(entity, ORIGINAL_SPEED_KEY)
        objc.set_local_value(entity, HAS_INITIALIZED_SLOW_KEY, 0)
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, originalMaxVel)
        objc.remove_shader(entity, SLOW_SHADER_NAME)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_KyrieBoss_immobilizeTickTrigger')
        objc.idle_entity(entity, 1)
    else
        objc.set_local_value(entity, SLOW_TIMER_KEY, slowTimerValue - 1)
    end
end

W_KyrieBoss_addHealthTrigger = function(entity)
    local healthAddTime = objc.tick_timer(entity, 'healthAddTimerKey', HEALTH_ADD_DELAY)
    local currentState = objc.get_local_value(entity, CURRENT_BOSS_STATE)
    if healthAddTime == 0 or currentState == BOSS_STATE_ACTIVE then
        local updatedAnimations = {}
        updatedAnimations[ANIMATION_STATE_MOVE_START] = 'startAnimationName'
        objc.update_animation_state(entity, updatedAnimations)

        objc.set_local_value(entity, IS_HEALING, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, HEALTH_TRIGGER)
    end
end

W_KyrieBoss_immunityTrigger = function(entity)
    local immunityTime = objc.tick_timer(entity, 'immunityTimerKey', IMMUNITY_TIME)
    objc.set_local_value(entity, IS_IMMUNE, 1)
    if immunityTime == 0 then
        objc.set_local_value(entity, IS_IMMUNE, 0)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, IMMUNITY_TRIGGER)
    end
end

W_KyrieBoss_idleAnimTrigger = function(entity)
    local hasLanded = objc.get_local_value(entity, HAS_LANDED)
    if hasLanded == 0 then
        objc.set_local_value(entity, HAS_LANDED, 1)
    elseif hasLanded == 1 then
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_LANDING_ANIM)
        objc.play_override_animation(entity, LANDING_ANIM_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
        objc.set_local_value(entity, HAS_LANDED, 2)
    end

    local rounds = objc.convert_frames_to_rounds(FRAMES_IN_LANDING_ANIM)
    local idleTime = objc.tick_timer(entity, 'idleAnimTimerKey', rounds - 1)
    if idleTime == 0 then
        objc.idle_entity(entity, RETREAT_TIMER * 100)
        objc.set_entity_air_height_variance(entity, -750)
        objc.play_override_animation(entity, IDLE_ANIM_NAME, RETREAT_TIMER * 1000)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, IDLE_TRIGGER)
    end
end

W_KyrieBoss_firstFreezeTrigger = function(entity)
    local firstFreezeDelay = objc.tick_timer(entity, 'firstFreezeDelayTimerKey', FIRST_FREEZE_DELAY)
    if firstFreezeDelay == 0 then
        local entityPos = objc.get_entity_position(entity)
        W_KyrieBoss_freezeMap(entity, entityPos[1] + 200, entityPos[2])
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, FIRST_FREEZE_TRIGGER)
    end
end

W_KyrieBoss_spawnWaveTrigger = function(entity)
    local spawnWaveDelay = objc.tick_timer(entity, 'spawnWaveDelayTimerKey', SPAWN_WAVE_DELAY)
    if spawnWaveDelay == 0 then
        local waveNumber = objc.get_local_value(entity, NUM_RETREATS)
        if waveNumber ~= 0 then
            W_KyrieBoss_spawnWaveNumber(entity, waveNumber)
        end
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, SPAWN_WAVE_TRIGGER)
    end
end

W_KyrieBoss_retreatDelayTrigger = function(entity)
    local retreatDelay = objc.tick_timer(entity, 'retreatDelayTimerKey', RETREAT_DELAY)
    if retreatDelay == 0 then
        local originalPositionEntity = objc.get_local_value(entity, ORIGINAL_POSITION_ENTITY)
        if originalPositionEntity ~= 0 then
            objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, 0)
            objc.move_entity_to_entity_with_motion(entity, originalPositionEntity, MOTION_STATE_WALK, MOVEMENT_VELOCITY)
        end
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, RETREAT_DELAY_TRIGGER)
    end
end

W_KyrieBoss_setAirHeightTrigger = function(entity)
    local airHeightDelay = objc.tick_timer(entity, 'airHeightDelay', AIR_HEIGHT_DELAY)
    if airHeightDelay == 0 then
        objc.set_entity_air_height_variance(entity, 50)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, AIR_HEIGHT_TRIGGER)
    end
end

W_KyrieBoss_deathTrigger = function(entity)
    local teamNumber = objc.get_team_number(entity)
    local baseEntity = objc.get_base_entity(teamNumber)
    objc.kill_entity(baseEntity)
end
