--keys
local CHANNELING_TICKS = 'numChannelingTicks'
local OLD_MAX_VELOCITY = 'oldMaxVelKey'
local PROGRESS_BAR = 'progressBarKey'
local SHOULD_CHANNEL = 'shouldBeCheckingKey'
local VFX_PLAYING = 'playVfxKey'
local HAS_INIT_CHANNEL = 'hasInitChannelActivity'

--Constants
local CHANNEL_ROUNDS = 10
local RESTART_CASTING_TIMER = 220

--Animation Values
local CASTING_ANIMATION = 'W_Cultist_Attack_04'
local CASTING_ANIMATION_2 = 'W_Cultist_Attack_04'
local CASTING_FRAMES = 700

local HEALTH_BONUS = 100

-- Frost aura
local CHECK_TICK = 3
local CHECK_KEY = 'checkKey'
local HAS_BEEN_SLOWED_KEY = 'hasBeenSlowedKey'
local FROST_AFFECTED_UNITS_ARRAY = 'frostAffectedUnitsArray'
local ORIGINAL_ATTACK_SPEED_FROST = 'originalAttackSpeedFrost'

local FROST_RADIUS = 1300
local MOVE_SPEED_SLOW = 50

local BUFF_ENTITY_ID = 'KyrieCultistBuffEntity'
local MOVE_TARGET_KEY = 'buffEntityMoveTargetKey'

W_KyrieCultist_tickTrigger = function(entity)
   local shouldChannel = objc.get_local_value(entity, SHOULD_CHANNEL)
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)

   --If I should not already be channeling, then check my position to
   --see if I should
   if shouldChannel == 0 then
      --If I'm past the midpoint, I should be channeling
      local position = objc.get_entity_position(entity)
      if teamNumber == 1 then
         if position[1] > 0 then
            objc.set_local_value(entity, SHOULD_CHANNEL, 1)
         end
      else
         if position[1] < 2250 then
            objc.set_local_value(entity, SHOULD_CHANNEL, 1)
         end
      end
   else
      local attackRate = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RATE)
      local channelRounds = objc.divide_with_floor(attackRate, 100)
      
      local restartCastingTimer = objc.tick_timer(entity, 'restartCastingTimerKey', RESTART_CASTING_TIMER)
      if restartCastingTimer == 0 then
        objc.sacrifice_entity(entity)
      end

      --Set up the necessary things to channel
      local hasInitChannel = objc.get_local_value(entity, HAS_INIT_CHANNEL)
      if hasInitChannel == 0 then
         --Stop myself and play the summoning animation
         local numberOfSubrounds = objc.convert_frames_to_subrounds(CASTING_FRAMES)
         objc.idle_entity(entity, numberOfSubrounds)
         local castAnim = CASTING_ANIMATION
         if teamNumber == 2 then
            castAnim = CASTING_ANIMATION_2
         end
         objc.play_override_animation(entity, castAnim, objc.subrounds_to_milliseconds(numberOfSubrounds))

         objc.set_local_value(entity, HAS_INIT_CHANNEL, 1)
      end

      --Tick down my channeling time
      local channelTimer = objc.tick_timer(entity, CHANNELING_TICKS, channelRounds)
      if channelTimer == 0 then
         local allies = objc.list_entities(teamNumber)
         for _, ally in pairs(allies) do
            local identifier = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_IDENTIFIER)
            if identifier == 'W_KyrieBoss' then
              local entityPos = objc.get_entity_position(entity)
              local buffEntity = objc.spawn_at_point(BUFF_ENTITY_ID, 1, playerNumber, entityPos[1], entityPos[2], 0, 0)
              objc.set_local_value(buffEntity[1], MOVE_TARGET_KEY, ally)
              objc.set_entity_value_for_key(buffEntity[1], ENTITY_VALUE_KEY_ATTACK_RANGE, 1)
            end
         end
      end
   end
end
