local LOBBER_PROJECTILE_DELAY = 20
local LOBBER_PROJECTILE_TIMER = 'lobberProjectileTimerKey'
local PROJECTILE_LOCATION = 'projectileLocationKey'
local PROJECTILE_RADIUS = 250
local IS_PROJECTILE_DAMAGE = 'isProjectileDamageKey'
local TARGET_ENTITY_ID = 'lobberProjectileTargetEntity'
local PROJECTILE_ENTITY_ID = 'W_LobberProjectileDummy'
local PROJECTILE_TARGET = 'projectileTargetEntityKey'

W_Lobber_damageTrigger = function(damage, targetEntity, sourceEntity)
    local isProjectileDamage = objc.get_local_value(sourceEntity, IS_PROJECTILE_DAMAGE)
    if isProjectileDamage == 1 then
        return damage
    end

    return 0
end

W_Lobber_onAttackTrigger = function(sourceEntity, targetEntity)
    local playerNumber = objc.get_player_number(sourceEntity)
    objc.set_local_value(sourceEntity, IS_PROJECTILE_DAMAGE, 0)
    objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_Lobber.lua', 'W_Lobber_waitForProjectileTrigger')
    local targetLocation = objc.get_entity_position(targetEntity)
    objc.set_local_value(sourceEntity, PROJECTILE_LOCATION, targetLocation)

    local projectileEntityArray = objc.spawn_at_entity(PROJECTILE_ENTITY_ID, 1, sourceEntity)
    local projectileTargetEntityArray = objc.spawn_at_point(TARGET_ENTITY_ID, 1, playerNumber, targetLocation[1], targetLocation[2], 0, 0)
    objc.set_local_value(projectileEntityArray[1], PROJECTILE_TARGET, projectileTargetEntityArray[1])
end

W_Lobber_waitForProjectileTrigger = function(entity)
    local projectileTimer = objc.tick_timer(entity, LOBBER_PROJECTILE_TIMER, LOBBER_PROJECTILE_DELAY)
    if projectileTimer == 0 then
        objc.set_local_value(entity, IS_PROJECTILE_DAMAGE, 1)
        local teamNumber = objc.get_team_number(entity)
        local targetLocation = objc.get_local_value(entity, PROJECTILE_LOCATION)
        local enemiesInRange = objc.list_enemies_in_radius_of_point(targetLocation[1], targetLocation[2], PROJECTILE_RADIUS, teamNumber)
        local damageToDeal = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_DAMAGE)
        local damageBuff = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_DAMAGE_BUFF)
        for _, enemy in pairs(enemiesInRange) do
            if objc.is_normal_unit(enemy) and objc.is_ground_unit(enemy) then
                objc.inflict_normal_damage(entity, enemy, damageToDeal + damageBuff)
                objc.flash_entity_for_damage(enemy, 250)
            end
        end
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Lobber_waitForProjectileTrigger')
    end
end