local MIND_CONTROL_ROUNDS = 100
local MIND_CONTROL_INITIAL_TARGET = 'mindControlTargetKey'
local MIND_CONTROL_STANDIN_TARGET = 'mindControlStandinTargetKey'
local IS_MIND_CONTROLLING = 'isMindControlling'

W_MindController_onAttackTrigger = function(sourceEntity, targetEntity)
    if objc.is_normal_unit(targetEntity) then
        local playerNumber = objc.get_player_number(sourceEntity)
        local teamNumber = objc.get_team_number(sourceEntity)
        objc.idle_entity(sourceEntity, MIND_CONTROL_ROUNDS * 100)

        local thisPosition = objc.get_entity_position(targetEntity)
        local thisId = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_IDENTIFIER)
        local currentHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
        local maxHealth = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_MAX_HEALTH)
        local damageToDo = maxHealth - currentHealth
        local directionX = 1
        if teamNumber == 2 then
          directionX = -1
        end
        local spawned = objc.spawn_at_point(thisId, 1, playerNumber, thisPosition[1], thisPosition[2], directionX, 0)
        objc.inflict_normal_damage(spawned[1], spawned[1], damageToDo)
        objc.teleport_entity(targetEntity, -10000, 0)
        objc.idle_entity(targetEntity, MIND_CONTROL_ROUNDS * 100)
        objc.set_targetable(targetEntity, false)

        objc.set_local_value(sourceEntity, MIND_CONTROL_INITIAL_TARGET, targetEntity)
        objc.set_local_value(sourceEntity, MIND_CONTROL_STANDIN_TARGET, spawned[1])

        objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_MindController.lua', 'W_MindController_waitOutMindControlTrigger')
    end
end

W_MindController_waitOutMindControlTrigger = function(entity)
    local mindControlTarget = objc.get_local_value(entity, MIND_CONTROL_STANDIN_TARGET)
    if mindControlTarget ~= 0 then
        local targetHealth = objc.get_entity_value_for_key(mindControlTarget, ENTITY_VALUE_KEY_CURRENT_HEALTH)
        if targetHealth > 0 then
            local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
            local healthDrainTick = objc.divide_with_ceiling(maxHealth, MIND_CONTROL_ROUNDS)
            objc.inflict_normal_damage(entity, entity, healthDrainTick)
        else
            local initialTarget = objc.get_local_value(entity, MIND_CONTROL_INITIAL_TARGET)
            if initialTarget ~= 0 then
                objc.sacrifice_entity(initialTarget)
            end
            objc.unidle_entity(entity)
            objc.set_local_value(entity, MIND_CONTROL_STANDIN_TARGET, 0)
        end
    else
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_MindController_waitOutMindControlTrigger')
    end
end

W_MindController_deathTrigger = function(entity)
    local mindControlTarget = objc.get_local_value(entity, MIND_CONTROL_STANDIN_TARGET)
    if mindControlTarget ~= 0 then
        local targetPos = objc.get_entity_position(mindControlTarget)
        local targetHealth = objc.get_entity_value_for_key(mindControlTarget, ENTITY_VALUE_KEY_CURRENT_HEALTH)
        local initialTarget = objc.get_local_value(entity, MIND_CONTROL_INITIAL_TARGET)
        objc.set_entity_value_for_key(initialTarget, ENTITY_VALUE_KEY_CURRENT_HEALTH, targetHealth)
        objc.teleport_entity(initialTarget, targetPos[1], targetPos[2])
        objc.set_targetable(initialTarget, true)
        objc.teleport_entity(mindControlTarget, -10000, 0)
        objc.sacrifice_entity(mindControlTarget)
    end
end
