-- Keys
local NUMBER_OF_HITS_KEY = 'numberOfHits'

-- Animation Values
local ONE_HIT_TRANSITION_ANIMATION_NAME = 'W_Negator_Shield1_Trans_01'
local NUMBER_OF_FRAMES_IN_ONE_HIT_TRANSITION_ANIMATION = 24
local ONE_HIT_MOVE_ANIMATION_STATE = 'move1AnimationName'
local ONE_HIT_ATTACK_ANIMATION_STATE = 'attack1AnimationName'

local TWO_HIT_TRANSITION_ANIMATION_NAME = 'W_Negator_Shield2_Trans_01'
local NUMBER_OF_FRAMES_IN_TWO_HIT_TRANSITION_ANIMATION = 24
local TWO_HIT_MOVE_ANIMATION_STATE = 'move2AnimationName'
local TWO_HIT_ATTACK_ANIMATION_STATE = 'attack2AnimationName'

local THREE_HIT_TRANSITION_ANIMATION_NAME = 'W_Negator_Shield3_Trans_01'
local NUMBER_OF_FRAMES_IN_THREE_HIT_TRANSITION_ANIMATION = 24
local THREE_HIT_MOVE_ANIMATION_STATE = 'move3AnimationName'
local THREE_HIT_ATTACK_ANIMATION_STATE = 'attack3AnimationName'

local HAS_BEED_PROTECTED_KEY = 'hasBeenProtectedKey'

--[[
This OnDamageTrigger gives the Negator the Negate UnitAbility.
--]]
W_Negator_onDamageTrigger = function(damage, targetEntity, sourceEntity)
    local hasBeenProtected = objc.get_local_value(targetEntity, HAS_BEED_PROTECTED_KEY)
    if damage > 0 and objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0 and hasBeenProtected == 0 then
        local negateLimit = objc.get_combat_entity_custom_stat(targetEntity, CUSTOM_ENTITY_STAT_KEY_SHOTS_BLOCKED)
        -- Update the animation to show the right number of shields
        local numberOfHits = objc.get_local_value(targetEntity, NUMBER_OF_HITS_KEY)
        local firstThreshold = objc.divide_with_floor(objc.divide_with_floor(negateLimit * 10, 3) + 5, 10)
        local secondThreshold = objc.divide_with_floor(objc.divide_with_floor(negateLimit * 10, 3) * 2 + 5, 10)

        if numberOfHits == firstThreshold-1 then
            -- Play transition animation
            local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_ONE_HIT_TRANSITION_ANIMATION)
            objc.idle_entity(targetEntity, numberOfSubrounds)
            objc.play_override_animation(targetEntity, ONE_HIT_TRANSITION_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

            -- Update animations
            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_MOVE] = ONE_HIT_MOVE_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = ONE_HIT_ATTACK_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = ONE_HIT_ATTACK_ANIMATION_STATE
            objc.update_animation_state(targetEntity, updatedAnimations)

        elseif numberOfHits == secondThreshold-1 then
            -- Play transition animation
            local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_TWO_HIT_TRANSITION_ANIMATION)
            objc.idle_entity(targetEntity, numberOfSubrounds)
            objc.play_override_animation(targetEntity, TWO_HIT_TRANSITION_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

            -- Update animations
            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_MOVE] = TWO_HIT_MOVE_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = TWO_HIT_ATTACK_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = TWO_HIT_ATTACK_ANIMATION_STATE
            objc.update_animation_state(targetEntity, updatedAnimations)

        elseif numberOfHits == negateLimit-1 then
            -- Play transition animation
            local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_THREE_HIT_TRANSITION_ANIMATION)
            objc.idle_entity(targetEntity, numberOfSubrounds)
            objc.play_override_animation(targetEntity, THREE_HIT_TRANSITION_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

            -- Update animations
            local updatedAnimations = {}
            updatedAnimations[ANIMATION_STATE_MOVE] = THREE_HIT_MOVE_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_GROUND] = THREE_HIT_ATTACK_ANIMATION_STATE
            updatedAnimations[ANIMATION_STATE_ATTACK_AIR] = THREE_HIT_ATTACK_ANIMATION_STATE
            objc.update_animation_state(targetEntity, updatedAnimations)
        end

            objc.set_local_value(targetEntity, NUMBER_OF_HITS_KEY, numberOfHits + 1)
        -- Get the negate ability!
        return UnitAbility_negate(targetEntity, damage, negateLimit)
    end
end
