local HAS_INIT = 'hasInitKey'
local HAS_CLONED_KEY = 'hasClonedKey'
local SHOULD_CLONE = 'shouldCloneKey'
local CLONE_COUNT = 15
local CLONE_TIMER = 'cloneTimerKey'
local CLONE_RADIUS = 3700
local KILL_COUNT = 20
local SHAPESHIFTER_IDENTIFIER = 'W_Shapeshifter'
local KILL_TIMER = 'killTimerKey'
local NUMBER_OF_FRAMES_IN_TRANSFORM_ANIMATION = 60
local ABILITY_ANIMATION_KEY_GROUND = 'abilityState'
local ABILITY_ANIMATION_KEY_AIR = 'abilityStateAir'
local CLONED_VFX_NAME = 'W_Cloner_Wing.lvl'
local IS_FOLLOWING_KEY = 'isFollowingKey'
local HOLOGRAMMED_UNIT_KEY = 'hologrammedUnitKey'
local HOLOGRAM_SHADER = 'skinned_toon_hologram'
local BUILD_VFX_NAME = 'W_Shapeshifter_Target.lvl'
local ADD_HOLOGRAM_TRIGGER_NAME = 'W_Shapeshifter_clonedUnitAddHologramTickTrigger'
local HAS_ADDED_SHADER = 'hasAddedShaderKey'
local SHADER_ADD_TIME = 0
local SHADER_ADD_TIMER_KEY = 'shaderAddTimerKey'
local DISSOLVE_SHADER_COLOR_DEFAULT = {red = 0, green = 255, blue = 255, alpha = 127}
local UNIFORM_NAME_DISSOLVE_COLOR = 'DissolveColor'
local RESEARCH_TIER_FACTOR_TIMES_1000 = 1788 --this must be an integer
local VALUE_OF_A_CARD_TIMES_100 = 275 --this must be an integer

local LEGENDARY_CUTOFF = 4


W_Shapeshifter_tickTrigger = function(entity)
    local shouldClone = objc.get_local_value(entity, SHOULD_CLONE)
    local hasInit = objc.get_local_value(entity, HAS_INIT)

    if hasInit == 0 then
        objc.set_local_value(entity, HAS_INIT, 1)
    else
        if shouldClone == 0 then

            local teamNumber = objc.get_team_number(entity)
            local enemyNumber = objc.get_enemy_team_number(entity)

            local allies = objc.list_entities_in_radius(entity, CLONE_RADIUS, teamNumber)
            local mostExpensiveEntity = entity
            local highestValue = 0
            for _,ally in pairs(allies) do
                if objc.is_normal_unit(ally) and ally ~= entity then
                    
                    local thisID = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_IDENTIFIER)

                    if not string.match(thisID, SHAPESHIFTER_IDENTIFIER) then
                        local thisCost = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_BUILD_COST)
                        local techTierMinusOne = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_REQUIRED_RESEARCH)
                        local thisValue = thisCost * 100 + VALUE_OF_A_CARD_TIMES_100
                        if techTierMinusOne > 0 then
                            thisValue = objc.divide_with_floor((thisValue * (RESEARCH_TIER_FACTOR_TIMES_1000)), 1000)
                        end
                        if techTierMinusOne > 1 then
                            thisValue = objc.divide_with_floor((thisValue * (RESEARCH_TIER_FACTOR_TIMES_1000)), 1000)
                        end
                        if objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_SPAWN_COUNT) > 1 then
                            spawnCount = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_SPAWN_COUNT)
                            triangleNumber = objc.divide_with_floor(spawnCount * (spawnCount + 1), 2)
                            thisValue = objc.divide_with_floor(thisValue, triangleNumber)
                        end
                        if thisValue > highestValue then
                            mostExpensiveEntity = ally
                            highestValue = thisValue
                        end
                    end
                end
            end

            local enemies = objc.list_entities_in_radius(entity, CLONE_RADIUS, enemyNumber)
            for _,enemy in pairs(enemies) do
                if objc.is_normal_unit(enemy) and enemy ~= entity and objc.is_targetable(enemy) then
                    
                    local thisID = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IDENTIFIER)

                    if not string.match(thisID, SHAPESHIFTER_IDENTIFIER) then
                        local thisCost = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_BUILD_COST)
                        local techTierMinusOne = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_REQUIRED_RESEARCH)
                        local thisValue = thisCost * 100 + VALUE_OF_A_CARD_TIMES_100
                        if techTierMinusOne > 0 then
                            thisValue = objc.divide_with_floor((thisValue * (RESEARCH_TIER_FACTOR_TIMES_1000)), 1000)
                        end
                        if techTierMinusOne > 1 then
                            thisValue = objc.divide_with_floor((thisValue * (RESEARCH_TIER_FACTOR_TIMES_1000)), 1000)
                        end
                        if objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_SPAWN_COUNT) > 1 then
                            spawnCount = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_SPAWN_COUNT)
                            triangleNumber = objc.divide_with_floor(spawnCount * (spawnCount + 1), 2)
                            thisValue = objc.divide_with_floor(thisValue, triangleNumber)
                        end
                        if thisValue > highestValue then
                            mostExpensiveEntity = enemy
                            highestValue = thisValue
                        end
                    end
                end
            end

            if mostExpensiveEntity ~= entity then
                objc.move_entity_to_entity_with_motion(entity, mostExpensiveEntity, MOTION_STATE_WALK, 200)
            end

        else
            local hasCloned = objc.get_local_value(entity, HAS_CLONED_KEY)
            local cloneTimer = objc.tick_timer(entity, CLONE_TIMER, CLONE_COUNT)
            local killTimer = objc.tick_timer(entity, KILL_TIMER, KILL_COUNT)
            if cloneTimer == 0 and hasCloned == 0 then
                local tier = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_TIER)
                local baseUnitIdentifier = objc.get_entity_value_for_key(shouldClone, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
                local baseUnitRarity = objc.get_entity_value_for_key(shouldClone, ENTITY_VALUE_KEY_RARITY)

                if baseUnitRarity == 0 then
                	tier = tier + 6
                end
                if baseUnitRarity == 1 then
                	tier = tier + 3
                end
                if baseUnitRarity == 3 then
                	tier = math.max(0, tier - 3)
                end
                if tier == 0 then
                	tier = ''
                end

                local spawned = objc.spawn_at_entity(baseUnitIdentifier .. tier, 1, entity)
                objc.add_trigger(spawned[1], TRIGGER_NAME_TICK_TRIGGER, 'W_Shapeshifter.lua', ADD_HOLOGRAM_TRIGGER_NAME)
                local hologrammedUnit = objc.get_local_value(entity, HOLOGRAMMED_UNIT_KEY)
                local hologrammedUnitHealth = objc.get_entity_value_for_key(hologrammedUnit, ENTITY_VALUE_KEY_CURRENT_HEALTH)
                if hologrammedUnitHealth > 0 then
                    objc.stop_vfx(hologrammedUnit, BUILD_VFX_NAME, NODE_ORIGIN, false)
                end
                local noDeathAnimations = {}
                noDeathAnimations[ANIMATION_STATE_DEATH] = ''
                objc.update_animation_state(entity, noDeathAnimations)
                objc.set_local_value(entity, HAS_CLONED_KEY, 1)
                local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_TRANSFORM_ANIMATION)
                objc.idle_entity(entity, numberOfSubrounds)
            end

            if killTimer == 0 then
                objc.sacrifice_entity(entity)
            end
        end
    end



end

W_Shapeshifter_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    objc.set_local_value(sourceEntity, SHOULD_CLONE, targetEntity)
    objc.play_vfx(targetEntity, BUILD_VFX_NAME, NODE_ORIGIN)
    if objc.get_team_number(targetEntity) ~= objc.get_team_number(sourceEntity) then
        objc.stun_entity(targetEntity,1600)
    end
    objc.set_local_value(sourceEntity, HOLOGRAMMED_UNIT_KEY, targetEntity)
    local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_TRANSFORM_ANIMATION)
    objc.idle_entity(sourceEntity, numberOfSubrounds)
    if objc.is_air_unit(targetEntity) then
        objc.play_override_animation_key(sourceEntity, ABILITY_ANIMATION_KEY_AIR, objc.subrounds_to_milliseconds(numberOfSubrounds))
    else
        objc.play_override_animation_key(sourceEntity, ABILITY_ANIMATION_KEY_GROUND, objc.subrounds_to_milliseconds(numberOfSubrounds))
    end
end

W_Shapeshifter_clonedUnitAddHologramTickTrigger = function(entity)
    local hasAddedShader = objc.get_local_value(entity, HAS_ADDED_SHADER)
    if hasAddedShader == 0 then
        --have to add a delay to prevent the shader from being overwritten after spawn
        local addShaderTime = objc.tick_timer(entity, SHADER_ADD_TIMER_KEY, SHADER_ADD_TIME)
        if addShaderTime == 0 then
            objc.apply_shader(entity, HOLOGRAM_SHADER)
            objc.set_local_value(entity, HAS_ADDED_SHADER, 1)
            objc.update_shader_uniform_color(entity, UNIFORM_NAME_DISSOLVE_COLOR, DISSOLVE_SHADER_COLOR_DEFAULT)
        end
    end
end
