local HAS_INIT = 'hasinit'
local HAS_REVEALED_KEY = 'hasRevealedKey'
local REGULAR_ATTACK_ANIMATION_STATE = 'attackAnimName'
local UNCLOAK_ANIMATION_STATE = 'attackSuperAnimName'
local CLOAKED_MOVE_STATE = 'stealthMoveAnim'
local NORMAL_MOVE_STATE = 'moveAnimName'
local OFFSET_AMOUNT = 500
local ENEMY_CHECK_TIMER = 2
local SEARCH_RADIUS = 15000
local DASH_RANGE = 3000
local FRAMES_IN_SUPER_ATTACK_ANIMATION = 45
local SUPER_ATTACK_ANIMATION = 'W_Sneaky_Battle_Attack_Super'
local FRAMES_IN_DASH_ANIMATION = 30
local DASH_ANIMATION = 'W_Sneaky_Battle_Dash_01'
local DASH_FINISH_TRIGGER = 'W_Sneaky_finishJumpingToTargetTrigger'
local IS_WAITING_FOR_SPAWN = 'isWaitingForSpawnKey'
local WAIT_FOR_SPAWN_DELAY = 10
local MOTION_STATE_DICT = 'oldMotionStateDictKey'
local ENEMY_RANGE_THRESHOLD = 2000
local HEIGHT_OF_CENTER_BLANK_AREA = 4200

local COMBO_DAMAGE_DELAY = 3
local COMBO_DAMAGE_KEY = 'comboDamageKey'
local SWINGS_KEY = 'currentSwingsKey'
local NUM_COMBO_SWINGS = 3
local COMBO_TARGET_KEY = 'comboTargetKey'

W_Sneaky_tickTrigger = function(entity)
    local hasinit = objc.get_local_value(entity, HAS_INIT)
    local timer = objc.tick_timer(entity, 'timer', 1) 

    if hasinit == 0 and timer == 0 then
        objc.set_targetable(entity, false)
        objc.set_local_value(entity, HAS_INIT, 1)
        objc.set_solid(entity, false)
        local stealthAnimations = {}
        stealthAnimations[ANIMATION_STATE_MOVE] = CLOAKED_MOVE_STATE
        stealthAnimations[ANIMATION_STATE_ATTACK_GROUND] = UNCLOAK_ANIMATION_STATE
        objc.update_animation_state(entity, stealthAnimations)
    end

    local spawnDelay = objc.tick_timer(entity, 'spawnDelayTimer', WAIT_FOR_SPAWN_DELAY)
    if spawnDelay == 0 then
        objc.set_local_value(entity, IS_WAITING_FOR_SPAWN, 1)
    end

    local isWaitingForSpawn = objc.get_local_value(entity, IS_WAITING_FOR_SPAWN)
    if isWaitingForSpawn ~= 0 then
        local hasRevealed = objc.get_local_value(entity, HAS_REVEALED_KEY)
        if hasRevealed == 0 then
            local enemyCheckTimer = objc.tick_timer(entity, 'enemyCheckTimer', ENEMY_CHECK_TIMER)
            if enemyCheckTimer == 0 then
                local closestEntity = W_Sneaky_findEnemyWithinXY(entity, DASH_RANGE, HEIGHT_OF_CENTER_BLANK_AREA)

                if closestEntity ~= entity then
                    objc.set_local_value(entity, HAS_REVEALED_KEY, 1)

                    local regularAnimations = {}
                    regularAnimations[ANIMATION_STATE_ATTACK_GROUND] = REGULAR_ATTACK_ANIMATION_STATE
                    regularAnimations[ANIMATION_STATE_MOVE] = NORMAL_MOVE_STATE
                    objc.update_animation_state(entity, regularAnimations)


                    --JUMP TO TARGET HERE
                    local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_DASH_ANIMATION)
                    objc.play_override_animation(entity, DASH_ANIMATION, objc.subrounds_to_milliseconds(numberOfSubrounds))
                    --local motionStateDict = objc.move_entity_to_entity_with_motion(entity, closestEntity, MOTION_STATE_WALK, 1000)

                    local targetSize = objc.get_entity_value_for_key(closestEntity, ENTITY_VALUE_KEY_SIZE)
                    local targetDirectionX = objc.get_entity_value_for_key(closestEntity, ENTITY_VALUE_KEY_DIRECTION_X)
                    local targetDirectionY = objc.get_entity_value_for_key(closestEntity, ENTITY_VALUE_KEY_DIRECTION_Y)

                    local teamNumber = objc.get_team_number(entity)
                    local entitySize = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_SIZE)
                

                    targetDirectionX = targetDirectionX * -1
                    targetDirectionY = targetDirectionY * -1

                    local behindDistance = objc.divide_with_ceiling(targetSize + entitySize, 2)

                    targetDirectionX, targetDirectionY = objc.resize_vector(targetDirectionX, targetDirectionY, behindDistance)

                    local targetDestinationY = objc.get_entity_value_for_key(closestEntity, ENTITY_VALUE_KEY_POSITION_Y) + targetDirectionY
                    local targetDestinationX = objc.get_entity_value_for_key(closestEntity, ENTITY_VALUE_KEY_POSITION_X) + targetDirectionX

                    targetDestinationX = targetDestinationX + targetDirectionX
                    targetDestinationY = targetDestinationY + targetDirectionY

                    -- Is subtracted off in code, so use a positive value.
                    local velocityDamping = 105
                    local startingVelocity = 760
                    -- Set maxVelocity and maxAcceleration to be the starting velocity so that
                    -- We immediately begin motion at the startingVelocity, and decay down using the deceleration
                    local maxVelocity = startingVelocity    
                    local maxAcceleration = startingVelocity
                    local timeoutRounds = 10
                    local motionStateDict = objc.smoothly_move_entity_to_location_with_trigger(entity, entity, closestEntity, targetDestinationX,
                                                                                               targetDestinationY, MOTION_STATE_LEAP,
                                                                                               startingVelocity, maxVelocity, maxAcceleration,
                                                                                               100, velocityDamping,
                                                                                               'W_Sneaky.lua', DASH_FINISH_TRIGGER,
                                                                                               timeoutRounds)

                    objc.set_local_value(entity, MOTION_STATE_DICT, motionStateDict)

                    --objc.add_trigger(entity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'W_Sneaky.lua', DASH_FINISH_TRIGGER)
                else
                    closestEntity = W_Sneaky_findEnemyWithinXY(entity, SEARCH_RADIUS, HEIGHT_OF_CENTER_BLANK_AREA)
                    if closestEntity ~= entity then
                        local movementSpeed = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY)
                        objc.move_entity_to_entity_with_motion(entity, closestEntity, MOTION_STATE_WALK, movementSpeed)
                    end
                end
            end
        end
    end
end


W_Sneaky_canTargetTrigger = function(entity, targetEntity)
    local hasRevealed = objc.get_local_value(entity, HAS_REVEALED_KEY)
    if hasRevealed == 0 then
        local targetRange = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE)
        if targetRange > ENEMY_RANGE_THRESHOLD then
            return NO
        end
    end
    return YES
end


W_Sneaky_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
   objc.set_targeting(sourceEntity, targetEntity)
end

W_Sneaky_finishJumpingToTargetTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local motionStateDict = objc.get_local_value(sourceEntity, MOTION_STATE_DICT)
    if motionStateDict ~= 0 then
        objc.swap_out_entity_motion_state_with_dictionary(sourceEntity, motionStateDict)
    end
    if targetingFailed == 0 then
        local facingY = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_Y) - objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_POSITION_Y)
        local facingX = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_X) - objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_POSITION_X)
        objc.set_entity_facing_direction(sourceEntity, facingX, facingY)
        --only deal extra damage if the unit is a combat unit, not buildings or base
        local dashAttackDamage = objc.get_combat_entity_custom_stat(sourceEntity, CUSTOM_ENTITY_STAT_KEY_DASH_DAMAGE)
        objc.set_local_value(sourceEntity, COMBO_DAMAGE_KEY, dashAttackDamage)

        objc.set_local_value(sourceEntity, COMBO_TARGET_KEY, targetEntity)
        objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_Sneaky.lua', 'W_Sneaky_dealDamageTrigger')

        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_SUPER_ATTACK_ANIMATION)
        objc.idle_entity(sourceEntity, numberOfSubrounds)
        objc.play_override_animation(sourceEntity, SUPER_ATTACK_ANIMATION, objc.subrounds_to_milliseconds(numberOfSubrounds))
    else
        objc.set_targetable(entity, true)
        objc.set_solid(entity, true)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Sneaky_dealDamageTrigger')
    end
end

W_Sneaky_dealDamageTrigger = function(entity)
    local currentSwings = objc.get_local_value(entity, SWINGS_KEY)
    if currentSwings >= NUM_COMBO_SWINGS then
        objc.set_targetable(entity, true)
        objc.set_solid(entity, true)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Sneaky_dealDamageTrigger')
        return
    end

    local damageTickTimer = objc.tick_timer(entity, 'damageTickKey', COMBO_DAMAGE_DELAY)
    if damageTickTimer == 0 then
        local comboDamage = objc.get_local_value(entity, COMBO_DAMAGE_KEY)
        local damageToDeal = objc.divide_with_ceiling(comboDamage, NUM_COMBO_SWINGS)
        local comboTarget = objc.get_local_value(entity, COMBO_TARGET_KEY)
        objc.inflict_normal_damage(entity, comboTarget, damageToDeal)
        objc.flash_entity_for_damage(comboTarget, 250)
        objc.set_local_value(entity, SWINGS_KEY, currentSwings + 1)
    end
end

W_Sneaky_actionSystemLoopStartTrigger = function(sourceEntity, targetEntity)
    local hasRevealed = objc.get_local_value(sourceEntity, HAS_REVEALED_KEY)
    if hasRevealed == 0 then
        objc.set_targetable(sourceEntity, true)
        objc.set_solid(sourceEntity, true)

        local regularAnimations = {}
        regularAnimations[ANIMATION_STATE_ATTACK_GROUND] = REGULAR_ATTACK_ANIMATION_STATE
        regularAnimations[ANIMATION_STATE_MOVE] = NORMAL_MOVE_STATE
        objc.update_animation_state(sourceEntity, regularAnimations)

        objc.set_local_value(sourceEntity, HAS_REVEALED_KEY, 1)
    end
end

W_Sneaky_findEnemyWithinXY = function(entity, x, y)
    local enemyTeamNumber = objc.get_enemy_team_number(entity)
    local myLocationX = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
    local myLocationY = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y)
    local enemies = objc.list_entities_in_rectangle(myLocationX, myLocationY, x, y, enemyTeamNumber)
    local closestEntity = entity
    local closestDist = 99999999
    for _, enemy in pairs(enemies) do
        if W_Sneaky_isValidTarget(enemy) then
            local thisDist = objc.quick_unit_distance_square(entity, enemy)
            local thisRange = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_ATTACK_RANGE)
            if thisDist < closestDist and objc.is_targetable(enemy) and thisRange > ENEMY_RANGE_THRESHOLD then
                closestDist = thisDist
                closestEntity = enemy
            end
        end
    end
    return closestEntity
end

W_Sneaky_isValidTarget = function(enemy)
    if objc.is_ground_unit(enemy) and (objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) then
        return true
    end
    return false
end



