local EATEN_ENEMIES_KEY = 'eatenEnemiesKey'
local STASIS_LOCATION_X = 40000
local STASIS_IDLE_TIMER = 'stasisIdleTimer'
local STASIS_IDLE_TICK_TIME = 10
local DAMAGE_SOURCE_KEY = 'damageSourceKey'
local NUM_DAMAGE_TICKS = 5
local STASIS_IDLE_TIME = 1000
local STASIS_IDLE_TRIGGER = 'W_StasisFrog_stasisTrigger'

W_StasisFrog_damageTrigger = function(damage, targetEntity, sourceEntity)
	local frog = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
	local targetID = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_IDENTIFIER)
	local eatenEnemiesArray = objc.get_local_value(frog, EATEN_ENEMIES_KEY)
	if eatenEnemiesArray == 0 then
		eatenEnemiesArray = {}
	end
	local teamNumber = objc.get_team_number(targetEntity)
	local baseEntity = objc.get_base_entity(teamNumber)
	if targetEntity == baseEntity or objc.is_barricade(targetEntity) then
		return damage
	end

	objc.set_targetable(targetEntity, false)
	objc.stun_entity(targetEntity, STASIS_IDLE_TIME)
	objc.set_local_value(targetEntity, DAMAGE_SOURCE_KEY, frog)
	objc.add_trigger(targetEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_StasisFrog.lua', STASIS_IDLE_TRIGGER)
	table.insert(eatenEnemiesArray, targetEntity)
	objc.set_local_value(frog, EATEN_ENEMIES_KEY, eatenEnemiesArray)
	objc.idle_entity(frog, STASIS_IDLE_TICK_TIME * 100 * NUM_DAMAGE_TICKS)
	objc.apply_shader(targetEntity, 'skinned_toon_hologram')
	return 0
end

W_StasisFrog_deathTrigger = function(entity)
	local thisPos = objc.get_entity_position(entity)
	local eatenEnemiesArray = objc.get_local_value(entity, EATEN_ENEMIES_KEY)
	if eatenEnemiesArray ~= 0 then
		for _, enemy in pairs(eatenEnemiesArray) do
			objc.set_targetable(enemy, true)
			objc.remove_trigger(enemy, TRIGGER_NAME_TICK_TRIGGER, STASIS_IDLE_TRIGGER)
			objc.remove_shader(enemy, 'skinned_toon_hologram')
		end
	end
end

--prevent the unit from attacking while in siberia
W_StasisFrog_stasisTrigger = function(entity)
	local stasisTimer = objc.tick_timer(entity, STASIS_IDLE_TIMER, STASIS_IDLE_TICK_TIME)
	if stasisTimer == 0 then
		local maxHealth = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
		local damageSource = objc.get_local_value(entity, DAMAGE_SOURCE_KEY)
		local damageToDo = objc.divide_with_ceiling(maxHealth, NUM_DAMAGE_TICKS)
		objc.inflict_direct_damage(damageSource, entity, damageToDo)
		objc.stun_entity(entity, STASIS_IDLE_TIME)
	end
end
