local KNOCKBACK_DISTANCE = 3700
local CRATER_OFFSET = 350
local KNOCKBACK_SPEED = 350
local BASE_BUFFER = 2000
local ORIGINAL_RANGE_KEY = 'originalRangeKey'
local MELEE_RANGE = 550
local STUN_LENGTH = 1000
local END_RANGE_TRIGGER_NAME = 'W_Sumo_endRangeTrigger'
local TURN_AROUND_TRIGGER_NAME = 'W_Sumo_turnAround_tickTrigger'
local SUMO_KEY = 'originatingSumoEntity'
local MAIN_TARGET_KEY = 'mainSumoTargetKey'
local SHOULD_KNOCK_BACK = 'shouldKnockBackKey'

W_Sumo_onAttackTrigger = function(sourceEntity, targetEntity)
	local description = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_DESCRIPTION)
	objc.set_local_value(targetEntity, SHOULD_KNOCK_BACK, description)
end

--[[
This OnDamageTrigger caused the Sumo to knockback whatever unit it hits.
--]]
W_Sumo_damageTrigger = function(damage, targetEntity, sourceEntity)
	local shouldKnockBack = objc.get_local_value(targetEntity, SHOULD_KNOCK_BACK)
	local sumo = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
	local description = objc.get_entity_value_for_key(sumo, ENTITY_VALUE_KEY_DESCRIPTION)
	if objc.is_normal_unit(targetEntity) and shouldKnockBack == description then
		local playerNumber = objc.get_player_number(sourceEntity)
		local teamNumber = objc.get_team_number(sourceEntity)
		local targetDestinationX = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_X)
		local targetDestinationY = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_Y)
		local basePositionX = math.abs(objc.get_base_position(teamNumber)[1])
		local sumoPosition = objc.get_entity_position(sumo)
		local craterOffsetModifier = 1
		local oldRange = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE)

		if sumoPosition[1] < targetDestinationX then
			targetDestinationX = targetDestinationX + KNOCKBACK_DISTANCE + oldRange
			if targetDestinationX > basePositionX - BASE_BUFFER then
				targetDestinationX = basePositionX - BASE_BUFFER
			end
		else
			targetDestinationX = targetDestinationX - KNOCKBACK_DISTANCE - oldRange
			if targetDestinationX < -basePositionX + BASE_BUFFER then
				targetDestinationX = -basePositionX + BASE_BUFFER
			end
			craterOffsetModifier = -1
		end

		local targetArray = objc.spawn_at_point('SumoTargetEffectEntity', 1, playerNumber, targetDestinationX, targetDestinationY, 0, 0)
		objc.set_local_value(targetArray[1], SUMO_KEY, sumo)
		objc.set_local_value(targetArray[1], MAIN_TARGET_KEY, targetEntity)
        --Commenting this out as a workaround until the way this interacts with move_entity_to_entity_with_motion is fixed
		--[[objc.set_local_value(targetEntity, ORIGINAL_RANGE_KEY, oldRange)
		objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, MELEE_RANGE)]]
		objc.add_trigger(targetEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'W_Sumo.lua', END_RANGE_TRIGGER_NAME)
		local oldParams = objc.move_entity_to_entity_with_motion(targetEntity, targetArray[1], MOTION_STATE_LEAP, KNOCKBACK_SPEED)
		objc.set_entity_should_reverse_facing_direction(targetEntity, true)
		--Set the offset on the EffectEntity for where to spawn the impact effect
		objc.set_local_value(targetArray[1], 'craterSpawnPosition', targetDestinationX - CRATER_OFFSET * craterOffsetModifier * 2)

		objc.set_local_value(targetEntity, SHOULD_KNOCK_BACK, 0)

		return 0
	end

	return damage
end

W_Sumo_endRangeTrigger = function(sourceEntity, targetEntity)
	--[[local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
	objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)]]

	objc.stun_entity(sourceEntity, STUN_LENGTH)
	objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_Sumo.lua', TURN_AROUND_TRIGGER_NAME)
	objc.remove_trigger(sourceEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, END_RANGE_TRIGGER_NAME)
end

W_Sumo_turnAround_tickTrigger = function(entity)
	local timer = objc.tick_timer(entity, 'timer', 10)

    if timer == 0 then
    	objc.set_entity_should_reverse_facing_direction(entity, false)
   		objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, TURN_AROUND_TRIGGER_NAME)
    end
end