local TELEPORT_DISTANCE = 3000
local TELEPORT_TIME = 3
local ALLY_CHECK_RADIUS = 1000
local ORIGINAL_RANGE_KEY = 'originalAttackRangeKey'
local TELEPORT_TARGET_IDENTIFIER = 'teleporterTargetEntity'
local CURRENTLY_TELEPORTING = 'currentlyTeleportingKey'
local HAS_FRIENDS_KEY = 'hasFriendsKey'
local HAS_TELEPORTED = 'hasTeleportedKey'
local STASIS_LOCATION_X = 10000
local PREVIOUS_X_KEY = 'previousXKey'
local PREVIOUS_Y_KEY = 'previousYKey'
local SPAWN_DELAY_TIMER_KEY = 'spawnDelayTimerKey'
local SPAWN_TIME_ROUNDS = 0
local UNITS_TO_TELEPORT = 'unitsToTeleportKey'
local INHALE_ANIMATION_TIMER = 'inhaleAnimationTimerKey'
local EXHALE_ANIMATION_TIMER = 'exhaleAnimationTimerKey'
local FRAMES_IN_TELEPORT_IN_ANIMATION = 25
local TELEPORT_IN_ANIMATION_NAME = 'W_Teleporter_Teleport_In'
local FRAMES_IN_TELEPORT_MOVE_ANIMATION = 30
local TELEPORT_MOVE_ANIMATION_NAME = 'W_Teleporter_Teleport_Move'
local FRAMES_IN_TELEPORT_OUT_ANIMATION = 30
local TELEPORT_OUT_ANIMATION_NAME = 'W_Teleporter_Teleport_Out'
local INHALE_TIMING_TRIGGER_NAME = 'W_Teleporter_waitForIngressTrigger'
local EXHALE_TIMING_TRIGGER_NAME = 'W_Teleporter_waitForOutgressTrigger'
local TIME_TO_EXHALE = 2
local INHALE_LOGIC_DELAY = 4

--Used to check if a unit is a miner.
local IS_MINER_KEY = "isMinerKey"

W_Teleporter_spawnTrigger = function(entity)

    --set range to extremely low so that move_entity_to_entity_with_motion doesn't end early
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE, 100)

    --check and see if there's anything under it, because we should find stuff if to teleport if there's not
    local teamNumber = objc.get_team_number(entity)
    local allies = objc.list_entities_in_radius(entity, ALLY_CHECK_RADIUS, teamNumber)
    local numActualAllies = 0 
    for _, ally in pairs(allies) do
        --only count allies I'd actually be teleporting
        if objc.is_normal_unit(ally) and objc.is_ground_unit(ally) and objc.get_local_value(ally, IS_MINER_KEY) ~= 1 then
            numActualAllies = numActualAllies + 1
        end
    end

    if numActualAllies == 0 then
        local allAllies = objc.list_entities(teamNumber)
        local mapSize = objc.get_map_size()
        local closestAlly = entity
        local closestDistance = mapSize[1] * mapSize[1]
        for _, ally in pairs(allAllies) do
            local thisDistance = objc.quick_unit_distance_square(entity, ally)
            if thisDistance < closestDistance and objc.is_normal_unit(ally) and objc.is_ground_unit(ally) and objc.get_local_value(ally, IS_MINER_KEY) ~= 1 then
                closestAlly = ally
                closestDistance = thisDistance
            end
        end

        if closestAlly ~= entity then
            local closestAllyId = objc.get_entity_value_for_key(closestAlly, ENTITY_VALUE_KEY_IDENTIFIER)
            objc.move_entity_to_entity_with_motion(entity, closestAlly, MOTION_STATE_WALK, 500)
        end
    else
        --How much would it suck if your teleporter could die when it first spawns?
        --A lot. That's how much.
        objc.set_targetable(entity, false)
        local originalRange = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_ATTACK_RANGE)
        objc.set_local_value(entity, ORIGINAL_RANGE_KEY, originalRange)
        --We have friends.
        objc.set_local_value(entity, HAS_FRIENDS_KEY, 1)
    end
end

W_Teleporter_tickTrigger = function(entity)
    local hasFriends = objc.get_local_value(entity, HAS_FRIENDS_KEY)
    if hasFriends ~= 0 then
        local hasTeleported = objc.get_local_value(entity, HAS_TELEPORTED)
        if hasTeleported == 0 then
            local spawnDelayTimer = objc.tick_timer(entity, SPAWN_DELAY_TIMER_KEY, SPAWN_TIME_ROUNDS)
            if hasFriends == 2 then
                spawnDelayTimer = 0
            end
            if spawnDelayTimer == 0 then
                local teamNumber = objc.get_team_number(entity)

                --Play 'suck in' animation, add timing trigger
                local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TELEPORT_IN_ANIMATION)
                objc.idle_entity(entity, numberOfSubrounds)
                objc.play_override_animation(entity, TELEPORT_IN_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))
                objc.add_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'W_Teleporter.lua', INHALE_TIMING_TRIGGER_NAME)
                objc.set_local_value(entity, HAS_TELEPORTED, 1)

            end
        elseif hasTeleported < INHALE_LOGIC_DELAY then
            objc.set_local_value(entity, HAS_TELEPORTED, hasTeleported + 1)
        elseif hasTeleported == INHALE_LOGIC_DELAY then
            --search for nearby ground allies and start them teleporting
            local teamNumber = objc.get_team_number(entity)
            local allies = objc.list_entities_in_radius(entity, ALLY_CHECK_RADIUS, teamNumber)
            local unitsToTeleport = {}
            for _, ally in pairs(allies) do
                if objc.is_ground_unit(ally) and objc.is_normal_unit(ally) and objc.get_local_value(ally, IS_MINER_KEY) ~= 1 then
                    local allyPos = objc.get_entity_position(ally)
                    objc.set_local_value(ally, PREVIOUS_X_KEY, allyPos[1])
                    objc.set_local_value(ally, PREVIOUS_Y_KEY, allyPos[2])
                    W_Teleporter_placeInStasis(ally)
                    table.insert(unitsToTeleport, ally)
                end
            end
            objc.set_local_value(entity, UNITS_TO_TELEPORT, unitsToTeleport)
            objc.set_local_value(entity, HAS_TELEPORTED, hasTeleported + 1)
        end
    end
end

W_Teleporter_waitForIngressTrigger = function(entity)
    local inhaleTimerRounds = objc.convert_frames_to_rounds(FRAMES_IN_TELEPORT_IN_ANIMATION)
    local inhaleAnimationTime = objc.tick_timer(entity, INHALE_ANIMATION_TIMER, inhaleTimerRounds)
    if inhaleAnimationTime == 0 then
        local playerNumber = objc.get_player_number(entity)

        --spawn target entity
        local thisPos = objc.get_entity_position(entity)
        local teamNumber = objc.get_team_number(entity)
        local xFactor = 1
        if teamNumber == 2 then
            xFactor = -1
        end
        local targetEntity = objc.spawn_at_point(TELEPORT_TARGET_IDENTIFIER, 1, playerNumber, thisPos[1] + (TELEPORT_DISTANCE * xFactor), thisPos[2])
        local teleportVelocity = objc.divide_with_ceiling(TELEPORT_DISTANCE, TELEPORT_TIME)
        --play moving animation
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TELEPORT_MOVE_ANIMATION)
        objc.play_override_animation(entity, TELEPORT_MOVE_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

        --actually move
        objc.set_local_value(entity, CURRENTLY_TELEPORTING, 1)
        objc.move_entity_to_entity_with_motion(entity, targetEntity[1], MOTION_STATE_WALK, teleportVelocity)

        --remove this logic since we're done
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, INHALE_TIMING_TRIGGER_NAME)
    end
end

W_Teleporter_onOverrideTargetingCompletionTrigger = function(sourceEntity, targetEntity, targetingFailed)
    local hasFriends = objc.get_local_value(sourceEntity, HAS_FRIENDS_KEY)
    local isCurrentlyTeleporting = objc.get_local_value(sourceEntity, CURRENTLY_TELEPORTING)

    --only do this logic if we didn't find allies under me
    --set this value to 2 so we know it moved, and won't do the delay
    if hasFriends == 0 then
        objc.set_local_value(sourceEntity, HAS_FRIENDS_KEY, 2)
    end

    --only do this logic if we've moved because we're teleporting
    if isCurrentlyTeleporting == 1 then
        --play the 'spit out' animation
        local numberOfSubrounds = objc.convert_frames_to_subrounds(FRAMES_IN_TELEPORT_OUT_ANIMATION)
        objc.idle_entity(sourceEntity, numberOfSubrounds)
        objc.play_override_animation(sourceEntity, TELEPORT_OUT_ANIMATION_NAME, objc.subrounds_to_milliseconds(numberOfSubrounds))

        --wait until the right point in the animation to spit out the units
        objc.add_trigger(sourceEntity, TRIGGER_NAME_TICK_TRIGGER, 'W_Teleporter.lua', EXHALE_TIMING_TRIGGER_NAME)

        --we're done teleporting, set range back to normal, and mark it as such
        local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
        objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
        objc.set_targetable(sourceEntity, true)
        objc.set_local_value(sourceEntity, CURRENTLY_TELEPORTING, 0)
    end
end

W_Teleporter_waitForOutgressTrigger = function(entity)
    local exhaleAnimationTime = objc.tick_timer(entity, EXHALE_ANIMATION_TIMER, TIME_TO_EXHALE)
    if exhaleAnimationTime == 0 then
        local unitsTeleporting = objc.get_local_value(entity, UNITS_TO_TELEPORT)
        if unitsTeleporting ~= 0 then
            for _, unit in pairs(unitsTeleporting) do
                W_Teleporter_removeFromStasis(unit)
            end
            objc.set_local_value(entity, UNITS_TO_TELEPORT, 0)
        end

        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, EXHALE_TIMING_TRIGGER_NAME)
    end
end

W_Teleporter_placeInStasis = function(entity)
    --teleport the unit to siberia, make it untargetable, and idle for the duration of the teleport
    objc.set_targetable(entity, false)
    objc.teleport_entity(entity, STASIS_LOCATION_X, 0)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
    objc.idle_entity(entity, TELEPORT_TIME * 100)
end

W_Teleporter_removeFromStasis = function(entity)
    --undo the things we did to send the unit to siberia
    local oldPosX = objc.get_local_value(entity, PREVIOUS_X_KEY)
    local oldPosY = objc.get_local_value(entity, PREVIOUS_Y_KEY)
    local teamNumber = objc.get_team_number(entity)
    local xOff = 1
    if teamNumber == 2 then
        xOff = -1
    end
    objc.set_targetable(entity, true)
    objc.teleport_entity(entity, oldPosX + (TELEPORT_DISTANCE * xOff), oldPosY)
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_VELOCITY, 60)
end
