////////////////////////////////////////////////////////////////////////////////////////////////////

// Here are some helper #defines & function to assist in bridging
// between the HLSL-based XLE shader code and our target GLSL variant

#if !defined(XLE_SHADER_BRIDGE_H)
#define XLE_SHADER_BRIDGE_H

#define float2 vec2
#define float3 vec3
#define float4 vec4
#define int2 ivec2
#define int3 ivec3
#define int4 ivec4
#define uint2 uvec2
#define uint3 uvec3
#define uint4 uvec4
#define bool2 bvec2
#define bool3 bvec3
#define bool4 bvec4
#define TextureCube samplerCube
#define Texture2D sampler2D
#define static
#define precise highp
#define BRANCH_DIRECTIVE
#define LOOP_DIRECTIVE


#define TAPER_OFF_FRESNEL 0
#define MAT_DOUBLE_SIDED_LIGHTING 0
#define MAT_CLAMP_SPEC 0
#define HAS_DIFFUSE_IBL 1
#define MAT_SEPARATE_REFRACTION_MAP 0

#define rcp(x) (1.0f/(x))
#define rsqrt(x) rcp(sqrt(x))
#define atan2(y, x) atan(y, x)
#define lerp(x, y, a) mix(x, y, a)
#define sincos(theta, s, c) { s = sin(theta); c=cos(theta); }

#define ddx dFdx
#define ddy dFdy

float saturate(float value)     { return clamp(value, 0.0f, 1.0f); }
float2 saturate(float2 value)   { return clamp(value, float2(0.0f), float2(1.0f)); }
float3 saturate(float3 value)   { return clamp(value, float3(0.0f), float3(1.0f)); }
float4 saturate(float4 value)   { return clamp(value, float4(0.0f), float4(1.0f)); }

uint reversebits(uint bits)
{
    bits = (bits << 16u) | (bits >> 16u);
    bits = ((bits & 0x55555555u) << 1u) | ((bits & 0xAAAAAAAAu) >> 1u);
    bits = ((bits & 0x33333333u) << 2u) | ((bits & 0xCCCCCCCCu) >> 2u);
    bits = ((bits & 0x0F0F0F0Fu) << 4u) | ((bits & 0xF0F0F0F0u) >> 4u);
    bits = ((bits & 0x00FF00FFu) << 8u) | ((bits & 0xFF00FF00u) >> 8u);
    return bits;
}

#include "library/texture_utility.sh"

float4 Load(sampler2D sampler, int3 coords)                                     { return textureLod(sampler, float2(coords.xy), 0.0f); }
float4 SampleLevel_Clamp(sampler2D sampler, float2 coords, float mipmap)        { return textureLod(sampler, saturate(coords), mipmap); }
float4 SampleLevelZero_Default(sampler2D sampler, float2 coords)                { return textureLod(sampler, coords, 0.0f); }
float4 SampleLevelZero_Default(samplerCube sampler, float3 coords)              { return textureLod(sampler, coords, 0.0f); }
float4 SampleLevel_Default(sampler2D sampler, float2 coords, float mipmap)      { return textureLod(sampler, coords, mipmap); }
float4 SampleLevel_Default(samplerCube sampler, float3 coords, float mipmap)    { return textureLod(sampler, coords, mipmap); }

#endif

