local ARROW_COUNT_KEY = 'arrowCount'
local ABILITY_ID_KEY = 'abilityIDKey'
local ARROW_AOE_SIZE = 350
local NUM_ARROW_ROWS = 5
local ARROWS_IN_ROW = 1
local RANDOM_RADIUS = 350
local LEFT_X_OFFSET = 1200
local RIGHT_X_OFFSET = 3900
local UP_Y_OFFSET = 0
local DOWN_Y_OFFSET = 0
local TIER_KEY = 'tierKey'

--[[
This TickTrigger is the manifestation of the BombingRun hero ability. It
spawns a set number of arrows at a given interval, and then kills itself.
--]]
fireArrowsTargetedScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)

   local intervalTimer = objc.tick_timer(entity, 'intervalTimer', 0)
   local teamNumber = objc.get_team_number(entity)

   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)

   if arrowCount >= (NUM_ARROW_ROWS * ARROWS_IN_ROW) then
      objc.sacrifice_entity(entity)
      return 0
   end

   if intervalTimer == 0 then
      fireArrowsTargetedScriptEntity_spawnArrow(teamNumber, entity)
   end
end

--[[
This function spawns a roxiAbility3EffectEntity at a random location on the map
--]]
fireArrowsTargetedScriptEntity_spawnArrow = function(teamNumber, entity)
   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)
   local effectIdentifier = 'roxiAbility3EffectEntity'

   local entityPos = objc.get_entity_position(entity)
   local xPos = entityPos[1]
   local yPos = entityPos[2]
   local startXPos = xPos
   local endXPos = xPos
   local xMod = 1

   if teamNumber == 1 then
      startXPos = xPos + LEFT_X_OFFSET
      endXPos = xPos + RIGHT_X_OFFSET
   elseif teamNumber == 2 then 
      startXPos = xPos - LEFT_X_OFFSET
      endXPos = xPos - RIGHT_X_OFFSET
      xMod = -1
      effectIdentifier = 'roxiAbility3EffectEntityPlayer2'
   else
      objc.scripting_error('Unknown player number encountered: ' .. teamNumber)
   end

   local startYPos = yPos + DOWN_Y_OFFSET
   local endYPos = yPos + UP_Y_OFFSET
   local yIndex = arrowCount % ARROWS_IN_ROW
   local yStep = objc.divide_with_floor((endYPos - startYPos), ARROWS_IN_ROW)
   local thisY = (yStep * yIndex) + startYPos

   local xIndex = objc.divide_with_floor(arrowCount, ARROWS_IN_ROW)
   local xStep = objc.divide_with_floor((endXPos - startXPos), NUM_ARROW_ROWS)
   local thisX = (xStep * xIndex) + startXPos

   local playerNumber = objc.get_player_number(entity)

   local spawnedArrowArray = objc.spawn_at_point(effectIdentifier, 1, playerNumber, thisX, thisY, xMod, 0)
   objc.set_local_value(spawnedArrowArray[1], ABILITY_ID_KEY, objc.get_entity_value(entity, 'identityComponent.description'))
   objc.set_local_value(entity, ARROW_COUNT_KEY, arrowCount + 1)
   tierOfRoxie = objc.get_local_value(entity, TIER_KEY)
   objc.set_local_value(spawnedArrowArray[1], TIER_KEY, tierOfRoxie)
end
