
#if !defined(FRAMEWORK_SH)
#define FRAMEWORK_SH

#include "surface.sh"

////////////////////////////////////////////////////////////////////////////////////////////////////

#if !defined(VS_IN_S_POSITION)
    #define VS_IN_S_POSITION 0
#endif
#if !defined(VS_IN_S_TEXCOORD)
    #define VS_IN_S_TEXCOORD 0
#endif
#if !defined(VS_IN_S_COLOR)
    #define VS_IN_S_COLOR 0
#endif
#if !defined(VS_IN_S_BONE_WEIGHTS)
    #define VS_IN_S_BONE_WEIGHTS 0
#endif
#if !defined(VS_IN_S_TANGENT_FRAME)
    #define VS_IN_S_TANGENT_FRAME 0
#endif

#if !defined(VS_OUT_S_TEXCOORD)
    #define VS_OUT_S_TEXCOORD 0
#endif
#if !defined(VS_OUT_S_COLOR)
    #define VS_OUT_S_COLOR 0
#endif
#if !defined(VS_OUT_S_TANGENT_FRAME)
    #define VS_OUT_S_TANGENT_FRAME 0
#endif
#if !defined(VS_OUT_S_WORLD_SPACE_POSITION)
    #define VS_OUT_S_WORLD_SPACE_POSITION 0
#endif
#if !defined(VS_OUT_S_SHADOW_SAMPLE)
    #define VS_OUT_S_SHADOW_SAMPLE 0
#endif
#if !defined(VS_OUT_S_COLOR)
    #define VS_OUT_S_COLOR 0
#endif
#if !defined(SIMPLE_VERTEX_COLOR)
    #define SIMPLE_VERTEX_COLOR 0
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
    //      C O M P I L E R   F E A T U R E   S E T

#if defined(METAL_INSTEAD_OF_OPENGL)
/* KenD -- Metal TODO -- do not support array inputs for vertex attributes in Metal */
#else

// DavidJ -- angle doesn't appear to support array inputs correctly; so disabling on windows
#if (__VERSION__ >= 300) && !CC3_PLATFORM_WINDOWS
    #define SUPPORT_ARRAY_INPUTS
#endif
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

#if !VS_IN_S_POSITION && !VS_IN_S_TEXCOORD && (!VS_IN_S_COLOR || SIMPLE_VERTEX_COLOR != 0) && !VS_IN_S_BONE_WEIGHTS && !VS_IN_S_TANGENT_FRAME && !defined(INSTANCING_ENABLED)
    #define VSIN float
#else

struct VSIN {
    #if VS_IN_S_POSITION
        vec4 _position;
    #endif
        
    #if VS_IN_S_TEXCOORD > 0
        vec2 _texCoord[VS_IN_S_TEXCOORD];
    #endif

    #if VS_IN_S_COLOR > 0 && SIMPLE_VERTEX_COLOR == 0
        vec4 _color[VS_IN_S_COLOR];
    #endif

    #if VS_IN_S_BONE_WEIGHTS
        vec4 _boneWeights;
        vec4 _boneIndices;
    #endif
        
    #if VS_IN_S_TANGENT_FRAME
        RawTangentFrame _tangentFrame;
    #endif
    
    #if defined(INSTANCING_ENABLED)
        uint _instanceID;
    #endif
};

#endif

struct VSOUT {
    vec4 _position;
    
    #if VS_OUT_S_TEXCOORD > 0
        vec2 _texCoord[VS_OUT_S_TEXCOORD];
    #endif
    
    #if VS_OUT_S_COLOR > 0 && SIMPLE_VERTEX_COLOR == 0
        vec4 _color[VS_OUT_S_COLOR];
    #endif
    
    #if VS_OUT_S_WORLD_SPACE_POSITION
        vec3 _worldSpacePosition;
    #endif
    
    #if VS_OUT_S_TANGENT_FRAME
        TangentFrame _tangentFrame;
    #endif
};

vec3 getPosition(VSIN vsin) {
    #if VS_IN_S_POSITION
        return vsin._position.xyz;
    #else
        return vec3(0,0,0);
    #endif
}

RawTangentFrame getTangentFrame(VSIN vsin) {
    #if VS_IN_S_TANGENT_FRAME
        return vsin._tangentFrame;
    #else
        RawTangentFrame result;
        result._axesA = vec3(0,0,0);
        result._axesB = vec3(0,0,0);
        result._handiness = 0.0;
        result._type = 0;
        return result;
    #endif
}

vec3 getNormal(VSIN vsin) {
    return asFullTangentFrame(getTangentFrame(vsin))._normal;
}

vec2 getTexCoord(VSIN vsin, int idx) {
    #if VS_IN_S_TEXCOORD > 0
        return vsin._texCoord[idx];
    #else
        return vec2(0,0);
    #endif
}

void setTexCoord(inout_VSOUT vsout, int idx, vec2 newValue) {
    #if VS_OUT_S_TEXCOORD > 0
        vsout._texCoord[idx] = newValue;
    #endif
}

vec4 getColor(VSIN vsin, int idx) {
    #if VS_IN_S_COLOR > 0 && SIMPLE_VERTEX_COLOR == 0
        return vsin._color[idx];
    #else
        return vec4(0,0,0,1);
    #endif
}

void setColor(inout_VSOUT vsout, int idx, vec4 newValue) {
    #if VS_OUT_S_COLOR > 0 && SIMPLE_VERTEX_COLOR == 0
        vsout._color[idx] = newValue;
    #endif
}

void setTangentFrame(inout_VSOUT vsout, TangentFrame tf) {
    #if VS_OUT_S_TANGENT_FRAME
        vsout._tangentFrame = tf;
    #endif
}

void setTangentFrame(inout_VSOUT vsout, RawTangentFrame tf) {
    #if VS_OUT_S_TANGENT_FRAME
        vsout._tangentFrame = asFullTangentFrame(tf);
    #endif
}

void setWorldSpacePosition(inout_VSOUT vsout, vec3 worldSpacePosition) {
    #if VS_OUT_S_WORLD_SPACE_POSITION
        vsout._worldSpacePosition = worldSpacePosition;
    #endif
}

#endif


////////////////////////////////////////////////////////////////////////////////////////////////////
    //      H E L P E R   M A C R O S

// #define S_POSITION              1
// #define S_TEXCOORD              2
// #define S_COLOR                 3
// #define S_BONE_WEIGHTS          4
// #define S_TANGENT_FRAME         5
// #define S_WORLD_SPACE_POSITION  6
// #define STITCH(X, Y)            X ## Y
// #define VS_IN(X)                STITCH(VS_IN_, X) != 0
// #define VS_OUT(X)               STITCH(VS_OUT_, X)

