
#if !defined(FRAMEWORK_GLSLES_VERT)
#define FRAMEWORK_GLSLES_VERT

#include "framework/framework.sh"
#include "framework/skinning.sh"

#include "repeat.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
    //      C O N F I G U R A T I O N

// DavidJ -- angle doesn't appear to support array inputs correctly; so disabling on windows
#if (__VERSION__ >= 300) && !CC3_PLATFORM_WINDOWS
    #define SUPPORT_ARRAY_INPUTS
#endif

#if defined(CC3_PLATFORM_ANDROID) && CC3_PLATFORM_ANDROID
    #undef SUPPORT_ARRAY_INPUTS
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
    //      I N P U T   A T T R I B U T E S

#if VS_IN_S_POSITION
    ATTRIBUTE vec4 a_cc3Position;
#endif

#if VS_IN_S_TEXCOORD > 0
    #if defined(SUPPORT_ARRAY_INPUTS)
        ATTRIBUTE vec2 a_cc3TexCoord[VS_IN_S_TEXCOORD];
    #else
        ATTRIBUTE vec2 a_cc3TexCoord;
    #endif
#endif

#if VS_IN_S_COLOR > 0
    #if defined(SUPPORT_ARRAY_INPUTS)
        ATTRIBUTE vec4 a_cc3Color[VS_IN_S_COLOR];
    #else
        ATTRIBUTE vec4 a_cc3Color;
    #endif
#endif

#if VS_IN_S_BONE_WEIGHTS
    ATTRIBUTE vec4 a_cc3BoneWeights;
    ATTRIBUTE vec4 a_cc3BoneIndices;
#endif

#if VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
    ATTRIBUTE vec3 a_cc3Normal;
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_FULL
    ATTRIBUTE vec3 a_cc3Normal;
    ATTRIBUTE vec3 a_cc3Tangent;
    ATTRIBUTE vec3 a_cc3Bitangent;
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
    ATTRIBUTE vec3 a_cc3Normal;
    ATTRIBUTE vec4 a_cc3Tangent;
#elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_TB
    ATTRIBUTE vec4 a_cc3Tangent;
    ATTRIBUTE vec3 a_cc3Bitangent;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
//      O U T P U T   A T T R I B U T E S   (   V A R Y I N G S   )

#if VS_OUT_S_TEXCOORD
    VARYING_OUT vec2 v_texCoord[VS_OUT_S_TEXCOORD];
#endif

#if VS_OUT_S_COLOR
    VARYING_OUT vec4 v_color[VS_OUT_S_COLOR];
#endif

#if VS_OUT_S_WORLD_SPACE_POSITION
    VARYING_OUT vec3 v_position_world;
#endif

#if VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
    VARYING_OUT vec3 v_N;
#elif VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL
    VARYING_OUT vec3 v_N;
    VARYING_OUT vec3 v_T;
    VARYING_OUT vec3 v_B;
#endif

#if VS_OUT_S_SHADOW_SAMPLE
    VARYING_OUT vec4 v_shadow;
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

void main() {
    VSIN vsin;
    #if VS_IN_S_POSITION
        vsin._position = a_cc3Position;
    #endif
    
    int c;
    #if VS_IN_S_TEXCOORD > 0
        #if defined(SUPPORT_ARRAY_INPUTS)
            for (c=0; c<VS_OUT_S_TEXCOORD; ++c) {
                vsin._texCoord[c] = a_cc3TexCoord[c];
            }
        #else
            vsin._texCoord[0] = a_cc3TexCoord;
        #endif
        vsin._texCoord[0] = vsin._texCoord[0];
    #endif
    
    #if VS_IN_S_COLOR > 0
        #if defined(SUPPORT_ARRAY_INPUTS)
            for (c=0; c<VS_OUT_S_COLOR; ++c) {
#if SIMPLE_VERTEX_COLOR > 0
                v_color[c] = a_cc3Color[c];
#else
                vsin._color[c] = a_cc3Color[c];
#endif
            }
        #else
#if SIMPLE_VERTEX_COLOR > 0
                v_color[0] = a_cc3Color;
#else
                vsin._color[0] = a_cc3Color;
#endif
        #endif
    #endif
    
    #if VS_IN_S_BONE_WEIGHTS
        vsin._boneWeights = a_cc3BoneWeights;
        vsin._boneIndices = a_cc3BoneIndices;
    #endif
        
    #if VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = vec3(0,0,0);
        vsin._tangentFrame._handiness = 0.0;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_FULL
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = a_cc3Tangent;
        vsin._tangentFrame._handiness = dot(cross(a_cc3Normal, a_cc3Tangent), a_cc3Bitangent) >= 0.0 ? 1.0 : -1.0;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
        vsin._tangentFrame._axesA = a_cc3Normal;
        vsin._tangentFrame._axesB = a_cc3Tangent.xyz;
        vsin._tangentFrame._handiness = a_cc3Tangent.w;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #elif VS_IN_S_TANGENT_FRAME == TANGENT_FRAME_NT
        vsin._tangentFrame._axesA = a_cc3Tangent.xyz;
        vsin._tangentFrame._axesB = a_cc3Bitangent;
        vsin._tangentFrame._handiness = a_cc3Tangent.w;
        vsin._tangentFrame._type = VS_IN_S_TANGENT_FRAME;
    #endif
    
    VSOUT vsout;
    vsMain(vsout, vsin);
    
    gl_Position = vsout._position;
    
    #if VS_OUT_S_TEXCOORD > 0
        FOR(c, 0, VS_OUT_S_TEXCOORD, v_texCoord[c] = vsout._texCoord[c])
    #endif
    
    #if VS_OUT_S_COLOR > 0 && SIMPLE_VERTEX_COLOR == 0
        FOR(c, 0, VS_OUT_S_COLOR, v_color[c] = vsout._color[c])
    #endif
    
    #if VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
        v_N = vsout._tangentFrame._normal;
    #elif VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL
        v_N = vsout._tangentFrame._normal;
        v_T = vsout._tangentFrame._tangent;
        v_B = vsout._tangentFrame._bitangent;
    #endif
    
    #if VS_OUT_S_WORLD_SPACE_POSITION
        v_position_world = vsout._worldSpacePosition;
    #endif
    
    #if VS_OUT_S_SHADOW_SAMPLE
        v_shadow = vec4(0,0,0,0);
    #endif
}

#endif
