local GAME_HAS_INIT = 'gameRulesHasInit'
local STARTING_SPEED_MULTIPLIER = 1.5
local MAX_SPEED_MULTIPLIER = 2
local START_OF_ESCALATION_IN_SECONDS = 120
local MAX_GAME_LENGTH_IN_SECONDS = 180

local INITIAL_MERCENARY_REGION_TEAM_ONE = objc.get_mercenary_placement_region(1, PLACEMENT_REGION_KEY_INITIAL)
local INITIAL_MERCENARY_REGION_TEAM_TWO = objc.get_mercenary_placement_region(2, PLACEMENT_REGION_KEY_INITIAL)
local TOP_MERCENARY_REGION_TEAM_ONE = {}
local BOTTOM_MERCENARY_REGION_TEAM_ONE = {}
local TOP_MERCENARY_REGION_TEAM_TWO = {}
local BOTTOM_MERCENARY_REGION_TEAM_TWO = {}

local INITIAL_TACTIC_REGION_TEAM_ONE = objc.get_tactic_placement_region(1, PLACEMENT_REGION_KEY_INITIAL)
local INITIAL_TACTIC_REGION_TEAM_TWO = objc.get_tactic_placement_region(2, PLACEMENT_REGION_KEY_INITIAL)
local TOP_TACTIC_REGION_TEAM_ONE = {}
local BOTTOM_TACTIC_REGION_TEAM_ONE = {}
local TOP_TACTIC_REGION_TEAM_TWO = {}
local BOTTOM_TACTIC_REGION_TEAM_TWO = {}

local ONE_CONDITION_MET_PLACEMENT_BONUS = 1800
local BOTH_CONDITIONS_MET_PLACEMENT_BONUS = 2400

local TACTICS_PLACEMENT_BONUS = 6000

local GRID_SIZE = 600
local GRID_OFFSET_X = GRID_SIZE
 -- There is an odd amount of cells in the Y so offset correctly for this (middle row is not placable until both regions have been extended)
local GRID_OFFSET_Y = GRID_SIZE / 2

local PLACEMENT_REGION_KEY_TOP = "Top"
local PLACEMENT_REGION_KEY_BOTTOM = "Bottom"
local TOP_LANE_CENTER_OBJECTIVE_TAG
local BOTTOM_LANE_CENTER_OBJECTIVE_TAG

gameRulesEntityTwoLane_tickTrigger = function(entity)
    local gameRulesHasInit = objc.get_local_value(entity, GAME_HAS_INIT)
    if gameRulesHasInit == 0 then
        objc.set_placement_transition_lerp_visibility(false)

        --[[For both players, initialize two empty rectangles, one for each lane.
        The empty rectangles start one placementCellSize inwards from the initial region's edge that is closest to the center.]]
        local teamOnePlacementX = INITIAL_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X] + INITIAL_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]
        local MERCENARY_SUB_REGION_WIDTH_TEAM_ONE = INITIAL_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2
        local MERCENARY_SUB_REGION_WIDTH_TEAM_TWO = INITIAL_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]/2

        BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X] = teamOnePlacementX - GRID_OFFSET_X
        BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y] = GRID_OFFSET_Y
        BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] = MERCENARY_SUB_REGION_WIDTH_TEAM_ONE - GRID_OFFSET_Y
        objc.set_mercenary_placement_region(1, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH])


        TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X] = teamOnePlacementX - GRID_OFFSET_X
        TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y] = -MERCENARY_SUB_REGION_WIDTH_TEAM_ONE
        TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] = MERCENARY_SUB_REGION_WIDTH_TEAM_ONE - GRID_OFFSET_Y
        -- add GRID_SIZE to width to initially fill the gap between the two even regions
        objc.set_mercenary_placement_region(1, PLACEMENT_REGION_KEY_TOP, TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                         TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                         TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH],
                                                                         TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] + GRID_SIZE)

        local teamTwoPlacementX = INITIAL_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X] - GRID_OFFSET_X
        BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X] = teamTwoPlacementX + GRID_OFFSET_X
        BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y] = GRID_OFFSET_Y
        BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] = MERCENARY_SUB_REGION_WIDTH_TEAM_TWO - GRID_OFFSET_Y
        objc.set_mercenary_placement_region(2, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH],
                                                                            BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH])


        TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X] = teamTwoPlacementX + GRID_OFFSET_X
        TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y] = -MERCENARY_SUB_REGION_WIDTH_TEAM_TWO
        TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] = MERCENARY_SUB_REGION_WIDTH_TEAM_TWO - GRID_OFFSET_Y
        -- add GRID_SIZE to width to initially fill the gap between the two even regions
        objc.set_mercenary_placement_region(2, PLACEMENT_REGION_KEY_TOP, TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X],
                                                                         TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                         TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH],
                                                                         TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] + GRID_SIZE)


        --Do the same thing for tactics
        local teamOnePlacementX = INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X] + INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]

        BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X] = teamOnePlacementX - GRID_OFFSET_X
        BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y] = GRID_OFFSET_Y
        BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] = INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2 - GRID_OFFSET_Y
        objc.set_tactic_placement_region(1, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                         BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                         BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH],
                                                                         BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH])

        TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X] = teamOnePlacementX - GRID_OFFSET_X
        TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y] = -INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2
        TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] = INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2 - GRID_OFFSET_Y
        objc.set_tactic_placement_region(1, PLACEMENT_REGION_KEY_TOP, TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                      TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                      TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH],
                                                                      TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH] + GRID_SIZE)

        local teamTwoPlacementX = INITIAL_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X] - GRID_OFFSET_X
        BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X] = teamTwoPlacementX + GRID_OFFSET_X
        BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y] = GRID_OFFSET_Y
        BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] = INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2 - GRID_OFFSET_Y
        objc.set_tactic_placement_region(2, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X],
                                                                         BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                         BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH],
                                                                         BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH])

        TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X] = teamTwoPlacementX + GRID_OFFSET_X
        TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y] = -INITIAL_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]/2
        TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH] = GRID_OFFSET_X
        TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] = INITIAL_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]/2 - GRID_OFFSET_Y
        objc.set_tactic_placement_region(2, PLACEMENT_REGION_KEY_TOP, TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X],
                                                                      TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                      TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH],
                                                                      TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH] + GRID_SIZE)

        --putting more stuff in here will make it run only once at the beginning of the game.
        objc.set_local_value(entity, GAME_HAS_INIT, 1)
    end

    if objc.get_global_value("placementHasChanged") == 1 then
        objc.set_placement_transition_lerp_visibility(true)

        --print("UPDATE PLACEMENT AREA")
        local barricadeTeamOneTopDestroyed = objc.get_global_value("BarricadeTeamOneTopDestroyed") == 1
        local barricadeTeamOneBottomDestroyed = objc.get_global_value("BarricadeTeamOneBottomDestroyed") == 1
        local barricadeTeamTwoTopDestroyed = objc.get_global_value("BarricadeTeamTwoTopDestroyed") == 1
        local barricadeTeamTwoBottomDestroyed = objc.get_global_value("BarricadeTeamTwoBottomDestroyed") == 1

        -- figure out which center objective is on which lane and save the result
        if not TOP_LANE_CENTER_OBJECTIVE_TAG or not BOTTOM_LANE_CENTER_OBJECTIVE_TAG then
            if objc.get_global_value("CenterObjective1Y") < 0 then
                TOP_LANE_CENTER_OBJECTIVE_TAG = "CenterObjective1"
                BOTTOM_LANE_CENTER_OBJECTIVE_TAG = "CenterObjective2"
            else
                TOP_LANE_CENTER_OBJECTIVE_TAG = "CenterObjective2"
                BOTTOM_LANE_CENTER_OBJECTIVE_TAG = "CenterObjective1"
            end
        end

        -- top lane ----------------------------------------------------------------
        local topPlacementTeam1Bonus = 0
        local topPlacementTeam2Bonus = 0
        local topTacticPlacementTeam1Bonus = 0
        local topTacticPlacementTeam2Bonus = 0

        -- team 1 related things
        local topCenterObjectiveControllingTeam = objc.get_global_value(TOP_LANE_CENTER_OBJECTIVE_TAG .. "CapturedByTeam")
        if topCenterObjectiveControllingTeam == 1 then
            --print("team 1 controls top lane objective")
            if barricadeTeamTwoTopDestroyed then
                --print("team 1 has destroyed enemy's top barrier")
                topPlacementTeam1Bonus = 2
                topTacticPlacementTeam1Bonus = 1
            else
                --print("team 1 has NOT destroyed enemy's top barrier")
                topPlacementTeam1Bonus = 1
            end
        else
            --print("team 1 does NOT control top lane objective")
            if barricadeTeamTwoTopDestroyed then
                --print("team 1 has destroyed top barrier")
                topPlacementTeam1Bonus = 1
                topTacticPlacementTeam1Bonus = 1
            end
        end

        -- team 2 related things
        if topCenterObjectiveControllingTeam == 2 then
            --print("team 2 controls top lane objective")
            if barricadeTeamOneTopDestroyed then
                --print("team 2 has destroyed enemy's top barrier")
                topPlacementTeam2Bonus = 2
                topTacticPlacementTeam2Bonus = 1
            else
                --print("team 2 has NOT destroyed enemy's top barrier")
                topPlacementTeam2Bonus = 1
            end
        else
            --print("team 2 does NOT control top lane objective")
            if barricadeTeamOneTopDestroyed then
                --print("team 2 has destroyed barrier")
                topPlacementTeam2Bonus = 1
                topTacticPlacementTeam2Bonus = 1
            end
        end


        -- bottom lane ------------------------------------------------------------
        local bottomPlacementTeam1Bonus = 0
        local bottomPlacementTeam2Bonus = 0
        local bottomTacticPlacementTeam1Bonus = 0
        local bottomTacticPlacementTeam2Bonus = 0

        -- team 1 related things
        local bottomCenterObjectiveControllingTeam = objc.get_global_value(BOTTOM_LANE_CENTER_OBJECTIVE_TAG .. "CapturedByTeam")
        if bottomCenterObjectiveControllingTeam == 1 then
            --print("team 1 controls bottom lane objective")
            if barricadeTeamTwoBottomDestroyed then
                --print("team 1 has destroyed enemy's bottom barrier")
                bottomPlacementTeam1Bonus = 2
                bottomTacticPlacementTeam1Bonus = 1
            else
                --print("team 1 has NOT destroyed enemy's bottom barrier")
                bottomPlacementTeam1Bonus = 1
            end
        else
            --print("team 1 does NOT control bottom lane objective")
            if barricadeTeamTwoBottomDestroyed then
                --print("team 1 has destroyed barrier")
                bottomPlacementTeam1Bonus = 1
                bottomTacticPlacementTeam1Bonus = 1
            end
        end

        -- team 2 related things
        if bottomCenterObjectiveControllingTeam == 2 then
            --print("team 2 controls bottom lane objective")
            if barricadeTeamOneBottomDestroyed then
                --print("team 2 has destroyed enemy's bottom barrier")

                bottomPlacementTeam2Bonus = 2
                bottomTacticPlacementTeam2Bonus = 1
            else
                --print("team 2 has NOT destroyed enemy's bottom barrier")
                bottomPlacementTeam2Bonus = 1
            end
        else
            --print("team 2 does NOT control bottom lane objective")
            if barricadeTeamOneBottomDestroyed then
                --print("team 2 has destroyed barrier")
                bottomPlacementTeam2Bonus = 1
                bottomTacticPlacementTeam2Bonus = 1
            end
        end


        -- calculating the length for top
        local topPlacementLengthTeam1 = TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]
        if topPlacementTeam1Bonus == 2 then
            topPlacementLengthTeam1 = topPlacementLengthTeam1 + ONE_CONDITION_MET_PLACEMENT_BONUS + BOTH_CONDITIONS_MET_PLACEMENT_BONUS
        elseif topPlacementTeam1Bonus == 1 then
            topPlacementLengthTeam1 = topPlacementLengthTeam1 + ONE_CONDITION_MET_PLACEMENT_BONUS
        end

        local topPlacementLengthTeam2 = TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH]
        local topPlacementXTeam2 = TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X]
        if topPlacementTeam2Bonus == 2 then
            topPlacementXTeam2 = topPlacementXTeam2 - ONE_CONDITION_MET_PLACEMENT_BONUS - BOTH_CONDITIONS_MET_PLACEMENT_BONUS
            topPlacementLengthTeam2 = topPlacementLengthTeam2 + ONE_CONDITION_MET_PLACEMENT_BONUS + BOTH_CONDITIONS_MET_PLACEMENT_BONUS
        elseif topPlacementTeam2Bonus == 1 then
            topPlacementXTeam2 = topPlacementXTeam2 - ONE_CONDITION_MET_PLACEMENT_BONUS
            topPlacementLengthTeam2 = topPlacementLengthTeam2 + ONE_CONDITION_MET_PLACEMENT_BONUS
        end

        -- calculating the length for bottom
        local bottomPlacementLengthTeam1 = BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]
        if bottomPlacementTeam1Bonus == 2 then
            bottomPlacementLengthTeam1 = bottomPlacementLengthTeam1 + ONE_CONDITION_MET_PLACEMENT_BONUS + BOTH_CONDITIONS_MET_PLACEMENT_BONUS
        elseif bottomPlacementTeam1Bonus == 1 then
            bottomPlacementLengthTeam1 = bottomPlacementLengthTeam1 + ONE_CONDITION_MET_PLACEMENT_BONUS
        end

        local bottomPlacementXTeam2 = BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_X]
        local bottomPlacementLengthTeam2 = BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH]
        if bottomPlacementTeam2Bonus == 2 then
            bottomPlacementXTeam2 = bottomPlacementXTeam2 - ONE_CONDITION_MET_PLACEMENT_BONUS - BOTH_CONDITIONS_MET_PLACEMENT_BONUS
            bottomPlacementLengthTeam2 = bottomPlacementLengthTeam2 + ONE_CONDITION_MET_PLACEMENT_BONUS + BOTH_CONDITIONS_MET_PLACEMENT_BONUS
        elseif bottomPlacementTeam2Bonus == 1 then
            bottomPlacementXTeam2 = bottomPlacementXTeam2 - ONE_CONDITION_MET_PLACEMENT_BONUS
            bottomPlacementLengthTeam2 = bottomPlacementLengthTeam2 + ONE_CONDITION_MET_PLACEMENT_BONUS
        end

        -- fill the gap in the middle depending on which is extended further
        local topPlacementWidthTeam1 = TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]
        local bottomPlacementYTeam1 = BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y]
        local bottomPlacementWidthTeam1 = BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]
        if bottomPlacementTeam1Bonus <= topPlacementTeam1Bonus then
            bottomPlacementYTeam1 = bottomPlacementYTeam1 - GRID_SIZE
            bottomPlacementWidthTeam1 = bottomPlacementWidthTeam1 + GRID_SIZE
        else
            topPlacementWidthTeam1 = topPlacementWidthTeam1 + GRID_SIZE
        end


        local topPlacementWidthTeam2 = TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]
        local bottomPlacementYTeam2 = BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y]
        local bottomPlacementWidthTeam2 = BOTTOM_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]
        if bottomPlacementTeam2Bonus <= topPlacementTeam2Bonus then
            bottomPlacementYTeam2 = bottomPlacementYTeam2 - GRID_SIZE
            bottomPlacementWidthTeam2 = bottomPlacementWidthTeam2 + GRID_SIZE
        else
            topPlacementWidthTeam2 = topPlacementWidthTeam2 + GRID_SIZE
        end

        -- setting placement regions
        objc.set_mercenary_placement_region(1, PLACEMENT_REGION_KEY_TOP, TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                         TOP_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                         topPlacementLengthTeam1,
                                                                         topPlacementWidthTeam1)
        objc.set_mercenary_placement_region(2, PLACEMENT_REGION_KEY_TOP, topPlacementXTeam2,
                                                                         TOP_MERCENARY_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                         topPlacementLengthTeam2,
                                                                         topPlacementWidthTeam2)
        objc.set_mercenary_placement_region(1, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_MERCENARY_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                            bottomPlacementYTeam1,
                                                                            bottomPlacementLengthTeam1,
                                                                            bottomPlacementWidthTeam1)
        objc.set_mercenary_placement_region(2, PLACEMENT_REGION_KEY_BOTTOM, bottomPlacementXTeam2,
                                                                            bottomPlacementYTeam2,
                                                                            bottomPlacementLengthTeam2,
                                                                            bottomPlacementWidthTeam2)

        -- calculating the length for team 1
        local topTacticsPlacementLengthTeam1 = TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]
        local topTacticsPlacementWidthTeam1 = TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]
        local bottomTacticsPlacementLengthTeam1 = BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_LENGTH]
        local bottomTacticsPlacementWidthTeam1 = BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_WIDTH]
        if topTacticPlacementTeam1Bonus == 1 then
            topTacticsPlacementLengthTeam1 = topTacticsPlacementLengthTeam1 + TACTICS_PLACEMENT_BONUS
        end
        if bottomTacticPlacementTeam1Bonus == 1 then
            bottomTacticsPlacementLengthTeam1 = bottomTacticsPlacementLengthTeam1 + TACTICS_PLACEMENT_BONUS
        end

        -- calculating the length for team 2
        local topTacticPlacementXTeam2 = TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X]
        local topTacticPlacementLengthTeam2 = TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH]
        local bottomTacticPlacementXTeam2 = BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_X]
        local bottomTacticPlacementLengthTeam2 = BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_LENGTH]
        if topTacticPlacementTeam2Bonus == 1 then
            topTacticPlacementXTeam2 = topTacticPlacementXTeam2 - TACTICS_PLACEMENT_BONUS
            topTacticPlacementLengthTeam2 = topTacticPlacementLengthTeam2 + TACTICS_PLACEMENT_BONUS
        end
        if bottomTacticPlacementTeam2Bonus == 1 then
            bottomTacticPlacementXTeam2 = bottomTacticPlacementXTeam2 - TACTICS_PLACEMENT_BONUS
            bottomTacticPlacementLengthTeam2 = bottomTacticPlacementLengthTeam2 + TACTICS_PLACEMENT_BONUS
        end

        -- fill the gap in the middle depending on which is extended further
        local bottomTacticsPlacementYTeam1 = BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y]
        if bottomTacticPlacementTeam1Bonus <= topTacticPlacementTeam1Bonus then
            bottomTacticsPlacementYTeam1 = bottomTacticsPlacementYTeam1 - GRID_SIZE
            bottomTacticsPlacementWidthTeam1 = bottomTacticsPlacementWidthTeam1 + GRID_SIZE
        else
            topTacticsPlacementWidthTeam1 = topTacticsPlacementWidthTeam1 + GRID_SIZE
        end
        local bottomTacticsPlacementYTeam2 = BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y]
        local bottomTacticsPlacementWidthTeam2 = BOTTOM_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]
        local topTacticsPlacementWidthTeam2 = TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_WIDTH]
        if bottomTacticPlacementTeam2Bonus <= topTacticPlacementTeam2Bonus then
            bottomTacticsPlacementYTeam2 = bottomTacticsPlacementYTeam2 - GRID_SIZE
            bottomTacticsPlacementWidthTeam2 = bottomTacticsPlacementWidthTeam2 + GRID_SIZE
        else
            topTacticsPlacementWidthTeam2 = topTacticsPlacementWidthTeam2 + GRID_SIZE
        end


        objc.set_tactic_placement_region(1, PLACEMENT_REGION_KEY_TOP, TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                      TOP_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_Y],
                                                                      topTacticsPlacementLengthTeam1,
                                                                      topTacticsPlacementWidthTeam1)
        objc.set_tactic_placement_region(2, PLACEMENT_REGION_KEY_TOP, topTacticPlacementXTeam2,
                                                                      TOP_TACTIC_REGION_TEAM_TWO[PLACEMENT_REGION_Y],
                                                                      topTacticPlacementLengthTeam2,
                                                                      topTacticsPlacementWidthTeam2)
        objc.set_tactic_placement_region(1, PLACEMENT_REGION_KEY_BOTTOM, BOTTOM_TACTIC_REGION_TEAM_ONE[PLACEMENT_REGION_X],
                                                                         bottomTacticsPlacementYTeam1,
                                                                         bottomTacticsPlacementLengthTeam1,
                                                                         bottomTacticsPlacementWidthTeam1)
        objc.set_tactic_placement_region(2, PLACEMENT_REGION_KEY_BOTTOM, bottomTacticPlacementXTeam2,
                                                                         bottomTacticsPlacementYTeam2,
                                                                         bottomTacticPlacementLengthTeam2,
                                                                         bottomTacticsPlacementWidthTeam2)
        ------------------------------------------------------------------------------
        objc.set_global_value("placementHasChanged", 0)
    end
end
