////////////////////////////////////////////////////////////////////////////////////////////////////
// general.frag
////////////////////////////////////////////////////////////////////////////////////////////////////

#define SUPPORT_GLSL300

#include "prefix.sh"

#if defined(METAL_INSTEAD_OF_OPENGL)
#include "framework/Metal/MetalPrefix.metal"

#undef MANTIS_PBR /* Metal HACK -- disable Mantis PBR */
#undef TEAM_COLOR /* Metal HACK -- disable team color */
#undef GRAY_SCALE /* Metal HACK -- disable team color */

#endif

#include "system_uniforms.sh"
vec3 DiffuseColorBlend(vec3 diffuseColor);
#include "mantis_lighting.sh"

#if !defined(SHADOW_GEN) || !SHADOW_GEN

////////////////////////////////////////////////////////////////////////////////////////////////////
//      I N P U T   A T T R I B U T E S   (   V A R Y I N G S   )

#if defined(METAL_INSTEAD_OF_OPENGL)
typedef struct
{
    float4 clipSpacePosition [[position]];

#if VS_OUT_S_TEXCOORD
    VARYING_IN vec2 v_texCoord;
#endif

#if VS_OUT_S_COLOR
    VARYING_IN vec4 v_color;
#endif

#else // defined(METAL_INSTEAD_OF_OPENGL)

#if VS_OUT_S_TEXCOORD > 0
    VARYING_IN vec2 v_texCoord[VS_OUT_S_TEXCOORD];
#endif

#if VS_OUT_S_COLOR > 0
    VARYING_IN vec4 v_color[VS_OUT_S_COLOR];
#endif

#endif // defined(METAL_INSTEAD_OF_OPENGL)

#if defined(MANTIS_PBR) && (__VERSION__ >= 300)
    #define VS_OUT_S_WORLD_SPACE_POSITION 1
#endif

#if !defined(VS_OUT_S_WORLD_SPACE_POSITION)
    #define VS_OUT_S_WORLD_SPACE_POSITION 0
#endif

#if VS_OUT_S_WORLD_SPACE_POSITION
    VARYING_IN vec3 v_position_world;
#endif

#if VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_NORMAL
    VARYING_IN vec3 v_N;
#elif VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL
    VARYING_IN vec3 v_N;
    VARYING_IN vec3 v_T;
    VARYING_IN vec3 v_B;
#endif

#if defined(METAL_INSTEAD_OF_OPENGL)

} OmegaData;

#endif

////////////////////////////////////////////////////////////////////////////////////////////////////


#if defined(METAL_INSTEAD_OF_OPENGL)

#define s_cc3Texture2Ds_BaseColorSampler s_cc3Texture2Ds0
#define s_cc3Texture2Ds_NormalMapSampler s_cc3Texture2Ds1
#define s_cc3Texture2Ds_PropertiesMapSampler s_cc3Texture2Ds2

/* KenD -- Metal TODO --in effort to remove arrays of members of vertex attributes, disallow arrays of tex coords.
 * That will bring consistency to both GLSL and Metal Shading Language.
 * Future todo: use v_texCoord0 instead of v_texCoord[0] or v_texCoord.
 * For now, because this shader is more up to date than OmegaData
 * (OmegaData is declared with v_texCoord), we change v_texCoord0 to v_texCoord.
 * Update: we shouldn't do this before including MetalFragmentFramework (which expands `v_texCoord0` to `omegadata_inout.v_texCoord` [which will later be `omegadata_inout.v_texCoord0`])
 */
//#define v_texCoord0 v_texCoord

#else // defined(METAL_INSTEAD_OF_OPENGL)

#define s_cc3Texture2Ds_BaseColorSampler s_cc3Texture2Ds[BaseColorSampler]
#define s_cc3Texture2Ds_NormalMapSampler s_cc3Texture2Ds[NormalMapSampler]
#define s_cc3Texture2Ds_PropertiesMapSampler s_cc3Texture2Ds[PropertiesMapSampler]

/* KenD -- TODO -- eventually, with shader refactor to move away from array inputs to vertex functions and array outputs, this will go away */
#define v_texCoord0 v_texCoord[0]

#if VS_OUT_S_TEXCOORD > 0
    uniform sampler2D s_cc3Texture2Ds[3];

    const int BaseColorSampler = 0;
    const int NormalMapSampler = 1;
    const int PropertiesMapSampler = 2;
#endif

#endif // defined(METAL_INSTEAD_OF_OPENGL)

#if defined(TEAM_COLOR) || defined(GRAY_SCALE)
    uniform struct
    {
        vec4 TeamColor;
    } perentity;
#endif


////////////////////////////////////////////////////////////////////////////////////////////////////

#if defined(METAL_INSTEAD_OF_OPENGL)
#include "../../Externals/cocos3d/cocos3d/PG/pg_shaders/framework/Metal/MetalFragmentFramework.metal"
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "wm_shadow.fsh"

#if defined(METAL_INSTEAD_OF_OPENGL)

#if VS_OUT_S_TEXCOORD > 0
#define GetNormal()         ( fn_GetNormal(omegaData_inout, s_cc3Texture2Ds_NormalMapSampler) )
#else
#define GetNormal()         ( fn_GetNormal(omegaData_inout) )
#endif

vec3 fn_GetNormal(thread OmegaData& omegaData_inout
#if VS_OUT_S_TEXCOORD > 0
                  , texture2d<float> s_cc3Texture2Ds_NormalMapSampler
#endif
               )
#else // defined(METAL_INSTEAD_OF_OPENGL)
vec3 GetNormal()
#endif
{
    #if (VS_OUT_S_TANGENT_FRAME == TANGENT_FRAME_FULL) && (VS_OUT_S_TEXCOORD > 0)
        vec3 normal = texture2D(s_cc3Texture2Ds_NormalMapSampler, v_texCoord0).rgb;
        vec3 normal_t = normal * 2.0 - 1.0;
        return normalize(mat3(v_T, v_B, v_N) * normal_t);
    #elif VS_OUT_S_TANGENT_FRAME != 0
        return v_N;
    #else
        return vec3(0.0, 0.0, 0.0);
    #endif
}

#if VS_OUT_S_WORLD_SPACE_POSITION
    vec3 GetDirectionToCamera()
    {
        vec3 V = (getCameraPositionWorld() - v_position_world);
        float V_length = length(V);
        return normalize(V);
    }
#endif

vec3 OverlayBlend(vec3 target, vec3 blend, float alpha)
{
    // DavidJ -- this is the overlay blend used in photoshop (with an alpha overlay).
    //  It has been optimized to reduce the number of
    vec3 C = vec3(target.r > 0.5, target.g > 0.5, target.b > 0.5);
    vec3 X = target * blend;
    X += C * (vec3(-0.5) + blend + target - 2.0 * X);
    return mix(target, 2.0 * X, alpha);
}

vec3 DiffuseColorBlend(vec3 diffuseColor)
{
    return diffuseColor;
}

vec3 DiffuseColorBlendSRGB(vec3 diffuseColor)
{
    #if defined(TEAM_COLOR)
        // Expecting to find the team color mask in the alpha channel of the diffuse texture
        vec3 teamColor = perentity.TeamColor.rgb;
        #if defined(MANTIS_PBR) && (__VERSION__ >= 300)
            teamColor = SRGBToLinear(teamColor);
        #endif
        diffuseColor.rgb = OverlayBlend(diffuseColor.rgb, teamColor, texture2D(s_cc3Texture2Ds_BaseColorSampler, v_texCoord0).a);
    #endif
    #if defined(GRAY_SCALE)
    if (perentity.TeamColor.a < 0.1) {
        float luminance = dot(diffuseColor.rgb, vec3(0.299, 0.587, 0.114));
        luminance = mix(0.33, 1.0, luminance);
        diffuseColor = vec3(luminance, luminance, luminance);
    }
    #endif
    return diffuseColor;
}

#if defined(MANTIS_PBR) && (__VERSION__ >= 300)
uniform sampler2D _Perlin;
#endif

float SmokeyNoise()
{
#if defined(MANTIS_PBR) && (__VERSION__ >= 300) && (VS_OUT_S_TEXCOORD > 0)
    vec2 noise0 = texture2D(_Perlin, vec2(0.1 * 2.72 * v_texCoord0.x - 0.0162 * getSceneTime(), 0.1 * 2.72 * v_texCoord0.y + 0.0077 * getSceneTime())).rg;

    vec2 secondTap = 0.6 * v_texCoord0;
    secondTap += .06 * noise0;
    float noise = texture2D(_Perlin, secondTap).r;

    return noise;
#else
    return 0.0;
#endif
}

vec3 SmokeySummon()
{
#if defined(MANTIS_PBR) && (__VERSION__ >= 300) && (VS_OUT_S_TEXCOORD > 0)
    vec3 N = GetNormal();
    vec3 V = GetDirectionToCamera();

    vec4 properties = texture2D(s_cc3Texture2Ds_PropertiesMapSampler, v_texCoord0);

    float smoke = SmokeyNoise();
    float F = 1.0f - saturate(dot(V, N));

    #if 0
        float fresnel = pow(F, lerp(5.0, 1.0, smoke));

        vec3 result;
        result.rgb = vec3(0.25 * fresnel, 0.7 * fresnel, fresnel);
        // result.rgb += vec3(1.0, 0.2, 0.2) * saturate(fresnel - 0.25);
        result.rgb += vec3(0.05, 0.05, 0.05);
        return result;
    #else
        float fresnel = pow(F, 3.0);
        vec3 result;
        float a = (fresnel + 0.125) * smoke;
        result.rgb = a * vec3(.5, .85, 2.0);
        return result;
    #endif

#else
    return vec3(1,1,1);
#endif
}

#if defined(METAL_INSTEAD_OF_OPENGL)
float4 fsMainGeneral(thread OmegaData& omegaData_inout
#if VS_OUT_S_TEXCOORD > 0
                     , texture2d<float> s_cc3Texture2Ds0
#endif
#if VS_OUT_S_TEXCOORD > 1
                     , texture2d<float> s_cc3Texture2Ds1
#endif
#if VS_OUT_S_TEXCOORD > 2
                     , texture2d<float> s_cc3Texture2Ds2
#endif
)
#else
void main()
#endif
{
    FRAGMENT_FUNCTION_BEGIN

    vec4 baseColor = vec4(1,1,1,1);
    #if VS_OUT_S_COLOR > 0
        baseColor = v_color[0];
    #endif
    
    #if VS_OUT_S_TEXCOORD > 0 && (VS_OUT_S_TEXCOORD > 0)
        vec4 diffuseTextureSample = texture2D(s_cc3Texture2Ds_BaseColorSampler, v_texCoord0);
        baseColor *= diffuseTextureSample;
    #endif

    #if defined(SMOKEY_SUMMON)
        baseColor.rgb *= 0.001;
        baseColor.rgb += SmokeySummon();
        baseColor.a = 1.0;
    #elif defined(MANTIS_PBR) && (__VERSION__ >= 300) && (VS_OUT_S_TEXCOORD > 0)
        vec3 N = GetNormal();
        vec3 V = GetDirectionToCamera();

        vec4 properties = texture2D(s_cc3Texture2Ds_PropertiesMapSampler, v_texCoord0);
        vec3 diffuseAlbedoSRGB = DiffuseColorBlendSRGB(baseColor.rgb);
        baseColor.rgb = CalculateMantisLighting(properties, diffuseAlbedoSRGB, N, V);
    #else
        baseColor.rgb = DiffuseColorBlendSRGB(baseColor.rgb);
    #endif

    baseColor.rgb *= ShadowCalculateMultiplier();

    #if defined(TEAM_COLOR)
        float a = min(max((getCameraPositionWorld().y - 20.0) / 5.0, 0.0), 1.0);
        baseColor.rgb = mix(baseColor.rgb, perentity.TeamColor.rgb, a);
    #endif

    gl_FragColor = baseColor;

    FRAGMENT_FUNCTION_END
}

#else // #if !defined(SHADOW_GEN) || !SHADOW_GEN

vec3 DiffuseColorBlend(vec3 diffuseColor) { return diffuseColor; }

#if defined(METAL_INSTEAD_OF_OPENGL)
    void fsMainGeneral(thread OmegaData& omegaData_inout) {} // (shadow gen version)
#else
    void main() {}      // (shadow gen version)
#endif

#endif


#if defined(METAL_INSTEAD_OF_OPENGL)

#define SPECIFIC_MAIN_FUNCTION_CALL \
    fsMainGeneral()

fragment float4
generalFragmentShader(OmegaData omegaData_inout [[stage_in]]
#if VS_OUT_S_TEXCOORD > 0
                      , texture2d<float> s_cc3Texture2Ds0
#endif
#if VS_OUT_S_TEXCOORD > 1
                      , texture2d<float> s_cc3Texture2Ds1
#endif
#if VS_OUT_S_TEXCOORD > 2
                      , texture2d<float> s_cc3Texture2Ds2
#endif
)
{
    return fsMainGeneral(omegaData_inout
#if VS_OUT_S_TEXCOORD > 0
                         , s_cc3Texture2Ds0
#endif
#if VS_OUT_S_TEXCOORD > 1
                         , s_cc3Texture2Ds1
#endif
#if VS_OUT_S_TEXCOORD > 2
                         , s_cc3Texture2Ds2
#endif
                         );
}

#else
#endif
