#define SUPPORT_GLSL300

#include "library/prefix.sh"

#if defined(METAL_INSTEAD_OF_OPENGL)
#include "framework/Metal/MetalPrefix.metal"
#endif

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef MANTIS_PBR
    #define VS_OUT_S_WORLD_SPACE_POSITION 1
#endif

#include "framework/framework.sh"
#include "framework/transforms.sh"
#include "framework/skinning.sh"

#include "repeat.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "wm_shadow.vsh"

#if defined(METAL_INSTEAD_OF_OPENGL)
void vsMainGeneral(thread VSOUT& vsout, thread const VSIN& vsin, constant CocosUniforms& CocosUniforms, constant CocosNodeBasedUniforms& CocosNodeBasedUniforms)
#else
void vsMain(out VSOUT vsout, VSIN vsin)
#endif
{
    vec3 worldSpacePosition;
    RawTangentFrame worldSpaceTangentFrame;
    
    #if defined(SKIN_BONE_TO_WORLD)
        bool transformViaModelSpace = false;
    #else
        bool transformViaModelSpace = true;
    #endif
    if (transformViaModelSpace) {
        vec3 modelSpaceP;
        RawTangentFrame modelSpaceTF;
#if defined(METAL_INSTEAD_OF_OPENGL)

        getCoords_Internal(modelSpaceP, modelSpaceTF, vsin, 0, CocosNodeBasedUniforms);
#else
        getModelSpace(modelSpaceP, modelSpaceTF, vsin);
#endif

        vsout._position = transformPoint(getModelToProj(), modelSpaceP);
        
        worldSpacePosition = transformPoint(getModelToWorld(), modelSpaceP).xyz;
        worldSpaceTangentFrame = transformTangentFrame(getModelToWorldNoScale(), modelSpaceTF);
        if (!modelToWorldNoScaleIsGood()) {
            worldSpaceTangentFrame = normalizeTangentFrame(worldSpaceTangentFrame);
        }
    } else {
#if defined(METAL_INSTEAD_OF_OPENGL)
        getCoords_Internal(worldSpacePosition, worldSpaceTangentFrame, vsin, 1, CocosNodeBasedUniforms);
#else
        getWorldSpace(worldSpacePosition, worldSpaceTangentFrame, vsin);
#endif

        vsout._position = transformPoint(getWorldToProj(), worldSpacePosition);
    }

    int c;
    #if VS_OUT_S_TEXCOORD < VS_IN_S_TEXCOORD
        FOR(c, 0, VS_OUT_S_TEXCOORD, setTexCoord(vsout, c, getTexCoord(vsin, c)))
    #else
        FOR(c, 0, VS_IN_S_TEXCOORD, setTexCoord(vsout, c, getTexCoord(vsin, c)))
    #endif

    #if VS_OUT_S_COLOR < VS_IN_S_COLOR
        FOR(c, 0, VS_OUT_S_COLOR, setColor(vsout, c, getColor(vsin, c)))
    #else
        FOR(c, 0, VS_IN_S_COLOR, setColor(vsout, c, getColor(vsin, c)))
    #endif

    setWorldSpacePosition(vsout, worldSpacePosition);
    setTangentFrame(vsout, worldSpaceTangentFrame);

    shadow_receive(worldSpacePosition);
}

#if defined(METAL_INSTEAD_OF_OPENGL)

#define NON_SPECIFIC_FUNCTION_PARAMETERS \
constant CocosUniforms& CocosUniforms, \
constant CocosNodeBasedUniforms& CocosNodeBasedUniforms


#define SPECIFIC_MAIN_FUNCTION_CALL \
    vsMainGeneral(vsout, vsin, CocosUniforms, CocosNodeBasedUniforms);

#include "framework/Metal/MetalVertexFramework.metal"


vertex OmegaData
generalVertexShader(OmegaVertex v_in [[stage_in]],
                    constant CocosUniforms& CocosUniforms [[buffer(1)]],
                    constant CocosNodeBasedUniforms& CocosNodeBasedUniforms [[buffer(2)]])
{
    VERTEX_FUNCTION_BEGIN

    nonSpecificVertexShader(omegaData_inout, v_in, CocosUniforms, CocosNodeBasedUniforms);

    VERTEX_FUNCTION_END
}

#else
#include "framework/glsles/framework_glsles.vert"
#endif
