#if defined(GL_ES)
    precision highp float;
#endif

// uniforms
uniform vec4 u_cc3MaterialDiffuseColor;
uniform sampler2D s_cc3Texture2Ds[1];
uniform vec2 u_cc3SceneTime;

uniform float nearDist;
uniform float farDist;

uniform float minOpacity;
uniform float maxOpacity;
uniform float noisePeriod;

// varyings
varying vec2 v_texCoord;
varying vec3 v_position_view;

#define M_PI 3.1415926535897932384626433832795

float fadeFactor(float cameraDist) {
    cameraDist = clamp(cameraDist, nearDist, farDist);
    return (cameraDist - nearDist) / (farDist - nearDist);
}

float blinkNoise(float time, float period) {
    float noiseTime = mod(time, period) * 2.0 * M_PI / period;
    float noise = sin(noiseTime) * (0.5 * cos(noiseTime * 0.6366 + 56.7272) + 0.5);
    return mix(minOpacity, maxOpacity, 0.5 * noise + 0.5);
}

// code
void main() {
    float cameraDist = distance(v_position_view, vec3(0.0, 0.0, 0.0));
    float opacity = fadeFactor(cameraDist) * blinkNoise(u_cc3SceneTime[0], noisePeriod);

    gl_FragColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    gl_FragColor.a *= opacity;
}
