#define SUPPORT_GLSL300
#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

#if __VERSION__ >= 300
    layout(location = 0) out vec4 primaryOutput;
    #define gl_FragColor primaryOutput
#endif


#define M_PI 3.1415926535897932384626433832795
#define M_GRAYSCALE vec3(0.2126, 0.7152, 0.0722)

//===================================================================================================================//
// UNIFORMS //
//===================================================================================================================//
uniform sampler2D s_cc3Texture2Ds[4];
uniform vec3 u_cc3CameraPositionGlobal;
uniform vec2 u_cc3SceneTime;

uniform vec4 innerColor;
uniform vec4 outerColor;
uniform vec4 borderColor;
uniform float fresnelFalloff;

uniform float scanlineScale;
uniform float scanlineSpeed;
uniform float scanlineStrength;

uniform float scanline2Scale;
uniform float scanline2Speed;
uniform float scanline2Strength;

uniform float fadeHeight;
uniform float fadeStrength;

uniform float fresnelStrength;
uniform float textureStrength;

//===================================================================================================================//
// VARYINGS //
//===================================================================================================================//




#if !defined(SHADOW_GEN)
    VARYING_IN vec3 v_position_world;
    VARYING_IN vec3 v_N;
    VARYING_IN vec2 v_texCoord;
#else
    vec3 v_N;
    vec3 v_position_world;
    vec2 v_texCoord;
#endif


#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    VARYING_IN vec4 v_material_diffuse_color;
#endif

#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    vec4 getMaterialColor() { return v_material_diffuse_color; }
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 getMaterialColor() { return u_cc3MaterialDiffuseColor; }
#endif


//===================================================================================================================//
// SHADER //
//===================================================================================================================//
void main() 
{   
    vec4 textureColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    vec4 diffuseColor = getMaterialColor() * textureColor;

    gl_FragColor = diffuseColor;

    vec3 N = normalize(v_N);
    vec3 V = (u_cc3CameraPositionGlobal - v_position_world);
    float V_length = length(V);
    V = normalize(V);

     
    // Fresnel //
    float fresnel = 1.0 - abs(dot(N, V));
    fresnel = pow(fresnel, fresnelFalloff);

    // Scanlines //
    float scanlines = cos(v_position_world.y / scanlineScale + u_cc3SceneTime.x * scanlineSpeed + textureColor.x * textureStrength) * 0.5 + 0.5;
    scanlines = pow(scanlines, 3.0) * scanlineStrength;

    float scanlines2 = cos(v_position_world.y / scanline2Scale + u_cc3SceneTime.x * scanline2Speed + textureColor.x * textureStrength) * 0.5 + 0.5;
    scanlines2 = pow(scanlines2, 3.0) * scanline2Strength - 0.3;

    scanlines += scanlines2;

    // Height Fade //
    float fade = pow(1.0 - min(1.0, v_position_world.y / fadeHeight), 1.0);

    gl_FragColor = mix(innerColor, outerColor, min(1.0, fade * fadeStrength));

    gl_FragColor = mix(gl_FragColor, borderColor, pow(clamp(fresnel * fresnelStrength + scanlines, 0.0, 1.0), 4.0));

        // Top Lighting //
    float lighting_up = v_N.y * 0.5 + 0.5;
    gl_FragColor.rgb *= mix(1.5, 1.5, lighting_up);

}
