local KNOCKBACK_WIDTH = 1500
local KNOCKBACK_HEIGHT = 4500
local KNOCKBACK_DISTANCE = 2700
local CRATER_OFFSET = 350
local KNOCKBACK_SPEED = 350
local BASE_BUFFER = 2000
local ORIGINAL_RANGE_KEY = 'originalRangeKey'
local MELEE_RANGE = 550
local END_RANGE_TRIGGER_NAME = 'commanderKnockback_endRangeTrigger'
local ABILITY_DAMAGE = 75
local TARGET_X_KEY = 'targetXKey'
local TARGET_Y_KEY = 'targetYKey'
local STUN_SUBROUNDS = 3000
local STANDIN_ENTITY_KEY = 'SumoStandinEntityKey'

knockbackSumoTimingEntity_deathTrigger = function(entity)
   local teamNumber = objc.get_team_number(entity)
   local targetPositionX = objc.get_local_value(entity, TARGET_X_KEY)
   local targetPositionY = objc.get_local_value(entity, TARGET_Y_KEY)
   local halfHeight = objc.divide_with_floor(KNOCKBACK_HEIGHT, 2)
   local enemiesInRange = objc.list_enemies_in_radius_of_point(targetPositionX, targetPositionY, halfHeight, teamNumber)
   for _, enemy in pairs(enemiesInRange) do
      local thisEnemyPos = objc.get_entity_position(enemy)
      if math.abs(thisEnemyPos[1] - targetPositionX) < KNOCKBACK_WIDTH and objc.is_normal_unit(enemy) then
         knockEnemyBack(entity, enemy, targetPositionX, targetPositionY)
      end
   end
end

knockEnemyBack = function(sourceEntity, targetEntity, sourcePositionX, sourcePositionY)
   if objc.is_normal_unit(targetEntity) then
      local teamNumber = objc.get_team_number(sourceEntity)
      local targetDestinationX = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_X)
      local targetDestinationY = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_POSITION_Y)
      local basePositionX = math.abs(objc.get_base_position(teamNumber)[1])
      local craterOffsetModifier = 1

      if sourcePositionX < targetDestinationX then
         targetDestinationX = targetDestinationX + KNOCKBACK_DISTANCE
         if targetDestinationX > basePositionX - BASE_BUFFER then
            targetDestinationX = basePositionX - BASE_BUFFER
         end
      else
         targetDestinationX = targetDestinationX - KNOCKBACK_DISTANCE
         if targetDestinationX < -basePositionX + BASE_BUFFER then
            targetDestinationX = -basePositionX + BASE_BUFFER
         end
         craterOffsetModifier = -1
      end


       local playerNumber = objc.get_player_number(sourceEntity)

      local targetArray = objc.spawn_at_point('SumoTargetEffectEntity', 1, playerNumber, targetDestinationX, targetDestinationY, 0, 0)
      local oldRange = objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE)
      objc.set_local_value(targetEntity, ORIGINAL_RANGE_KEY, oldRange)
      objc.set_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, MELEE_RANGE)
      objc.add_trigger(targetEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, 'knockbackTargeted.lua', END_RANGE_TRIGGER_NAME)
      local standinEntity = objc.get_local_value(sourceEntity, STANDIN_ENTITY_KEY)
      objc.set_local_value(targetEntity, STANDIN_ENTITY_KEY, standinEntity)
      if objc.is_ground_unit(targetEntity) then
         local oldParams = objc.move_entity_to_entity_with_motion(targetEntity, targetArray[1], MOTION_STATE_LEAP, KNOCKBACK_SPEED)
      else
         local oldParams = objc.move_entity_to_entity_with_motion(targetEntity, targetArray[1], MOTION_STATE_WALK, KNOCKBACK_SPEED)
      end
      
      --Set the offset on the EffectEntity for where to spawn the impact effect
      objc.set_local_value(targetArray[1], 'craterSpawnPosition', targetDestinationX - CRATER_OFFSET * craterOffsetModifier * 2)
   end
end

commanderKnockback_endRangeTrigger = function(sourceEntity, targetEntity)
   local standinEntity = objc.get_local_value(sourceEntity, STANDIN_ENTITY_KEY)
   local originalRange = objc.get_local_value(sourceEntity, ORIGINAL_RANGE_KEY)
   objc.set_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_ATTACK_RANGE, originalRange)
   objc.stun_entity(sourceEntity, STUN_SUBROUNDS)
   objc.inflict_normal_damage(standinEntity, sourceEntity, ABILITY_DAMAGE)
   objc.flash_entity_for_damage(sourceEntity, 250)
   objc.remove_trigger(sourceEntity, TRIGGER_NAME_ON_OVERRIDE_TARGETING_COMPLETION_TRIGGER, END_RANGE_TRIGGER_NAME)
end
