local MAX_SPAWNS = 4
local WAIT_TIMER = 30
local SPAWN_COUNTER = 'spawnCounter'
local BASE_EDGE = 9500
local SPAWN_FORWARD_EDGE = 8500
local HAS_INITIALIZED = 'hasInitialized'
local REFLECTION_TAG_KEY = 'reflectionTagKey'

--[[
This TickTrigger find a number of random units to duplicate and creates copies
of them.
--]]
kuroAbility1ScriptEntity_tickTrigger = function(entity)
   local hasInitialized = objc.get_local_value(entity, HAS_INITIALIZED)
  
   --If the ability hasn't started yet, duplicate some units
   if hasInitialized == 0 then
      kuroAbility1ScriptEntity_reflectUnits(entity)
      objc.set_local_value(entity, HAS_INITIALIZED, 1)
      return 0
   end

   --If the script wasn't able to duplicate the full amount of units, it waits and tries again later
   local tickTimer = objc.tick_timer(entity, 'tickTime', WAIT_TIMER)

   if tickTimer == 0 then
      kuroAbility1ScriptEntity_reflectUnits(entity)
   end

   return 0
end


kuroAbility1ScriptEntity_reflectUnits = function(entity)
   local teamNumber = objc.get_team_number(entity)
   local baseEntity = objc.get_base_entity(teamNumber)
   local currentSpawnCounter = objc.get_local_value(entity, SPAWN_COUNTER)

   local unitsToDuplicate = {}
   local allyList = objc.list_entities(teamNumber)
   local validTargets = kuroAbility1ScriptEntity_validTargetList(allyList, teamNumber)

   local targetCount = listCount(validTargets)

   for i = 1, MAX_SPAWNS do
      --[[if targetCount > 0 then
         local allyNumberToDuplicate = objc.roll_number(1, targetCount)
         table.insert(unitsToDuplicate, validTargets[allyNumberToDuplicate])
         table.remove(validTargets, allyNumberToDuplicate)
         targetCount = listCount(validTargets)
      end]]
      local mostExpensiveEntity = 0
      local mostExpensiveCost = 0

      for _,ally in pairs(validTargets) do
         local hasBeenCopied = objc.get_local_value(ally, REFLECTION_TAG_KEY)
         if hasBeenCopied == 0 then
            local thisCost = objc.get_entity_value(ally, 'aiComponent.cost')
            if thisCost > mostExpensiveCost then
               mostExpensiveCost = thisCost
               mostExpensiveEntity = ally
            end
         end
      end

      if mostExpensiveEntity ~= 0 then
         kuroAbility1ScriptEntity_copyUnit(mostExpensiveEntity)
      end
   end

   objc.kill_entity(entity)
end


kuroAbility1ScriptEntity_validTargetList = function(listOfEntities, teamNumber)
   local validTargets = {}
   for _, entity in pairs(listOfEntities) do
      if objc.is_normal_unit(entity) and objc.get_local_value(entity, REFLECTION_TAG_KEY) == 0 then
         local entityPosition = objc.get_entity_position(entity)

         if teamNumber == 1 then
            if entityPosition[1] > BASE_EDGE * (-1) then
               table.insert(validTargets, entity)
               --objc.scripting_error('Adding for Player 1:  ' .. objc.get_entity_value(entity, 'identityComponent.identifier'))
            end
         elseif teamNumber == 2 then
            if entityPosition[1] < BASE_EDGE then
               table.insert(validTargets, entity)
               --objc.scripting_error('Adding for Player 2:  ' .. objc.get_entity_value(entity, 'identityComponent.identifier'))
            end
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end
      end
   end

   return validTargets
end

kuroAbility1ScriptEntity_copyUnit = function(ally)
   local allyUnitType = objc.get_entity_value(ally, 'identityComponent.identifier')
   local allyUnitPosition = objc.get_entity_position(ally)
   local xOffset = 600
   local yOffset = 0
   local xOffsetModifier = -1
   local yOffsetModifier = 1
   local xFacing = 1
   local overrideMaxVelocity = 2000
   local targetRadius = 100

   if teamNumber == 2 then
      xOffsetModifier = 1
      xFacing = -1
   end

   --If they're too close to the base, we need to spawn them forward instead of back into the base
   if math.abs(allyUnitPosition[1]) > SPAWN_FORWARD_EDGE then
      xOffset = 0
      yOffset = 600
      if allyUnitPosition[2] > 0 then
         yOffsetModifier = -1
      end
      xOffsetModifier = xOffsetModifier * (-1)
   end


   local playerNumber = objc.get_player_number(ally)

   --Play a VFX on the unit to be duplicated
   objc.play_vfx(ally, 'Commander_ReflecStorm.lvl', 'node_origin')
   --Spawn a duplicate unit
   local duplicateUnitArray = objc.spawn_at_point(allyUnitType, 1, playerNumber, allyUnitPosition[1], allyUnitPosition[2], xFacing, 0)
   --Make sure it can move right away
   objc.set_entity_value(duplicateUnitArray[1], 'spawnComponent.holdPositionCountForSpawning', 0)
   --Play a VFX on the duplicate unit
   objc.play_vfx(duplicateUnitArray[1], 'Commander_ReflecClone.lvl', 'node_origin')
   --Move the spawned duplicate unit backwards for visual effect
   objc.move_entity_to_location_with_velocity(duplicateUnitArray[1], allyUnitPosition[1] + (xOffset * xOffsetModifier), allyUnitPosition[2] + (yOffset * yOffsetModifier), overrideMaxVelocity, targetRadius)

   --Tag both the original unit and its duplicate don't get duplicated again
   objc.set_local_value(ally, REFLECTION_TAG_KEY, 1)
   objc.set_local_value(duplicateUnitArray[1], REFLECTION_TAG_KEY, 1)
end
