local HAS_INITIALIZED_KEY = 'hasInitialized'
local SPAWN_COUNTER_KEY = 'spawnCounter'
local SPAWNED_ENTITY_KEY = 'spawnedEntityKey'


--[[
This TickTrigger initially finds the strongest unit the player has, then creates
a set number of copies of it. Once it's created them, it will kill itself.
--]]
kuroAbility3ScriptEntity_tickTrigger = function(entity)
   local hasInitialized = objc.get_local_value(entity, HAS_INITIALIZED_KEY)
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)
   local spawnCounter = objc.get_local_value(entity, SPAWN_COUNTER_KEY)
   local spawnTimer = objc.tick_timer(entity, 'spawnTimer', 4)
   local mostPowerfulFactoryUnitIdentifier = 'mostPowerfulFactoryUnitIdentifier' .. teamNumber

   local maxNumberToSpawn = 4

   if spawnCounter >= maxNumberToSpawn then
      objc.kill_entity(entity)
   end
   
   if hasInitialized == 0 then
      objc.set_local_value(entity, HAS_INITIALIZED_KEY, 1)
      objc.set_targetable(entity, false)   

      local unitFactories = objc.get_unit_spawn_factories(teamNumber)
      local mostPowerfulFactory = 'None'

      for _, factory in pairs(unitFactories) do
         if kuroAbility3ScriptEntity_IsMorePowerfulFactory(factory, mostPowerfulFactory) == true then
            mostPowerfulFactory = factory
         end
      end

      local unitSpawnIdentifier = objc.get_spawn_identifier_for_factory(mostPowerfulFactory)
      objc.set_global_value(mostPowerfulFactoryUnitIdentifier, unitSpawnIdentifier)
   end

   if spawnTimer == 0 then
      kuroAbility3ScriptEntity_SpawnVisualEffect(entity, playerNumber, teamNumber, spawnCounter)
   end
end

kuroAbility3ScriptEntity_IsMorePowerfulFactory = function(newFactory, mostPowerfulFactory)
   if mostPowerfulFactory == 'None' then
      return true
   elseif objc.get_entity_value(newFactory, 'aiComponent.supply') > objc.get_entity_value(mostPowerfulFactory, 'aiComponent.supply') then
      return true
   elseif objc.get_entity_value(newFactory, 'aiComponent.supply') == objc.get_entity_value(mostPowerfulFactory, 'aiComponent.supply') then
      if objc.get_entity_value(newFactory, 'aiComponent.damageComponent.attackDamage') > objc.get_entity_value(mostPowerfulFactory, 'aiComponent.damageComponent.attackDamage') then
         return true
      end
   end

   return false
end

kuroAbility3ScriptEntity_SpawnVisualEffect = function(entity, playerNumber, teamNumber, spawnCounter)
   local positionX = 0
   local positionY = 0
   local directionX = 0
   local directionY = 0
   local farBorderX = 9500
   local spawnBoxSizeX = 3500
   local spawnBoxSizeY = 2500

   local spawningXBufferBetweenUnits = 1000
   local spawningYBufferBetweenUnits = 1000

   local tries = 0
   local maxNumberOfTries = 300

   local spawnPositionXKey = ''
   local spawnPositionYKey = ''

   while tries < maxNumberOfTries do
      tries = tries + 1

      --Reset the starting X and Y positions for each calculation
      positionY = 0
      if teamNumber == 1 then
         positionX = farBorderX * (-1)
         directionX = 1
      elseif teamNumber == 2 then
         positionX = farBorderX
         directionX = -1
      else
         objc.scripting_error('Encountered unknown player number ' .. teamNumber)
      end

      local randomAdjustmentX = objc.roll_number(1, spawnBoxSizeX)
      local randomAdjustmentY = objc.roll_number(1, spawnBoxSizeY)

      if randomAdjustmentY > objc.objc.divide_with_floor(spawnBoxSizeY, 2) then
         randomAdjustmentY = randomAdjustmentY - spawnBoxSizeY
      end

      positionX = positionX + (randomAdjustmentX * directionX)
      positionY = positionY + randomAdjustmentY

      local isValidLocation = true
      if spawnCounter ~= 0 then
         for index = 1, spawnCounter do
            local spawnedEntityKey = SPAWNED_ENTITY_KEY .. index

            local spawnedEntity = objc.get_local_value(entity, spawnedEntityKey)
            local spawnedEntityPosition = objc.get_entity_position(spawnedEntity)
            local spawnPositionX = spawnedEntityPosition[1]
            local spawnPositionY = spawnedEntityPosition[2]

            if math.abs(math.abs(positionX) - math.abs(spawnPositionX)) < spawningXBufferBetweenUnits and math.abs(math.abs(positionY) - math.abs(spawnPositionY)) < spawningYBufferBetweenUnits then
               isValidLocation = false
               break
            end
         end
      end

      if isValidLocation then
         break
      end
   end
   
   local spawnedEntityArray = objc.spawn_at_point('kuroAbility3SpawnEffect', 1, playerNumber, positionX, positionY, directionX, directionY)
   spawnCounter = spawnCounter + 1
   objc.set_local_value(entity, SPAWN_COUNTER_KEY, spawnCounter)
   
   local spawnedEntityKey = SPAWNED_ENTITY_KEY .. spawnCounter
   objc.set_local_value(entity, spawnedEntityKey, spawnedEntityArray[1])
  
end
