local GUNSHIPS_ALREADY_SPAWNED_KEY = 'gunshipsAlreadySpawned'
local MOVE_OFFSET_FROM_BASE = 3000
local SPAWN_OFFSET_FROM_BASE = 3000

--[[
This TickTrigger spawns 3 gunships offscreen along with with spawn VFX,
brings them on screen near the base, and then kills the VFX.
--]]
legionCommanderGunshipsScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)

   local tickValue = objc.tick_timer(entity, 'tickTime', 11)
   local gunshipsAlreadySpawned = objc.get_local_value(entity, GUNSHIPS_ALREADY_SPAWNED_KEY)
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)

   if gunshipsAlreadySpawned == 0 then
      local basePosition = objc.get_base_position(teamNumber)
      local positionX = 0
      local positionY = -500
      local directionX = 0
      gunshipEntitiesList = {}

      if teamNumber == 1 then
         positionX = basePosition[1] - SPAWN_OFFSET_FROM_BASE
         directionX = 1
      elseif teamNumber == 2 then
         positionX = basePosition[1] + SPAWN_OFFSET_FROM_BASE
         directionX = -1
      else
         objc.scripting_error('Encountered unknown player number ' .. teamNumber)
      end

      for i = 1,2 do
         local spawnedGunship = objc.spawn_at_point('B_Gunship', 1, playerNumber, positionX, positionY, directionX, 0)
         objc.set_entity_value(spawnedGunship[1], 'spawnComponent.holdPositionCountForSpawning', 0)
         objc.set_local_value(spawnedGunship[1], 'hasCheckedForFactory', 1)
         table.insert(gunshipEntitiesList, spawnedGunship[1])
         positionY = positionY + 500
      end

      for i, gunship in pairs(gunshipEntitiesList) do
         local xAdjustment = 0
         if teamNumber == 1 then
            xAdjustment = basePosition[1] + MOVE_OFFSET_FROM_BASE
         elseif teamNumber == 2 then
            xAdjustment = basePosition[1] - MOVE_OFFSET_FROM_BASE
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end

         objc.play_vfx(gunship, 'commanderGunshipSpawn.lvl', 'B_Gunship_Rig.pod')

         --if i == 2 then
            --objc.set_local_value(gunship, 'centerGunship', 1)
         --end

         local overrideMaxVelocity = 3000
         local targetRadius = 1000

         local currentPosition = objc.get_entity_position(gunship)
         objc.move_entity_to_location_with_velocity(gunship, xAdjustment, currentPosition[2], overrideMaxVelocity, targetRadius)
      end
     
      objc.set_local_value(entity, GUNSHIPS_ALREADY_SPAWNED_KEY, 1)
   end

   if tickValue == 0 and gunshipsAlreadySpawned == 1 then
      local currentPosition = objc.get_entity_position(gunshipEntitiesList[1])
      for _, gunship in pairs(gunshipEntitiesList) do
         objc.stop_vfx(gunship, 'commanderGunshipSpawn.lvl', 'B_Gunship_Rig.pod', false)
         --objc.set_combat_text(gunship, 'VFX should have been killed')
         --if objc.get_local_value(gunship, 'centerGunship') == 1 then
         objc.play_vfx(gunship, 'commanderFlashScreen.lvl', 'B_Gunship_Rig.pod')
            --objc.set_combat_text(gunship, 'Center Gunship')
         --end
      end
      --objc.spawn_at_point('legionCommanderGunshipsFlash', 1, playerNumber, currentPosition[1], 0, 0, 0)
      
      objc.kill_entity(entity)
   end
end
