local GUNSHIPS_ALREADY_SPAWNED_KEY = 'gunshipsAlreadySpawned'
local MOVE_OFFSET_FROM_BASE = 3000
local SPAWN_OFFSET_FROM_BASE = 3000
local GUNSHIP_SPAWN_VELOCITY = 3000
local RANGE_OFFSET = 2300
local GUNSHIPS_TO_SPAWN = 1
local TARGET_ENTITY_IDENTIFIER = 'gunshipAbilityTargetEntity'
local GUNSHIPS_STARTED_MOVING = 'gunshipsStartedMoving'
local THIS_TARGET_KEY = 'thisTargetKey'
local GUNSHIP_ENTITIES_LIST_KEY = 'gunshipEntitiesList'

--[[
This TickTrigger spawns 3 gunships offscreen along with with spawn VFX,
brings them on screen near the base, and then kills the VFX.
--]]
legionCommanderGunshipsTargetedScriptEntity_tickTrigger = function(entity)
    objc.set_targetable(entity, false)

    local tickValue = objc.tick_timer(entity, 'tickTime', 11)
    local gunshipsAlreadySpawned = objc.get_local_value(entity, GUNSHIPS_ALREADY_SPAWNED_KEY)
    local gunshipsStartedMoving = objc.get_local_value(entity, GUNSHIPS_STARTED_MOVING)
    local teamNumber = objc.get_team_number(entity)
    local playerNumber = objc.get_player_number(entity)

    if gunshipsAlreadySpawned == 0 then
        local basePosition = objc.get_base_position(teamNumber)
        local positionX = 0
        local positionY = -500
        local directionX = 0
        local gunshipEntitiesList = {}

        if teamNumber == 1 then
            positionX = basePosition[1] - SPAWN_OFFSET_FROM_BASE
            directionX = 1
        elseif teamNumber == 2 then
            positionX = basePosition[1] + SPAWN_OFFSET_FROM_BASE
            directionX = -1
        else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
        end

        for i = 1,GUNSHIPS_TO_SPAWN do
            local spawnedGunship = objc.spawn_at_point('B_Gunship', 1, playerNumber, positionX, positionY, directionX, 0)
            objc.set_entity_value(spawnedGunship[1], 'spawnComponent.holdPositionCountForSpawning', 0)
            objc.set_local_value(spawnedGunship[1], 'hasCheckedForFactory', 1)
            table.insert(gunshipEntitiesList, spawnedGunship[1])
            positionY = positionY + 500
        end

        for i, gunship in pairs(gunshipEntitiesList) do
            objc.play_vfx(gunship, 'commanderGunshipSpawn.lvl', 'B_Gunship_Rig.pod')

            local targetRadius = 800

            local currentPosition = objc.get_entity_position(gunship)
            local myXPosition = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
            local myYPosition = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y) + currentPosition[2]
            local target = objc.spawn_at_point(TARGET_ENTITY_IDENTIFIER, 1, playerNumber, myXPosition + (RANGE_OFFSET * directionX), myYPosition, 0, 0)
            objc.set_local_value(entity, THIS_TARGET_KEY, target[1])
            objc.set_local_value(entity, GUNSHIP_ENTITIES_LIST_KEY, gunshipEntitiesList)
            --objc.move_entity_to_location_with_velocity(gunship, myXPosition, myYPosition, overrideMaxVelocity, targetRadius)
        end
      
        objc.set_local_value(entity, GUNSHIPS_ALREADY_SPAWNED_KEY, 1)
    end

    gunshipEntitiesList = objc.get_local_value(entity, GUNSHIP_ENTITIES_LIST_KEY)

    if gunshipsAlreadySpawned == 1 and gunshipsStartedMoving == 0 then
      for _, gunship in pairs(gunshipEntitiesList) do
         local thisTarget = objc.get_local_value(entity, THIS_TARGET_KEY)
         objc.move_entity_to_entity_with_motion(gunship, thisTarget, MOTION_STATE_WALK, GUNSHIP_SPAWN_VELOCITY)
         objc.set_local_value(entity, GUNSHIPS_STARTED_MOVING, 1)
      end
    end

    if tickValue == 0 and gunshipsAlreadySpawned == 1 then
        local currentPosition = objc.get_entity_position(gunshipEntitiesList[1])
        for _, gunship in pairs(gunshipEntitiesList) do
             objc.stop_vfx(gunship, 'commanderGunshipSpawn.lvl', 'B_Gunship_Rig.pod', false)
            objc.play_vfx(gunship, 'commanderFlashScreen.lvl', 'B_Gunship_Rig.pod')
        end
        
        objc.kill_entity(entity)
    end
end
