local HAS_MECHAZOR_ALREADY_SPAWNED_KEY = 'hasMechazorAlreadySpawned'
local HAS_LANDING_EFFECTS_PLAYED_KEY = 'hasLandingEffectsPlayed'
local LANDING_EFFECT_KEY = 'landingEffect'
local OFFSET_FROM_BASE = 3500
local SCRIPT_DELAY = 10
local DELAY_KEY = 'delayTimerKey'
local FINISHED_DELAY = 'finishedDelayKey'
--[[
This TickTrigger spawns a Mechazor and plays the appropriate spawning VFX where
the Mechazor spawns, and then kills the VFX after a set amount of time.
--]]
legionCommanderMechazorScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)
   
   local delayTimer = objc.tick_timer(entity, DELAY_KEY, SCRIPT_DELAY)
   if delayTimer == 0 then
      objc.set_local_value(entity, FINISHED_DELAY, 1)
   end

   local finishedDelay = objc.get_local_value(entity, FINISHED_DELAY)
   if finishedDelay == 1 then   
      local spawnTimer = objc.tick_timer(entity, 'spawnTimer', 0)
      local hasMechazorAlreadySpawned = objc.get_local_value(entity, HAS_MECHAZOR_ALREADY_SPAWNED_KEY)
      
      local landingTimer = objc.tick_timer(entity, 'landingTimer', 19)
      local hasLandingEffectsPlayed = objc.get_local_value(entity, HAS_LANDING_EFFECTS_PLAYED_KEY)

      local killSwitch = objc.tick_timer(entity, 'killSwitch', 150)
      local killLandingEffectsTimer = objc.tick_timer(entity, 'killLandingEffectsTimer', 80)  -- this needs to be lower than the killSwitch timer
      
      local teamNumber = objc.get_team_number(entity)
      local playerNumber = objc.get_player_number(entity)

      local positionX = 0
      local positionY = 0
      local directionX = 0
      local directionY = 0

      local basePosition = objc.get_base_position(teamNumber)

      if teamNumber == 1 then
         --positionX = basePosition[1] + OFFSET_FROM_BASE
         directionX = 1
      elseif teamNumber == 2 then
         --positionX = basePosition[1] - OFFSET_FROM_BASE
         directionX = -1
      else
         objc.scripting_error('Encountered unknown player number ' .. teamNumber)
      end

      if spawnTimer == 0 and hasMechazorAlreadySpawned == 0 then
         local thisPos = objc.get_entity_position(entity)
         objc.spawn_at_point('B_Mechazor', 1, playerNumber, thisPos[1], thisPos[2], directionX, directionY)
         objc.set_local_value(entity, HAS_MECHAZOR_ALREADY_SPAWNED_KEY, 1)
      end

      if landingTimer == 0 and hasLandingEffectsPlayed == 0 then
         local landingVFXArray = objc.spawn_at_point('legionCommanderMechazorLandingEffect', 1, playerNumber, positionX, positionY, directionX, directionY)

         -- We know that we only spawn 1 VFX here. So only hold on to that one VFX
         objc.set_local_value(entity, LANDING_EFFECT_KEY, landingVFXArray[1])

         objc.set_local_value(entity, HAS_LANDING_EFFECTS_PLAYED_KEY, 1)
      end

      if killLandingEffectsTimer == 0 and hasLandingEffectsPlayed == 1 then
         local landingVFX = objc.get_local_value(entity, LANDING_EFFECT_KEY)
         if landingVFX ~= nil then
            objc.kill_entity(landingVFX)
         end 

      end

      if killSwitch == 0 then
         objc.kill_entity(entity)
      end
   end


   return 0
end
