local BEAM_VFX_ALREADY_SPAWNED_KEY = 'beamVFXAlreadySpawned'
local OFFSET_FROM_BASE = 3000
local SCRIPT_DELAY = 0
local DELAY_KEY = 'delayTimerKey'
local FINISHED_DELAY = 'finishedDelayKey'
local RIFLES_PER_ROW = 4

--[[
This TickTrigger initially spawns the beamVFXAlreadySpawned, and then spawns
two rows of 4 B_Rifleman.
--]]
legionCommanderRiflesScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)
   local delayTimer = objc.tick_timer(entity, DELAY_KEY, SCRIPT_DELAY)
   if delayTimer == 0 then
      objc.set_local_value(entity, FINISHED_DELAY, 1)
   end

   local finishedDelay = objc.get_local_value(entity, FINISHED_DELAY)
   if finishedDelay == 1 then 
      local tickValue = objc.tick_timer(entity, 'tickTime', 13)
      local beamVFXAlreadySpawned = objc.get_local_value(entity, BEAM_VFX_ALREADY_SPAWNED_KEY)

      if beamVFXAlreadySpawned == 0 then
         local teamNumber = objc.get_team_number(entity)
         local playerNumber = objc.get_player_number(entity)
         local positionX = 0
         local positionY = 0
         local basePosition = objc.get_base_position(teamNumber)

         if teamNumber == 1 then
            positionX = basePosition[1] + OFFSET_FROM_BASE
         elseif teamNumber == 2 then
            positionX = basePosition[1] - OFFSET_FROM_BASE
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end

         objc.spawn_at_point('legionCommanderSpawnBeam', 1, playerNumber, positionX, 0, 0, 0)
         objc.set_local_value(entity, BEAM_VFX_ALREADY_SPAWNED_KEY, 1)
      end
      
      if tickValue == 0 then
         legionCommanderRiflesScriptEntity_SpawnEntity('B_Rifleman', entity)
         objc.kill_entity(entity)
      end
   end

end

--[[
This function spawns two rows of 4 B_Rifleman by the player's base.
--]]
legionCommanderRiflesScriptEntity_SpawnEntity = function(entityToSpawn, teamNumberEntity)
   local teamNumber = objc.get_team_number(teamNumberEntity)
   local playerNumber = objc.get_player_number(entity)
   local basePosition = objc.get_base_position(teamNumber)
   local positionX = 0
   local positionY = -150

   local directionX = 0
   local directionY = 0

   if teamNumber == 1 then
      positionX = basePosition[1] + OFFSET_FROM_BASE
      directionX = 1
   elseif teamNumber == 2 then
      positionX = basePosition[1] - OFFSET_FROM_BASE
      directionX = -1
   else
      objc.scripting_error('Encountered unknown player number ' .. teamNumber)
   end

   for i = 1,RIFLES_PER_ROW do
      local spawnedEntities = objc.spawn_at_point(entityToSpawn, 1, playerNumber, positionX, positionY, directionX, directionY)
      for _, entity in pairs(spawnedEntities) do
         objc.play_vfx(entity, 'commanderSpawn.lvl', 'B_Rifleman_Rig.pod')
         objc.set_local_value(entity, 'hasCheckedForFactory', 1)
         objc.set_local_value(entity, 'unableToSpawnMultiple', 1)
      end   
      positionY = positionY + 300
   end

   if teamNumber == 1 then
      positionX = basePosition[1] + OFFSET_FROM_BASE + 800
   elseif teamNumber == 2 then
      positionX = basePosition[1] - OFFSET_FROM_BASE - 800
   else
      objc.scripting_error('Encountered unknown player number ' .. teamNumber)
   end

   positionY = -150
   
   for i = 1,RIFLES_PER_ROW do
      local spawnedEntities = objc.spawn_at_point(entityToSpawn, 1, playerNumber, positionX, positionY, directionX, directionY)
      for _, entity in pairs(spawnedEntities) do
         objc.play_vfx(entity, 'commanderSpawn.lvl', 'B_Rifleman_Rig.pod')
         objc.set_local_value(entity, 'hasCheckedForFactory', 1)
         objc.set_local_value(entity, 'unableToSpawnMultiple', 1)
      end
      positionY = positionY + 300
   end
end
