local ARROW_COUNT_KEY = 'arrowCount'
local ABILITY_ID_KEY = 'abilityIDKey'
local ARROW_AOE_SIZE = 500
local NUM_ARROW_ROWS = 8
local ARROWS_IN_ROW = 1
local RANDOM_RADIUS = 200
local LEFT_X_OFFSET = 0
local RIGHT_X_OFFSET = 9000
local UP_Y_OFFSET = 0
local DOWN_Y_OFFSET = 0
local SCALE_LEVEL = 'scaleLevelKey'

lineExplosionEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)

   local intervalTimer = objc.tick_timer(entity, 'intervalTimer', 2)
   local teamNumber = objc.get_team_number(entity)

   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)

   if arrowCount >= (NUM_ARROW_ROWS * ARROWS_IN_ROW) then
      objc.sacrifice_entity(entity)
      return 0
   end

   if intervalTimer == 0 then
      lineExplosionEntity_spawnArrow(teamNumber, entity)
   end
end

lineExplosionEntity_spawnArrow = function(teamNumber, entity)
   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)
   local effectIdentifier = 'lineExplosionIndividualEntity'

   local entityPos = objc.get_entity_position(entity)
   local xPos = entityPos[1]
   local yPos = entityPos[2]
   local startXPos = xPos
   local endXPos = xPos
   local xMod = 1

   if teamNumber == 1 then
      startXPos = xPos + LEFT_X_OFFSET
      endXPos = xPos + RIGHT_X_OFFSET
   elseif teamNumber == 2 then
      startXPos = xPos - LEFT_X_OFFSET
      endXPos = xPos - RIGHT_X_OFFSET
      xMod = -1
      effectIdentifier = 'lineExplosionIndividualEntity'
   else
      objc.scripting_error('Unknown player number encountered: ' .. teamNumber)
   end

   local thisY = yPos + objc.roll_number(-RANDOM_RADIUS, RANDOM_RADIUS)

   local xIndex = objc.divide_with_floor(arrowCount, ARROWS_IN_ROW)
   local xStep = objc.divide_with_floor((endXPos - startXPos), NUM_ARROW_ROWS)
   local thisX = (xStep * xIndex) + startXPos

   local playerNumber = objc.get_player_number(entity)

   local spawnedArrowArray = objc.spawn_at_point(effectIdentifier, 1, playerNumber, thisX, thisY, xMod, 0)
   objc.set_local_value(spawnedArrowArray[1], ABILITY_ID_KEY, objc.get_entity_value(entity, 'identityComponent.description'))
   objc.set_local_value(entity, ARROW_COUNT_KEY, arrowCount + 1)
   tierOfRoxie = objc.get_local_value(entity, SCALE_LEVEL)
   objc.set_local_value(spawnedArrowArray[1], SCALE_LEVEL, tierOfRoxie)
end
