local HAS_FIRED_KEY = 'hasFired'
local ABILITY_ID_KEY = 'abilityIDKey'
local DUMMY_ENTITY_ID = 'RoxieAbilityDummyEntity'
local DAMAGE_AMOUNT = 2000
local DAMAGE_RADIUS = 590
local TIER_KEY = 'tierKey'

--[[
This TickTrigger waits a given amount of time, and then damages all ground
units within a given radius of it.
--]]
lineExplosionIndividualEntity_tickTrigger = function(entity)
   local enemyNumber = objc.get_enemy_team_number(entity)
   local playerNumber = objc.get_player_number(entity)
   local fireArrowDamage = DAMAGE_AMOUNT

   local hasFired = objc.get_local_value(entity, HAS_FIRED_KEY)
   local damageDelay = objc.tick_timer(entity, 'damageDelay', 10)
   -- duration is long enough so that the effect can play out before being killed
   local duration = objc.tick_timer(entity, 'duration', 100)

   local abilityID = objc.get_local_value(entity, ABILITY_ID_KEY)

   if hasFired == 0 and damageDelay == 0 then
      local dummyEntity = objc.spawn(DUMMY_ENTITY_ID, 1, playerNumber)
      for _, enemy in pairs(objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyNumber)) do
         if (objc.is_normal_unit(enemy) or objc.is_commander(enemy)) and objc.get_local_value(enemy, ABILITY_ID_KEY) ~= abilityID then
            objc.inflict_normal_damage(dummyEntity[1], enemy, fireArrowDamage)
            objc.flash_entity_for_damage(enemy, 250)
            objc.set_local_value(enemy, ABILITY_ID_KEY, abilityID)
         end
      end

      objc.sacrifice_entity(dummyEntity[1])
      objc.set_local_value(entity, HAS_FIRED_KEY, 1)
   end

   if duration == 0 then
      objc.sacrifice_entity(entity)
   end

   return 0
end
