local HAS_INIT = 'hasInitKey'
local CENTRAL_OBJECTIVE_MARKER_NAME = 'GroundCircle'
local MINER_WAYPOINT = 'MinerWaypoint'
local LEGION_BARRIER = 'B_Barricade'
local ROGUE_BARRIER = 'R_Barricade'
local ASCENSION_BARRIER = 'W_Barricade'
local SECOND_BARRIER_DISTANCE_FROM_CENTER = 2600
local MINER_PLACEMENT_BUFFER = -100

longMap2vE_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end
        
        local p1Mines = objc.location_marker_node_names_with_prefix("P1_Mine")
        for k,v in pairs(p1Mines) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalMineAtPosition(x, y, 1)
        end

        local p2Mines = objc.location_marker_node_names_with_prefix("P2_Mine")
        for k,v in pairs(p2Mines) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalMineAtPosition(x, y, 2)
        end

        if is2vE == 1 then
            local p3Mines = objc.location_marker_node_names_with_prefix("P3_Mine")
            for k,v in pairs(p3Mines) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalMineAtPosition(x, y, 3)
            end

            local p4Mines = objc.location_marker_node_names_with_prefix("P4_Mine")
            for k,v in pairs(p4Mines) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalMineAtPosition(x, y, 4)
            end
        end

        local x, y = objc.location_of_marker_node_named("P1_DropOff")
        spawnMinerWaypointAtPosition(x,y,1)

        local x, y = objc.location_of_marker_node_named("P2_DropOff")
        spawnMinerWaypointAtPosition(x,y,2)

        local team1Entities = objc.list_entities(1)
        for _,unit in pairs(team1Entities) do
            if objc.is_barricade(unit) then
                objc.sacrifice_entity(unit)
            end
        end

        spawnBarrier(-1*SECOND_BARRIER_DISTANCE_FROM_CENTER, 0, 1)

        local playerNumbers = objc.get_player_numbers_for_team_number(1) -- if this is a 2v2 or a 2ve, set base income to 75%
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                objc.set_entity_value_for_key(objc.get_base_entity(1), ENTITY_VALUE_KEY_INCOME_TIER, 1)
                objc.set_entity_value_for_key(objc.get_base_entity(2), ENTITY_VALUE_KEY_INCOME_TIER, 1)
            end
        end

        objc.spawn('gameRulesEntity', 1, 1)
        objc.set_local_value(entity, HAS_INIT, 1)
    elseif hasInit == 1 then
        --[[
        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end

        local px, py = objc.location_of_marker_node_named("P1_DropOff")
        local p1Miners = objc.location_marker_node_names_with_prefix("P1_MinePP")
        for k,v in pairs(p1Miners) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalWorker(px - MINER_PLACEMENT_BUFFER, py, 1)
        end

        local px, py = objc.location_of_marker_node_named("P2_DropOff")
        local p2Miners = objc.location_marker_node_names_with_prefix("P2_MinePP")
        for k,v in pairs(p2Miners) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalWorker(px + MINER_PLACEMENT_BUFFER, py, 2)
        end

        if is2vE == 1 then
            local p3Miners = objc.location_marker_node_names_with_prefix("P3_MinePP")
            for k,v in pairs(p3Miners) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalWorker(x, y, 3)
            end

            local p4Miners = objc.location_marker_node_names_with_prefix("P4_MinePP")
            for k,v in pairs(p4Miners) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalWorker(x, y, 4)
            end
        end]]

        objc.set_local_value(entity, HAS_INIT, 2)
    end
end

spawnCrystalMineAtPosition = function(xPos, yPos, playerNumber)
    objc.spawn_skinned_entity_with_key_at_point(SKINNED_ENTITY_KEY_CRYSTAL_MINE, 1, playerNumber, xPos, yPos, 1, 0)
end

spawnMinerWaypointAtPosition = function(xPos, yPos, playerNumber)
    objc.spawn_at_point(MINER_WAYPOINT, 1, playerNumber, xPos, yPos, 1, 0)
end

spawnCrystalObjectiveAtPosition = function(xPos, yPos)
    local centerObjective = objc.spawn_skinned_entity_with_key_at_point(SKINNED_ENTITY_KEY_CENTRAL_OBJECTIVE, 1, 0, xPos, yPos, 0, 0)
    objc.set_entity_value_for_key(centerObjective[1], ENTITY_VALUE_KEY_INCOME_TIER, 0)
    return centerObjective[1]
end

spawnCrystalWorker = function(xPos, yPos, side)
    local faction = objc.get_commander_factions(side)
    if faction[1] == FACTION_BLUE then
        objc.spawn_at_point(ASCENSION_MINER, 1, side, xPos, yPos, 1, 0)
    elseif faction[1] == FACTION_GREEN then
        objc.spawn_at_point(LEGION_MINER, 1, side, xPos, yPos, 1, 0)
    elseif faction[1] == FACTION_RED then
        objc.spawn_at_point(ROGUE_MINER, 1, side, xPos, yPos, 1, 0)
    else
        objc.spawn_at_point(ROGUE_MINER, 1, side, xPos, yPos, 1, 0)
    end
end

spawnBarrier = function(xPos, yPos, side)
    local faction = objc.get_commander_factions(side)
    local xdirectionToFace = 0
    if side == 1 then
        xdirectionToFace = 1
    else
        xdirectionToFace = -1
    end
    if faction[1] == FACTION_BLUE then
        objc.spawn_at_point(ASCENSION_BARRIER, 1, side, xPos, yPos, xdirectionToFace, 0)
    elseif faction[1] == FACTION_GREEN then
        objc.spawn_at_point(LEGION_BARRIER, 1, side, xPos, yPos, xdirectionToFace, 0)
    elseif faction[1] == FACTION_RED then
        objc.spawn_at_point(ROGUE_BARRIER, 1, side, xPos, yPos, xdirectionToFace, 0)
    else
        objc.spawn_at_point(ROGUE_BARRIER, 1, side, xPos, yPos, xdirectionToFace, 0)
    end
end
