local HAS_BIG_SLOW_INIT = 'hasBigSlowInit'
local ORIGINAL_PLANE_OF_EXISTENCE = 'originalPlaneOfExistenceKey'

local STAT_BUFF_DENOMINATOR = 2
local STAT_BUFF_NUMERATOR = 3

local MOVE_SPEED_NUMERATOR = 3
local MOVE_SPEED_DENOMINATOR = 5

local CHECK_TICK = 3
local CHECK_KEY = 'checkKey'

local UNIT_CHECK_RADIUS = 1000

local PLANE_MOVE_DELAY = 1
local FALLING_TIME = 1000

mod_BigSlowUnits_tickTrigger = function(entity)
    local checkTimer = objc.tick_timer(entity, CHECK_KEY, CHECK_TICK)
    if checkTimer == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            local hasInit = objc.get_local_value(unit, HAS_BIG_SLOW_INIT)
            if objc.is_normal_unit(unit) and hasInit == 0 then
                local teamNumber = objc.get_team_number(unit)
                local identifier = objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER)
                local nearbyAllies = objc.list_entities_in_radius(unit, UNIT_CHECK_RADIUS, teamNumber)
                local spawnPosition = 0
                local numAdds = 1
                for _, ally in pairs(nearbyAllies) do
                    local thisIdentifier = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_IDENTIFIER)
                    if thisIdentifier == identifier and ally ~= unit then
                        objc.idle_entity(ally, FALLING_TIME)
                        objc.set_local_value(ally, HAS_BIG_SLOW_INIT, 1)
                        numAdds = numAdds + 1
                        spawnPosition = objc.get_entity_position(ally)
                        objc.set_solid(ally, false)
                        objc.add_trigger(ally, TRIGGER_NAME_TICK_TRIGGER, 'mod_BigSlowUnits.lua', 'mod_BigSlowUnits_deathDelayTrigger')
                    end
                end

                if numAdds > 1 then
                    local bigUnit = objc.spawn_at_point(identifier, 1, teamNumber, spawnPosition[1], spawnPosition[2], -1, 0)
                    local originalPlane = objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_PLANE_OF_EXISTENCE)
                    objc.set_local_value(bigUnit[1], ORIGINAL_PLANE_OF_EXISTENCE, originalPlane)
                    mod_BigSlowUnits_setToOppositePlane(bigUnit[1], originalPlane)
                    objc.add_trigger(bigUnit[1], TRIGGER_NAME_TICK_TRIGGER, 'mod_BigSlowUnits.lua', 'mod_BigSlowUnits_movePlaneTrigger')
                    objc.set_entity_scale_percentage(bigUnit[1], 175)
                    objc.idle_entity(bigUnit[1], FALLING_TIME)
                    objc.set_local_value(bigUnit[1], HAS_BIG_SLOW_INIT, 1)

                    local currentHealth = objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_CURRENT_HEALTH)
                    local maxHealth = objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_MAX_HEALTH)
                    local attackDamage = objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_ATTACK_DAMAGE)
                    local attackBuff = objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_DAMAGE_BUFF)

                    local newAttackDamage = math.floor(attackDamage * numAdds)
                    local newMaxHealth = math.floor(maxHealth * numAdds)
                    local newCurrentHealth = math.floor(currentHealth * numAdds)

                    local damageToSet = (objc.divide_with_ceiling(newAttackDamage, STAT_BUFF_DENOMINATOR) * STAT_BUFF_NUMERATOR) - attackDamage

                    objc.set_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_CURRENT_HEALTH, objc.divide_with_ceiling(newCurrentHealth, STAT_BUFF_DENOMINATOR) * STAT_BUFF_NUMERATOR)
                    objc.set_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_MAX_HEALTH, objc.divide_with_ceiling(newMaxHealth, STAT_BUFF_DENOMINATOR) * STAT_BUFF_NUMERATOR)
                    objc.set_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_DAMAGE_BUFF, attackBuff + damageToSet)

                    local newSpeed = objc.divide_with_ceiling(objc.get_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_MAX_VELOCITY) * MOVE_SPEED_NUMERATOR, MOVE_SPEED_DENOMINATOR)
                    objc.set_entity_value_for_key(bigUnit[1], ENTITY_VALUE_KEY_MAX_VELOCITY, newSpeed)

                    objc.set_solid(unit, false)
                    objc.idle_entity(unit, FALLING_TIME)
                    objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'mod_BigSlowUnits.lua', 'mod_BigSlowUnits_deathDelayTrigger')
                end

                objc.set_local_value(unit, HAS_BIG_SLOW_INIT, 1)
            end
        end
    end
end

mod_BigSlowUnits_setToOppositePlane = function(entity, originalPlane)
    if originalPlane == NUM_PLANE_OF_EXISTENCE_AIR then
        objc.move_entity_to_planes_of_existence(entity, PLANE_OF_EXISTENCE_GROUND, 0, EASE_IN, 60)
    elseif originalPlane == NUM_PLANE_OF_EXISTENCE_GROUND then
        objc.move_entity_to_planes_of_existence(entity, PLANE_OF_EXISTENCE_AIR, 0, EASE_IN, 60)
    end
end

mod_BigSlowUnits_movePlaneTrigger = function(entity)
    local planeMoveDelay = objc.tick_timer(entity, 'moveDelayTimer', PLANE_MOVE_DELAY)
    if planeMoveDelay == 0 then
        local originalPlane = objc.get_local_value(entity, ORIGINAL_PLANE_OF_EXISTENCE)
        if originalPlane == NUM_PLANE_OF_EXISTENCE_AIR then
            objc.move_entity_to_planes_of_existence(entity, PLANE_OF_EXISTENCE_AIR, FALLING_TIME, EASE_IN, 60)
        elseif originalPlane == NUM_PLANE_OF_EXISTENCE_GROUND then
            objc.set_entity_air_height_variance(entity, 1000)
            objc.move_entity_to_planes_of_existence(entity, PLANE_OF_EXISTENCE_GROUND, FALLING_TIME, EASE_IN, 60)
        end
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'mod_BigSlowUnits_movePlaneTrigger')
    end
end

mod_BigSlowUnits_deathDelayTrigger = function(entity)
    local deathDelay = objc.tick_timer(entity, 'deathDelayTimer', objc.divide_with_floor(FALLING_TIME, 100))
    if deathDelay == 0 then
        local entityPos = objc.get_entity_position(entity)
        local teamNumber = objc.get_team_number(entity)
        objc.spawn_at_point('SumoCraterEffectEntity', 1, teamNumber, entityPos[1], entityPos[2], 0, 0)
        objc.sacrifice_entity(entity)
    end
end