
#define SUPPORT_GLSL300
#include "prefix.sh"
#include "texture_utility.sh"
#include "projection_utility.sh"

uniform sampler2D s_cc3Texture2Ds[2];
VARYING_IN vec2 v_texCoord;

#if __VERSION__ >= 300
    #define o_fragColor destColor
#else
    #define o_fragColor gl_FragColor
#endif

vec3 sampleColor(vec2 offset) {
    // DavidJ -- NOTE -- getting much better results with a slight
    //      offset back in the coords. Without this, we see noticable creeping
    //      in the pixels (ie, the downsample doesn't appear to to be correctly centered)
    //      The reason for this offset is unclear... It doesn't appear to be related to
    //      fragment/pixel coordinate differences. It could be floating point creep problems,
    //      off-by-one problems in reciprocalViewportSize, or just general inaccuracy related to
    //      that function.
    const vec2 accuracyOffset = vec2(-0.25, -0.25);
    return sampleLevelZero(s_cc3Texture2Ds[0], (offset + accuracyOffset) * reciprocalViewportSize() + v_texCoord.xy).rgb;
    // return sampleLevelZero(s_cc3Texture2Ds[0], (gl_FragCoord.xy + offset) * reciprocalViewportSize()).rgb;
}

float sampleDepthIndirect(highp sampler2D depthSampler, vec2 tc) {
    return sampleLevelZero(depthSampler, tc).x;
}

float sampleDepth(vec2 offset) {
    const vec2 accuracyOffset = vec2(-0.25, -0.25);
    return sampleDepthIndirect(s_cc3Texture2Ds[1], (offset + accuracyOffset) * reciprocalViewportSize() + v_texCoord.xy);
    // return sampleDepthIndirect(s_cc3Texture2Ds[1], (gl_FragCoord.xy + offset) * reciprocalViewportSize());
}

void main() {
    
    // Here, we downsample both the color and depth buffers in a single pass, using a filter
    // intended for use with multi-resolution rendering.
    //
    // For color, we're using a 2x2 box filter (ie, just taking the average of 4 pixels)
    // For depth, we're using min and max in a chequerboard pattern
    
    vec3 c00 = sampleColor(vec2(0.0, 0.0));
    vec3 c01 = sampleColor(vec2(0.0, 0.5));
    vec3 c10 = sampleColor(vec2(0.5, 0.0));
    vec3 c11 = sampleColor(vec2(0.5, 0.5));
    
    o_fragColor.rgb = (c00 + c01 + c10 + c11) * 0.25;
    o_fragColor.a = 0.0;    // initialize the alpha channel with zeroes for this pass
    
    float d00 = sampleDepth(vec2(0.0, 0.0));
    float d01 = sampleDepth(vec2(0.0, 0.5));
    float d10 = sampleDepth(vec2(0.5, 0.0));
    float d11 = sampleDepth(vec2(0.5, 0.5));
    
    float minDepth = min(min(d00, d01), min(d10, d11));
    float maxDepth = max(max(d00, d01), max(d10, d11));
    
    bool chequerboard = ((int(gl_FragCoord.x) + int(gl_FragCoord.y)) & 0x1) != 0x0;
    gl_FragDepth = chequerboard ? minDepth : maxDepth;
}
