//#define DISTORT_NORMALS
//#define LIGHTMAP_FOAM
//#define DEFORM_MESH
//#define PERTURB_NORMALS

#include "prefix_tow.sh"

#if defined(DEFORM_MESH) && defined(PERTURB_NORMALS)
#if CC3_PLATFORM_IOS
#extension GL_OES_standard_derivatives : require
#endif
#endif

#if defined(GL_ES)
    precision highp float;
#endif

uniform sampler2D s_cc3Texture2Ds[4];
uniform samplerCube s_cc3TextureCube;

uniform float noiseLevel;

uniform vec3 normalClamp;

uniform float reflectionLevel;
uniform float reflectionPower;

uniform vec3 deepOceanBlendFactor;
uniform vec3 shallowOceanBlendFactor;

uniform vec4 foamColor;
uniform float foamMultiplier;

varying vec3 v_viewpoint_world;

varying vec2 v_waveTexCoord;

#ifdef DISTORT_NORMALS
varying vec2 v_noiseTexCoord;
#endif

#if defined(DEFORM_MESH) && defined(PERTURB_NORMALS)
varying vec3 v_normal_world;
#endif

#ifdef LIGHTMAP_FOAM
varying vec2 v_foamTexCoord;
varying vec2 v_lightMapTexCoord;
#endif

#include "shadow.fsh"

#if defined(DEFORM_MESH) && defined(PERTURB_NORMALS)
mat3 cotangent_frame(vec3 N, vec3 V, vec2 texCoord) {
    vec3 dp1 = dFdx(V);
    vec3 dp2 = dFdy(V);
    vec2 duv1 = dFdx(texCoord);
    vec2 duv2 = dFdy(texCoord);

    vec3 dp2perp = cross(dp2, N);
    vec3 dp1perp = cross(N, dp1);
    vec3 T = dp2perp * duv1.x + dp1perp * duv2.x;
    vec3 B = dp2perp * duv1.y + dp1perp * duv2.y;
    float invmax = inversesqrt(max(dot(T,T), dot(B,B)));
    return mat3(T * invmax, B * invmax, N);
}
#endif

void main() {
#ifdef DISTORT_NORMALS
    vec4 noiseTexel = texture2D(s_cc3Texture2Ds[1], v_noiseTexCoord);
    vec2 delta = noiseLevel * (noiseTexel.xy * 2.0 - 1.0);
    vec2 waveTexCoord = v_waveTexCoord + delta;
#else
    vec2 waveTexCoord = v_waveTexCoord;
#endif

    vec4 oceanTexel = texture2D(s_cc3Texture2Ds[0], waveTexCoord);
    vec4 normalTexel = texture2D(s_cc3Texture2Ds[1], waveTexCoord);

    vec3 normal = normalTexel.xyz * normalClamp * 2.0 - normalClamp;
    vec3 V = v_viewpoint_world;

#if defined(DEFORM_MESH) && defined(PERTURB_NORMALS)
    mat3 TBN = cotangent_frame(normalize(v_normal_world), -V, waveTexCoord);
    vec3 N = TBN * normal;
#else
    vec3 N = vec3(normal.x, normal.z, -normal.y);
#endif

    vec3 R = reflect(-V, N);

    vec3 reflectionColor = textureCube(s_cc3TextureCube, R).rgb;
    float glossiness = normalTexel.a;

#ifdef LIGHTMAP_FOAM
    vec4 foamTexel = texture2D(s_cc3Texture2Ds[2], v_foamTexCoord);
    vec4 lightMapTexel = texture2D(s_cc3Texture2Ds[3], v_lightMapTexCoord);

    float foamMask = lightMapTexel.r;
    float shadowMask = lightMapTexel.g;
    float colorLerp = lightMapTexel.b;
    float opacityMod = lightMapTexel.a;

    vec3 oceanColor = oceanTexel.rgb * mix(deepOceanBlendFactor, shallowOceanBlendFactor, colorLerp);

    vec3 color = mix(oceanColor, reflectionColor, reflectionLevel * glossiness);

    float foamStrength = foamMask * foamTexel.r * foamMultiplier;
    color = mix(color, foamColor.rgb, min(0.92, foamStrength)) * vec3(shadowMask);
    float opacity = mix(1.0, foamStrength, min(0.92, foamStrength)) * opacityMod;

#else
    vec3 color = mix(oceanTexel.rgb, reflectionColor, reflectionLevel * glossiness);
    float opacity = oceanTexel.a;
#endif

    gl_FragColor = vec4(color, opacity);

    shadow_receive();
}
