//#define DISTORT_NORMALS
//#define LIGHTMAP_FOAM
//#define DEFORM_MESH
//#define PERTURB_NORMALS

#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

attribute vec4 a_cc3Position;
attribute vec4 a_cc3Normal;
attribute vec4 a_cc3Color;
attribute vec2 a_cc3TexCoord0;

#ifdef LIGHTMAP_FOAM
attribute vec2 a_cc3TexCoord1;
attribute vec2 a_cc3TexCoord2;
#endif

// uniforms
uniform mat4 u_cc3MatrixModel;
uniform mat4 u_cc3MatrixView;
uniform mat4 u_cc3MatrixProj;
uniform vec2 u_cc3SceneTime;
uniform vec3 u_cc3CameraPositionGlobal;

uniform float waveMinHeight;
uniform float waveMaxHeight;

uniform float waveLength;
uniform float waveFrequency;

uniform vec2 waveSpeed;
uniform vec2 noiseSpeed;
uniform vec2 foamSpeed;

uniform float waveTexCoordFactor;
uniform float noiseTexCoordFactor;

// varyings
varying vec3 v_viewpoint_world;

varying vec2 v_waveTexCoord;
varying vec2 v_noiseTexCoord;

#ifdef PERTURB_NORMALS
varying vec3 v_normal_world;
#endif

#ifdef LIGHTMAP_FOAM
varying vec2 v_foamTexCoord;
varying vec2 v_lightMapTexCoord;
#endif

#include "shadow.vsh"

vec2 sanitize_time2(float originalTime, vec2 multiplier) {
    float moddedTime = mod(originalTime, 1.0 / (1e-5 + max(abs(multiplier.x), abs(multiplier.y))));
    return moddedTime * multiplier;
}

void main() {
    vec4 position_world = u_cc3MatrixModel * a_cc3Position;

    v_waveTexCoord = waveTexCoordFactor * a_cc3TexCoord0 + sanitize_time2(u_cc3SceneTime[0], waveSpeed);

#ifdef DISTORT_NORMALS
    v_noiseTexCoord = noiseTexCoordFactor * a_cc3TexCoord0 + sanitize_time2(u_cc3SceneTime[0], noiseSpeed);
#endif

#ifdef LIGHTMAP_FOAM
    v_foamTexCoord = a_cc3TexCoord1 + sanitize_time2(u_cc3SceneTime[0], foamSpeed);
    v_lightMapTexCoord = a_cc3TexCoord2;
#endif

#ifdef DEFORM_MESH
    float waveHeight = mix(waveMinHeight, waveMaxHeight, a_cc3Color.r);

    float dist = distance(position_world.xz, vec2(0.0));
    float phase = waveLength * dist + u_cc3SceneTime[0] * waveFrequency;

    position_world.y += waveHeight * (cos(phase) + 1.0) / 2.0;

    float jacobianXZ = 0.5 * waveLength * waveHeight * sin(phase) / dist;
#ifdef PERTURB_NORMALS
    v_normal_world = mat3(u_cc3MatrixModel) * vec3(-jacobianXZ * position_world.x, 1.0, -jacobianXZ * position_world.z);
#endif
#endif

    v_viewpoint_world = u_cc3CameraPositionGlobal - position_world.xyz;

    gl_Position = u_cc3MatrixProj * u_cc3MatrixView * position_world;

    shadow_receive(position_world);
}
