local LINE_ENTITY_TYPE_KEY = 'lineEntityTypeKey'
local SHOULD_CAPTURE_KEY = 'shouldBeCapturedKey'
local IS_BUFFED_KEY = 'isLineBuffedKey'
local BUFF_AMOUNT = 30
local BUFF_TICK = 5
local BUFF_KEY = 'buffTickTimerKey'
local DESC_ID_KEY = 'descriptionIdKey'
local HAS_INIT = 'hasInit'

objectiveDamageEntity_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        local thisId = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_DESCRIPTION)
        local descIdKey = 'buffedBy' .. thisId
        objc.set_local_value(entity, DESC_ID_KEY, descIdKey)
        objc.set_local_value(entity, HAS_INIT, 1)
    end

    local buffTickTime = objc.tick_timer(entity, BUFF_KEY, BUFF_TICK)
    if buffTickTime == 0 then
        local teamNumber = objc.get_team_number(entity)
        local buffedByThisKey = objc.get_local_value(entity, DESC_ID_KEY)
        local allies = objc.list_entities(teamNumber)

        for _, ally in pairs(allies) do
            local buffedByThis = objc.get_local_value(ally, buffedByThisKey)
            if buffedByThis == 0 and objc.is_normal_unit(ally) then
                local isBuffed = objc.get_local_value(ally, IS_BUFFED_KEY)
                if isBuffed == 0 then
                    local currentBuff = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF)
                    objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF, currentBuff + BUFF_AMOUNT)
                    objc.set_local_value(ally, IS_BUFFED_KEY, 1)
                elseif isBuffed == 1 then
                    local currentBuff = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF)
                    objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF, currentBuff + (BUFF_AMOUNT * 2))
                    objc.set_local_value(ally, IS_BUFFED_KEY, 2)
                end

                objc.set_local_value(ally, buffedByThisKey, 1)
            end
        end
    end
end

objectiveDamageEntity_deathTrigger = function(entity)
    local entityPos = objc.get_entity_position(entity)
    local teamNumber = objc.get_team_number(entity)
    local thisId = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_DESCRIPTION)
    local buffedByThisKey = objc.get_local_value(entity, DESC_ID_KEY)
    local allies = objc.list_entities(teamNumber)

    for _, ally in pairs(allies) do
        local buffedByThis = objc.get_local_value(ally, buffedByThisKey)
        if buffedByThis == 1 and objc.is_normal_unit(ally) then
            local isBuffed = objc.get_local_value(ally, IS_BUFFED_KEY)
            if isBuffed == 1 then
                local currentBuff = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF)
                objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF, currentBuff - BUFF_AMOUNT)
                objc.set_local_value(ally, IS_BUFFED_KEY, 0)
            elseif isBuffed == 2 then
                local currentBuff = objc.get_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF)
                objc.set_entity_value_for_key(ally, ENTITY_VALUE_KEY_DAMAGE_BUFF, currentBuff - (BUFF_AMOUNT * 2))
                objc.set_local_value(ally, IS_BUFFED_KEY, 1)
            end

            objc.set_local_value(ally, buffedByThisKey, 0)
        end
    end


    local spawned = objc.spawn_at_point('IncomeLineEntityNeutral', 1, 1, 0, entityPos[2], 1, 0)
    objc.set_local_value(spawned[1], LINE_ENTITY_TYPE_KEY, 'damageBuff')
end
