#define SUPPORT_GLSL300
#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

#if __VERSION__ >= 300
    layout(location = 0) out vec4 primaryOutput;
    #if defined(BORDER) && (BORDER_METHOD == BORDER_METHOD_PIXEL_BASED_MRT)
        layout(location = 1) out int secondaryDepth;
    #endif
    #define gl_FragColor primaryOutput

    #define textureCube texture
#endif

#if CC3_PLATFORM_WINDOWS // win32
    #undef INSTANCING_ENABLED
#else
    #if !defined(INSTANCING_HARD_DISABLE)
        #define INSTANCING_ENABLED
    #else
        #undef INSTANCING_ENABLED
    #endif
#endif

#include "environmentconfig.sh"
#define M_PI 3.1415926535897932384626433832795
#if defined(INSTANCING_ENABLED)
    #define MAX_SKINNED_INSTANCES 10
#else
    #define MAX_SKINNED_INSTANCES 1
#endif
#define M_GRAYSCALE vec3(0.2126, 0.7152, 0.0722)

//===================================================================================================================//
// UNIFORMS //
//===================================================================================================================//
uniform sampler2D s_cc3Texture2Ds[7];
uniform samplerCube u_reflectionMap;
uniform sampler2D sphereMap;
uniform sampler2D cloudTexture;
uniform samplerCube nebulaMap;

uniform vec3 u_cc3CameraPositionGlobal;
uniform vec2 u_cc3SceneTime;
uniform mat4 u_cc3MatrixView;

uniform vec3 GlobalUniform_RegionMaskChannel;

#if defined(OCEAN_BACKGROUND)
    uniform vec3 FixedLightDirection;
#endif

// Sampler Indices //
const int normal_map_index = 0;
const int utility0_map_index = 1; //repeat U+V r = normal noise map  g = foam  b = fresnel curves -- ALL
const int regionmask_map_index = 2;
const int regionreveal_map_index = 3;
const int ao_map_index = 4; //RGB multiply

#if defined(WORLD_WATER_LIGHTMAP)
    const int light_map_index = ao_map_index + 1; //RGB additive
#else
    const int light_map_index = ao_map_index; //RGB additive
#endif

#if defined(WORLD_WATERFALL) || defined(WORLD_WATERSHORE)
    const int utility1_map_index = light_map_index + 1; 
        //repeat U+V(FALLS) repeat U + clamp V(SHORE) r = BLACK/empty  g = shore1/waterfall1  b = shore2/waterfall2 -- SHORE + FALLS ONLY
#endif

#ifdef SHADER_DEBUG
    uniform int DebugIndex;
    uniform float DebugFloat;
    uniform vec3 DebugVec3;
    uniform vec4 DebugVec4;
#endif


uniform float RegionMaskMultiplier;


uniform float u_cc3ObjectTime[MAX_SKINNED_INSTANCES];
uniform sampler2D dissolveMask;

uniform vec4 WaterShallowColor;
uniform vec4 WaterFoamColor;
uniform vec4 WaterDeepColor;

uniform float RegionReveal;
uniform float RegionStartTime;
uniform float RegionDuration;

uniform float SpecularPower;
uniform vec3 WorldPosition;
uniform vec3 WorldRotation;

uniform vec2 WaterUVScale;
uniform vec2 WaterScrollSpeed;

#if !defined(WORLD_WATERSHORE)
    uniform float FresnelIndex;
    #if !defined(WORLD_WATERFALL)
        uniform float ReflectFresnelIndex;
    #endif
#endif

#if defined(WORLD_WATERSHORE)
    uniform vec3 WaveSine1;
    uniform vec3 WaveSine2;
    uniform vec2 WaveUV1;
    uniform vec2 WaveUV2;

#endif

#if defined(WORLD_WATERFALL)
    uniform vec2 Stream1UV;
    uniform vec2 Stream2UV;
#endif

vec3 DiffuseColorBlend(vec3 diffuseColor) { return diffuseColor; }

#include "property_map.fsh"
#include "mantis_lighting.sh"        // DavidJ -- these #include are commented out temporarily, until we release a new version of the level editor
#include "tiled_lighting.sh"
#include "shadow.fsh"

//===================================================================================================================//
// VARYINGS //
//===================================================================================================================//

#ifdef INSTANCING_ENABLED
    #if defined(INTEGER_ATTRIBUTES)
        VARYING_IN int v_instanceID;
    #else
        VARYING_IN float v_instanceID; // "int" with small constant added to handle flooring
    #endif
    #define INSTANCE_ID (int(v_instanceID))
#else
    #define INSTANCE_ID 0
#endif

VARYING_IN vec3 v_position_world;

VARYING_IN vec3 v_N;
VARYING_IN vec3 v_T;
VARYING_IN vec3 v_B;

VARYING_IN vec2 v_texCoord;
VARYING_IN vec2 v_lightMapCoord;

#if defined(INSTANCING_ENABLED)
    VARYING_IN vec4 v_material_diffuse_color;
#endif

VARYING_IN vec4 v_vertexColor;

VARYING_IN vec3 v_world_light_dir;


//===================================================================================================================//
// SHADER //
//===================================================================================================================//

#if defined(INSTANCING_ENABLED) && !defined(SHADOW_GEN)
    vec4 getMaterialColor() { return v_material_diffuse_color; }
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 getMaterialColor() { return u_cc3MaterialDiffuseColor; }
#endif

//===================================================================================================================//
void main() 
{
#if defined(SHADOW_GEN)
    
#else

    // Vertex Colors //

    float shallowColorVertBlend = v_vertexColor.r;

    #if defined(WORLD_WATERFALL)
        float waterSlopeColorVertBlend = v_vertexColor.g;
    #endif

    #if !defined(WORLD_WATERFALL) && !defined(WORLD_WATERSHORE)
        float bumpValueVertBlend = v_vertexColor.g;
    #endif

    float foamValueVertBlend = v_vertexColor.b;
    float alphaColorVertBlend = v_vertexColor.a;


    // View Vector //
        vec3 V = (u_cc3CameraPositionGlobal - v_position_world);
        float V_length = length(V);
        V = normalize(V);


    // UVs //

    vec2 lightmap_uvs = v_lightMapCoord;

#if !defined(WORLD_WATERSHORE) && !defined(WORLD_WATERFALL) && defined(WORLD_WATER_LIGHTMAP)
    lightmap_uvs = v_texCoord;
#endif

        //time for scrolling
        const float repeatTime = 10.0e3;
        float time = mod(u_cc3ObjectTime[INSTANCE_ID], repeatTime);

        //rotation matrix
        vec3 c = cos(WorldRotation * M_PI/180.0);
        vec3 s = sin(WorldRotation * M_PI/180.0);

        mat3 rotMatrix = mat3(
        (c.y * c.z) + (s.x * s.y * s.z),
        c.x * s.z, 
        (c.y * s.x * s.z) - (c.z * s.y),

        (c.z * s.x * s.y) - (c.y * s.z),
        c.x * c.z,
        (c.y * c.z * s.x) + (s.y * s.z),

        c.x * s.y,
        -s.x,
        c.x * c.y
        );

        //UVs for region reveal mask
        vec3 dir = normalize(v_position_world - WorldPosition);
        dir = rotMatrix * dir;
        float u = atan(dir.z, dir.x) * 0.5 / M_PI + 0.25;
        float v = dir.y * 0.35 + 0.5;
        vec2 world_uvs = vec2(u * -1.0, v);

        //tiled wave - used for normals/bump/spec
        vec2 tiled_uv = lightmap_uvs;

        vec2 water_uvs = tiled_uv * WaterUVScale;
        vec2 water_uvs2 = (water_uvs + (vec2(-u_cc3SceneTime.x, u_cc3SceneTime.x + 0.5) * WaterScrollSpeed)) * 1.33;
        water_uvs += (vec2(u_cc3SceneTime.x, 0.0) * WaterScrollSpeed);

        float foam_uv_scale_mult = 4.0;
        float foam_speed = time * 0.1 / foam_uv_scale_mult;
        vec2 foam_scale = WaterUVScale * vec2(foam_uv_scale_mult, foam_uv_scale_mult);

        //shore 1 and 2

        #if defined(WORLD_WATERSHORE)

            float waveFreq1 = WaveSine1.x;
            float waveCycle1 = WaveSine1.y;
            float waveAmp1 = WaveSine1.z;
            float waveFreq2 = WaveSine2.x;
            float waveCycle2 = WaveSine2.y;
            float waveAmp2 = WaveSine2.z;
            float waveHeightOffset1 = WaveUV1.y;
            float waveHeightOffset2 = WaveUV2.y;

            float posOffset1 = v_texCoord.x * waveCycle1;
            float posOffset2 = v_texCoord.x * waveCycle2;
            float wave_move1 = sin(posOffset1 + time * waveFreq1) * 0.5;
            float wave_move2 = sin(posOffset2 + time * waveFreq2) * 0.5;

            vec2 waveMod1 = vec2(1.0, clamp(v_texCoord.y - waveHeightOffset1 + wave_move1 * waveAmp1, 0.0, 1.0));
            vec2 waveMod2 = vec2(1.0, clamp(v_texCoord.y - waveHeightOffset2 + wave_move2 * waveAmp2, 0.0, 1.0));;


            float xCoord0 = v_texCoord.x + waveMod1.x;
            float xCoord1 = v_texCoord.x + waveMod2.x;
            float yCoord0 = clamp(v_texCoord.y + waveMod1.y, 0.0, 1.0);
            float yCoord1 = clamp(v_texCoord.y + waveMod2.y, 0.0, 1.0);


            vec2 watershoreUV0 = vec2(xCoord0, yCoord0) * vec2(WaveUV1.x, 1.0);
            vec2 watershoreUV1 = vec2(xCoord1, yCoord1) * vec2(WaveUV2.x, 1.0);

        #endif

        #if defined(WORLD_WATERFALL)
            float offset = 0.0;

            vec4 waterfallScrollCoord = vec4(v_texCoord.x + Stream1UV.x * time, offset + v_texCoord.y + Stream1UV.y * time, v_texCoord.x + Stream2UV.x * time, offset + v_texCoord.y + Stream2UV.y * time);

        #endif

        #if !defined(WORLD_WATERSHORE)
            float fresnel_u = 1.0 - min(max(0.0, dot(V, normalize(v_N))), 0.99);
        #endif

    // Texture Samplers //

        float noise_map = texture2D(s_cc3Texture2Ds[utility0_map_index], tiled_uv * WaterUVScale).r;

        vec4 water_normal0 = texture2D(s_cc3Texture2Ds[normal_map_index], water_uvs);
        vec4 water_normal1 = texture2D(s_cc3Texture2Ds[normal_map_index], water_uvs2);


        float foam_tex0 = texture2D(s_cc3Texture2Ds[utility0_map_index], (lightmap_uvs * foam_scale + vec2(-foam_speed, foam_speed))).g;
        float foam_tex1 = texture2D(s_cc3Texture2Ds[utility0_map_index], (lightmap_uvs * foam_scale * 0.35 + vec2(foam_speed, foam_speed))).g;

        vec4 aoMap = texture2D(s_cc3Texture2Ds[ao_map_index], lightmap_uvs);

        #if defined(WORLD_WATER_LIGHTMAP)
            vec4 lightMap = texture2D(s_cc3Texture2Ds[light_map_index], lightmap_uvs);
        #endif

        #if !defined(WORLD_WATERSHORE)
            #if !defined(WORLD_WATERFALL)
                float reflect_fresnel = texture2D(s_cc3Texture2Ds[utility0_map_index], vec2(fresnel_u, ReflectFresnelIndex)).b;
            #endif
            float water_fresnel = texture2D(s_cc3Texture2Ds[utility0_map_index], vec2(fresnel_u, FresnelIndex)).b;
        #endif

        #if defined(WORLD_WATERSHORE)
            vec4 tex_move1 = texture2D (s_cc3Texture2Ds[utility1_map_index], watershoreUV0);   
            vec4 tex_move2 = texture2D (s_cc3Texture2Ds[utility1_map_index], watershoreUV1);   
        #endif

        #if defined(WORLD_WATERFALL)

            vec4 waveTex1 = texture2D (s_cc3Texture2Ds[utility1_map_index], vec2(5.0, 3.0) * waterfallScrollCoord.xy); 
            vec4 waveTex2 = texture2D (s_cc3Texture2Ds[utility1_map_index], vec2(1.0, 1.0) * waterfallScrollCoord.zw);

            vec4 waveTex3 = texture2D (s_cc3Texture2Ds[utility1_map_index], vec2(0.25, 0.5) * waterfallScrollCoord.xy); 
        #endif

        vec4 region_tex = texture2D(s_cc3Texture2Ds[regionmask_map_index], world_uvs);   

        vec4 region_reveal_tex = texture2D(s_cc3Texture2Ds[regionreveal_map_index], world_uvs);     
        vec3 region_reveal_masks = GlobalUniform_RegionMaskChannel * region_reveal_tex.rgb;
        float region_reveal_mask = region_reveal_masks.r + region_reveal_masks.g + region_reveal_masks.b;    

    // Normals //

        vec3 water_normal = mix(vec3(0.5, 0.5, 1.0), water_normal0.rgb + water_normal1.rgb, noise_map);

        #if defined(WORLD_WATERFALL)
            water_normal = mix(water_normal, vec3(waveTex3.b, waveTex2.g, 1.0), clamp(waterSlopeColorVertBlend - 0.5, 0.0, 1.0));
        #endif

        vec3 normal_t = water_normal * 2.0 - 1.0;

        water_normal = normalize(mat3(v_T, v_B, v_N) * normal_t);
      


    // Diffuse //

        //base diffuse = base colors + foam * bump(ocean only)

        gl_FragColor.rgb = getMaterialColor().rgb;
        gl_FragColor.rgb *= mix(WaterDeepColor.rgb, WaterShallowColor.rgb, shallowColorVertBlend);

        //foam base color
        float foamBlend = 0.0;
        #if defined(WORLD_WATERFALL)
            foamBlend = waveTex3.b * waterSlopeColorVertBlend;
        #elif defined(WORLD_WATERSHORE)
            foamBlend = clamp(v_texCoord.y * 4.0, 0.0, 1.0);
        #endif
        
        gl_FragColor.rgb = mix (gl_FragColor.rgb, WaterFoamColor.rgb, foamBlend);
        
        float water_bump = clamp( dot( water_normal, v_N), 0.0, 1.0 );

        #if !defined(WORLD_WATERFALL) && !defined(WORLD_WATERSHORE)
            float water_bump_clamp = clamp(bumpValueVertBlend * 4.0, 0.0, 1.0);

            vec3 base = vec3(1.0, 1.0, 1.0);
            gl_FragColor.rgb *= mix(1.0, water_bump, water_bump_clamp);

        #endif

        //foam

        float foamTexBlend = clamp(max(foam_tex0, foam_tex1), 0.0, 1.0);
        float foamVertBlend = foamValueVertBlend * alphaColorVertBlend;

        foamTexBlend = mix(foamTexBlend, 2.0, clamp(foamVertBlend, 0.0, 1.0)) * noise_map * water_bump;

        float foamVal = clamp(foamVertBlend * foamTexBlend, 0.0, 1.0);

        gl_FragColor.rgb = mix(gl_FragColor.rgb, WaterFoamColor.rgb * WaterFoamColor.a * 4.0, foamVal);

       
#if defined(OCEAN_BACKGROUND)
    vec3 world_light = FixedLightDirection * mat3(u_cc3MatrixView);
#else
    vec3 world_light = v_world_light_dir;
#endif

    // Lighting //


        vec3 aoMapBias = mix(vec3(1.0), aoMap.rgb, 0.75);
        float light_value = 1.0;

        light_value = clamp(dot(v_N, normalize(world_light)) * 0.65 + 0.15, 0.0, 1.0);

        vec3 lighting = mix(EnvironmentConfig.WorldAmbientColor, EnvironmentConfig.WorldDiffuseColor, light_value);

        lighting *= aoMap.rgb;


        //spec

        vec3 water_r = reflect(-normalize(world_light), water_normal);

        float spec_dot_value = dot(V, water_r);
        float spec_value = pow(max(0.0, spec_dot_value), SpecularPower);

        vec3 water_spec = vec3(spec_value).rrr;
        water_spec *= alphaColorVertBlend;


        #if defined(WORLD_WATERFALL)
            lighting.rgb *= vec3(mix(0.8, 1.0, water_spec.r));
            lighting.rgb += water_spec * alphaColorVertBlend * aoMapBias;
        #else
            gl_FragColor.rgb += water_spec * (1.0 - max(shallowColorVertBlend, foamValueVertBlend)) * aoMapBias;
        #endif

        gl_FragColor.rgb *= lighting;

        #if defined(WORLD_WATER_LIGHTMAP)
            gl_FragColor.rgb += lightMap.rgb; //make it a lightmask? share with ao map?
        #endif


        // Atmospheric fresnel + reflection //
        #if !defined(WORLD_WATERSHORE)
        
            vec3 R = reflect(-V, normalize(water_normal));
            vec3 environment = textureCube(nebulaMap, R).rgb;
            #if !defined(WORLD_WATERFALL)
                gl_FragColor.rgb += environment.rgb * reflect_fresnel * aoMapBias;
                gl_FragColor.rgb += water_fresnel * aoMapBias; 
            #else
                gl_FragColor.rgb += water_fresnel * aoMapBias * vec3(1.0 - waterSlopeColorVertBlend); 
            #endif

        #endif    

            


    // Opacity //

        #if defined(WORLD_WATERSHORE)

            float shore_mask1 = tex_move1.g * (1.0 - v_texCoord.y); 
            float shore_mask2 = tex_move2.b * (1.0 - v_texCoord.y); 

        #endif
            

        gl_FragColor.a = getMaterialColor().a;



        #if defined(WORLD_WATERSHORE)
            gl_FragColor.a *= max(shore_mask1, shore_mask2) * alphaColorVertBlend * alphaColorVertBlend;
        #endif

        #if defined(WORLD_WATERFALL)
            gl_FragColor.a *= alphaColorVertBlend;   
            float texAlpha = mix(1.0, waveTex2.g, waterSlopeColorVertBlend);         
            gl_FragColor.a *= mix(waveTex3.b * waveTex2.g * texAlpha, 1.0, alphaColorVertBlend * alphaColorVertBlend);
            gl_FragColor.a *= mix(1.0, waveTex3.b, waterSlopeColorVertBlend * 0.25);
        #endif



    // Region Darkening //

 

        float region_mask = min(1.0, region_tex.r);

        float region_reveal = pow(((u_cc3ObjectTime[0] - RegionStartTime) / RegionDuration), 1.0) * RegionDuration + region_reveal_mask * 2.0 - RegionDuration;
        region_reveal = region_reveal * RegionReveal;
        region_mask = clamp(region_mask + clamp(region_reveal, 0.0, region_reveal_mask), 0.0, 1.0);
        region_mask = (1.0 - region_mask) * RegionMaskMultiplier;

        gl_FragColor.rgb = mix(gl_FragColor.rgb, EnvironmentConfig.RegionDarkColor.rgb, region_mask * EnvironmentConfig.RegionDarkColor.a);



    // Depth Fog //
        #if !defined(WORLD_WATER_NOFOG)
            float fog_depth = length(v_position_world - u_cc3CameraPositionGlobal);

            float fog_blend = clamp((fog_depth - EnvironmentConfig.FogNearFarMax.x) / EnvironmentConfig.FogNearFarMax.y, 0.0, 1.0);
            gl_FragColor.rgb = mix(gl_FragColor.rgb, EnvironmentConfig.FogColor.rgb, fog_blend * EnvironmentConfig.FogNearFarMax.z);
        #endif

    // Shadows //

        world_shadow_receive();


    #ifdef SHADER_DEBUG
        // Diffuse //
        if(DebugIndex == 1)
            gl_FragColor = vec4(textureColor.rgb, 1.0);
 
        // UV1 //
        else if(DebugIndex == 2)
            gl_FragColor = vec4(v_texCoord, 0.0, 0.0);

        // Vertex Color //
        else if(DebugIndex == 3)
            gl_FragColor = v_vertexColor;

 
        // Vertex Normals //
        else if(DebugIndex == 4)
            gl_FragColor = vec4(1.0);

        // Alpha //
        else if(DebugIndex == 5)
            gl_FragColor = vec4(gl_FragColor.aaa, 1.0);

        // Lighting //
        else if(DebugIndex == 6)
        {
            gl_FragColor = vec4(1.0);
        }

    #endif
    
#endif
}
