#define SUPPORT_GLSL300
#include "prefix.sh"

#include "framework/framework.sh"
#include "framework/transforms.sh"
#include "framework/skinning.sh"
#include "framework/system_uniforms.sh"
#include "repeat.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

void vsMain(out VSOUT vsout, VSIN vsin)
{
    #if defined(PROJ3D)
        vec3 modelSpaceP;
        RawTangentFrame modelSpaceTF;
        getModelSpace(modelSpaceP, modelSpaceTF, vsin);
        vsout._position = transformPoint(getModelToProj(), modelSpaceP);
    #else
        vec3 p = getPosition(vsin);
        // DavidJ -- now using real pixel coordinates for input to this vertex shader,
        //      so we need to use the "true" viewport (ie, unlike u_cc3CameraViewport, this uniform
        //      isn't modified for the retina scale factor)
        p.x = p.x / getViewport().z *  2.0 - 1.0;
        p.y = p.y / getViewport().w * -2.0 + 1.0;
        vsout._position = vec4(p.xyz, 1.0);
    #endif
    
    int c;
    #if VS_OUT_S_TEXCOORD < VS_IN_S_TEXCOORD
        FOR(c, 0, VS_OUT_S_TEXCOORD, setTexCoord(vsout, c, getTexCoord(vsin, c)))
    #else
        FOR(c, 0, VS_IN_S_TEXCOORD, setTexCoord(vsout, c, getTexCoord(vsin, c)))
    #endif

    #if VS_OUT_S_COLOR < VS_IN_S_COLOR
        FOR(c, 0, VS_OUT_S_COLOR, setColor(vsout, c, getColor(vsin, c)))
    #else
        FOR(c, 0, VS_IN_S_COLOR, setColor(vsout, c, getColor(vsin, c)))
    #endif
}

#include "framework/glsles/framework_glsles.vert"
