//#define DRAW_IN_GRAYSCALE
//#define OVERDRAW
//#define ALPHA_TEST
//#define CUSTOM_SHADER_*
//#define DEBUG

#define SUPPORT_GLSL300

#if !CC3_PLATFORM_ANDROID       // Instancing disabled on Android, because some low end GLES3.0 devices don't support using gl_InstanceID as an array indexor
    #define INSTANCING_ENABLED
#endif

#include "prefix.sh"

////////////////////////////////////////////////////////////////////////////////////////////////////
#define float2 vec2
#define float3 vec3
#define float4 vec4

#include "wm_dependencies.sh"
////////////////////////////////////////////////////////////////////////////////////////////////////

#include "programmable_blending.sh"

VARYING_IN vec2 v_texCoord;
VARYING_IN vec4 v_color;
VARYING_IN vec2 v_time;

uniform sampler2D un_diffuseMap;

vec4  getDiffuseColor()         { return ParticleFrameMaterial_packed[3]; }
vec4  getClampValues()          { return ParticleFixedMaterial_clampValues; }
bool  getUseColor()             { return bool(ParticleFixedMaterial_useColor > 0.5); }
bool  getMultiplyColorWithRGB() { return bool(ParticleFixedMaterial_multiplyColorWithRGB > 0.5); }
float getAlphaTestValue()       { return ParticleFrameMaterial_packed[1].z; }
#if !defined(GL_ES)
float getHideShadedWireframe()  { return 1.0; }// { return ParticleFrameMaterial_packed[1].w; }
#endif

#ifdef CUSTOM_SHADER_gray_distortion
#include "particle_gray_distortion.sh"
#else
// default pixel color is simply the texture
vec4 ComputePixelColor() {
    return texture2D(un_diffuseMap, v_texCoord);
}
#endif

void main() {
    /* Do not write to gl_FragColor before applying programmable blending.
     * gl_FragColor is defined as destColor, which, in the case of programmable blending
     * with OpenGL ES version 3.0+, stores the previous fragment color.
     * We need that color for programmable blending to be effective.
     * In the case of lower OpenGL ES versions, programmable blending obtains the previous
     * fragment color from gl_LastFragData.
     */

    vec4 texel = ComputePixelColor();
    vec3 colorMultiplier = mix(vec3(1.0), texel.rgb, float(getMultiplyColorWithRGB()));
    vec4 sourceColor = vec4(mix(texel.rgb, v_color.rgb * colorMultiplier, float(getUseColor())),
                        texel.a * v_color.a) * getDiffuseColor();

    // Apply programmable blending
    apply_programmable_blending(sourceColor);

#if defined(DEPTH_WRITE_ONLY)
    // In depth-only pass, color mask is turned off, we just perform an alpha-test here
    if (gl_FragColor.a < 0.001) {
        discard;
    }
    return;
#endif
    
#if 0 // visualize back faces
    if (!gl_FrontFacing) {
        gl_FragColor.rgb = vec3(1) - gl_FragColor.rgb;
    }
#endif

#if defined(DRAW_IN_GRAYSCALE)
    float color = dot(gl_FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
    gl_FragColor = vec4(color, color, color, gl_FragColor.a);
#endif

#if defined(OVERDRAW)
    gl_FragColor = vec4(0.25, 0.05, 0.025, 0.08);
#endif

#if !defined(GL_ES)
    // wireframe is enabled with a getHideShadedWireframe()-value of 0, resulting in black lines with full alpha
    gl_FragColor = vec4(gl_FragColor.rgb * getHideShadedWireframe(), gl_FragColor.a + 1.0-getHideShadedWireframe());
#endif

#if defined(DEBUG) || defined(OVERDRAW)
    // If the vertex shader suggested to discard the particle, we discard the fragment
    if (v_color.a == 0.0) {
//        gl_FragColor = vec4(0.0, 1.0, 0.0, 1.0); /* uncomment and do not discard for debug visualization */
        discard;
    }
#endif
#if defined(OVERDRAW)
    return;
#endif

#if defined(ALPHA_TEST)
    if(gl_FragColor.a < getAlphaTestValue()) {
        discard;
    }
#endif
    float clampValue = step(0.5, v_time.y) * mix(getClampValues().y, getClampValues().z, 2.0 * (v_time.y - 0.5)) +
                       (1.0 - step(0.5, v_time.y)) * mix(getClampValues().x, getClampValues().y, 2.0 * v_time.y);
    clampValue *= getClampValues().w;
    if (gl_FragColor.a < clampValue) {
        discard;
    }
}
