#ifdef USE_PROPERTY_MAP    
    void ApplyLighting(vec4 properties, inout vec3 result, vec3 position, vec3 N, vec3 V, vec3 R, float NdotV, vec3 rim)
    {
        // Properties //
        float SpecularStrength = 1.0;
        float GlossStrength = 1.0;
        float MetallicStrength = 1.0;
        vec3 FresnelColor = vec3(0.8, 1.05, 1.2);
        vec3 RimColor = vec3(0.75, 0.6, 0.45);
   
        float Spotlight_Height = 1000.0;
        float Spotlight_Radius = 5000.0;
        
        float Ramp_Start = 1200.0;
        float Ramp_End = -300.0;
        vec3 RampTop = vec3(0.15, 0.1, 0.05);
        vec3 RampBottom = vec3(0.8, 0.92, 1.0);


        properties.g = clamp(properties.g * GlossStrength, 0.0, 1.0);
        float spec_power = exp2(8.0 * properties.g + 1.0);
        float spec_normalize = 0.2 * (spec_power + 8.0) / (8.0 * 3.1415926);// * pow(NdotV, spec_power); 
        properties.r = clamp(spec_normalize, 0.0, 1.0);
        properties.b = clamp(properties.b * MetallicStrength, 0.0, 1.0);

        float mat_mask = properties.r;
        float mat_gloss = properties.g;
        float mat_metallic = properties.b;
        float mat_cloth = properties.a;

        // Reflection //
        float refl = R.y;
        vec3 refl_weights = vec3(max(0.0, refl), 1.0 - abs(refl), max(0.0, -refl));
        vec3 reflection = EnvironmentConfig.LightTop * refl_weights.x + EnvironmentConfig.LightSide * refl_weights.y + EnvironmentConfig.LightBottom * refl_weights.z;

        // Diffuse // 
        result.rgb *= mix(FresnelColor, vec3(1.0), NdotV);

        // Lighting //
        float up = N.y;
        vec3 light_weights = vec3(max(0.0, up), 1.0 - abs(up), max(0.0, -up));
        vec3 light = EnvironmentConfig.LightTop * light_weights.x + EnvironmentConfig.LightSide * light_weights.y + EnvironmentConfig.LightBottom * light_weights.z;
        result.rgb *= light * LightStrength;

        // Metallic //
        result.rgb *= mix(vec3(1.0), reflection * LightStrength, mat_metallic);

        float metallic_shift = 0.7;
        float metallic_size = 2.0 + 6.0 * mat_gloss;
        float metallic_aniso = 1.0 - clamp(abs((dot(N, V) - metallic_shift) * metallic_size), 0.0, 1.0);
        float metallic_result = mix(1.0, 0.7 + metallic_aniso * SpecularStrength, mat_metallic);
        result.rgb *= metallic_result;

        // Specular //
        float spec = pow(NdotV, spec_power) * mat_mask * (1.0 - mat_metallic);   
        result.rgb += spec * reflection * LightStrength * SpecularStrength;


        #ifdef COMMANDER
            // Sphere/Rim //
            vec3 rim_n = normalize((getWorldToCamera() * vec4(N, 0.0)).xyz);
            vec3 r = reflect(rim, rim_n);
            float m = 2.0 * sqrt(pow(r.x, 2.0) + pow(r.y, 2.0) + pow(r.z + 1.0, 2.0));
            float rim_x = max(0.0, dot(r, normalize(vec3(-1.0, 0.5, -0.8))));
            result.rgb += pow(rim_x, 10.0) * RimColor;

            // Environment Cube //
            vec3 environment = textureCube(u_reflectionMap, R).rgb;
            result.rgb += mat_cloth * environment.r * 0.3 * reflection * LightStrength * SpecularStrength;// * mat_mask;

            // Environment Sphere //     
            // float sphere_mip = properties.r * 8.0;
            vec2 sphere_uv = r.xy / m + 0.5;
            vec3 sphere_tex = texture2D(sphereMap, sphere_uv).rgb;
            // result.rgb += mat_cloth * sphere_tex.r * 0.3;

            // Spotlight //
            float spotlight_distance = distance(position, vec3(0.0, Spotlight_Height, 0.0));
            float spotlight = 1.0 - min(1.0, spotlight_distance / Spotlight_Radius);
            result.rgb *= spotlight * spotlight;

            // Ramp //
            float ramp_pos = min(1.0, (position.y - Ramp_End) / (Ramp_Start - Ramp_End));
            vec3 ramp = mix((1.0 - RampBottom) * -1.0, RampTop, ramp_pos);
            //result.rgb += ramp;
        #endif
    }
#endif
