local EXPLOSION_TIMER = 70
local EXPLOSION_ENTITY = 'psychoSamBossFlameEntity'
local BARRICADE_INIT = 'barricadeInit'

local RETREAT_VELOCITY = 100

local BOSS_ACTIVATE_KEY = "bossActivateKey"
local SAM_INTRO_KEY = 'samIntroKey'
local FIRE_COUNT_KEY = "fireCountKey"
local FIRES_BEFORE_ROTATING = 4
local BOSS_LANE_ROTATION_KEY = "bossLaneRotationKey"
local MOVE_TO_SHIP = "moveToShip"
local BOSS_FIRST_ROTATION_KEY = "boss1stRotation"

psychoSamBoss_tickTrigger = function(entity)
    local barricadeInit = objc.get_local_value(entity, BARRICADE_INIT)
    if barricadeInit == 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'R_Barricade' then
                objc.set_local_value(entity, BARRICADE_INIT, 1)
            end
        end
    elseif barricadeInit == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER) == 'R_Barricade' then
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_HEALTH, 4000)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_CURRENT_HEALTH, 4000)
            end
        end
        objc.set_local_value(entity, BARRICADE_INIT, 2)
    end

    local psychoSamIntro = objc.get_global_value(SAM_INTRO_KEY)
    if psychoSamIntro == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                objc.set_entity_scale_percentage(unit, 150)
                objc.set_entity_facing_direction(unit, -1, 0)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_ATTACK_RANGE, 3000)
                local newMaxHP = 3000
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_HEALTH, newMaxHP)
                objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_CURRENT_HEALTH, newMaxHP)
                objc.idle_entity_indefinitely(unit)
                objc.play_override_animation_key(unit, "spawnAnimationName", objc.convert_frames_to_subrounds(180))
                objc.add_trigger(unit, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'psychoSamBoss.lua', 'psychoSamBoss_spellResistant')
                objc.set_global_value(SAM_INTRO_KEY, 0)
                break
            end
        end
    end

    -- Initialize Psycho Sam's explosion
    local bossActivate = objc.get_global_value(BOSS_ACTIVATE_KEY)
    if bossActivate == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                objc.play_override_animation_key(unit, "abilityAnim", objc.convert_frames_to_subrounds(60))
                objc.set_global_value(BOSS_ACTIVATE_KEY, 0)
                break
            end
        end
    end

    local boss1stRotation = objc.get_global_value(BOSS_FIRST_ROTATION_KEY)
    if boss1stRotation == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                objc.unidle_entity(unit)
                objc.move_entity_to_location_with_velocity(unit, 4000, 2500, RETREAT_VELOCITY, 100)

                if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_POSITION_Y) > 2400 then
                    objc.spawn_at_point(EXPLOSION_ENTITY, 1, 2, 4000, 2500, -1, 0)
                    objc.idle_entity_indefinitely(unit)
                    objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss.lua', 'psychoSamBoss_LongFlameTrigger')
                    objc.play_override_animation_key(unit, "basicAttackAnim", objc.convert_frames_to_subrounds(120))
                    objc.set_entity_facing_direction(unit, -1, 0)
                    objc.set_global_value(BOSS_FIRST_ROTATION_KEY, 0)
                    objc.set_global_value(BOSS_LANE_ROTATION_KEY, 1)
                end
                break
            end
        end
    end

    -- Psycho Sam part 2 explosions on each lane
    local bossLaneRotation = objc.get_global_value(BOSS_LANE_ROTATION_KEY)
    if bossLaneRotation ~= 0 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                -- If it's time, start moving to a new lane
                local laneRotationTimer = objc.tick_timer(entity, "laneRotationTimerKey", 240)
                if laneRotationTimer == 0 then
                    if bossLaneRotation == 1 then
                        objc.unidle_entity(unit)
                        objc.remove_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss_LongFlameTrigger')
                        objc.move_entity_to_location_with_velocity(unit, 4000, -2400, RETREAT_VELOCITY, 100)
                    elseif bossLaneRotation == 2 then
                        objc.unidle_entity(unit)
                        objc.remove_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss_LongFlameTrigger')
                        objc.move_entity_to_location_with_velocity(unit, 4000, 2500, RETREAT_VELOCITY, 100)
                    end
                    break
                end

                -- If we're moving to a new lane, check when we've reached there
                if bossLaneRotation == 1 then
                    if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_POSITION_Y) < -2000 then
                        objc.spawn_at_point(EXPLOSION_ENTITY, 1, 2, 4000, -2500, -1, 0)
                        objc.set_local_value(unit, FIRE_COUNT_KEY, 0)
                        objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss.lua', 'psychoSamBoss_LongFlameTrigger')
                        objc.play_override_animation_key(unit, "basicAttackAnim", objc.convert_frames_to_subrounds(120))
                        objc.idle_entity_indefinitely(unit)
                        objc.set_entity_facing_direction(unit, -1, 0)
                        objc.set_global_value(BOSS_LANE_ROTATION_KEY, 2)
                    end
                elseif bossLaneRotation == 2 then
                    if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_POSITION_Y) > 2400 then
                        objc.spawn_at_point(EXPLOSION_ENTITY, 1, 2, 4000, 2500, -1, 0)
                        objc.set_local_value(unit, FIRE_COUNT_KEY, 0)
                        objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss.lua', 'psychoSamBoss_LongFlameTrigger')
                        objc.play_override_animation_key(unit, "basicAttackAnim", objc.convert_frames_to_subrounds(120))
                        objc.idle_entity_indefinitely(unit)
                        objc.set_entity_facing_direction(unit, -1, 0)
                        objc.set_global_value(BOSS_LANE_ROTATION_KEY, 1)
                    end
                end
                break
            end
        end
    end

    local moveToShip = objc.get_global_value(MOVE_TO_SHIP)
    if moveToShip == 1 then
        local base = objc.get_base_entity(2)
        objc.set_targetable(base, false)
        objc.add_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'psychoSamBoss.lua', 'base_invulnerable')

        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == EXPLOSION_ENTITY then
                objc.kill_entity(unit)
            elseif objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                objc.unidle_entity(unit)
                psychoSamBoss_killFlame()
                objc.remove_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'psychoSamBoss_LongFlameTrigger')
                objc.stop_override_animation(unit)
                objc.move_entity_to_location_with_velocity(unit, 6000, 0, RETREAT_VELOCITY, 100)
                objc.set_global_value(MOVE_TO_SHIP, 2)
            end
        end
    elseif moveToShip == 2 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                local bossPositionX = objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_POSITION_X)
                if bossPositionX > 5900 then
                    objc.set_entity_facing_direction(unit, -1, 0)
                    objc.play_override_animation_key(unit, "abilityAnim", objc.convert_frames_to_subrounds(60))
                    local wipe_radius = 7000
                    local playerunitsInRadius = objc.list_entities_in_radius(unit, wipe_radius, 1)
                    for _, playerunit in pairs(playerunitsInRadius) do
                        if (objc.is_building_unit(playerunit) or objc.is_normal_unit(playerunit) or objc.is_commander(playerunit) or objc.is_base(playerunit) or objc.is_barricade(playerunit) or (objc.is_boss_unit(playerunit) == 1)) and (objc.get_entity_value_for_key(playerunit, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) then
                            local attackDamage = objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_ATTACK_DAMAGE)
                            objc.inflict_normal_damage(unit, playerunit, 400)
                            objc.flash_entity_for_damage(playerunit, 250)
                        end
                    end
                    objc.set_flow_field_enabled_for_entity(unit, false)
                    objc.set_entity_value_for_key(unit, ENTITY_VALUE_KEY_MAX_VELOCITY, 0)
                    objc.set_global_value(MOVE_TO_SHIP, 3)
                end
                break
            end
        end
    elseif moveToShip == 3 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'R_PyroTutorial' then
                objc.set_entity_facing_direction(unit, -1, 0)
                objc.set_global_value(MOVE_TO_SHIP, 0)
                break
            end
        end
    end
end

psychoSamBoss_LongFlameTrigger = function(entity)
    local fireCount = objc.get_local_value(entity, FIRE_COUNT_KEY)
    local flameAnimationTimer = objc.tick_timer(entity, "laneRotationTimerKey", 40)
    if flameAnimationTimer == 0 then
        if fireCount < FIRES_BEFORE_ROTATING then
            objc.play_override_animation_key(entity, "basicAttackAnim", objc.convert_frames_to_subrounds(120))
            objc.set_local_value(entity, FIRE_COUNT_KEY, fireCount + 1)
        else
            psychoSamBoss_killFlame()
        end
    end
end

psychoSamBoss_killFlame = function()
    local team2Entities = objc.list_entities(2)
    for _,unit in pairs(team2Entities) do
        if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'psychoSamBossFlameEntity' then
            objc.sacrifice_entity(unit)
        end
    end
end


base_invulnerable = function(entity)
    return 0
end

-- OnDamageTrigger
psychoSamBoss_spellResistant = function(damage, targetEntity, sourceEntity)
    local source = objc.get_entity_value_for_key(sourceEntity, ENTITY_VALUE_KEY_SOURCE_ENTITY)
    local sourceIdentifier = objc.get_entity_value_for_key(source, ENTITY_VALUE_KEY_BASE_TIER_IDENTIFIER)
    if sourceIdentifier == 'W_Blast' or sourceIdentifier == 'W_Fog' then
        return objc.divide_with_ceiling(damage, 3)
    end
    return damage
end



