import os
import xml.etree.ElementTree as ET

ASSETS_FOLDER = os.path.dirname(os.path.abspath(__file__))
CCB_FOLDER = os.path.abspath(os.path.join(ASSETS_FOLDER, '..'))

ASSETS_FILE_TYPES = ['.png', '.fnt', '.plist', '.ttf']
CCB_FILE_TYPE = '.ccb'

def generateAssetsFilepathMap():
    assets_filepath_map = {}
    for root, dirs, files in os.walk(ASSETS_FOLDER):
        for f in files:
            ext = os.path.splitext(f)[-1]
            if ext in ASSETS_FILE_TYPES:
                if f in assets_filepath_map:
                    raise Exception('Duplicate filename %s', f)
                assets_filepath_map[f] = os.path.join(root, f)
    return assets_filepath_map

def relinkCCBs(assets_filepath_map):
    for root, dirs, files in os.walk(CCB_FOLDER):
        for f in files:
            if os.path.splitext(f)[-1] == CCB_FILE_TYPE:
                filepath = os.path.join(root, f)
                print "Relinking file %s" % f
                tree = ET.parse(filepath)
                tree_root = tree.getroot()
                made_changes = False
                for string in tree_root.iter('string'):
                    if not string.text or os.path.splitext(string.text)[-1] not in ASSETS_FILE_TYPES:
                        continue
                    basename = os.path.basename(string.text)
                    try:
                        new_absolute_path = assets_filepath_map[basename]
                        new_rel_path = os.path.normpath(os.path.relpath(new_absolute_path, os.path.dirname(filepath)))
                        if string.text == new_rel_path:
                            continue
                        print "Re-referencing %s as %s" % (string.text, new_rel_path)
                        string.text = new_rel_path
                        made_changes = True
                    except KeyError:
                        print "Cannot find %s for CCB %s" % (basename, f)
                if made_changes:
                    tree.write(filepath, method='html')

if __name__ == '__main__':
    assets_filepath_map = generateAssetsFilepathMap()
    relinkCCBs(assets_filepath_map)

