local HAS_ABILITY_STARTED_KEY = 'hasAbilityStarted'
local TARGET_QUEUED_FOR_DAMAGE = 'targetQueuedForDamage'
local KILL_COUNTER = 'killCounter'
local FOG_DISTANCE = 22000

--[[
This TickTrigger finds random units in battle and deals damage to them.
--]]
roxiAbility1ScriptEntity_tickTrigger = function(entity)
   local maxKills = 4
   local damagePerShot = 15
   local delayAdjustment = 3 -- THIS HAS TO BE LOWER THAN THE TICK TIMER
   local tickTimerInitialValue = 4 -- THIS TIMER HAS TO BE LONGER THAN THE DELAY TIMER

   local tickTimer = objc.tick_timer(entity, 'tickTime', tickTimerInitialValue) 
   local teamNumber = objc.get_team_number(entity)
   local enemyPlayer = objc.get_enemy_team_number(entity)
   local killCounter = objc.get_local_value(entity, KILL_COUNTER)
   local hasAbilityStarted = objc.get_local_value(entity, HAS_ABILITY_STARTED_KEY)
   local targetQueuedForDamage = objc.get_local_value(entity, TARGET_QUEUED_FOR_DAMAGE)
   local baseEntity = objc.get_base_entity(teamNumber)
   local playerNumber = objc.get_player_number(entity)

   if hasAbilityStarted == 0 then
      objc.reveal_fog(baseEntity, FOG_DISTANCE)
      objc.set_local_value(entity, HAS_ABILITY_STARTED_KEY, 1)
   end

   if tickTimer <= 0 then
      local validTargets = {}
      
      local enemyList = objc.list_entities(enemyPlayer)
      RoxiAbility1ScriptEntity_buildValidTargetList(enemyList, teamNumber, validTargets)

      local targetCount = listCount(validTargets)

      if targetCount > 0 then
         
         local enemyNumberToShoot = objc.roll_number(1, targetCount)
         local target = validTargets[enemyNumberToShoot]
         objc.set_local_value(entity, TARGET_QUEUED_FOR_DAMAGE, target)
         local targetLocation = objc.get_entity_position(target)
         local targetVelocity = objc.get_entity_value(target, 'velocityComponent.primaryMaxVelocity')

         local teamNumberAdjustment = 1
         local directionY = 0

         if teamNumber == 1 then
            teamNumberAdjustment = -1
            directionX = 1
         elseif teamNumber == 2 then
            teamNumberAdjustment = 1
            directionX = -1
         else
            objc.scripting_error('Unknown player number:' .. teamNumber)
         end
         
         local spawnIdentifier
         local killEffectIdentifier
         local spawnQuantity = 1
         local targetPositionX = targetLocation[1] + (delayAdjustment * targetVelocity * (teamNumberAdjustment * 2))
         local targetPositionY = targetLocation[2]
         local directionY = 0
         local targetRemainingHealth = objc.get_entity_value(target, 'healthDefenseComponent.health')

         if objc.is_ground_unit(target) then
            spawnIdentifier = 'roxiAbility1EffectEntityGround'
            killEffectIdentifier = 'roxiAbility1KillEffectEntityGround'
         elseif objc.is_air_ground_unit(target) then
            spawnIdentifier = 'roxiAbility1EffectEntityAir'
            killEffectIdentifier = 'roxiAbility1KillEffectEntityAir'
         elseif objc.is_air_unit(target) then
            spawnIdentifier = 'roxiAbility1EffectEntityHighAir'
            killEffectIdentifier = 'roxiAbility1KillEffectEntityHighAir'
         else
            objc.scripting_error('Unit ' .. objc.get_entity_value(target, 'description') .. ' has unknown plane of existence')
         end

         objc.spawn_at_point(spawnIdentifier, spawnQuantity, playerNumber, targetPositionX, targetPositionY, directionX, directionY)

         if targetRemainingHealth <= damagePerShot then
            objc.set_local_value(entity, KILL_COUNTER, killCounter + 1)
            objc.spawn_at_point(killEffectIdentifier, 1, playerNumber, targetPositionX, targetPositionY, 0, 0)
         end
      end
   end

   if targetQueuedForDamage ~= 0 then
      local delayTimer = objc.tick_timer(entity, 'delayTimer', delayAdjustment)  -- THIS TIMER HAS TO BE SHORTER THAN THE TICKTIMER
      if delayTimer == 0 then
         objc.inflict_direct_damage(entity, targetQueuedForDamage, damagePerShot)
         objc.set_local_value(entity, TARGET_QUEUED_FOR_DAMAGE, 0)
      end

      if killCounter >= maxKills then
         objc.kill_entity(entity)
         objc.unreveal_fog(baseEntity, FOG_DISTANCE)
         --objc.scripting_error('Ending Ability')
         return 0
      end
   end

   return 0
end


RoxiAbility1ScriptEntity_buildValidTargetList = function(listOfEntities, teamNumber, validTargets)
   for _, entity in pairs(listOfEntities) do
      if objc.is_normal_unit(entity) then
         local entityPosition = objc.get_entity_position(entity)
         if teamNumber == 1 then
            if entityPosition[1] < 9500 then
               table.insert(validTargets, entity)
            end
         elseif teamNumber == 2 then
            if entityPosition[1] > -9500 then
               table.insert(validTargets, entity)
            end
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end
      end
   end
end
