local HAS_FIRED_KEY = 'hasFired'
local ABILITY_ID_KEY = 'abilityIDKey'
local DUMMY_ENTITY_ID = 'RoxieAbilityDummyEntity'
local DAMAGE_AMOUNT = {1480, 1661, 1865, 2093, 2349, 2637, 2960, 3322, 3729, 4186, 4699, 5274, 5920, 6645}
local DAMAGE_RADIUS = 590
local TIER_KEY = 'tierKey'
local ARROW_IMPACT_VFX = 'imp_Fire.lvl'

--[[
This TickTrigger waits a given amount of time, and then damages all ground
units within a given radius of it.
--]]
roxiAbility3EffectEntity_tickTrigger = function(entity)
   local playerNumber = objc.get_player_number(entity)
   local enemyNumber = objc.get_enemy_team_number(entity)
   local roxie = objc.get_commander_id(playerNumber)
   local roxieTier = objc.get_local_value(entity, TIER_KEY)
   local fireArrowDamage = DAMAGE_AMOUNT[roxieTier+1]

   local hasFired = objc.get_local_value(entity, HAS_FIRED_KEY)
   local damageDelay = objc.tick_timer(entity, 'damageDelay', 0)
   -- duration is long enough so that the effect can play out before being killed
   local duration = objc.tick_timer(entity, 'duration', 100)

   local abilityID = objc.get_local_value(entity, ABILITY_ID_KEY)

   if hasFired == 0 and damageDelay == 0 then
      local dummyEntity = objc.spawn(DUMMY_ENTITY_ID, 1, playerNumber)
      for _, enemy in pairs(objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyNumber)) do
         if (objc.is_building_unit(enemy) or objc.is_normal_unit(enemy) or objc.is_commander(enemy) or objc.is_base(enemy) or objc.is_barricade(enemy) or (objc.is_boss_unit(enemy) == 1)) and (objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IS_BEING_PROTECTED) == 0) and objc.get_local_value(enemy, ABILITY_ID_KEY) ~= abilityID then
            objc.inflict_normal_damage(dummyEntity[1], enemy, fireArrowDamage)
            objc.play_vfx(enemy, ARROW_IMPACT_VFX, NODE_IMPACT)
            objc.set_local_value(enemy, ABILITY_ID_KEY, abilityID)
         end
      end

      objc.sacrifice_entity(dummyEntity[1])
      objc.set_local_value(entity, HAS_FIRED_KEY, 1)
   end

   if duration == 0 then
      objc.sacrifice_entity(entity)
   end

   return 0
end
