local HAS_FIRED_KEY = 'hasFired'
local ABILITY_ID_KEY = 'abilityIDKey'
local THIS_ENTITY_ID = 'roxiAbility3EffectEntitySmall'
local DAMAGE_AMOUNT = 200
local DAMAGE_RADIUS = 350

--[[
This TickTrigger waits a given amount of time, and then damages all ground
units within a given radius of it.
--]]
roxiAbility3EffectEntitySmall_tickTrigger = function(entity)
   local teamNumber = objc.get_team_number(entity)
   local enemyNumber = objc.get_enemy_team_number(entity)

   local hasFired = objc.get_local_value(entity, HAS_FIRED_KEY)
   local damageDelay = objc.tick_timer(entity, 'damageDelay', 12)
   -- duration is long enough so that the effect can play out before being killed
   local duration = objc.tick_timer(entity, 'duration', 100)

   local abilityID = objc.get_local_value(entity, ABILITY_ID_KEY)

   if hasFired == 0 and damageDelay == 0 then
      for _, enemy in pairs(objc.list_entities_in_radius(entity, DAMAGE_RADIUS, enemyNumber)) do
         local identifier = objc.get_entity_value_for_key(enemy, ENTITY_VALUE_KEY_IDENTIFIER)
         if objc.is_normal_unit(enemy) and objc.get_local_value(enemy, ABILITY_ID_KEY) ~= abilityID and identifier ~= THIS_ENTITY_ID then
            objc.inflict_normal_damage(entity, enemy, DAMAGE_AMOUNT)
            objc.set_local_value(enemy, ABILITY_ID_KEY, abilityID)
         end
      end
      objc.set_local_value(entity, HAS_FIRED_KEY, 1)
   end

   if duration == 0 then
      objc.kill_entity(entity)
   end

   return 0
end
