local ARROW_COUNT_KEY = 'arrowCount'
local ABILITY_ID_KEY = 'abilityIDKey'
local ARROW_AOE_SIZE = 1200
local NUM_ARROW_ROWS = 6
local ARROWS_IN_ROW = 3
local RANDOM_RADIUS = 300
--[[
This TickTrigger is the manifestation of the BombingRun hero ability. It
spawns a set number of arrows at a given interval, and then kills itself.
--]]
roxiAbility3ScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)

   local intervalTimer = objc.tick_timer(entity, 'intervalTimer', 1)
   local teamNumber = objc.get_team_number(entity)
   local playerNumber = objc.get_player_number(entity)

   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)

   if arrowCount >= (NUM_ARROW_ROWS * ARROWS_IN_ROW) then
      objc.kill_entity(entity)
      return 0
   end

   if intervalTimer == 0 then
      roxiAbility3ScriptEntity_spawnArrow(playerNumber, teamNumber, entity)
      roxiAbility3ScriptEntity_spawnArrow(playerNumber, teamNumber, entity)
      roxiAbility3ScriptEntity_spawnArrow(playerNumber, teamNumber, entity)
   end
end

--[[
This function spawns a roxiAbility3EffectEntity at a random location on the map
--]]
roxiAbility3ScriptEntity_spawnArrow = function(playerNumber, teamNumber, entity)
   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)
   local xMod = 1
   local effectIdentifier = 'roxiAbility3EffectEntity'
   if teamNumber == 1 then
        -- Do nothing. We've already initialized this value.
   elseif teamNumber == 2 then 
      xMod = -1
      effectIdentifier = 'roxiAbility3EffectEntityPlayer2'
   else
      objc.scripting_error('Unknown player number encountered: ' .. teamNumber)
   end

   local xRand = objc.roll_number(-RANDOM_RADIUS, RANDOM_RADIUS)
   local yRand = objc.roll_number(-RANDOM_RADIUS, RANDOM_RADIUS)

   local startXPos = -6000
   local endXPos = 6500
   local xIndex = objc.divide_with_floor(arrowCount, ARROWS_IN_ROW)
   local xStep = objc.divide_with_floor((endXPos - startXPos), NUM_ARROW_ROWS)
   local thisX = (xStep * xIndex) + startXPos
   thisX = math.floor(thisX * xMod)

   local startYPos = -2000
   local endYPos = 2900
   local yIndex = arrowCount % ARROWS_IN_ROW
   local yStep = objc.divide_with_floor((endYPos - startYPos), ARROWS_IN_ROW)
   local thisY = (yStep * yIndex) + startYPos

   local spawnedArrowArray = objc.spawn_at_point(effectIdentifier, 1, playerNumber, thisX + xRand, thisY + yRand, xMod, 0)
   objc.set_local_value(spawnedArrowArray[1], ABILITY_ID_KEY, objc.get_entity_value(entity, 'identityComponent.description'))
   objc.set_local_value(entity, ARROW_COUNT_KEY, arrowCount + 1)

   --[[local xBorder = 9500
   local yBorder = 1200
   local effectLocationX = objc.roll_number(1, xBorder)
   local effectLocationY = objc.roll_number(1, yBorder)
   local sideModifierX = objc.roll_number(1,2000)
   local sideModifierY = objc.roll_number(1,2000)
   local effectIdentifier = 'roxiAbility3EffectEntity'

   if sideModifierX < 1000 then
      sideModifierX = -1
   elseif sideModifierX >= 1000 then
      sideModifierX = 1 
   end

   if sideModifierY < 1000 then
      sideModifierY = -1
   elseif sideModifierY >= 1000 then
      sideModifierY = 1 
   end

   local directionX = 1
   local directionY = 0

   if teamNumber == 1 then
        -- Do nothing. We've already initialized this value.
   elseif teamNumber == 2 then 
        directionX = -1
        directionY = 0
        effectIdentifier = 'roxiAbility3EffectEntityPlayer2'
   else
      objc.scripting_error('Unknown player number encountered: ' .. teamNumber)
   end

   objc.spawn_at_point(effectIdentifier, 1, playerNumber, effectLocationX * sideModifierX, effectLocationY * sideModifierY, directionX, directionY)

   local arrowCount = objc.get_local_value(entity, ARROW_COUNT_KEY)
   objc.set_local_value(entity, ARROW_COUNT_KEY, arrowCount + 1)]]
end
