local MAX_SPAWNS = 5
local WAIT_TIMER = 30
local SPAWN_COUNTER = 'upgradeCounter'
local BASE_EDGE = 9500
local SPAWN_FORWARD_EDGE = 8500
local HAS_INITIALIZED = 'hasInitialized'
local REFLECTION_TAG_KEY = 'reflectionTagKey'

--[[
This TickTrigger find a number of random units to duplicate and creates copies
of them.
--]]
scrapperAbility1EffectEntity_tickTrigger = function(entity)
   local hasInitialized = objc.get_local_value(entity, HAS_INITIALIZED)
  
   --If the ability hasn't started yet, duplicate some units
   if hasInitialized == 0 then
      scrapperAbility1EffectEntity_replaceUnits(entity)
      objc.set_local_value(entity, HAS_INITIALIZED, 1)
      return 0
   end

   --If the script wasn't able to duplicate the full amount of units, it waits and tries again later
   local tickTimer = objc.tick_timer(entity, 'tickTime', WAIT_TIMER)

   if tickTimer == 0 then
      --scrapperAbility1EffectEntity_replaceUnits(entity)
   end

   return 0
end


scrapperAbility1EffectEntity_replaceUnits = function(entity)
   local teamNumber = objc.get_team_number(entity)
   local baseEntity = objc.get_base_entity(teamNumber)
   local currentSpawnCounter = objc.get_local_value(entity, SPAWN_COUNTER)
   local playerNumber = objc.get_player_number(entity)

   local unitsToReplace = {}
   local allyList = objc.list_entities(teamNumber)
   local validTargets = scrapperAbility1EffectEntity_validTargetList(allyList, teamNumber)

   local targetCount = listCount(validTargets)


   for i = 1, MAX_SPAWNS do
      if targetCount > 0 then
         local allyNumberToDuplicate = objc.roll_number(1, targetCount)
         table.insert(unitsToReplace, validTargets[allyNumberToDuplicate])
         table.remove(validTargets, allyNumberToDuplicate)
         targetCount = listCount(validTargets)
      end
   end
 
   if listCount(unitsToReplace) > 0 then
      for _, ally in pairs(unitsToReplace) do
         local allyUnitType = objc.get_entity_value(ally, 'identityComponent.identifier')
         local allyUnitPosition = objc.get_entity_position(ally)
         local xOffset = 0
         local yOffset = 0
         local xOffsetModifier = -1
         local yOffsetModifier = 1
         local xFacing = 1
         local overrideMaxVelocity = 2000
         local targetRadius = 10

         if teamNumber == 2 then
            xOffsetModifier = 1
            xFacing = -1
         end
         
         --Play a VFX on the unit to be replaced
         objc.play_vfx(ally, 'Commander_ReflecStorm.lvl', 'node_origin')
         --Determine what unit should replace it
         local unitToReplace = scrapperAbility1EffectEntity_determineReplacement(ally)
         --Destroy the original unit
         objc.kill_entity(ally)
         --Spawn a replacement unit
         local duplicateUnitArray = objc.spawn_at_point(unitToReplace, 1, playerNumber, allyUnitPosition[1], allyUnitPosition[2], xFacing, 0)
         --Make sure it can move right away
         objc.set_entity_value(duplicateUnitArray[1], 'spawnComponent.holdPositionCountForSpawning', 0)
         --Play a VFX on the duplicate unit
         objc.play_vfx(duplicateUnitArray[1], 'Commander_ReflecClone.lvl', 'node_origin')
         
         --Tag both the original unit and its duplicate don't get duplicated again
         objc.set_local_value(ally, REFLECTION_TAG_KEY, 1)
         objc.set_local_value(duplicateUnitArray[1], REFLECTION_TAG_KEY, 1)

         --Increment the spawn counter
         currentSpawnCounter = currentSpawnCounter + 1
         objc.set_local_value(entity, SPAWN_COUNTER, currentSpawnCounter)
         
         --Stop the script if it has reached the spawn limit
         if currentSpawnCounter >= MAX_SPAWNS then
            objc.kill_entity(entity)
            break
         end
      end
   end
end


scrapperAbility1EffectEntity_validTargetList = function(listOfEntities, teamNumber)
   local validTargets = {}
   for _, entity in pairs(listOfEntities) do
      if objc.is_normal_unit(entity) and objc.get_local_value(entity, REFLECTION_TAG_KEY) == 0 then
         local entityPosition = objc.get_entity_position(entity)

         if teamNumber == 1 then
            if entityPosition[1] > BASE_EDGE * (-1) then
               table.insert(validTargets, entity)
               --objc.scripting_error('Adding for Player 1:  ' .. objc.get_entity_value(entity, 'identityComponent.identifier'))
            end
         elseif teamNumber == 2 then
            if entityPosition[1] < BASE_EDGE then
               table.insert(validTargets, entity)
               --objc.scripting_error('Adding for Player 2:  ' .. objc.get_entity_value(entity, 'identityComponent.identifier'))
            end
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end
      end
   end

   return validTargets
end

scrapperAbility1EffectEntity_determineReplacement = function(entity)
   --[[local targeting = scrapperAbility1EffectEntity_checkCanTarget(objc.get_entity_value(entity, 'velocityComponent.primaryTargetingComponent.requiredAttributes'))
   if objc.is_ground_unit(entity) then
      if targeting == 2 then
         return 'R_RustyTank'
      elseif targeting == 1 then
         return 'R_Jeep'
      elseif target == 0 then
         return 'R_Junker'
      else
         return 'nil'
      end
   elseif objc.is_air_unit(entity) then
      if targeting == 2 then
         return 'R_Dragonfly'
      elseif targeting == 1 then
         return 'R_SurgeSprite'
      elseif target == 0 then
         return 'R_Dragonfly'
      else
         return 'nil'
      end
   else
      objc.scripting_error('Encountered unknown unit type attempting to use scrapperAbility1')
   end]]--

   local teamNumber = objc.get_team_number(entity)
   local mostPowerfulFactoryUnitIdentifier = 'mostPowerfulFactoryUnitIdentifier' .. teamNumber

   local unitFactories = objc.get_unit_spawn_factories(teamNumber)
   local mostPowerfulFactory = 'None'

   for _, factory in pairs(unitFactories) do
      if scrapperAbility1EffectEntity_IsMorePowerfulT2Factory(factory, mostPowerfulFactory) == true then
         mostPowerfulFactory = factory
      end
   end

   local unitSpawnIdentifier = objc.get_spawn_identifier_for_factory(mostPowerfulFactory)
   return unitSpawnIdentifier
end

scrapperAbility1EffectEntity_IsMorePowerfulT2Factory = function(newFactory, mostPowerfulFactory)
   local newCost = objc.get_entity_value(newFactory, 'deathComponent.bounty')
   local mostPowerfulCost = objc.get_entity_value(newFactory, 'deathComponent.bounty')

   if mostPowerfulFactory == 'None' then
      return true
   elseif objc.get_entity_value(newFactory, 'aiComponent.supply') == 2 then--and tonumber(newCost) > tonumber(mostPowerfulCost) then
      return true
   end

   return false
end

--returns 0 if ground, 1 if air, 2 if both
scrapperAbility1EffectEntity_checkCanTarget = function(targets)
   local isGround = 0
   local isAir = 0
   for _, value in pairs(table) do
      if value == 'ground' then
         isGround = 1
      elseif value == 'air' then
         isAir = 1
      end
   end

   if isGround and isAir then
      return 2
   elseif isAir then
      return 1
   elseif isGround then
      return 2
   else
      return -1
   end
end
