//#define ADDITIVE
//#define USE_VERTEX_COLOR
//#define INSTANCING_HARD_DISABLE

//#define SHADER_DEBUG 1

#define SUPPORT_GLSL300
#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

#if __VERSION__ >= 300
    out vec4 primaryOutput;
    #define gl_FragColor primaryOutput
#endif

#if CC3_PLATFORM_WINDOWS // win32
    #undef INSTANCING_ENABLED
#else
    #if !defined(INSTANCING_HARD_DISABLE)
        #define INSTANCING_ENABLED
    #else
        #undef INSTANCING_ENABLED
    #endif
#endif

#if defined(INSTANCING_ENABLED)
    #define MAX_SKINNED_INSTANCES 10
#else
    #define MAX_SKINNED_INSTANCES 1
#endif

// uniforms

#ifdef FOGOFWAR
    uniform vec4 TintColor;

    uniform vec4 BattlePlacement_WarBattlePlacementProperties0;
    uniform vec4 BattlePlacement_WarBattlePlacementProperties1;
    uniform float BattlePlacement_BorderPulse0;
    uniform float BattlePlacement_BorderPulse1;
    uniform vec3 BattlePlacement_GridVisibility;
    uniform vec4 BattlePlacement_ZoneColor;
    uniform float BattlePlacement_GridSize;

    //new outside red grid
    uniform vec4 BattlePlacement_WarBattlePlacementProperties2;
    uniform vec4 BattlePlacement_WarBattlePlacementProperties3;
    uniform vec3 BattlePlacement_GridVisibilityOutside;
    uniform vec4 BattlePlacement_ZoneColorOutside;  
    uniform float BattlePlacement_GridSizeOutside;

#endif

#ifdef SHADER_DEBUG
    uniform int DebugIndex;
    uniform float DebugFloat;
    uniform vec3 DebugVec3;
    uniform vec4 DebugVec4;
#endif

uniform sampler2D s_cc3Texture2Ds[2];
uniform float u_cc3ObjectTime[MAX_SKINNED_INSTANCES];

#if defined(DISTORTED)
    uniform float distortionMultiplier;
    uniform sampler2D heatDistortion;
#endif

#if defined(DISTORT_ADJUST)
    uniform vec2 DistortUVScale;
#endif

#ifdef DISSOLVE
uniform sampler2D dissolveMask;

uniform float DissolveDelay; //how long before mesh starts dissolving
uniform float DissolveSpread; //how long it takes the dissolution effect to spread across the whole mesh
uniform float DissolveTimeTo1plasma; //how long until a pixel turns from white flare to plasma, from the time it starts dissolving
uniform float DissolveTimeTo2smoke; //how long until a pixel turns from plasma to smoke, from the time it starts dissolving
uniform float DissolveTimeTo3disappear; //how long until a pixel turns from smoke to full transparency, from the time it starts dissolving
#endif

#if !defined(SHADOW_GEN)

// varyings
#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    VARYING_IN vec4 v_material_diffuse_color;
#endif

#if defined(FOGOFWAR)
    VARYING_IN vec3 v_position_world;
    VARYING_IN vec3 v_N;
#endif

#ifdef INSTANCING_ENABLED
    #if defined(INTEGER_ATTRIBUTES)
        VARYING_IN int v_instanceID;
    #else
        VARYING_IN float v_instanceID; // "int" with small constant added to handle flooring
    #endif
    #define INSTANCE_ID (int(v_instanceID))
#else
    #define INSTANCE_ID 0
#endif

VARYING_IN vec2 v_texCoord;

#ifdef NUM_TEXTURES

#else
    #define NUM_TEXTURES 1
#endif
#if NUM_TEXTURES > 1 || defined(DISTORTED)
    VARYING_IN vec2 v_texCoord2;
#endif

#ifdef USE_VERTEX_COLOR
VARYING_IN vec4 v_vertexColor;
#endif

#endif

#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    vec4 getMaterialColor() { return v_material_diffuse_color; }
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 getMaterialColor() { return u_cc3MaterialDiffuseColor; }
#endif

#include "shadow.fsh"

vec4 combine(vec4 color1, vec4 color2) {
#ifdef ADDITIVE
    return color1 + color2;
#elif defined MULTIPLICATIVE
    return color1 * color2;
#else
    return mix(color1, color2, color2.a);
#endif
}

vec4 grid_render(vec4 _GridCoords, vec3 world_pos, vec4 gColor, float gSize)
{
#if defined(FOGOFWAR)

    //colors and settings
    vec4 _Color = vec4(gColor.rgb, 1.0) * vec4(TintColor.rgb, 1.0);
    vec4 _BorderColor = _Color * vec4(1.0);
    vec4 _GridColor = _Color * vec4(1.0); 
    vec4 _BkgdColor = _Color * vec4(1.0, 1.0, 1.0, 0.5); 
    vec2 _Offset = vec2(0.0, gSize/2.0); 
    vec4 _GridSettings = vec4(gSize, 8.0, 80.0, 0.0);  //(X: Grid Size Y: Grid Thickness Z: Border Size W: Grid Edge Clamp)

    vec2 uv = world_pos.xz;

    //background and apply center offset
    vec4 col = _BkgdColor;
    uv.xy += _Offset.xy;
    _GridCoords.xy += _Offset.xy;
    _GridCoords.zw += _Offset.xy;

    //grid lines
    vec2 zone_lines = abs(fract(uv / _GridSettings.x) - 0.5);
    float grid = clamp((max(zone_lines.x, zone_lines.y) - 0.465) * 20.0, 0.0, 1.0);
    if(gSize > 0.0)
    {
        col += mix(vec4(0.0), vec4(_GridColor.rgb * _GridColor.rgb, _GridColor.a), grid);
    }
    else if(gSize < 0.0)
    {
        vec4 gridTickTex = texture2D(s_cc3Texture2Ds[1], uv.xy / _GridSettings.x);//
        col += mix(vec4(0.0), vec4(_GridColor.rgb * _GridColor.rgb, _GridColor.a), gridTickTex.r);
    }
    //create center offset to hide center border
    float center_border_offset_a = 0.0;
    float center_border_offset_b = 0.0;
    if(_GridCoords.y > _GridCoords.w)
    {
        center_border_offset_a = _GridSettings.z;
    }
    else
    {
        center_border_offset_b = _GridSettings.z;
    }

    //apply border
    if(uv.x - _GridSettings.z < min(_GridCoords.x, _GridCoords.z) || uv.x + _GridSettings.z > max(_GridCoords.x, _GridCoords.z) ||
    uv.y - center_border_offset_a < min(_GridCoords.y, _GridCoords.w) || uv.y + center_border_offset_b > max(_GridCoords.y, _GridCoords.w)
    || v_vertexColor.r > 0.0)
    {
        col = _BorderColor;
    } 

    //clamp out the rest of the rect too allow for compositing multiple rects
    if(uv.x < min(_GridCoords.x, _GridCoords.z) || uv.x > max(_GridCoords.x, _GridCoords.z) ||
    uv.y < min(_GridCoords.y, _GridCoords.w) || uv.y > max(_GridCoords.y, _GridCoords.w))
    {
        col = _Color;
        col.a *= _GridSettings.w;
        col.rgb *= col.a;
    }
    col.a *= gColor.a;
    return col;
#else
    return vec4(0.5);
#endif
}

//can remove this swizzle if/when the code adjusts
vec4 grid_swizzle(vec4 _GridCoords)
{
#if defined(FOGOFWAR)    
    float newX = _GridCoords.x; //center X
    float newY = _GridCoords.w; //center Y
    float newZ = _GridCoords.y; //outer X
    float newW = _GridCoords.z; //outer Y
    return vec4(newX, newY, newZ, newW);
#else
    return vec4(0.5);
#endif
}

vec4 grid_render_top(vec4 _GridCoords, vec3 world_pos, vec4 gColor, float gSize)
{
#if defined(FOGOFWAR)

    _GridCoords = grid_swizzle(_GridCoords);

    float center_y = max(_GridCoords.y, _GridCoords.w);
    float outer_y = min(_GridCoords.y, _GridCoords.w);

    _GridCoords = vec4(_GridCoords.x, center_y, _GridCoords.z, outer_y);

    return grid_render(_GridCoords, world_pos, gColor, gSize);
#else
    return vec4(0.5);
#endif
}

vec4 grid_render_bottom(vec4 _GridCoords, vec3 world_pos, vec4 gColor, float gSize)
{
#if defined(FOGOFWAR)

    _GridCoords = grid_swizzle(_GridCoords);

    float center_y = min(_GridCoords.y, _GridCoords.w);
    float outer_y = max(_GridCoords.y, _GridCoords.w);

    _GridCoords = vec4(_GridCoords.x, center_y, _GridCoords.z, outer_y);

    return grid_render(_GridCoords, world_pos, gColor, gSize);
#else
    return vec4(0.5);
#endif
}

void main() {
    
#if !defined(SHADOW_GEN)
    
#ifdef DISTORTED

    
    #if defined(DISTORT_ADJUST)
        vec4 noiseTexel = texture2D(s_cc3Texture2Ds[1], v_texCoord2 * DistortUVScale);
    #else
        vec4 noiseTexel = texture2D(heatDistortion, v_texCoord2);
    #endif
    vec2 uv_offset = distortionMultiplier * (noiseTexel.xy * 2.0 - 1.0);
    vec2 finalTexCoord = v_texCoord + uv_offset;
    vec4 textureColor = texture2D(s_cc3Texture2Ds[0], finalTexCoord);

#else // DISTORTED

    vec4 textureColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);

#if NUM_TEXTURES > 1
    vec4 texture2 = texture2D(s_cc3Texture2Ds[1], v_texCoord2);
    textureColor = combine(textureColor, texture2);
#endif

#endif // DISTORTED

    gl_FragColor = getMaterialColor() * textureColor;

#ifdef USE_VERTEX_COLOR
    gl_FragColor *= v_vertexColor;
#endif

#ifdef DISSOLVE
    vec4 flareColor = vec4(1, 1, 1, 1);
    vec4 plasmaColor = vec4(1, 0, 0, 0.8);
    vec4 smokeColor = vec4(0.1, 0.1, 0.1, 0.5);

    float time = u_cc3ObjectTime[int(INSTANCE_ID)] - DissolveDelay - texture2D(dissolveMask, v_texCoord).r * DissolveSpread;
    gl_FragColor = float(time < 0.0) * gl_FragColor +
    float(time >= 0.0 && time < DissolveTimeTo1plasma) * flareColor +
    float(time >= DissolveTimeTo1plasma && time < DissolveTimeTo2smoke) * plasmaColor +
    float(time >= DissolveTimeTo2smoke) * vec4(1, 1, 1, 1.0 - min((time - DissolveTimeTo2smoke) / DissolveTimeTo3disappear, 1.0)) * smokeColor;
#endif

#if defined(FOGOFWAR)
    gl_FragColor = vec4(0.0); 



#ifdef SHADER_DEBUG
            //to display a very large grid exceeding placement mesh
            vec4 u_war_battle_placement_properties_XX0 = vec4( 
                -10000.0, //start
                20000.0, //length (X)
                -10000.0, //outer edge (Z)
                10000.0 //center (Z)
                );

            // to display a single lane map (default merc positions) - two players
            vec4 u_war_battle_placement_properties_XX1a = vec4( 
                -15000, //start
                -3000, //length (X)
                -2700, //outer edge (Z)
                -300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX1b = vec4(
                -15000, //start
                -3000, //length (X)
                2700, //outer edge (Z)
                -300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX1c = vec4(
                3000, //start
                15000, //length (X)
                -2700, //outer edge (Z)
                -300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX1d = vec4(
                3000, //start
                15000, //length (X)
                2700, //outer edge (Z)
                -300 //center (Z)
                );

            //to display a dual lane map (default merc positions) - two players
            vec4 u_war_battle_placement_properties_XX2a = vec4( 
                -15000, //start
                -3000, //length (X)
                -3900, //outer edge (Z)
                300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX2b = vec4( 
                -15000, //start
                -3000, //length (X)
                3900, //outer edge (Z)
                300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX2c = vec4(
                3000, //start
                15000, //length (X)
                -3900, //outer edge (Z)
                -300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX2d = vec4(
                3000, //start
                15000, //length (X)
                3900, //outer edge (Z)
                -300 //center (Z)
                );

            // starting player 1 dual lane
            vec4 u_war_battle_placement_properties_XX3a = vec4( 
                -10000.0, //start
                -3000.0, //length (X)
                -3900, //outer edge (Z)
                0.0 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX3b = vec4( 
                -10000.0, //start
                -3000.0, //length (X)
                3900, //outer edge (Z)
                0.0 //center (Z)
                );

            //extremes test
            vec4 u_war_battle_placement_properties_XX4a = vec4( 
                -10000.0, //start
                -6000.0, //length (X)
                -3900, //outer edge (Z)
                -300.0 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX4b = vec4( 
                -10000.0, //start
                6000.0, //length (X)
                3900, //outer edge (Z)
                -300.0 //center (Z)
                );

            //new test for refactor
            vec4 u_war_battle_placement_properties_XX5a = vec4( 
                -10000.0, //start
                -3000.0, //length (X)
                -3900, //outer edge (Z)
                300.0 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX5b = vec4( 
                -10000.0, //start
                -600.0, //length (X)
                3900, //outer edge (Z)
                300.0 //center (Z)
                );

            //to display a dual lane map red negative space
            vec4 u_war_battle_placement_properties_XX6a = vec4( 
                -15000, //start
                -3000, //length (X)
                -3900, //outer edge (Z)
                300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX6b = vec4( 
                -15000, //start
                -3000, //length (X)
                3900, //outer edge (Z)
                300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX6c = vec4(
                -3000, //start
                15000, //length (X)
                -3900, //outer edge (Z)
                -300 //center (Z)
                );
            vec4 u_war_battle_placement_properties_XX6d = vec4(
                -3000, //start
                15000, //length (X)
                3900, //outer edge (Z)
                -300 //center (Z)
                );


        vec4 gColor = vec4(0.5, 1.0, 0.5, 0.8);
        float gSize = 600.0;
        vec4 gColorOutside = vec4(1.0, 0.5, 0.5, 0.8);
        float gSizeOutside = 0.0;

        int test_index = DebugIndex;

        if(test_index == 0)//0
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX0, v_position_world, gColor, gSize);//top
        }        

        if(test_index == 1)//1
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX1a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX1b, v_position_world, gColor, gSize);//bottom            
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX1c, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX1d, v_position_world, gColor, gSize);//bottom
        }

        if(test_index == 2)//2
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX2a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX2b, v_position_world, gColor, gSize);//bottom            
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX2c, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX2d, v_position_world, gColor, gSize);//bottom
        }

        if(test_index == 3)//3
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX3a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX3b, v_position_world, gColor, gSize);//bottom
        }

        if(test_index == 4)//4
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX4a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX4b, v_position_world, gColor, gSize);//bottom
        }

        if(test_index == 5)//5
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX5a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX5b, v_position_world, gColor, gSize);//bottom
        }


        if(test_index == 6)//6
        {
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX6a, v_position_world, gColor, gSize);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX6b, v_position_world, gColor, gSize);//bottom            
            gl_FragColor += grid_render_top(u_war_battle_placement_properties_XX6c, v_position_world, gColorOutside, gSizeOutside);//top
            gl_FragColor += grid_render_bottom(u_war_battle_placement_properties_XX6d, v_position_world, gColorOutside, gSizeOutside);//bottom
        }

#endif

            if(BattlePlacement_GridVisibility.x > 0.1)
            {
                gl_FragColor += grid_render_top(BattlePlacement_WarBattlePlacementProperties0, v_position_world, BattlePlacement_ZoneColor, BattlePlacement_GridSize);//top
            }

            if(BattlePlacement_GridVisibility.y > 0.1)
            {
                gl_FragColor += grid_render_bottom(BattlePlacement_WarBattlePlacementProperties1, v_position_world, BattlePlacement_ZoneColor, BattlePlacement_GridSize);//bottom
            }

            if(BattlePlacement_GridVisibilityOutside.x > 0.1)
            {
                gl_FragColor += grid_render_top(BattlePlacement_WarBattlePlacementProperties2, v_position_world, BattlePlacement_ZoneColorOutside, BattlePlacement_GridSizeOutside);//top - outside
            }

            if(BattlePlacement_GridVisibilityOutside.y > 0.1)
            {
                gl_FragColor += grid_render_bottom(BattlePlacement_WarBattlePlacementProperties3, v_position_world, BattlePlacement_ZoneColorOutside, BattlePlacement_GridSizeOutside);//bottom - outside
            }

    #endif

    shadow_receive();
    
#endif
}
