
#define SUPPORT_GLSL300
#include "prefix_tow.sh"

//===================================================================================================================//
// INSTANCING //
//===================================================================================================================//

#if CC3_PLATFORM_WINDOWS // win32

    #if !defined(INSTANCING_HARD_DISABLE)
        #if !defined(UNIFORM_BASED_INSTANCING)
            #define INSTANCE_ID 0
        #else
            #define INSTANCE_ID (int(a_cc3InstanceId))
        #endif
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#elif defined(GL_ES) // ios

    #if !defined(INSTANCING_HARD_DISABLE)
        #extension GL_EXT_draw_instanced: enable
        #define INSTANCE_ID gl_InstanceIDEXT
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#else // osx

    #if !defined(INSTANCING_HARD_DISABLE)
        #extension GL_ARB_draw_instanced: enable
        #define INSTANCE_ID gl_InstanceIDARB
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#endif

//===================================================================================================================//
// INTERFACE //
//===================================================================================================================//

#if (defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(GLASS)) && defined(USE_NORMAL_MAP) && !defined(SHADOW_GEN)
    #define VSIN_TANGENT_FRAME
#endif

#if !defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)
    #define VSOUT_POSITION_WORLD 1
#endif

#if (!defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)) || defined(ALPHA_TEST)
    #if defined(LIGHTMAP_ENABLED) || defined(WORLD)
        #define VSOUT_TEX_COORD 2
    #else
        #define VSOUT_TEX_COORD 1
    #endif
#endif

#if (defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER)) && !defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)
    #define VSOUT_MATERIAL_DIFFUSE_COLOR 1
#endif

#if defined(INSTANCING_ENABLED)
    #define VSOUT_INSTANT_ID 1
#endif

#if (defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(GLASS) || defined(WORLD)) && !defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)
    #define VSOUT_NORMAL 1
#endif

#if defined(VSOUT_NORMAL) && defined(USE_NORMAL_MAP)
    #define VSOUT_TANGENT_FRAME 1
#endif

#if defined(COMMANDER) && !defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)
    #define VSOUT_RIM 1
#endif

#if (defined(DISSOLVE) || defined(GACHA) || defined(HOLOGRAM_BUILD) || defined(COMMANDER) || defined(WORLD)) && !defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)
    #define VSOUT_POSITION_LOCAL
#endif

//===================================================================================================================//
// ATTRIBUTES //
//===================================================================================================================//

#if defined(GL_ES)
    precision highp float;
#endif

ATTRIBUTE vec4 a_cc3Position;

#if (!defined(SHADOW_GEN) && !defined(SECOND_PASS_GEO_BORDERS)) || defined(ALPHA_TEST)
    ATTRIBUTE vec2 a_cc3TexCoord0;
    #if defined(LIGHTMAP_ENABLED) || defined(WORLD)
        ATTRIBUTE vec2 a_cc3TexCoord1;
    #endif
#endif

ATTRIBUTE vec4 a_cc3BoneWeights;
#if defined(INTEGER_ATTRIBUTES)
    ATTRIBUTE ivec4 a_cc3BoneIndices;
#else
    ATTRIBUTE vec4 a_cc3BoneIndices;
#endif

#if !defined(SHADOW_GEN)
    ATTRIBUTE vec3 a_cc3Normal;
    #if defined(VSIN_TANGENT_FRAME)
        ATTRIBUTE vec3 a_cc3Tangent;
        ATTRIBUTE vec3 a_cc3Bitangent;
    #endif

    #if defined(WAVE) || defined(WIND) || defined(VERTPUSH_BORDER) || defined(UV_POSITION_SCROLL) || defined(WORLD_LAVA_BLEND) || defined(HEIGHT_BLEND)
        ATTRIBUTE vec4 a_cc3Color;
    #endif
#else
    #if defined(WAVE)
        ATTRIBUTE vec4 a_cc3Color;
    #endif
#endif

#if defined(UNIFORM_BASED_INSTANCING)
    #if defined(INTEGER_ATTRIBUTES)
        ATTRIBUTE int a_cc3InstanceId;
    #else
        ATTRIBUTE float a_cc3InstanceId;
    #endif
#endif

#if defined(INSTANCING_ENABLED)
    #define MAX_BONE_MATRICES 56
    #define MAX_SKINNED_INSTANCES 10
#else
    #define MAX_BONE_MATRICES 56
    #define MAX_SKINNED_INSTANCES 1
#endif

//===================================================================================================================//
// UNIFORMS //
//===================================================================================================================//

#include "system_uniforms.sh"

#ifdef INSTANCING_ENABLED
    // unsupported
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 currentMaterialDiffuseColor() { return u_cc3MaterialDiffuseColor; }
    mat4 currentModelMatrix() { return getModelToWorld(); }
    int currentBoneMatrixOffset() { return 0; }
#endif

uniform int u_cc3VertexBoneCount;

#define SKIN_4x3
#if defined(SKIN_4x3)
    // DavidJ -- NOTE -- the elements of "u_cc3BoneMatricesModel4x3" are in transposed order
    //      relative to u_cc3BoneMatricesModel. This is required to ensure that the matrix
    //      can fit in 3 full vec4 rows.
    uniform vec4 u_cc3BoneMatricesModel4x3[3*MAX_BONE_MATRICES];
#else
    uniform mat4 u_cc3BoneMatricesModel[MAX_BONE_MATRICES];
#endif

uniform mat4 u_cc3MatrixModelView;

#ifdef SCROLLING_TEXTURE
    #ifdef NO_LOOP_TIME
        uniform float u_cc3ObjectTime[MAX_SKINNED_INSTANCES];
    #endif
#endif

#if defined(GACHA) && defined(SCROLLING_TEXTURE) && defined(NO_LOOP_TIME)
    // If you hit this error, please contact DavidJ.
    #error DavidJ -- NOTE -- this case will fail to link currently, because u_cc3ObjectTime[] is a different type in texture.frag
#endif

#ifdef SCROLLING_TEXTURE
    uniform vec2 scrollingSpeed;
    #if defined(NUM_TEXTURES)
        #if NUM_TEXTURES > 1
            uniform vec2 scrollingSpeed2;
        #endif
    #endif
    #ifdef NO_LOOP_TIME
        uniform float timeOffset;
    #endif
#endif

#ifdef WAVE
    uniform vec3 waveDisplacement;
    uniform vec3 waveOffset;
    uniform vec3 waveFrequency;
    uniform vec3 waveSpeed;
#endif

#ifdef WIND
    uniform float WindStrength;
    uniform float WindScale;
    uniform float WindSpeed;
    uniform float WindPhase;
    uniform sampler2D cloudTexture;
#endif

#ifdef UV_POSITION_SCROLL
    uniform vec3 scrollAxesMultipliers;
    uniform vec2 scrollUVDirection;
#endif

#if defined(VERTPUSH_BORDER)
#elif defined(SECOND_PASS_GEO_BORDERS)
    vec3 cameraPositionModel() { unsupported }
#endif

#ifdef DISSOLVE
    uniform vec4 DissolveStrengthDurationDirectionFade;
#endif

#if defined(WORLD_LAVA_BLEND)
    uniform vec2 UV3Speed;
    uniform vec2 UV3Tiling;
#endif

//===================================================================================================================//
// VARYINGS //
//===================================================================================================================//

#if defined(VSOUT_POSITION_WORLD)
    VARYING_OUT vec3 v_position_world;
#endif

#if defined(VSOUT_TEX_COORD)
    VARYING_OUT vec2 v_texCoord;
    #if defined(VSOUT_TEX_COORD) && VSOUT_TEX_COORD > 1
        VARYING_OUT vec2 v_lightMapCoord;
    #endif
#endif

#if defined(VSOUT_MATERIAL_DIFFUSE_COLOR)
    VARYING_OUT vec4 v_material_diffuse_color;
#endif

#if defined(VSOUT_INSTANT_ID)
    #if defined(INTEGER_ATTRIBUTES)
        VARYING_OUT int v_instanceID;
    #else
        VARYING_OUT float v_instanceID; // "int" with small constant added to handle flooring
    #endif
#endif

#if defined(VSOUT_NORMAL) || defined(FOGOFWAR) 
    VARYING_OUT vec3 v_N;
#endif

#if defined(VSOUT_TANGENT_FRAME)
    VARYING_OUT vec3 v_T;
    VARYING_OUT vec3 v_B;
#endif

#if defined(VSOUT_RIM)
    VARYING_OUT vec3 v_rim;
#endif

#ifdef DISSOLVE
    vec4 currentDissolveStrengthDurationDirectionFade() { return DissolveStrengthDurationDirectionFade; }
#endif

#if defined(VSOUT_POSITION_LOCAL)
    VARYING_OUT vec3 v_position_local;
#endif

#if defined(WORLD) || defined(WORLD_WATER) || defined(WORLD_CLOUDS) || defined(SPEC_LIGHT)
    VARYING_OUT vec3 v_world_light_dir;
#endif

#if defined(WORLD_LAVA_BLEND) || defined(HEIGHT_BLEND)
    VARYING_OUT vec4 v_vertexColor;
    #if defined(WORLD_LAVA_BLEND) 
        VARYING_OUT vec2 v_flow_uv1;
        VARYING_OUT vec2 v_flow_uv2;
        VARYING_OUT float v_flow_blend;

        VARYING_OUT vec4 v_flow_aux;
        VARYING_OUT vec2 v_flow_pre;
    #endif
#endif

//===================================================================================================================//
// SHADER //
//===================================================================================================================//

#include "shadow.vsh"
#include "environmentconfig.sh"

#if defined(VSIN_TANGENT_FRAME)
    vec3 GetAttributeTangent()      { return a_cc3Tangent; }
    vec3 GetAttributeBitangent()    { return a_cc3Bitangent; }
#else
    vec3 GetAttributeTangent()      { return vec3(1.0, 0.0, 0.0); }
    vec3 GetAttributeBitangent()    { return vec3(0.0, 0.0, 1.0); }
#endif

#if !defined(SHADOW_GEN)
    vec3 GetAttributeNormal()       { return a_cc3Normal; }
#else
    vec3 GetAttributeNormal()       { return vec3(0.0, 1.0, 0.0); }
#endif

// private
// DavidJ -- NOTE --
//      Enable OPT_SKINNING to enable optimised skinning code. This has 2 critical changes
//      1) we now combine the weighted post-transform positions and normals, instead of weighting
//          and combining the transform matrices. This reduces the number of temporary registers
//          used and the number of instructions in the loop
//      2) improved looping code (as per PVR examples) to encourage the compiler to produce the
//          best unrolled loop
#define OPT_SKINNING
#if defined(OPT_SKINNING)
	vec3 finalPosition_local;
    vec3 finalNormal_local;
    vec3 finalTangent_local;
    vec3 finalBitangent_local;
#else
	vec4 finalPosition_local;
	#if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(GLASS)
	    vec3 finalNormal_local;
	    #if defined(USE_NORMAL_MAP)
	        vec3 finalTangent_local;
	        vec3 finalBitangent_local;
	    #endif
	#endif
#endif

vec3 skinPositionSingleWeight(vec3 position, int boneIndex, float weight) {
    #if defined(SKIN_4x3)
        vec4 boneMatrixA = u_cc3BoneMatricesModel4x3[boneIndex*3+0];
        vec4 boneMatrixB = u_cc3BoneMatricesModel4x3[boneIndex*3+1];
        vec4 boneMatrixC = u_cc3BoneMatricesModel4x3[boneIndex*3+2];
        vec3 result;
        result.x = dot(boneMatrixA, vec4(position.xyz, 1.0));
        result.y = dot(boneMatrixB, vec4(position.xyz, 1.0));
        result.z = dot(boneMatrixC, vec4(position.xyz, 1.0));
        return result * weight;
    #else
        mat4 boneMatrix = u_cc3BoneMatricesModel[boneIndex];
        return (boneMatrix * vec4(position.xyz, 1.0) * weight).xyz;
    #endif
}

vec3 skinDirectionSingleWeight(vec3 direction, int boneIndex, float weight) {
    #if defined(SKIN_4x3)
        vec4 boneMatrixA = u_cc3BoneMatricesModel4x3[boneIndex*3+0];
        vec4 boneMatrixB = u_cc3BoneMatricesModel4x3[boneIndex*3+1];
        vec4 boneMatrixC = u_cc3BoneMatricesModel4x3[boneIndex*3+2];
        vec3 result;
        result.x = dot(boneMatrixA.xyz, direction.xyz);
        result.y = dot(boneMatrixB.xyz, direction.xyz);
        result.z = dot(boneMatrixC.xyz, direction.xyz);
        return result * weight;
    #else
        mat4 boneMatrix = u_cc3BoneMatricesModel[boneIndex];
        return mat3(boneMatrix) * direction.xyz * weight;
    #endif
}

void vertexToEyeSpace(vec4 position, vec3 normal, vec3 tangent, vec3 bitangent) {

	#if defined(OPT_SKINNING) //////////////////////////////////
	
		ivec4 boneIndices = ivec4(a_cc3BoneIndices);
	    vec4 boneWeights = a_cc3BoneWeights;
	    
        vec3 workingPosition = skinPositionSingleWeight(position.xyz, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);

        finalNormal_local = skinDirectionSingleWeight(normal, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
        finalTangent_local = skinDirectionSingleWeight(tangent, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
        finalBitangent_local = skinDirectionSingleWeight(bitangent, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
    
	    for (int i = 1; i < 4; ++i)
		{
			if (i < u_cc3VertexBoneCount)
			{
				// "rotate" the vector components
				boneIndices = boneIndices.yzwx;
				boneWeights = boneWeights.yzwx;
	
                workingPosition += skinPositionSingleWeight(position.xyz, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
            
                finalNormal_local += skinDirectionSingleWeight(normal, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
                finalTangent_local += skinDirectionSingleWeight(tangent, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
                finalBitangent_local += skinDirectionSingleWeight(bitangent, currentBoneMatrixOffset() + boneIndices.x, boneWeights.x);
			}
		}
		
		finalPosition_local = workingPosition.xyz;
	    
	#else //////////////////////////////////
	
	    ivec4 boneIndices = ivec4(a_cc3BoneIndices);
	    vec4 boneWeights = a_cc3BoneWeights;
	
	    mat4 transform = boneWeights[0] * u_cc3BoneMatricesModel[currentBoneMatrixOffset() + boneIndices[0]];
	    for (int i = 1; i < 4 && i < u_cc3VertexBoneCount; ++i) {
	        // Add position contribution from this bone
	        transform += boneWeights[i] * u_cc3BoneMatricesModel[currentBoneMatrixOffset() + boneIndices[i]];
	    }
	    finalPosition_local = transform * position;
	    #if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(GLASS)
	        finalNormal_local = (transform * vec4(normal, 0)).xyz;
	        #if defined(USE_NORMAL_MAP)
	            finalTangent_local = (transform * vec4(tangent, 0)).xyz;
	            finalBitangent_local = (transform * vec4(bitangent, 0)).xyz;
	        #endif
	    #endif
	    
	#endif //////////////////////////////////
	
}

#if defined(VERTPUSH_BORDER) || defined(SECOND_PASS_GEO_BORDERS)
    vec3 doVertPushBorders(vec3 position, vec3 normal, float object_scale, vec3 cameraPosition) {
        #ifdef USE_NORMAL_MAP
            float border_width = 0.002;
        #else
            float border_width = 0.001;
        #endif
        float dist = distance(cameraPosition, position.xyz);     // DavidJ -- precalculate this once per object!
        position.xyz -= normal * (border_width * dist * object_scale);
        return position;
    }
#endif

// DavidJ --    in cases where:
//                  1) we don't need to write out finalPosition_world
//                  2) finalPosition_world isn't modified post-transform,
//                  3) we don't use finalPosition_world for any calculations within the vertex shader
//              we can avoid using modelMatrix, and just go straight from local -> clip space
//              this reduces the number of input uniforms used by this shader 
#if !defined(VSOUT_POSITION_WORLD) && !(defined(VSOUT_NORMAL) && defined(VERTPUSH_BORDER)) && !defined(INSTANCING_ENABLED) && !defined(VSOUT_SHADOW_COORD)
    #define USE_MVP_PATH
#endif

#if defined(SECOND_PASS_GEO_BORDERS)
    #define USE_MVP_PATH
#endif

void main() {
    vec4 position = a_cc3Position;
    
    #if defined(WORLD_LAVA_BLEND) || defined(HEIGHT_BLEND)
        v_vertexColor = a_cc3Color;
    #endif

    #ifdef WAVE
        vec3 maxDisplacement = mix(vec3(0.0), waveDisplacement, vec3(a_cc3Color.g));
        position.xyz += maxDisplacement * cos(waveFrequency * position.xyz + waveSpeed * vec3(getSceneTime() + waveOffset));
    #endif

    vertexToEyeSpace(position, GetAttributeNormal(), GetAttributeTangent(), GetAttributeBitangent());
    
    #if defined(VSOUT_POSITION_LOCAL)
        v_position_local = finalPosition_local.xyz;
    #endif

    #if !defined(USE_MVP_PATH)
        mat4 modelMatrix = currentModelMatrix();
    
        vec4 finalPosition_world = modelMatrix * vec4(finalPosition_local.xyz, 1.0);
    #endif

    #ifdef WIND
        float wind_mask = a_cc3Color.g;
        float wind_phase = a_cc3Color.b;
  
        vec2 wind_uv = finalPosition_world.xz / WindScale - EnvironmentConfig.WindDirection * (getSceneTime() * WindSpeed + wind_phase * WindPhase);
        vec2 wind_tex = texture2D(cloudTexture, wind_uv).xy; 

        float wind_forward = dot(wind_tex, EnvironmentConfig.WindDirection);
        wind_forward = wind_forward * 0.5 + 0.5;
        float wind_side = dot(wind_tex, vec2(-EnvironmentConfig.WindDirection.y, EnvironmentConfig.WindDirection.x));
        vec3 wind = wind_mask * vec3((-EnvironmentConfig.WindDirection.y * wind_side * 1.0 + EnvironmentConfig.WindDirection.x * wind_forward) * WindStrength, -WindStrength * 1.0 * (wind_forward + abs(wind_side) - 1.0), (EnvironmentConfig.WindDirection.x * wind_side * 1.0 + +EnvironmentConfig.WindDirection.y * wind_forward) * WindStrength); 

        finalPosition_world.xyz += wind;
    #endif

    #if defined(WORLD) || defined(WORLD_WATER) || defined(WORLD_CLOUDS) || defined(SPEC_LIGHT)
        v_world_light_dir = EnvironmentConfig.WorldLightDirection * mat3(getWorldToCamera());
    #endif
    
    #if defined(VSOUT_TEX_COORD)
        v_texCoord = a_cc3TexCoord0;
        #if VSOUT_TEX_COORD > 1
            v_lightMapCoord = a_cc3TexCoord1;
        #endif
    #endif
    
    #if defined(WORLD_LAVA_BLEND)

//uv0 - tiling/lm set   uv1 - flow set

        v_flow_pre = vec2(0.0);
        v_flow_aux = vec4(0.0);

        v_flow_uv1 = (v_lightMapCoord * UV3Tiling) + (getSceneTime() * UV3Speed);
        v_flow_uv2 = (v_lightMapCoord * UV3Tiling) + (getSceneTime() * vec2(-UV3Speed.x, UV3Speed.y) + vec2(0.0, 0.25));
        v_flow_blend = 0.0;
    #endif

    #if defined(VSOUT_MATERIAL_DIFFUSE_COLOR)
        v_material_diffuse_color = currentMaterialDiffuseColor();

        #ifdef VERTPUSH_BORDER
            v_material_diffuse_color.rgb *= a_cc3Color.r;
        #endif
    #endif

    #if defined(VSOUT_NORMAL) || defined(FOGOFWAR) 
        v_N = normalize(mat3(modelMatrix) * finalNormal_local);
        #if defined(VSOUT_TANGENT_FRAME)
            v_T = normalize(mat3(modelMatrix) * finalTangent_local);
            v_B = normalize(mat3(modelMatrix) * finalBitangent_local);
        #endif

        #if defined(VERTPUSH_BORDER) || defined(SECOND_PASS_GEO_BORDERS)
            float object_scale = length(modelMatrix[0].xyz);
            finalPosition_world.xyz = doVertPushBorders(finalPosition_world.xyz, v_N, object_scale * (1.0 - a_cc3Color.r), getCameraPositionWorld());
            #ifdef COMMANDER
                finalPosition_world.y = mix(0.1, finalPosition_world.y, a_cc3Color.r);
            #endif

            #ifdef DISSOLVE
                finalPosition_world.y += float(currentDissolveStrengthDurationDirectionFade().w == 0.0) * 20000.0 * (1.0 - a_cc3Color.r);
            #endif
        #endif
    #elif defined(SECOND_PASS_GEO_BORDERS)
        finalPosition_local.xyz = doVertPushBorders(finalPosition_local.xyz, finalNormal_local.xyz, -1.0f, cameraPositionModel());
    #endif
    
    #if defined(VSOUT_RIM)
        v_rim = normalize(vec3(u_cc3MatrixModelView * position));
    #endif

    #if defined(SCROLLING_TEXTURE) && defined(VSOUT_TEX_COORD)
        #ifdef NO_LOOP_TIME
            float time = timeOffset + u_cc3ObjectTime[INSTANCE_ID];
            float time2 = time;
        #else
            float time = mod(getSceneTime(), 1.0 / max(abs(scrollingSpeed.x), abs(scrollingSpeed.y)));
            #if defined(NUM_TEXTURES)
                #if NUM_TEXTURES > 1
                    float time2 = mod(getSceneTime(), 1.0 / max(abs(scrollingSpeed2.x), abs(scrollingSpeed2.y)));
                #endif
            #endif
        #endif

        v_texCoord += time * scrollingSpeed;

        #if defined(NUM_TEXTURES)
            #if NUM_TEXTURES > 1
                v_texCoord2 += time2 * scrollingSpeed2;
            #endif
        #endif
    #endif
    
    #if defined(USE_MVP_PATH)
        gl_Position = getModelToProj() * vec4(finalPosition_local.xyz, 1.0);
    #else
        gl_Position = getWorldToProj() * vec4(finalPosition_world.xyz, 1.0);
    #endif

    #if defined(VSOUT_POSITION_WORLD)
        v_position_world = finalPosition_world.xyz;
    #endif

    #if defined(UV_POSITION_SCROLL) && defined(VSOUT_TEX_COORD)
        vec3 forwardDirection = (modelMatrix * vec4(scrollAxesMultipliers, 0)).xyz;
        vec3 staticOffset = (modelMatrix * vec4(finalPosition_local.xyz - a_cc3Position.xyz, 0.0)).xyz;
        vec3 offset = modelMatrix[3].xyz + staticOffset;
        v_texCoord += scrollUVDirection * dot(forwardDirection, offset) * a_cc3Color.b;
    #endif

    // A small constant must be added because interpolation could cause the value to fall slightly below the original
    // int, thus causing the typecast to access the lower index.
    
    #if defined(VSOUT_INSTANT_ID)
        #if defined(INTEGER_ATTRIBUTES)
            v_instanceID = INSTANCE_ID;
        #else
            v_instanceID = float(INSTANCE_ID) + 0.1;
        #endif
    #endif

    #if !defined(USE_MVP_PATH)
        shadow_receive(finalPosition_world);
    #endif
}
