local HAS_INIT = 'hasInitKey'
local BLUE_SPAWN_ANIMATION_NAME = 'Barricade_Ascension_Spawn'
local GREEN_SPAWN_ANIMATION_NAME = 'Barricade_Legion_Spawn'
local RED_SPAWN_ANIMATION_NAME = 'Barricade_Rogue_Spawn'
local TEAM_ONE_ANIMATION_NAME = '_BlueTeam'
local TEAM_TWO_ANIMATION_NAME = '_RedTeam'
local NUMBER_OF_FRAMES_IN_SPAWN_ANIMATION = 90

-- "W_Barricade"
-- "B_Barricade"
-- "R_Barricade"

-- Only spawn barricades for Team 1. This is used in Campaign.
spawnP1OneBarricade_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        objc.set_local_value(entity, HAS_INIT, 1)
    elseif hasInit == 1 then

        spawnBarricade(-3300, 0, 1, 1, "BarricadeTeamOne")

        objc.set_local_value(entity, HAS_INIT, 2)
    end
end



spawnBarricade = function(xPos, yPos, side, facing, tag)
    local faction = objc.get_commander_factions(side)
    local commanderID = objc.get_commander_id(side)
    local barrierID = objc.get_object_info("Commander",commanderID,"barricadeUnitIdentifier")
    local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_SPAWN_ANIMATION)
    local animation_name

    if faction[1] == FACTION_BLUE then
        animation_name = BLUE_SPAWN_ANIMATION_NAME
        barrierID = "W_OneLaneBarricade"
    elseif faction[1] == FACTION_GREEN then
        animation_name = GREEN_SPAWN_ANIMATION_NAME
        barrierID = "B_OneLaneBarricade"
    elseif faction[1] == FACTION_RED then
        animation_name = RED_SPAWN_ANIMATION_NAME
        barrierID = "R_OneLaneBarricade"
    else
        barrierID = "R_Barricade"
        animation_name = RED_SPAWN_ANIMATION_NAME
        scripting_error("No valid Barricade found for this Commander")
    end

    if side == 1 then
        animation_name = animation_name .. TEAM_ONE_ANIMATION_NAME
    elseif side == 2 then
        animation_name = animation_name .. TEAM_TWO_ANIMATION_NAME
    else
        animation_name = animation_name .. TEAM_ONE_ANIMATION_NAME
        scripting_error("No valid barrier animation found for this side")
    end

    local entities = objc.spawn_at_point(barrierID, 1, side, xPos, yPos, facing, 0)
    objc.set_local_value(entities[1], LUA_KEY_TAG, tag)
    objc.update_team_barricade_health(entities[1])
    objc.play_override_animation(entities[1], animation_name, objc.subrounds_to_milliseconds(numberOfSubrounds))
end
