local HAS_INIT = 'hasInitKey'
local BLUE_SPAWN_ANIMATION_NAME = 'Barricade_Ascension_Spawn'
local GREEN_SPAWN_ANIMATION_NAME = 'Barricade_Legion_Spawn'
local RED_SPAWN_ANIMATION_NAME = 'Barricade_Rogue_Spawn'
local TEAM_ONE_ANIMATION_NAME = '_BlueTeam'
local TEAM_TWO_ANIMATION_NAME = '_RedTeam'
local NUMBER_OF_FRAMES_IN_SPAWN_ANIMATION = 90

--THESE ARE LISTED SO THEY MADE IT INTO THE WHITELIST
local ascensionShipBarricade = 'Barricade_Ascension_Ship.lvl'
local rogueShipBarricade = 'Barricade_Rogue_Ship.lvl'
local legionShipBarricade = 'Barricade_Legion_Ship.lvl'

-- "W_Barricade"
-- "B_Barricade"
-- "R_Barricade"

spawnShipBarricade_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        objc.set_local_value(entity, HAS_INIT, 1)
    elseif hasInit == 1 then

        local team1Base = objc.get_base_entity(1)
        spawnShipBarricade(objc.get_entity_position(team1Base)[1] + 1350, 0, 1, 1, "ShipBarricadeOne")
        local team2Base = objc.get_base_entity(2)
        spawnShipBarricade(objc.get_entity_position(team2Base)[1] - 1350, 0, 2, -1, "ShipBarricadeTwo")

        objc.set_local_value(entity, HAS_INIT, 2)
    end
end



spawnShipBarricade = function(xPos, yPos, side, facing, tag)
    local faction = objc.get_commander_factions(side)
    local commanderID = objc.get_commander_id(side)
    local barrierID = objc.get_object_info("Commander",commanderID,"barricadeUnitIdentifier") .. "_Ship"
    local numberOfSubrounds = objc.convert_frames_to_subrounds(NUMBER_OF_FRAMES_IN_SPAWN_ANIMATION)
    local animation_name

    if faction[1] == FACTION_BLUE then
        animation_name = BLUE_SPAWN_ANIMATION_NAME
    elseif faction[1] == FACTION_GREEN then
        animation_name = GREEN_SPAWN_ANIMATION_NAME
    elseif faction[1] == FACTION_RED then
        animation_name = RED_SPAWN_ANIMATION_NAME
    else
        barrierID = "R_Barricade_Ship"
        animation_name = RED_SPAWN_ANIMATION_NAME
        scripting_error("No valid Barricade found for this Commander")
    end

    if side == 1 then
        animation_name = animation_name .. TEAM_ONE_ANIMATION_NAME
    elseif side == 2 then
        animation_name = animation_name .. TEAM_TWO_ANIMATION_NAME
    else
        animation_name = animation_name .. TEAM_ONE_ANIMATION_NAME
        scripting_error("No valid barrier animation found for this side")
    end

    local entities = objc.spawn_at_point(barrierID, 1, side, xPos, yPos, facing, 0)
    objc.set_local_value(entities[1], LUA_KEY_TAG, tag)
    objc.set_solid(entities[1], false)
    objc.play_override_animation(entities[1], animation_name, objc.subrounds_to_milliseconds(numberOfSubrounds))

    local base = objc.get_base_entity(side)
    objc.set_targetable(base, false)
    objc.add_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'spawnShipBarricade.lua', 'spawnShipBarricade_invulnerable')
    objc.add_trigger(entities[1], TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'spawnShipBarricade.lua', 'spawnShipBarricade_transferDamageToShip')
    objc.add_trigger(entities[1], TRIGGER_NAME_REACHED_OVERTIME_TRIGGER, 'spawnShipBarricade.lua', 'spawnShipBarricade_killDuringOvertime')
end

spawnShipBarricade_transferDamageToShip = function(damage, targetEntity, sourceEntity)
    local base = objc.get_base_entity(objc.get_team_number(targetEntity))
    objc.inflict_direct_damage(targetEntity, base, damage)
    return 0
end

spawnShipBarricade_invulnerable = function(damage, targetEntity, sourceEntity)
    return 0
end

spawnShipBarricade_killDuringOvertime = function(entity)
    local base = objc.get_base_entity(objc.get_team_number(entity))
    objc.set_targetable(base, true)
    objc.remove_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, 'spawnShipBarricade_invulnerable')
    objc.sacrifice_entity(entity)
end
