
//#define USE_LIGHT
//#define USE_NORMAL_MAP
//#define SPECULAR_ENABLED

#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

// uniforms
uniform vec4 u_cc3MaterialDiffuseColor;

uniform sampler2D s_cc3Texture2Ds[4];

uniform samplerCube u_irradianceMap;
uniform samplerCube u_reflectionMap;

uniform vec3 u_cc3CameraPositionGlobal;

// varyings
varying vec3 v_position_world;

#ifdef USE_LIGHT
varying vec3 v_N;
#ifdef USE_NORMAL_MAP
varying vec3 v_T;
varying vec3 v_B;
#endif
#endif

varying vec2 v_texCoord;

#include "shadow.fsh"

// code
vec3 thresholdedValue(float baseValue, float threshold, vec3 smaller, vec3 bigger) {
    return float(baseValue < threshold) * smaller + float(baseValue >= threshold) * bigger;
}

void main() {
    vec4 epidermis = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    vec4 dermis = texture2D(s_cc3Texture2Ds[1], v_texCoord);
    float translucency = texture2D(s_cc3Texture2Ds[2], v_texCoord).r;

//    gl_FragColor = vec4(vec3(translucency), 1); return;

    // normal
#ifdef USE_LIGHT
#ifdef USE_NORMAL_MAP
    vec3 normal = texture2D(s_cc3Texture2Ds[3], v_texCoord).rgb;
    vec3 normal_t = normal * 2.0 - 1.0;
    vec3 N = normalize(mat3(v_T, v_B, v_N) * normal_t);
    N = thresholdedValue(length(N), 1e-3, vec3(0, 0, 1), N);
#else
    vec3 N = v_N;
#endif
#endif

    vec3 backlight = vec3(0);

#ifdef USE_LIGHT
    // for dynamic objects

    epidermis.a = u_cc3MaterialDiffuseColor.a;

    epidermis = vec4(1);//epidermis * textureCube(u_irradianceMap, N);
//    gl_FragColor = textureCube(u_irradianceMap, N); return;

    vec3 V = normalize(u_cc3CameraPositionGlobal - v_position_world);

    backlight = textureCube(u_irradianceMap, -V).rgb;
//    gl_FragColor = vec4(vec3(vDotN), 1); return;

#ifdef SPECULAR_ENABLED
    vec3 R = reflect(-V, N);
    vec3 specularLevel = vec3(texture2D(s_cc3Texture2Ds[3], v_texCoord).a);
    vec4 specularColor = vec4(specularLevel, 1.0) * textureCube(u_reflectionMap, R);
    // add specular
    epidermis.rgb = epidermis.rgb + specularColor.rgb;
#endif
#endif

    vec4 diffuseColor = mix(epidermis, dermis * vec4(backlight, 0), translucency);

    gl_FragColor = u_cc3MaterialDiffuseColor * diffuseColor;

    shadow_receive();
}
