#define SUPPORT_GLSL300
#include "prefix_tow.sh"

//===================================================================================================================//
// INSTANCING //
//===================================================================================================================//

#if CC3_PLATFORM_WINDOWS // win32

    #if !defined(INSTANCING_HARD_DISABLE)
        #if !defined(UNIFORM_BASED_INSTANCING)
            #define INSTANCE_ID 0
        #else
            #define INSTANCE_ID (int(a_cc3InstanceId))
        #endif
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#elif defined(GL_ES) // ios

    #if !defined(INSTANCING_HARD_DISABLE)
        #extension GL_EXT_draw_instanced: enable
        #define INSTANCE_ID gl_InstanceIDEXT
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#else // osx

    #if !defined(INSTANCING_HARD_DISABLE)
        #extension GL_ARB_draw_instanced: enable
        #define INSTANCE_ID gl_InstanceIDARB
        #define INSTANCING_ENABLED
    #else
        #define INSTANCE_ID 0
        #undef INSTANCING_ENABLED
    #endif

#endif

//#define LIGHTMAP_ENABLED
//#define FAKE_LIGHTMAP_ENABLED
//#define SCROLLING_TEXTURE
//#define NO_LOOP_TIME
//#define WAVE

//===================================================================================================================//
// ATTRIBUTES //
//===================================================================================================================//

#if defined(GL_ES)
    precision highp float;
#endif

ATTRIBUTE vec4 a_cc3Position;
ATTRIBUTE vec3 a_cc3Normal;
ATTRIBUTE vec3 a_cc3Tangent;
ATTRIBUTE vec3 a_cc3Bitangent;
ATTRIBUTE vec2 a_cc3TexCoord0;
#if defined(WAVE) || defined(WIND) || defined(VERTPUSH_BORDER) || defined(USE_VERTEX_COLOR) || defined(HEIGHT_BLEND) || defined(WATER_BLEND) || defined(WORLD_REGION) || defined(WORLD_DETAIL) || defined(WORLD_WATER) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(OVERLAY) || defined(REFLECT)
    ATTRIBUTE vec4 a_cc3Color;
#endif

#if defined(LIGHTMAP_ENABLED) || defined(WATER_BLEND) || defined(WORLD) || defined(WORLD_WATER) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(ANIMATED_TEX)
    ATTRIBUTE vec2 a_cc3TexCoord1;
#endif

#if defined(UNIFORM_BASED_INSTANCING)
    #if defined(INTEGER_ATTRIBUTES)
        ATTRIBUTE int a_cc3InstanceId;
    #else
        ATTRIBUTE float a_cc3InstanceId;
    #endif
#endif

#if defined(INSTANCING_ENABLED)
    #define MAX_SKINNED_INSTANCES 10
    #define MAX_STATIC_INSTANCES 64
#else
    #define MAX_SKINNED_INSTANCES 1
    #define MAX_STATIC_INSTANCES 1
#endif

#if !defined(SHADOW_GEN)
    #define VSOUT_POSITION_WORLD 1
    #define VSOUT_POSITION_LOCAL 1
#endif

#if !defined(SHADOW_GEN) && !defined(NO_DIFFUSE_TEXTURE)
    #define VSOUT_TEX_COORD 1
#endif

#if (defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(PARALLAX) || defined(WORLD_REGION) || defined(WORLD_WATER) || defined(WORLDREGIONONLY)) && !defined(SHADOW_GEN)
    #define VSOUT_NORMAL 1
#endif

#if defined(VSOUT_NORMAL) && (defined(USE_NORMAL_MAP) || defined(PARALLAX) || defined(WORLD_WATER) || defined(WORLD_DETAIL))
    #define VSOUT_TANGENT_FRAME 1
#endif

//===================================================================================================================//
// UNIFORMS //
//===================================================================================================================//

#include "system_uniforms.sh"

#ifdef INSTANCING_ENABLED
    // unsupported
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 currentMaterialDiffuseColor() { return u_cc3MaterialDiffuseColor; }
    mat4 currentModelMatrix() { return getModelToWorld(); }
#endif

uniform mat4 u_cc3MatrixProj;

uniform float u_cc3ObjectTime[MAX_SKINNED_INSTANCES];

#ifdef SCROLLING_TEXTURE
    uniform vec2 scrollingSpeed;
    #if defined(NUM_TEXTURES)
        #if NUM_TEXTURES > 1
            uniform vec2 scrollingSpeed2;
        #endif
    #endif
    #ifdef NO_LOOP_TIME
        uniform float timeOffset;
    #endif
#endif

#ifdef RADIAL_TEXTURE
    uniform float radialSpeed;
#endif

#ifdef WAVE
    uniform vec3 waveDisplacement;
    uniform vec3 waveOffset;
    uniform vec3 waveFrequency;
    uniform vec3 waveSpeed;
#endif

#ifdef UV_POSITION_SCROLL
    uniform vec3 scrollAxesMultipliers;
    uniform vec2 scrollUVDirection;
#endif

#ifdef BILLBOARD
    uniform vec3 BillboardDirection;
    uniform vec2 BillboardFrontBack;
    uniform float BillboardDepthOffset;
#endif

#ifdef WIND
    uniform float WindStrength;
    uniform float WindScale;
    uniform float WindSpeed;
    uniform float WindPhase;
    uniform sampler2D cloudTexture;
#endif

#if defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
    uniform vec2 UV3Tiling;
    #if defined(NO_FLOW)
        uniform vec2 UV3Speed;
    #else
        uniform float NoiseStrength;
        uniform float WaveLength;
        uniform float FlowStrength;   
    #endif
#endif

//===================================================================================================================//
// VARYINGS //
//===================================================================================================================//

#if defined(VSOUT_POSITION_WORLD)
    VARYING_OUT vec3 v_position_world;
#endif

VARYING_OUT vec3 v_position_local;



#if defined(VSOUT_TEX_COORD) || defined(WORLD_WATER) || defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(ALPHA_TEST)
    VARYING_OUT vec2 v_texCoord;
#endif

#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    VARYING_OUT vec4 v_material_diffuse_color;
#endif
#ifdef INSTANCING_ENABLED
    #if defined(INTEGER_ATTRIBUTES)
        VARYING_OUT int v_instanceID
    #else
        VARYING_OUT float v_instanceID; // "int" with small constant added to handle flooring
    #endif
#endif

#if defined(ANIMATED_TEX)
    #define NUM_TEXTURES 2
#endif

#if defined(NUM_TEXTURES) && defined(VSOUT_TEX_COORD)
    #if NUM_TEXTURES > 1
        VARYING_OUT vec2 v_texCoord2;
    #endif
#endif

#if defined(VSOUT_NORMAL) || defined(FOGOFWAR) || defined(REFLECT) || defined(OVERLAY) || defined(REFLECT_CUBE)
    VARYING_OUT vec3 v_N;
    #if defined(VSOUT_TANGENT_FRAME) || defined(REFLECT_CUBE)
        VARYING_OUT vec3 v_T;
        VARYING_OUT vec3 v_B;
    #endif
#endif

#if defined(LIGHTMAP_ENABLED) || defined(FAKE_LIGHTMAP_ENABLED) || defined(COMPOSITE) || defined(WORLD) || defined(WORLD_WATER) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(NO_FLOW)
    VARYING_OUT vec2 v_lightMapCoord;
#endif

#if defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
    VARYING_OUT vec2 v_flow_uv1;
    VARYING_OUT vec2 v_flow_uv2;
    VARYING_OUT float v_flow_blend;
    #if defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || (defined(WATER_BLEND) && defined(NO_FLOW))
       VARYING_OUT vec4 v_flow_aux;
       VARYING_OUT vec2 v_flow_pre;
    #endif
#endif

#if defined(USE_VERTEX_COLOR) || defined(HEIGHT_BLEND) || defined(WORLD_WATER) || defined(WORLD_REGION) || defined(WORLD_DETAIL) || defined(OVERLAY) || defined(REFLECT)
    VARYING_OUT vec4 v_vertexColor;
#endif

#if defined(WORLD) || defined(WORLD_WATER) || defined(WORLD_CLOUDS) || defined(WORLD_DETAIL) || defined(REFLECT) || defined(SPEC_LIGHT) || defined(WORLDREGIONONLY)
    VARYING_OUT vec3 v_world_light_dir;
    #if defined(REFLECT) 
        VARYING_OUT vec3 v_world_normal_dir;
    #endif
#endif

#if defined(ANIMATED_TEX)
    VARYING_OUT vec2 v_animCoord;
#endif

//===================================================================================================================//
// SHADER //
//===================================================================================================================//

#include "shadow.vsh"
#include "environmentconfig.sh"
#include "system_uniforms.sh"

#define M_PI 3.1415926535897932384626433832795

#if defined(SHADOW_GEN) && (!defined(INSTANCING_ENABLED) && !defined(VSOUT_TEX_COORD) && !defined(WIND) && !defined(VSOUT_NORMAL) && !defined(VSOUT_POSITION_WORLD) && !defined(BILLBOARD))
    #define USE_MVP_PATH 1
#endif

void main() {
    vec4 position = a_cc3Position;

    #ifdef WAVE
        vec3 maxDisplacement = mix(vec3(0.0), waveDisplacement, vec3(a_cc3Color.g));
        position.xyz += maxDisplacement * cos(waveFrequency * position.xyz + waveSpeed * vec3(getSceneTime() + waveOffset));
    #endif

    //#if !defined(USE_MVP_PATH)
        mat4 modelMatrix = currentModelMatrix();
        vec4 position_world = modelMatrix * position;
    //#endif

    #ifdef WIND
        float wind_mask = a_cc3Color.g;
        float wind_phase = a_cc3Color.b;
  
        vec2 wind_uv = position_world.xz / WindScale - EnvironmentConfig.WindDirection * (getSceneTime() * WindSpeed + wind_phase * WindPhase);
        vec2 wind_tex = texture2D(cloudTexture, wind_uv).xy; 

        float wind_forward = dot(wind_tex, EnvironmentConfig.WindDirection);
        wind_forward = wind_forward * 0.5 + 0.5;
        float wind_side = dot(wind_tex, vec2(-EnvironmentConfig.WindDirection.y, EnvironmentConfig.WindDirection.x));
        vec3 wind = wind_mask * vec3((-EnvironmentConfig.WindDirection.y * wind_side * 1.0 + EnvironmentConfig.WindDirection.x * wind_forward) * WindStrength, -WindStrength * 1.0 * (wind_forward + abs(wind_side) - 1.0), (EnvironmentConfig.WindDirection.x * wind_side * 1.0 + +EnvironmentConfig.WindDirection.y * wind_forward) * WindStrength); 

        position_world.xyz += wind;
    #endif

    #if defined(WORLD) || defined(WORLD_WATER) || defined(WORLD_CLOUDS) || defined(SPEC_LIGHT) || defined(WORLDREGIONONLY)
        v_world_light_dir = EnvironmentConfig.WorldLightDirection * mat3(getWorldToCamera());
    #endif
    
    #if defined(VSOUT_POSITION_LOCAL)
        v_position_local = a_cc3Position.xyz;
    #else
        v_position_local = vec3(0.0);
    #endif
    #if defined(VSOUT_TEX_COORD) || defined(WORLD_WATER) || defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(ALPHA_TEST)
        v_texCoord = a_cc3TexCoord0;
    #endif
    #if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
        v_material_diffuse_color = currentMaterialDiffuseColor();

        #ifdef VERTPUSH_BORDER
            v_material_diffuse_color.rgb *= a_cc3Color.r;
        #endif
    #endif

    #if defined(NUM_TEXTURES) && defined(VSOUT_TEX_COORD)
        #if NUM_TEXTURES > 1 && !defined(ANIMATED_TEX)
            v_texCoord2 = a_cc3TexCoord0;
        #elif defined(ANIMATED_TEX)
            v_texCoord2 = a_cc3TexCoord1;
        #endif
    #endif

    #if defined (WORLD_LAVA_BLEND)
        v_texCoord = a_cc3TexCoord1;
    #endif

    #if defined(USE_VERTEX_COLOR) || defined(HEIGHT_BLEND) || defined(WORLD_REGION) || defined(WORLD_DETAIL) || defined(WORLD_WATER) || defined(OVERLAY) || defined(REFLECT)
        v_vertexColor = a_cc3Color;
        #if defined(WORLD_REGION)
            v_vertexColor.a *= currentMaterialDiffuseColor().a;
        #endif
    #endif

    #if ((defined(VSOUT_NORMAL) || defined(FOGOFWAR) || defined(REFLECT) || defined(OVERLAY) || defined(REFLECT_CUBE)) && !defined(SHADOW_GEN))
        v_N = normalize(mat3(modelMatrix) * a_cc3Normal);
        #if defined(VSOUT_TANGENT_FRAME) || defined(REFLECT_CUBE)
            v_T = normalize(mat3(modelMatrix) * a_cc3Tangent);
            v_B = normalize(mat3(modelMatrix) * a_cc3Bitangent);
        #endif

        #ifdef VERTPUSH_BORDER
            #ifdef USE_NORMAL_MAP
                float border_width = 0.002;
            #else
                float border_width = 0.0015;
            #endif

            float object_scale = length(modelMatrix[0].xyz);
            float dist = distance(getCameraPositionWorld(), position_world.xyz);
            position_world.xyz -= v_N * (border_width * dist) * (1.0 - a_cc3Color.r);

            #ifdef DISSOLVE
                position_world.y += float(currentDissolveStrengthDurationDirectionFade().w == 0.0) * 20000.0 * (1.0 - a_cc3Color.r);
            #endif
        #endif
    #endif

    #if defined(WORLD) || defined(WORLD_WATER) || defined(WORLD_CLOUDS) || defined(WORLD_DETAIL) || defined(REFLECT) || defined(OVERLAY)
        vec3 vv = normalize(position_world.xyz - getCameraPositionWorld());
        #if defined(REFLECT)
            v_world_normal_dir = mat3(getWorldToCamera()) * reflect(v_N, vv);
        #endif
    #endif

    #if defined(LIGHTMAP_ENABLED) || defined(FAKE_LIGHTMAP_ENABLED) || defined(COMPOSITE) || defined(WORLD) || defined(WORLD_WATER) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(NO_FLOW)
        #if (defined(WORLD_LAVA_BLEND) && !defined(NO_FLOW)) || defined(FAKE_LIGHTMAP_ENABLED)
            v_lightMapCoord = a_cc3TexCoord0;
        #else
            v_lightMapCoord = a_cc3TexCoord1;
        #endif
    #endif
    
    #if defined(SCROLLING_TEXTURE) && defined(VSOUT_TEX_COORD)
        #ifdef NO_LOOP_TIME
            float time = timeOffset + u_cc3ObjectTime[INSTANCE_ID];
            float time2 = time;
        #else
            // We use mod to avoid overflow of texture coordinates (with lowp floats,
            //  there is texture degradation with texture coordinates exceeding ~10e4
            //  and the degradation is very obvious when exceeding 10e5).
            //  We loop infrequently enough that hopefully no one will notice the loop.
            const float repeatTime = 10.0e3;
            float time = mod(u_cc3ObjectTime[INSTANCE_ID], repeatTime);
            #if defined(NUM_TEXTURES)
                #if NUM_TEXTURES > 1
                    float time2 = mod(u_cc3ObjectTime[INSTANCE_ID], repeatTime);
                #endif
            #endif
        #endif

        v_texCoord += time * scrollingSpeed;

        #if defined(NUM_TEXTURES)
            #if NUM_TEXTURES > 1
                v_texCoord2 += time2 * scrollingSpeed2;
            #endif
        #endif
    #endif

    #ifdef BILLBOARD
        position.xyz = mat3(modelMatrix) * position.xyz;
        position.xyz += (getWorldToCamera() * modelMatrix[3]).xyz;
        gl_Position = u_cc3MatrixProj * position;
        gl_Position.z += BillboardDepthOffset;

        vec3 v = normalize(getCameraPositionWorld() - modelMatrix[3].xyz);
        float billboard_fade = mix(BillboardFrontBack.y, BillboardFrontBack.x, dot(v, normalize(BillboardDirection)) * 0.5 + 0.5);
        v_material_diffuse_color.rgb *= billboard_fade;
    #else
        #if !defined(USE_MVP_PATH)
            gl_Position = getWorldToProj() * vec4(position_world.xyz, 1.0);
        #else
            gl_Position = getModelToProj() * vec4(position.xyz, 1.0);
        #endif
    #endif

    #if defined(VSOUT_POSITION_WORLD)
        v_position_world = position_world.xyz;
    #endif

    #if defined(RADIAL_TEXTURE) && defined(VSOUT_TEX_COORD)
        float time = mod(getSceneTime(), 2.0 * M_PI / radialSpeed);
        float angle = time * radialSpeed;
        mat2 rotationMatrix = mat2(vec2( cos(angle), sin(angle)),
                                   vec2(-sin(angle), cos(angle)));
        v_texCoord = v_texCoord * 2.0 - 1.0;
        v_texCoord = rotationMatrix * v_texCoord;
        v_texCoord = v_texCoord * 0.5 + 0.5;
    #endif

    #if defined(UV_POSITION_SCROLL) && defined(VSOUT_TEX_COORD)
        vec3 offset = modelMatrix[3].xyz;
        v_texCoord += scrollUVDirection * dot(scrollAxesMultipliers, offset);
    #endif

    #if defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
        vec2 flow_coord = v_lightMapCoord;
        vec2 diffuseCoord = v_texCoord;

        #if defined(WATER_BLEND)
            flow_coord = v_texCoord;
        #endif

        #if defined(NO_FLOW)
            v_flow_pre = vec2(0.0);
            v_flow_aux = vec4(0.0);

            v_flow_uv1 = (flow_coord * UV3Tiling) + (getSceneTime() * UV3Speed);
            v_flow_uv2 = (flow_coord * UV3Tiling) + (getSceneTime() * vec2(-UV3Speed.x, UV3Speed.y) + vec2(0.0, 0.25));
            v_flow_blend = 0.0;
        #else

            #if defined(WORLD_LAVA_BLEND)
                flow_coord = v_texCoord;
                diffuseCoord = v_lightMapCoord;
            #endif

            float flow_noise = a_cc3Color.a * NoiseStrength;
            float flow_timer = (getSceneTime() + flow_noise) / WaveLength;
            vec2 flow = (vec2(flow_coord.r, 1.0 - flow_coord.g) - 0.5) * FlowStrength;

            #if defined(LAVA_BLEND)|| defined(WORLD_LAVA_BLEND)
                v_flow_pre = flow;
                v_flow_aux.xy = UV3Tiling;
                v_flow_aux.z = flow_timer;
            #endif

            #if defined(WORLD_LAVA_BLEND)
                //flow = vec2(0.0, FlowStrength);
            #endif

            float wave_1 = fract(flow_timer);
            float wave_2 = fract(flow_timer + 0.5);

            v_flow_uv1 = diffuseCoord * UV3Tiling + wave_1 * flow - (flow * 0.5);
            v_flow_uv2 = diffuseCoord * UV3Tiling + wave_2 * flow - (flow * 0.5);
            v_flow_blend = abs(wave_1 - 0.5) * 2.0;
        #endif
    #endif

    #if defined(ANIMATED_TEX)
        v_animCoord = vec2(getSceneTime(), getSceneTime());
    #endif

    // A small constant must be added because interpolation could cause the value to fall slightly below the original
    // int, thus causing the typecast to access the lower index.
    #ifdef INSTANCING_ENABLED
        #if defined(INTEGER_ATTRIBUTES)
            v_instanceID = INSTANCE_ID;
        #else
            v_instanceID = float(INSTANCE_ID) + 0.1;
        #endif
    #endif

    #if !defined(SHADOW_GEN)
        shadow_receive(position_world);
    #endif
}
