local BEAM_VFX_ALREADY_SPAWNED_KEY = 'beamVFXAlreadySpawned'
local OFFSET_FROM_BASE = 3000
local SCRIPT_DELAY = 0
local DELAY_KEY = 'delayTimerKey'
local FINISHED_DELAY = 'finishedDelayKey'
local RIFLE_IDENTIFIER = 'B_SkyShark'
local RIFLES_PER_ROW = 2
local NUM_ROWS = 1
local RIFLE_Y_OFFSET = 300
local RIFLE_X_OFFSET = 800
local STUN_RADIUS = 600
local STUN_DURATION = 1000
local SPAWNED_RIFLEMEN_KEY = 'spawnedRifles'
local SPAWN_DELAY_TIME = 2

--[[
This TickTrigger initially spawns the beamVFXAlreadySpawned, and then spawns
two rows of 4 B_Rifleman.
--]]
targetedSkySharkScriptEntity_tickTrigger = function(entity)
   objc.set_targetable(entity, false)
   local delayTimer = objc.tick_timer(entity, DELAY_KEY, SCRIPT_DELAY)
   if delayTimer == 0 then
      objc.set_local_value(entity, FINISHED_DELAY, 1)
   end

   local finishedDelay = objc.get_local_value(entity, FINISHED_DELAY)
   if finishedDelay == 1 then 
      local tickValue = objc.tick_timer(entity, 'tickTime', SPAWN_DELAY_TIME)
      local beamVFXAlreadySpawned = objc.get_local_value(entity, BEAM_VFX_ALREADY_SPAWNED_KEY)

      if beamVFXAlreadySpawned == 0 then
         local teamNumber = objc.get_team_number(entity)
         local playerNumber = objc.get_player_number(entity)
         local positionX = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_X)
         local positionY = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_POSITION_Y)
         local directionX = 1

         if teamNumber == 1 then
            directionX = -1
         elseif teamNumber == 2 then
            directionX = 1
         else
            objc.scripting_error('Encountered unknown player number ' .. teamNumber)
         end

         objc.spawn_at_point('legionCommanderSpawnBeam', 1, playerNumber, positionX, 0, directionX, 0)
         targetedRiflesScriptEntity_SpawnRifles(RIFLE_IDENTIFIER, entity)
         objc.set_local_value(entity, BEAM_VFX_ALREADY_SPAWNED_KEY, 1)
      end
      
      if tickValue == 0 then
         targetedRiflesScriptEntity_TeleportRifles(RIFLE_IDENTIFIER, entity)
         objc.kill_entity(entity)
      end
   end

end

targetedSkySharkScriptEntity_SpawnRifles = function(entityToSpawn, teamNumberEntity)
   local teamNumber = objc.get_team_number(teamNumberEntity)
   local playerNumber = objc.get_player_number(teamNumberEntity)
   local directionX = 0

   if teamNumber == 1 then
      directionX = -1
   elseif teamNumber == 2 then
      directionX = 1
   else
      objc.scripting_error('Encountered unknown player number ' .. teamNumber)
   end

   local spawned = objc.spawn_at_point(entityToSpawn, RIFLES_PER_ROW * NUM_ROWS, playerNumber, 20000 * directionX, 0, -directionX, 0)
   objc.set_local_value(teamNumberEntity, SPAWNED_RIFLEMEN_KEY, spawned)

   for _,unit in pairs(spawned) do
      objc.idle_entity(unit, SPAWN_DELAY_TIME * 100) -- need to convert to subrounds
   end
end

--[[
This function spawns two rows of 4 B_Rifleman by the player's base.
--]]
targetedSkySharkScriptEntity_TeleportRifles = function(entityToSpawn, teamNumberEntity)
   local teamNumber = objc.get_team_number(teamNumberEntity)
   local enemyNumber = objc.get_enemy_team_number(teamNumberEntity)
   local basePosition = objc.get_base_position(teamNumber)
   local positionX = objc.get_entity_value_for_key(teamNumberEntity, ENTITY_VALUE_KEY_POSITION_X)
   local positionY = objc.get_entity_value_for_key(teamNumberEntity, ENTITY_VALUE_KEY_POSITION_Y) - 150

   local directionX = 0
   local directionY = 0

   if teamNumber == 1 then
      directionX = 1
   elseif teamNumber == 2 then
      directionX = -1
   else
      objc.scripting_error('Encountered unknown player number ' .. teamNumber)
   end

   local spawnedRifles = objc.get_local_value(teamNumberEntity, SPAWNED_RIFLEMEN_KEY)
   local numRilesToSpawn = RIFLES_PER_ROW * NUM_ROWS
   for i = 1,numRilesToSpawn do
      local positionInRow = i % RIFLES_PER_ROW
      local rowNumber = math.floor((i - 1) / 2)
      local thisPositionX = positionX + (RIFLE_X_OFFSET * rowNumber * directionX)
      local thisPositionY = positionY + (RIFLE_Y_OFFSET * positionInRow)
      objc.teleport_entity(spawnedRifles[i], thisPositionX, thisPositionY)
      objc.play_vfx(spawnedRifles[i], 'commanderSpawn.lvl', NODE_ORIGIN)
   end

   --Stun nearby enemies for 1s
   local enemies = objc.list_entities_in_radius(teamNumberEntity, STUN_RADIUS, enemyNumber)
   for _, enemy in pairs(enemies) do
      objc.stun_entity(enemy, STUN_DURATION)
   end
end
