//#define NO_DIFFUSE_TEXTURE
//#define LIGHTMAP_ENABLED
//#define ALPHA_TEST
//#define USE_NORMAL_MAP
//#define BYPASS_NORMAL_MAP
//#define USE_LIGHT
//#define USE_VERTEX_COLOR
//#define FRESNEL_ENABLED
//#define ENVIRONMENT
//#define INSTANCING_HARD_DISABLE
//#define MANTIS_PBR

#define SUPPORT_GLSL300
#include "prefix_tow.sh"

#if defined(GL_ES)
    precision highp float;
#endif

#include "system_uniforms.sh"

#if __VERSION__ >= 300
    layout(location = 0) out vec4 primaryOutput;
    #if defined(BORDER) && (BORDER_METHOD == BORDER_METHOD_PIXEL_BASED_MRT)
        layout(location = 1) out int secondaryDepth;
    #endif
    #define gl_FragColor primaryOutput

    #define textureCube texture
#endif

#if CC3_PLATFORM_WINDOWS // win32
    #undef INSTANCING_ENABLED
#else
    #if !defined(INSTANCING_HARD_DISABLE)
        #define INSTANCING_ENABLED
    #else
        #undef INSTANCING_ENABLED
    #endif
#endif

#if defined(INSTANCING_ENABLED)
    #define MAX_SKINNED_INSTANCES 10
#else
    #define MAX_SKINNED_INSTANCES 1
#endif

#define M_PI 3.1415926535897932384626433832795
#define M_GRAYSCALE vec3(0.2126, 0.7152, 0.0722)

//===================================================================================================================//
// UNIFORMS //
//===================================================================================================================//

uniform sampler2D s_cc3Texture2Ds[7];

uniform samplerCube u_reflectionMap;
uniform sampler2D sphereMap;
uniform sampler2D cloudTexture;
uniform samplerCube nebulaMap;

// Sampler Indices //
#if (defined(WORLD) && defined(HEIGHT_BLEND)) || defined(WORLD_DETAIL)
    const int region_map_index = 3;
#else
    const int region_map_index = 1;
#endif

#if defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLDREGIONONLY)
    const int cloud_map_index = region_map_index + 2;
    uniform vec3 GlobalUniform_RegionMaskChannel;
#else
    const int cloud_map_index = region_map_index;   
#endif

#if defined(WORLD_AO)
    const int light_map_index = cloud_map_index + 1;
#elif defined(PRELIT) && defined(REFLECT)
    const int light_map_index = 5;
#else
    const int light_map_index = cloud_map_index;   
#endif


#ifdef SHADER_DEBUG
    uniform int DebugIndex;
    uniform float DebugFloat;
    uniform vec3 DebugVec3;
    uniform vec4 DebugVec4;
#endif

#ifdef ALPHA_TEST
    uniform float AlphaThreshold;
#endif

#if defined(FRESNEL_ENABLED) || defined(SHIELD)
    uniform vec4 fresnelColor;
    uniform float fresnelFalloff;
#endif

#if defined(FOGOFWAR)
    uniform vec4 BattlePlacement_WarBattlePlacementProperties0;
    uniform vec4 BattlePlacement_WarBattlePlacementProperties1;
    uniform vec4 BattlePlacement_ZoneColor;
    uniform float BattlePlacement_BorderPulse0;
    uniform float BattlePlacement_BorderPulse1;
    uniform vec3 BattlePlacement_GridVisibility;
#endif

#if defined(BLEND_REFLECT) || defined(REFLECT_CUBE)
    uniform vec3 ReflectBlends;
#endif

#if (defined(BORDER) || defined(NO_BORDER)) && defined(USE_LIGHT)
    uniform float SpecularIntensity;
    uniform float SpecularPower;
    uniform float SpecularCustomUsage;
    uniform vec4 SpecularColor;
#endif

#if defined(REFLECT)
    uniform float SpecMult;
    uniform float SpecPower;
    uniform float ReflectionValue;
    uniform float SelfIllumination;
#endif

#if defined(OVERLAY)
    uniform vec2 OverlayUVTile;
    uniform vec4 Overlay1Color;
    uniform vec4 Overlay2Color;
#endif

#if defined(USE_LIGHT) || defined(USE_PROPERTY_MAP)
    float LightStrength = 1.0;

    uniform vec3 LightEnvTop;
    uniform vec3 LightEnvSide;
    uniform vec3 LightEnvBottom;

    uniform vec3 GlobalUniform_LightCharTop;
    uniform vec3 GlobalUniform_LightCharSide;
    uniform vec3 GlobalUniform_LightCharBottom;

    #if (defined(BORDER) || defined(NO_BORDER)) && defined(USE_LIGHT)
        uniform vec3 GlobalUniform_SpecularLightDir;
        uniform float SpecularLightAlign;
    #endif

#endif

#ifdef HEIGHT_BLEND
    uniform vec2 UV1Tiling;
    #if !defined(WORLD_LAVA_BLEND) && !defined(LAVA_BLEND)
        uniform vec4 Diffuse1Color;
        uniform vec4 Diffuse2Color;
        uniform vec4 Diffuse3Color;
    #endif
    uniform float Height1Influence;
    uniform vec2 UV2Tiling;
    uniform float Height2Influence;
#if !defined(NO_FLOW)
    uniform vec2 UV3Tiling;
#endif
    uniform float Height3Influence;
    uniform float MaskInfluence;
    uniform float Shift;
    uniform vec3 LayerEmissive;
#endif

#ifdef SHIELD
    uniform float ShieldScale;
    uniform float ShieldBorderAmount;
    uniform vec2 ShieldScroll;
    uniform float ShieldFade;
    uniform float ShieldPower;
#endif

#ifdef GRID
    uniform float GridTime;
    uniform vec3 GridUnitPos;
    uniform vec3 GridUnitPos2;
    uniform float BattlePlacement_WaveTimePercentage;
    uniform vec2 BattlePlacement_FactoryLocations[8];
#endif

#ifdef BLINK
    uniform vec3 blinkColor;
    uniform vec2 blinkIntensityFrequency;
#endif

#ifdef EMISSIVE
    uniform vec4 EmissiveColor;
#endif

#ifdef COLOR_FADE
    uniform vec4 FadeColor;
#endif

#ifdef UNIT_HIGHLIGHT
    uniform vec4 HighlightColor;
#endif
 
#ifdef FINAL_TINT
    uniform vec4 FinalTint;
#endif

uniform vec4 ColorFadeOverride;
uniform vec2 ColorFadeStartAndDuration; // .x is the start "object time" for the animation, and .y is the duration time

#if (defined(WORLD_LAVA_BLEND) && defined(USE_NORMAL_MAP)) || defined(REFLECT_CUBE)
    uniform vec2 NormalUVTiling;
#endif

#if defined(WORLD_LAVA_BLEND) && defined(USE_SPEC)
    uniform vec3 SpecularColor;
    uniform float SpecularPower;
#endif

#ifdef BLUEPRINT
    uniform vec4 BlueprintColor;
    uniform float u_dissolveHighestHeight;
    uniform float u_dissolveLowestHeight;
#endif

#if defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLDREGIONONLY)
    uniform float RegionMaskMultiplier;
#endif

uniform float u_cc3ObjectTime[MAX_SKINNED_INSTANCES];

#if defined(DISSOLVE) || defined(HOLOGRAM) || defined(HOLOGRAM2)  || defined(FREEZE2) || defined(ANIMATED_PATH) || defined(WORLD) || defined(WORLD_CLOUDS) || defined(PBR_ANIMATED_BLEND) || defined(GACHA) || defined(WORLDREGIONONLY)
    uniform sampler2D dissolveMask;
#endif

#if defined(DISSOLVE) || defined(HOLOGRAM)
    uniform vec4 DissolveColor;
    uniform vec4 DissolveStrengthDurationDirectionFade;
    uniform vec3 DissolveWidthTexwidthTexscale;
#elif defined(GACHA)
    const vec4 DissolveColor = vec4(0.2, 0.8, 1.0, 1.0);
    const vec4 DissolveStrengthDurationDirectionFade = vec4(8.0, 1.5, 1.0, 1.0);
    const vec3 DissolveWidthTexwidthTexscale = vec3(500.0, 100.0, 200.0);
#endif

#ifdef GLASS
    uniform vec4 GlassFrontColor;
    uniform vec4 GlassSideColor;
    uniform float GlassFresnel;
#endif

#if defined(WATER_BLEND)
    uniform float WaterShallowMultiplier;
    uniform vec4 WaterShallowColor;
    uniform vec4 WaterDeepColor;
#endif

#ifdef PARALLAX
    uniform vec4 Layer1DepthScaleBlendFade;
    uniform vec4 Layer2DepthScaleBlendFade;
    uniform vec4 Layer3DepthScaleBlendFade;
    uniform vec4 Layer4DepthScaleBlendFade;
    uniform vec4 Layer1BackgroundColor;
    uniform vec4 Layer2BackgroundColor;
    uniform vec4 Layer3BackgroundColor;
    uniform vec4 Layer4BackgroundColor;
    uniform vec2 Layer1Scroll;
    uniform vec2 Layer2Scroll;
    uniform vec2 Layer3Scroll;
    uniform vec2 Layer4Scroll;
    uniform float MaskSubtract;        
#endif

#ifdef WORLD_ATMOSPHERE
    uniform vec4 InnerColor;
    uniform vec4 MiddleColor;
    uniform vec4 OuterColor;
    uniform float InnerRadius;
    uniform float MiddleRadius;
    uniform float OuterRadius;
#endif

#ifdef WORLD_REGION
    uniform float RegionFresnelPower;
    uniform float RegionFacingAlpha;
    uniform float RegionEdgeAlpha;
#endif

#if defined(WORLD_REGION_PULSE)
    uniform float RegionBorderPulseValue;
    uniform vec4 RegionPulseColor;
#endif

#if defined(WORLD) || defined(WORLDREGIONONLY)
    uniform float RegionFogBias;
#endif

#if defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLDREGIONONLY)
    uniform float RegionReveal;
    uniform float RegionStartTime;
    uniform float RegionDuration;
#endif  

#ifdef WORLD_DETAIL
    uniform float DetailLayer1Gloss;
    uniform float DetailLayer1Specular;
    uniform vec4 DetailLayer1FrontColor;
    uniform vec4 DetailLayer1SideColor;
    uniform float DetailLayer1Fresnel;   
    uniform float DetailLayer1Reflection;   
    uniform float DetailLayer1Influence;
    uniform float DetailLayer2Gloss;
    uniform float DetailLayer2Specular;
    uniform vec4 DetailLayer2FrontColor;
    uniform vec4 DetailLayer2SideColor;
    uniform float DetailLayer2Fresnel;
    uniform float DetailLayer2Reflection;   
    uniform float DetailLayer2Influence;
    uniform float MaskInfluence;
    uniform float Shift;
#endif

#if defined(WORLD) || defined(WORLD_DETAIL) || defined(WORLD_CLOUDS) || defined(WORLDREGIONONLY)
    uniform vec3 WorldPosition;
    uniform vec3 WorldRotation;
#endif

#ifdef ANIMATED_PATH
    uniform float PathSize;
    uniform float PathDuration;
#endif

#if defined(MANTIS_PBR)
    uniform float GlossBoost;
#endif

#if defined(COMMANDER)
    uniform float RimMultiplier;
    uniform vec3 RimFillColor;
    uniform vec3 RimCustomColor;
    uniform vec4 FadeColor;
    uniform float RimMaskUsage;
#endif

#if defined(PRELIT)
    uniform vec4 ShadowColor;    //make into env config setting
#endif

#if defined(HOLOGRAM_SCALE)
    uniform float HologramScale;
    uniform float HologramAlpha;
#endif

#if defined(ANIMATED_TEX)
    uniform vec2 BaseOffsets;
    #if defined(USE_PROGRESS)
        uniform float GlobalUniform_ProgressA;
        uniform float GlobalUniform_ProgressB;
        uniform vec2 ProgressAB;
    #endif
    uniform vec2 AlphaOffsets;
    uniform vec2 Base2ScrollSpeed;
    uniform float Base2Blend;
    uniform float Intensity;
#endif

#if defined(UNIT_OCCLUSION_RENDERING)
    uniform vec4 BattleEntity_OcclusionHighlightColor;
#endif

#include "environmentconfig.sh"
#include "property_map.fsh"
#include "mantis_lighting2.sh"  
#include "tiled_lighting.sh"
#include "shadow.fsh"

//===================================================================================================================//
// VARYINGS //
//===================================================================================================================//

#ifdef INSTANCING_ENABLED
    #if defined(INTEGER_ATTRIBUTES)
        VARYING_IN int v_instanceID;
    #else
        VARYING_IN float v_instanceID; // "int" with small constant added to handle flooring
    #endif
    #define INSTANCE_ID (int(v_instanceID))
#else
    #define INSTANCE_ID 0
#endif

#if !defined(SHADOW_GEN)

VARYING_IN vec3 v_position_world;

#if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(SHIELD) || defined(GLASS) || defined(PARALLAX) || defined(WORLD_CLOUDS) || defined(WORLD_REGION) || defined(WORLD) || defined(WORLD_DETAIL) || defined(FOGOFWAR) || defined(REFLECT_CUBE) || defined(WORLDREGIONONLY)
    VARYING_IN vec3 v_N;
    #if defined(USE_NORMAL_MAP) || defined(PARALLAX) || defined(WORLD_DETAIL) || defined(REFLECT_CUBE)
        VARYING_IN vec3 v_T;
        VARYING_IN vec3 v_B;
    #endif
#endif

VARYING_IN vec2 v_texCoord;

#if defined(ANIMATED_TEX)
    VARYING_IN vec2 v_texCoord2;
    VARYING_IN vec2 v_animCoord;
#endif

#if defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)
    VARYING_IN vec4 v_material_diffuse_color;
#endif

#if defined(USE_VERTEX_COLOR) || defined(HEIGHT_BLEND) || defined(WORLD_REGION) || defined(WORLD_DETAIL) || defined(OVERLAY) || defined(REFLECT)
    VARYING_IN vec4 v_vertexColor;
#endif

#if defined(LIGHTMAP_ENABLED) || defined(COMPOSITE) || defined(WORLD_REGION) || defined(WORLD) || defined(WORLD_DETAIL) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND) || defined(PRELIT)
    VARYING_IN vec2 v_lightMapCoord;
#endif

#ifdef COMMANDER
    VARYING_IN vec3 v_rim;
#else
    vec3 v_rim = vec3(0.0);
#endif

#if defined(DISSOLVE) || defined(GACHA) || defined(HOLOGRAM_BUILD) || defined(COMMANDER) || defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLD_ATMOSPHERE) || defined(FOGOFWAR) || defined(SHIELD)
    VARYING_IN vec3 v_position_local;
#else
    vec3 v_position_local = vec3(0.0);
#endif

#if defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
    VARYING_IN vec2 v_flow_uv1;
    VARYING_IN vec2 v_flow_uv2;
    VARYING_IN float v_flow_blend;
    #if defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
       VARYING_IN vec4 v_flow_aux;
       VARYING_IN vec2 v_flow_pre;
    #endif
#endif

#else

    #if defined(ALPHA_TEST)
        VARYING_IN vec2 v_texCoord;
    #endif

#endif

#if defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLD_DETAIL) || defined(BLEND_REFLECT) || defined(REFLECT) || defined(SPEC_LIGHT) || defined(WORLDREGIONONLY)
    VARYING_IN vec3 v_world_light_dir;
    #if defined(REFLECT) || defined(BLEND_REFLECT)
        VARYING_IN vec3 v_world_normal_dir;
    #endif
#endif

//===================================================================================================================//
// SHADER //
//===================================================================================================================//
#define BlendSoftLightf(base, blend)    ((blend < 0.5) ? (2.0 * base * blend + base * base * (1.0 - 2.0 * blend)) : (sqrt(base) * (2.0 * blend - 1.0) + 2.0 * base * (1.0 - blend)))
#define BlendSoftLight(base, blend)     Blend(base, blend, BlendSoftLightf)
#define Blend(base, blend, funcf)       vec3(funcf(base.r, blend.r), funcf(base.g, blend.g), funcf(base.b, blend.b))

#ifdef BLINK
    vec3 currentBlinkColor() { return blinkColor; }
    float currentBlinkFrequency() { return blinkIntensityFrequency.y; }
    float currentBlinkIntensity() { return blinkIntensityFrequency.x; }
#endif

#if defined(DISSOLVE) || defined(HOLOGRAM) || defined(GACHA)
    vec4 currentDissolveColor() { return DissolveColor; }
    vec4 currentDissolveStrengthDurationDirectionFade() { return DissolveStrengthDurationDirectionFade; }
#endif

#if (defined(INSTANCING_ENABLED) || defined(VERTPUSH_BORDER) || defined(BILLBOARD)) && !defined(SHADOW_GEN)
    vec4 getMaterialColor() { return v_material_diffuse_color; }
#else
    uniform vec4 u_cc3MaterialDiffuseColor;
    vec4 getMaterialColor() { return u_cc3MaterialDiffuseColor; }
#endif

float grid_render(vec3 v_N, vec4 u_war_battle_placement_properties_XX, float BorderPulse, vec3 world_pos)//note this is deprecated - need to resolve with scrolling.frag, which is the current version of this function
{
#if defined(FOGOFWAR)
    float placement_x = u_war_battle_placement_properties_XX.x;
    float zone_width = u_war_battle_placement_properties_XX.y;
    float zone_length = u_war_battle_placement_properties_XX.z;

    float zone_offset = abs(placement_x);

    float grid_size = 600.0;
    vec2 normalized_pos = vec2(
        clamp((world_pos.x - u_war_battle_placement_properties_XX.x) /
        (u_war_battle_placement_properties_XX.y - u_war_battle_placement_properties_XX.x), 0.0, 1.0),
        clamp((world_pos.z - u_war_battle_placement_properties_XX.w) /
        (u_war_battle_placement_properties_XX.z - u_war_battle_placement_properties_XX.w), 0.0, 1.0)
    );

    vec2 zone_uv = normalized_pos;

    #if defined(TEX01)
        vec4 zone_tex = sampleLevelZero(s_cc3Texture2Ds[1], zone_uv);
    #else
        vec4 zone_tex = sampleLevelZero(s_cc3Texture2Ds[3], zone_uv);
    #endif

    float zone_tex_border = zone_tex.r;
    float zone_tex_wave_flash = zone_tex.g;
    float zone_tex_alpha = zone_tex.b;
    float zone_tex_background = zone_tex_alpha * 0.2;


    float zone_height_mask = 1.0 - clamp(abs((world_pos.y + 100.0) * 0.005), 0.0, 1.0); //temp disable
    float zone_pulse = sin(getSceneTime() * 6.28) * 0.5 + 1.0;
    float zone_edge = 1.0 - abs(clamp(-(world_pos.y + 100.0) * 0.005, 0.0, 1.0) - 0.5) * 2.0;

    float zone_edge_pulse = zone_tex_border * BorderPulse;

    vec2 zone_lines = abs(fract(world_pos.xz / grid_size) - 0.5);
    float zone_grid = clamp((max(zone_lines.x, zone_lines.y) - 0.45) * 10.0, 0.0, 1.0);

    float edge_dot = dot(v_N, vec3(0.0, 1.0, 0.0));

    float edge_falloff = pow(edge_dot, 4.0);
    zone_edge_pulse *= edge_falloff;

    float grid = (zone_grid * zone_pulse + zone_edge + zone_edge_pulse + zone_tex_background * 2.0) * zone_tex_alpha * edge_falloff;
    return grid;
#else
    return 0.0;
#endif           
}

//===================================================================================================================//
void main() 
{
#if defined(SHADOW_GEN)
    // In "SHADOW_GEN" mode, we're only writting to the depth buffer. So we don't have to set
    // any color values, or doing anything else here... We just return.
    // (except if we're doing ALPHA_TEST, in which case we must still do the discard.
    #if defined(ALPHA_TEST) && !defined(NO_DIFFUSE_TEXTURE)
        if (texture2D(s_cc3Texture2Ds[0], v_texCoord).a < AlphaThreshold) { discard; }
    #endif
    
#elif defined(SECOND_PASS_GEO_BORDERS)
    
    #if defined(ALPHA_TEST) && !defined(NO_DIFFUSE_TEXTURE)
        if (texture2D(s_cc3Texture2Ds[0], v_texCoord).a < AlphaThreshold) { discard; }
    #endif
    gl_FragColor = vec4(0, 0, 0, 1);

#elif defined(UNIT_OCCLUSION_RENDERING)
    // UNIT_OCCLUSION_RENDERING is defined by the engine when it's ready to perform the occlusion highlight pass.
    // Client controls it by setting the _enableOcclusionHighlight property in the scene's technique descriptor.

    // Flat shading requires a macro that defines a solid color to be used.
    gl_FragColor = BattleEntity_OcclusionHighlightColor;

#else
    
    #ifdef NO_DIFFUSE_TEXTURE
        vec4 textureColor = vec4(1.0);
    #else
        vec4 textureColor = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    #endif

    #ifdef COMPOSITE
        vec4 composite_mask = texture2D(s_cc3Texture2Ds[3], v_lightMapCoord);
        vec4 diffuse1 = texture2D(s_cc3Texture2Ds[1], v_texCoord);
        vec4 diffuse2 = texture2D(s_cc3Texture2Ds[2], v_texCoord);
        vec4 composite = vec4(0, 0, 0, 1);
        composite.rgb += textureColor.rgb * composite_mask.r;
        composite.rgb += diffuse1.rgb * composite_mask.g;
        composite.rgb += diffuse2.rgb * composite_mask.b;
        textureColor.rgb = composite.rgb;
    #endif
    
    vec4 diffuseColor = getMaterialColor() * textureColor;

    #ifdef PARALLAX
        vec3 v = normalize(getCameraPositionWorld() - v_position_world);
        vec2 offset = vec2(dot(v_T, v), dot(v_B, v));

        vec4 layer_1_tex = texture2D(s_cc3Texture2Ds[0], ((v_texCoord + getSceneTime() * Layer1Scroll) - offset * Layer1DepthScaleBlendFade.x - 0.5) / Layer1DepthScaleBlendFade.y + 0.5);
        vec4 layer_2_tex = texture2D(s_cc3Texture2Ds[1], ((v_texCoord + getSceneTime() * Layer2Scroll) - offset * Layer2DepthScaleBlendFade.x - 0.5) / Layer2DepthScaleBlendFade.y + 0.5);
        vec4 layer_3_tex = texture2D(s_cc3Texture2Ds[2], ((v_texCoord + getSceneTime() * Layer3Scroll) - offset * Layer3DepthScaleBlendFade.x - 0.5) / Layer3DepthScaleBlendFade.y + 0.5);
        vec4 layer_4_tex = texture2D(s_cc3Texture2Ds[3], ((v_texCoord + getSceneTime() * Layer4Scroll) - offset * Layer4DepthScaleBlendFade.x - 0.5) / Layer4DepthScaleBlendFade.y + 0.5);

        // Fade //
        float fresnel = max(0.0, dot(v, v_N));
        layer_1_tex.rgb *= Layer1BackgroundColor.rgb * mix(Layer1DepthScaleBlendFade.w, 1.0, fresnel);
        layer_2_tex.rgb *= Layer2BackgroundColor.rgb * mix(Layer2DepthScaleBlendFade.w, 1.0, fresnel);
        layer_3_tex.rgb *= Layer3BackgroundColor.rgb * mix(Layer3DepthScaleBlendFade.w, 1.0, fresnel);
        layer_4_tex.rgb *= Layer4BackgroundColor.rgb * mix(Layer4DepthScaleBlendFade.w, 1.0, fresnel);

        // Blend/Additive //
        float blend_1 = layer_2_tex.a * (1.0 - Layer2DepthScaleBlendFade.z);
        vec3 layer_blend = mix(layer_1_tex.rgb, layer_2_tex.rgb, blend_1) + layer_2_tex.rgb * layer_2_tex.a * Layer2DepthScaleBlendFade.z;

        float blend_2 = layer_3_tex.a * (1.0 - Layer3DepthScaleBlendFade.z);
        layer_blend = mix(layer_blend, layer_3_tex.rgb, blend_2) + layer_3_tex.rgb * layer_3_tex.a * Layer3DepthScaleBlendFade.z;

        float blend_3 = layer_4_tex.a * (1.0 - Layer4DepthScaleBlendFade.z);
        diffuseColor.rgb = mix(layer_blend, layer_4_tex.rgb, blend_3) + layer_4_tex.rgb * layer_4_tex.a * Layer4DepthScaleBlendFade.z;

        diffuseColor.a = min(1.0, layer_1_tex.a + layer_2_tex.a + layer_3_tex.a + layer_4_tex.a);

        // Mask //
        vec4 mask = texture2D(s_cc3Texture2Ds[4], v_texCoord);
        diffuseColor = clamp(diffuseColor, 0.0, 1.0);
        diffuseColor = mix(diffuseColor * mask, max(vec4(0.0), diffuseColor - (1.0 - mask)), MaskSubtract);
    #endif 

    // alpha test
    #ifdef ALPHA_TEST
        if (textureColor.a < AlphaThreshold) 
        {
            discard;
        }
    #endif

    #ifdef ANIMATED_PATH
        float path_size = 1.0 / PathSize;
        float path_clip = clamp(-PathSize - (v_texCoord.y * path_size) + (u_cc3ObjectTime[int(INSTANCE_ID)] / PathDuration * path_size), 0.0, 1.0);
        diffuseColor.a *= path_clip;
    #endif

    // normal
    #if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(USE_PROPERTY_MAP) || defined(SHIELD) || defined(GLASS) || defined(REFLECT_CUBE)
        #if defined(USE_NORMAL_MAP) && !defined(BYPASS_NORMAL_MAP)
            #if defined(WORLD_LAVA_BLEND)
                vec3 normal = texture2D(s_cc3Texture2Ds[1], v_texCoord * NormalUVTiling).rgb;
            #else
                vec3 normal = texture2D(s_cc3Texture2Ds[1], v_texCoord).rgb;
            #endif
            
            vec3 normal_t = normal * 2.0 - 1.0;
            vec3 N = normalize(mat3(v_T, v_B, v_N) * normal_t);
        #elif defined(REFLECT_CUBE)
            vec3 normal = texture2D(s_cc3Texture2Ds[5], v_texCoord * NormalUVTiling).rgb;
            vec3 normal_t = normal * 2.0 - 1.0;
            vec3 N = normalize(mat3(v_T, v_B, v_N) * normal_t);

        #else
            vec3 N = normalize(v_N);
        #endif
    #endif

    #if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(USE_PROPERTY_MAP) || defined(SHIELD) || defined(GLASS) || defined(FOG) || defined(WORLD_CLOUDS) || defined(WORLD_REGION) || defined(WORLD) || defined(WORLD_DETAIL) || defined(REFLECT_CUBE)
        vec3 V = (getCameraPositionWorld() - v_position_world);
        float V_length = length(V);
        V = normalize(V);
    #endif

    //lava color pre-blend
    #if defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
        vec3 self_illum_mask = vec3(0.0);
    #endif

    #if defined(REFLECT) || defined(BLEND_REFLECT)
        vec3 n = normalize(v_world_normal_dir); 
        vec2 sphere_uv = vec2(0.5, 0.5) + 0.5 * vec2(n.x, n.y);
        #if defined(BLEND_REFLECT)
            vec3 reflection = texture2D(s_cc3Texture2Ds[4], sphere_uv).rgb;
        #else
            vec3 reflection = texture2D(s_cc3Texture2Ds[2], sphere_uv).rgb;
            float reflMask = texture2D(s_cc3Texture2Ds[3], v_texCoord).r;
            float self_illum_mask = texture2D(s_cc3Texture2Ds[3], v_texCoord).r;
            #if defined(OVERLAY)
                float overlay1Mask = texture2D(s_cc3Texture2Ds[3], v_texCoord * OverlayUVTile.x).g;
                float overlay2Mask = texture2D(s_cc3Texture2Ds[3], v_texCoord * OverlayUVTile.y).b;
            #endif
        #endif
    #endif
    //-----------------------------------------------------------------//
    // Height Blend
    //  - Height maps are stored in the alpha of each layer
    //  - Influence values control how much each height map is used
    //  - Vertex red/green channels control layer blends
    //  - Vertex blue channel controls color softening
    //-----------------------------------------------------------------//
    #ifdef HEIGHT_BLEND
        vec2 diffuse12Coord = v_texCoord;
        #if defined(WORLD_LAVA_BLEND) 
            diffuse12Coord = v_lightMapCoord;
        #endif

        vec4 mask = v_vertexColor;
        vec4 diffuse1 = texture2D(s_cc3Texture2Ds[0], diffuse12Coord * UV1Tiling);
        vec4 diffuse2 = texture2D(s_cc3Texture2Ds[1], diffuse12Coord * UV2Tiling);       

        #if defined(WATER_BLEND) || defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)

            #if defined(NO_FLOW)
                vec4 flow_1 = texture2D(s_cc3Texture2Ds[2], v_flow_uv1);
                vec4 flow_2 = texture2D(s_cc3Texture2Ds[2], v_flow_uv2);
                vec4 diffuse3 = (flow_1 * 0.5) + (flow_2 * 0.5);
            #else
                vec4 flow_1 = texture2D(s_cc3Texture2Ds[2], v_flow_uv1);   
                vec4 flow_2 = texture2D(s_cc3Texture2Ds[2], v_flow_uv2);
                vec4 diffuse3 = vec4(mix(flow_1.rgb, flow_2.rgb, v_flow_blend), 1.0);
            #endif

        #else
            vec4 diffuse3 = texture2D(s_cc3Texture2Ds[2], v_texCoord * UV3Tiling);
        #endif

        #if defined(WORLD_LAVA_BLEND) || defined(LAVA_BLEND)
        #else
            #if defined(BLEND_REFLECT) || defined(REFLECT_CUBE)
                #if defined(REFLECT_CUBE)
                    vec3 nN = vec3(N.r, N.g, N.b * diffuse1.a);
                    vec3 R = reflect(-V, nN);
                    vec3 reflection = textureCube(nebulaMap, R).rgb;
                #endif
            

                diffuse1.rgb = mix(diffuse1.rgb + (reflection * ReflectBlends.x * (diffuse1.a * diffuse1.a)), Diffuse1Color.rgb, mask.b * 0.5);
                diffuse2.rgb = mix(diffuse2.rgb + (reflection * ReflectBlends.y * (diffuse2.a * diffuse2.a)), Diffuse2Color.rgb, mask.b * 0.5);
                diffuse3.rgb = mix(diffuse3.rgb + (reflection * ReflectBlends.z * (diffuse3.a * diffuse3.a)), Diffuse3Color.rgb, mask.b * 0.5);

            #else
                diffuse1.rgb = mix(diffuse1.rgb, Diffuse1Color.rgb, mask.b * 0.5);
                diffuse2.rgb = mix(diffuse2.rgb, Diffuse2Color.rgb, mask.b * 0.5);
                diffuse3.rgb = mix(diffuse3.rgb, Diffuse3Color.rgb, mask.b * 0.5);

            #endif
        #endif

        // Blend 1 //
        float height1 = (0.5 - diffuse1.a) * Height1Influence;
        float height2 = (diffuse2.a - 0.5) * Height2Influence;
        float blend1 = clamp(height1 + height2 + (mask.r - 0.5) * MaskInfluence + 0.5 + Shift, 0.0, 1.0);

        #if defined(WORLD_LAVA_BLEND) && defined(USE_NORMAL_MAP)
            vec3 diffuse = diffuse1.rgb;
            float layer_emissive = LayerEmissive[0];
            float height_prev = height1;
        #else
            vec3 diffuse = mix(diffuse1.rgb, diffuse2.rgb, blend1);
            float layer_emissive = mix(LayerEmissive[0], LayerEmissive[1], blend1);    
            float height_prev = mix(height1, (0.5 - diffuse2.a) * Height2Influence, blend1);               
        #endif

        // Blend 2 //
        float height3 = (diffuse3.a - 0.5) * Height3Influence;  
        float blend2 = clamp(height_prev + height3 + (mask.g - 0.5) * MaskInfluence + 0.5 + Shift, 0.0, 1.0);
        layer_emissive = mix(layer_emissive, LayerEmissive[2], blend2);   

        #if defined(WATER_BLEND)
            float edge = (1.0 - blend2) * blend2;
            vec3 flow_n = normalize((diffuse3.xyz * 2.0 - 1.0) * vec3(1.0, 1.0, 2.0)).xzy;

            diffuse *= clamp(1.0 - flow_n.y * blend2 * 1.0, 0.0, 1.0);
            flow_n.xz *= 1.0 * blend2;
            float gloss = mix(2.0, 500.0, blend2);
            float spec_up = clamp(dot(normalize(flow_n + vec3(-0.4, 0, 0.9)), V), 0.0, 1.0);
            vec3 spec = 1.0 * pow(spec_up, gloss) * vec3(0.8, 1.0, 1.0) * blend2;

            vec3 R = reflect(-V, normalize(flow_n));
            vec3 environment = textureColor.a * textureCube(u_reflectionMap, R).rgb;
            environment *= (1.0 - clamp(dot(normalize(flow_n), V), 0.0, 1.0)) * 1.0;
            
            diffuse *= mix(vec3(1.0), WaterShallowColor.rgb * WaterShallowMultiplier, edge);
            diffuse = mix(diffuse, diffuse + WaterDeepColor.rgb + spec + environment, blend2);
        #elif defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)

            float edge = (1.0 - blend2) * blend2;

            #if defined(LAVA_BLEND) || defined(WORLD_LAVA_BLEND)
                vec2 lookupCoord = v_texCoord;
                #if defined(WORLD_LAVA_BLEND)
                    lookupCoord = v_lightMapCoord;
                #endif

                float u_lookup = sin(diffuse3.r) * 0.95;// * (1.0 - edge) * (1.0 - lookupCoord.y));
                //float u_lookup = (diffuse3.r * (1.0 - edge) * (1.0 - lookupCoord.y));
                #if defined(WORLD_LAVA_BLEND)
                    vec3 lava_color = texture2D(s_cc3Texture2Ds[5], vec2(u_lookup, 0.0)).rgb;
                #else
                    vec3 lava_color = texture2D(s_cc3Texture2Ds[4], vec2(u_lookup, 0.0)).rgb;
                #endif

            #endif


            diffuse = mix(diffuse, lava_color, blend2);
            //self_illum_mask
        #elif defined(REFLECT_CUBE)

            diffuse = mix(diffuse, diffuse3.rgb, blend2);
        #else
            diffuse = mix(diffuse, diffuse3.rgb, blend2);
        #endif


        gl_FragColor.rgb = diffuse;

        #if defined(REFLECT_CUBE) && defined(BLEND_ALPHA)
            float frnl = dot(V, N);
            gl_FragColor.rgb = mix(diffuse, Diffuse3Color.rgb, frnl);
            gl_FragColor.a = clamp(ReflectBlends.z + frnl, 0.0, 1.0);//0.75;//mix(1.0, v_vertexColor.b, v_vertexColor.g);
        #else
            gl_FragColor.a = 1.0;
        #endif
    #else
        gl_FragColor = diffuseColor;
    #endif

    float region_height_mask = 0.0;

    #if defined(WORLD_LAVA_BLEND) && defined(USE_SPEC) && defined(USE_NORMAL_MAP)
        float NdotV = max(0.0, dot(normal, V));
        gl_FragColor.rgb += pow(NdotV, SpecularPower) * SpecularColor;
    #endif

    #if defined(WORLD) || defined(WORLD_CLOUDS) || defined(WORLDREGIONONLY)
 
        vec3 c = cos(WorldRotation * M_PI/180.0);
        vec3 s = sin(WorldRotation * M_PI/180.0);

        //rotation matrix
        mat3 rotMatrix = mat3(
        (c.y * c.z) + (s.x * s.y * s.z),
        c.x * s.z, 
        (c.y * s.x * s.z) - (c.z * s.y),

        (c.z * s.x * s.y) - (c.y * s.z),
        c.x * c.z,
        (c.y * c.z * s.x) + (s.y * s.z),

        c.x * s.y,
        -s.x,
        c.x * c.y
        );


        // World UVs //
        vec3 dir = normalize(v_position_world - WorldPosition);
        dir = rotMatrix * dir;

        float u = atan(dir.z, dir.x) * 0.5 / M_PI + 0.25;
        float v = dir.y * 0.35 + 0.5;
        vec2 world_uv = vec2(u * -1.0, v);
        vec3 binormal = cross(normalize(v_N), vec3(0.0, 1.0, 0.0));
        vec3 tangent = cross(normalize(v_N), binormal);      
        float world_radius = length(v_position_world);  

       // Lighting //
        vec3 adjustedNormal = v_N;
        #if defined(WORLD_LAVA_BLEND) && defined(USE_NORMAL_MAP)
            adjustedNormal = N;
        #endif
        vec3 center_dir = normalize(v_position_world);
        float center_value = dot(normalize(adjustedNormal), center_dir) * 0.5 + 0.5;
        float light_value = pow(clamp(dot(normalize(adjustedNormal), normalize(v_world_light_dir)) * 0.5 + 0.5, 0.0, 1.0), 1.0);

        vec3 lighting = mix(EnvironmentConfig.WorldAmbientColor, EnvironmentConfig.WorldDiffuseColor, light_value);
        //lighting *= mix(0.1, 1.0, center_value);

        // Region Mask //
        vec4 region_tex = texture2D(s_cc3Texture2Ds[region_map_index], world_uv);
        float region_mask = min(1.0, region_tex.r + region_height_mask);

        // Region Reveal //
        vec4 region_reveal_tex = texture2D(s_cc3Texture2Ds[region_map_index + 1], world_uv);     
        vec3 region_reveal_masks = GlobalUniform_RegionMaskChannel * region_reveal_tex.rgb;
        float region_reveal_mask = region_reveal_masks.r + region_reveal_masks.g + region_reveal_masks.b;
        float region_reveal = pow(((u_cc3ObjectTime[int(INSTANCE_ID)] - RegionStartTime) / RegionDuration), 1.0) * RegionDuration + region_reveal_mask * 2.0 - RegionDuration;
        region_reveal = region_reveal * RegionReveal;
        region_mask = clamp(region_mask + clamp(region_reveal, 0.0, region_reveal_mask), 0.0, 1.0);
        region_mask = (1.0 - region_mask) * RegionMaskMultiplier;

    #endif



    #if defined(WORLD)
        // Cloud Shadows //
        vec4 clouds_tex = texture2D(s_cc3Texture2Ds[cloud_map_index], world_uv * 1.0  + vec2(0.00, 0.04) - vec2(0.005, 0.0) * getSceneTime());
        vec4 clouds_tex2 = texture2D(s_cc3Texture2Ds[cloud_map_index], world_uv * 1.0  + vec2(0.00, 0.04) - vec2(0.0, 0.0) * getSceneTime());
        clouds_tex = (clouds_tex - 0.5 + clouds_tex2 + 0.5);
        clouds_tex = abs(fract(clouds_tex + vec4(0.1, 0.2, 0.3, 0.4) * 0.3 * getSceneTime()) - 0.5) * 2.0;

        float cloud_poles = 1.0 - clamp((abs(v_position_world.y) - 3800.0) / 1000.0, 0.0, 1.0);
        float clouds = (clouds_tex.r + clouds_tex.g + clouds_tex.b) / 3.0;
        clouds = clamp((clouds - 0.6) * 1.6 * cloud_poles, 0.0, 1.0);
        float cloud_height = max(0.0, world_radius - 7000.0) / 20.0;    
    #endif

    #ifdef WORLD
        float height_blend = max(0.0, world_radius - 5000.0) / 500.0;
        float water_edge = 1.0 - clamp(abs(world_radius - 4750.0) / 50.0, 0.0, 1.0);
        //gl_FragColor.rgb *= vec3(water_edge * 1.0 + 1.0);
        #if !defined(LAVA_BLEND) &&  !defined(WORLD_LAVA_BLEND)
            //gl_FragColor.rgb *= mix(1.0, 0.7, clouds);
        #endif
        gl_FragColor.a *= getMaterialColor().a;
    #endif

    #ifdef WORLD_CLOUDS
        // Clouds //      
        vec4 clouds_tex = texture2D(s_cc3Texture2Ds[cloud_map_index], world_uv * 1.0 - vec2(0.0025, 0.0) * getSceneTime());
        vec4 clouds_tex2 = texture2D(s_cc3Texture2Ds[cloud_map_index], world_uv * 1.0 - vec2(0.0, 0.0) * getSceneTime());
        clouds_tex = (clouds_tex - 0.5 + clouds_tex2 + 0.5);
        clouds_tex = abs(fract(clouds_tex + vec4(0.1, 0.2, 0.3, 0.4) * 0.1 * getSceneTime()) - 0.5) * 2.0;

        float clouds = (clouds_tex.r + clouds_tex.g + clouds_tex.b) / 3.0;
        float cloud_poles = 1.0 - clamp((abs(v_position_world.y) - 5000.0) / 1000.0, 0.0, 1.0);
        clouds = clamp((clouds - 0.6) * 1.6 * cloud_poles, 0.0, 1.0);
        float cloud_height = max(0.0, world_radius - 7000.0) / 20.0;  
        float clouds_fresnel = pow(1.0 - abs(dot(V, normalize(v_N))), 3.0);      
        gl_FragColor.a = max(0.0, clouds - clouds_fresnel * 2.0) * 1.0;
        gl_FragColor.rgb = lighting * gl_FragColor.a * 0.7;
    #endif

    #ifdef WORLD_REGION
        vec4 region_tex = texture2D(s_cc3Texture2Ds[0], v_texCoord);
        vec4 material_color = getMaterialColor();
        vec3 region_color = vec3(region_tex.r) * material_color.rgb;
        gl_FragColor.rgb = region_color.rgb;

        float region_fresnel = pow(1.0 - max(0.0, dot(V, normalize(v_N))), RegionFresnelPower);      
        gl_FragColor.a = region_tex.a * mix(RegionFacingAlpha, RegionEdgeAlpha, region_fresnel);

        #if defined(WORLD_REGION_PULSE)
            vec3 pulseColor = vec3(RegionBorderPulseValue * region_tex.g) + (RegionPulseColor.rgb * vec3(region_tex.r));
            gl_FragColor.rgb = pulseColor;
            gl_FragColor.a *= RegionPulseColor.a;
        #endif    
    #endif


    #if defined(WORLD) || defined(WORLDREGIONONLY)
        // Scan Lines //
        //float scan_fresnel = clamp(pow(1.0 - max(0.0, dot(V, normalize(v_N))), 1.5), 0.0, 1.0);
        //vec3 scan_axis = abs(fract(v_position_world.xyz / 2000.0 + getSceneTime() * 0.6) - 0.5) * 2.0;
        //scan_axis = (scan_axis - 0.9) * 10.0;  
        //float scan_lines = max(0.0, scan_axis.x) + max(0.0, scan_axis.y) + max(0.0, scan_axis.z);
        //scan_lines = pow(scan_lines, 3.0);
        //scan_fresnel = clamp((scan_fresnel - 0.8) * 10.0, 0.0, 1.0);
        //float scan_pulse = pow(abs(fract(getSceneTime() * 1.0) - 0.5) * 2.0, 2.0);
        #if (defined(WORLD) || defined(WORLDREGIONONLY)) && !defined(PRELIT)
            gl_FragColor.rgb *= lighting;
        #endif

        //region darkening
        float regionDarkValue = EnvironmentConfig.RegionDarkColor.a;
        vec3 regionDarkColor = EnvironmentConfig.RegionDarkColor.rgb;
        float regionMask = region_mask;


        float maxDarkenValue = (1.0 - RegionFogBias) * regionDarkValue;
        gl_FragColor.rgb = mix(gl_FragColor.rgb, regionDarkColor, regionMask * maxDarkenValue);


        //gl_FragColor.rgb *= 1.0 + scan_fresnel * 1.0;
        //gl_FragColor += scan_lines;
        //gl_FragColor.rgb *= 1.0 + scan_pulse * 1.0 * vec3(2.0, 1.0, 0.0);
        //gl_FragColor.rgb = region_tex.rgb;

        // Depth Fog //
        float fog_depth = length(v_position_world - getCameraPositionWorld());

        float fog_blend = clamp((fog_depth - EnvironmentConfig.FogNearFarMax.x) / EnvironmentConfig.FogNearFarMax.y, 0.0, 1.0);

        gl_FragColor.rgb = mix(gl_FragColor.rgb, EnvironmentConfig.FogColor.rgb, fog_blend * EnvironmentConfig.FogNearFarMax.z);


    #endif

    #ifdef WORLD_DETAIL
        vec4 mask = v_vertexColor; 

        // Diffuse //
        vec4 detail_tex1 = texture2D(s_cc3Texture2Ds[0], v_texCoord);
        vec4 detail_tex2 = texture2D(s_cc3Texture2Ds[1], v_lightMapCoord);

        // Blend //
        float height1 = (0.5 - detail_tex1.a) * DetailLayer1Influence;
        float height2 = (detail_tex2.a - 0.5) * DetailLayer2Influence;
        float blend1 = clamp(height1 + height2 + (mask.r - 0.5) * MaskInfluence + 0.5 + Shift, 0.0, 1.0);
        vec3 detail_tex = mix(detail_tex1.rgb, detail_tex2.rgb, blend1);

        // Normals //
        vec4 detail_normal1 = texture2D(s_cc3Texture2Ds[2], v_texCoord);
        vec4 detail_normal2 = texture2D(s_cc3Texture2Ds[3], v_lightMapCoord);

        vec4 detail_normal = mix(detail_normal1, detail_normal2, blend1);
        float detail_smoothness = detail_normal.a;
        vec3 detail_n = normalize(detail_normal.xyz * 2.0 - 1.0);
        vec3 N = normalize(mat3(v_T, v_B, v_N) * detail_n);

        // Specular //
        float detail_gloss = mix(DetailLayer1Gloss, DetailLayer2Gloss, blend1);
        float spec_up = clamp(dot(N, V), 0.0, 1.0);
        float spec = pow(spec_up, detail_gloss) * mix(DetailLayer1Specular, DetailLayer2Specular, blend1);

        // Reflection //
        vec3 R = reflect(-V, N);
        vec3 environment = textureCube(nebulaMap, R).rgb * mix(DetailLayer1Reflection, DetailLayer2Reflection, blend1);

        // Fresnel //
        float fresnel = max(0.0, dot(V, normalize(N)));
        fresnel = pow(1.0 - fresnel, mix(DetailLayer1Fresnel, DetailLayer2Fresnel, blend1));
        vec3 detail_layer1_color = mix(DetailLayer1FrontColor.rgb, DetailLayer1SideColor.rgb, fresnel);
        vec3 detail_layer2_color = mix(DetailLayer2FrontColor.rgb, DetailLayer2SideColor.rgb, fresnel);
        vec3 detail_color = mix(detail_layer1_color, detail_layer2_color, blend1);

        // Sphere Map //
        //vec2 sphere_uv = (v_world_normal_dir).xy * vec2(0.5) + 0.5;
        //vec4 sphere_tex = texture2D(s_cc3Texture2Ds[3], sphere_uv);
        //gl_FragColor.rgb = mix(vec3(0.5, 0.4, 0.3), vec3(1.0), sphere_tex.rgb) * 2.0;

        gl_FragColor.rgb = detail_tex.rgb * detail_color + spec + environment;
    #endif

    #if defined(WORLD_AO) && !defined(PRELIT)
        // Ambient Occlusion //
        vec2 lmCoords = v_lightMapCoord;
        #if defined(WORLD_LAVA_BLEND) //use lightmap?
            lmCoords = v_texCoord;
            vec4 light_tex = texture2D(s_cc3Texture2Ds[6], lmCoords);
            #if !defined(USE_NORMAL_MAP)
                gl_FragColor.rgb *= light_tex.rgb;   
            #endif
        #else
            vec4 light_tex = texture2D(s_cc3Texture2Ds[light_map_index], lmCoords);
            gl_FragColor.rgb *= pow(light_tex.rgb, vec3(2.0));   
        #endif

        

    #endif


    #if defined(WORLD) && defined(HEIGHT_BLEND)
        float fresnel = clamp(pow(1.0 - max(0.0, dot(V, normalize(v_N))), 1.0), 0.0, 1.0);
        //gl_FragColor.rgb *= mix(1.0, .9 + fresnel * 0.25, blend1);  
    #elif defined(WORLD)
        float fresnel = clamp(pow(1.0 - max(0.0, dot(V, normalize(v_N))), 1.0), 0.0, 1.0);
        //gl_FragColor.rgb *= .9 + fresnel * 0.25;  
    #endif

    #ifdef WORLD_ATMOSPHERE
        vec4 atmosphere_tex = texture2D(s_cc3Texture2Ds[0], v_texCoord * 1.0 - vec2(0.01, 0.0) * getSceneTime());
        float atmosphere_radius = length(v_position_local);
        float atmosphere_blend = (atmosphere_radius - InnerRadius) / (OuterRadius - InnerRadius);
        atmosphere_blend = pow(clamp(atmosphere_blend, 0.0, 1.0), 0.5) * 2.0 - 1.0;

        float inner_blend = (atmosphere_radius - InnerRadius) / (MiddleRadius - InnerRadius);
        inner_blend = pow(clamp(inner_blend, 0.0, 1.0), 3.0);
        float outer_blend = (atmosphere_radius - MiddleRadius) / (OuterRadius - MiddleRadius);
        outer_blend = pow(clamp(outer_blend, 0.0, 1.0), 0.3);
        
        vec4 atmosphere_color = mix(InnerColor, MiddleColor, inner_blend);
        atmosphere_color = mix(atmosphere_color, OuterColor, outer_blend);

        gl_FragColor = atmosphere_color * (0.9 + atmosphere_tex.r * 0.2);
        gl_FragColor.a *= getMaterialColor().a;

        // Depth //

        // float depth = texture2D(s_cc3Texture2Ds[1], gl_FragCoord.xy / vec2(2048.0, 1536.0) / 1.0 + vec2(0.0, 0.0)).x;
    
        // float zNear = 10.0;
        // float zFar = 200000.0;
        // float linear_depth =  (2.0 * zNear) / (zFar + zNear - depth * (zFar - zNear));
    
        // gl_FragColor.rgb = vec3(abs(linear_depth - v_position_world.z) * 10.0);
        // gl_FragColor.a = 0.0;

    #endif

    #ifdef USE_PROPERTY_MAP
        vec3 L = vec3(0.0, 1.0, 0.0);
        vec3 H = normalize(V - L);
        vec3 R = reflect(-V, N);
        float NdotV = max(0.0, dot(N, V));

        vec4 property_tex = texture2D(s_cc3Texture2Ds[2], v_texCoord);
        ApplyLighting(property_tex, gl_FragColor.rgb, v_position_local, N, V, R, NdotV, v_rim);
    #endif
    
    #ifdef MANTIS_PBR
        vec4 properties = texture2D(s_cc3Texture2Ds[2], v_texCoord);

        vec3 diffuseAlbedoSRGB = gl_FragColor.rgb;
        #if defined(PBR_ANIMATED_BLEND) && defined(COMMANDER)
            vec2 variantUVTile = vec2(4.0, 4.0);
            vec2 variantUVSpeed = vec2(0.001, 0.01);
            vec2 variantUV = v_texCoord * variantUVTile + (variantUVSpeed * vec2(u_cc3ObjectTime[int(INSTANCE_ID)]));
            vec2 blendUVTile = variantUVTile;//vec2(5.0, 5.0);
            vec2 blendUVSpeed = vec2(0.0, 0.015);
            vec2 blendUV = v_texCoord * blendUVTile + (blendUVSpeed * vec2(u_cc3ObjectTime[int(INSTANCE_ID)]));
            vec4 variant_diffuse = texture2D(s_cc3Texture2Ds[4], variantUV);
            vec4 variant_normal = texture2D(s_cc3Texture2Ds[5], variantUV);

            float blendAmount = texture2D(s_cc3Texture2Ds[6], blendUV).b;
            vec4 variant_material = texture2D(s_cc3Texture2Ds[6], variantUV);
            float materialMask = properties.b;
            blendAmount = blendAmount * (materialMask * materialMask * materialMask);
            diffuseAlbedoSRGB = mix(diffuseAlbedoSRGB.rgb, variant_diffuse.rgb, blendAmount);
            variant_material.b = properties.b;
            properties.rgb = mix(properties.rgb, variant_material.rgb, blendAmount);

            vec3 xnormal_t = vec3(variant_normal.r, variant_normal.g, variant_normal.b * blendAmount) * 2.0 - 1.0;
            vec3 xN = normalize(mat3(v_T, v_B, v_N) * xnormal_t);
            N = mix(N, xN, blendAmount);
        #endif
        vec3 diffuseIBL = GetPBRLighting(N);
        vec3 pbrLighting = ApplyPBRLighting(diffuseIBL, properties, diffuseAlbedoSRGB, v_position_local, N, V, GlossBoost);
        gl_FragColor.rgb = pbrLighting;

        #if defined(COMMANDER)

            vec3 n = normalize(mat3(getWorldToCamera()) * reflect(N, V));
            vec2 sphere_uv = vec2(0.5, 0.5) + 0.5 * vec2(n.x, n.y);

            //matcap rim
            vec3 mask_tex = texture2D(s_cc3Texture2Ds[3], sphere_uv).rgb;
            float lighting_rim_mask = mask_tex.r;
            float color_rim_mask = mask_tex.g;
            float user_mask = mask_tex.b;

            vec3 foreground = RimFillColor.rgb * color_rim_mask;
            
            foreground += diffuseIBL.rgb * lighting_rim_mask;

            vec3 customRim = RimCustomColor.rgb * user_mask;

            foreground += customRim; //old custom rim application

            //rim mask from AO property map - use diffuse alpha later for more flexibility - when all converted, can be deprecated
            foreground *= mix(properties.b, diffuseColor.a, RimMaskUsage);

            foreground *= RimMultiplier;



            vec3 background = mix(gl_FragColor.rgb, FadeColor.rgb, FadeColor.a);

            //additive
            gl_FragColor.rgb = foreground + background;

            //screen
            //gl_FragColor.rgb = 1.0-(1.0-foreground) * (1.0-background);

            //rim only - testing
            //gl_FragColor.rgb = foreground;

            //lighting only
            //gl_FragColor.rgb = pbrLighting.rgb;



        foreground = mix(vec3(0.5), RimCustomColor.rgb * 1.0, user_mask);
        background = gl_FragColor.rgb;
        
        //overlay
        vec3 LightmapTuning = vec3(2.0, 2.0, 0.5);
        vec3 mult = step(background, vec3(LightmapTuning.z)); 
        vec3 multOp = vec3(LightmapTuning.x) * foreground * background;

        vec3 screenOp = vec3(1.0) - vec3(LightmapTuning.y) * (vec3(1.0)-foreground) * (vec3(1.0)-background);       
        //gl_FragColor.rgb = ((vec3(1.0)-mult) * screenOp) + (mult * multOp);




            //none
            //gl_FragColor.rgb = background;

            //normal map
            //gl_FragColor.rgb = texture2D(s_cc3Texture2Ds[1], v_texCoord).rgb;
        #endif
    #endif

    #if defined(USE_LIGHT) && defined(TILED_LIGHTING)
        gl_FragColor.rgb = LinearToSRGB(SRGBToLinear(gl_FragColor.rgb) + CalculateTiledLighting(v_position_world, vec3(0.0f, 1.0f, 0.0f)));
    #endif

    #if defined(PRELIT)

        #if defined(LM01)
            vec4 illumination_map = texture2D(s_cc3Texture2Ds[1], v_lightMapCoord);
        #elif defined(LM05)
            vec4 illumination_map = texture2D(s_cc3Texture2Ds[5], v_lightMapCoord);
        #else
            vec4 illumination_map = texture2D(s_cc3Texture2Ds[4], v_lightMapCoord);
        #endif
       

        //lightmap illum mask - alpha
        vec3 luminance = vec3(dot(gl_FragColor.rgb, vec3(0.299, 0.587, 0.114)));
        vec3 luminanceLM = vec3(dot(illumination_map.rgb, vec3(0.299, 0.587, 0.114)));
        vec3 LightmapTuning = vec3(1.0, 1.0, 1.0);

        //Simon's Special Blend: softlight with alpha illum control
        vec3 valueA = (vec3(2.0) * illumination_map.rgb - vec3(1.0)) * (gl_FragColor.rgb - (gl_FragColor.rgb * gl_FragColor.rgb)) + gl_FragColor.rgb;
        vec3 valueB = (vec3(2.0) * illumination_map.rgb - vec3(1.0)) * (vec3(sqrt(gl_FragColor.rgb)) - gl_FragColor.rgb) + gl_FragColor.rgb;
        //gl_FragColor.rgb = mix(valueA, valueB, illumination_map.a);

        //photoshop-style blends

        vec3 foreground = illumination_map.rgb;
        vec3 background = gl_FragColor.rgb;

        //calibration
        //background = vec3(0.5);

        //soft light
        //gl_FragColor.rgb = BlendSoftLight(background, foreground);

        //screen - need to use different map
        //gl_FragColor.rgb = vec3(1.0)-(vec3(1.0)-foreground) * (vec3(1.0)-background);

        //pure multiply
        gl_FragColor.rgb *= illumination_map.rgb;
        
        //overlay
        LightmapTuning = vec3(2.0, 2.0, 0.5);
        vec3 mult = step(background, vec3(LightmapTuning.z)); 
        vec3 multOp = vec3(LightmapTuning.x) * foreground * background;
//ShadowColor.a
        vec3 screenOp = vec3(1.0) - vec3(LightmapTuning.y) * (vec3(1.0)-foreground) * (vec3(1.0)-background);       
        //gl_FragColor.rgb = ((vec3(1.0)-mult) * screenOp) + (mult * multOp);


        //Simon's Special Light Blend
        //x(1.0): highlights amount y(1.0): shadows amount z(1.0): lm curve - lower values skew towards more shadow
        LightmapTuning = vec3(4.0, 0.65, 1.0);
        float LightSaturation = 4.0;
        foreground = pow(foreground.rgb, vec3(LightSaturation));
        float light_shadow_curve = 1.0 - pow(luminanceLM.x, LightmapTuning.z);
        vec3 additive_light = mix(vec3(0.0), foreground, LightmapTuning.x);
        vec3 multiply_light = mix(vec3(1.0), foreground, LightmapTuning.y);

        background *= mix(vec3(1.0), multiply_light, light_shadow_curve);

        //add method
        background += mix(vec3(0.0), additive_light, light_shadow_curve);

        //screen method
        //background = vec3(1.0)-(vec3(1.0)-additive_light) * (vec3(1.0)-background);

        //gl_FragColor.rgb = background;

        //shadow color

        float shadow_amount = luminanceLM.x;
        shadow_amount = pow(1.0 - shadow_amount, 4.0);

        float shadow_power = 1.0;//ShadowColor.a

        //gl_FragColor.rgb += mix(vec3(0.0), vec3(shadow_amount) * ShadowColor.rgb, shadow_power);

        //gl_FragColor.rgb += (vec3(shadow_amount) * ShadowColor.rgb);


//testing
        //gl_FragColor.rgb = foreground;


//illumination_map.a = 0.25;
//temp disable overbright
        //gl_FragColor.rgb = mix(gl_FragColor.rgb, gl_FragColor.rgb + (luminance * illumination_map.rgb), illumination_map.a);

    #endif

    #if defined(USE_LIGHT) && !defined(USE_PROPERTY_MAP) && !defined(MANTIS_PBR) && !defined(WORLD)
        float up = N.y;
        vec3 light_weights = vec3(max(0.0, up), 1.0 - abs(up), max(0.0, -up));

        #if defined(BORDER) || defined(VERTPUSH_BORDER) || defined(NO_BORDER)
            vec3 topLight = GlobalUniform_LightCharTop;
            vec3 sideLight = GlobalUniform_LightCharSide;
            vec3 bottomLight = GlobalUniform_LightCharBottom;
        #else
            // temporary assignment until we can breakout env light settings
            vec3 topLight = EnvironmentConfig.LightTop;
            vec3 sideLight = EnvironmentConfig.LightSide;
            vec3 bottomLight = EnvironmentConfig.LightBottom;            
            //vec3 topLight = LightEnvTop;
            //vec3 sideLight = LightEnvSide;
            //vec3 bottomLight = LightEnvBottom;
        #endif

		#if defined(LEVEL_ED_RENDERCORE_COMPATIBILITY_MODE)
			// DavidJ -- LEVEL_ED_RENDERCORE_COMPATIBILITY_MODE is enabled when we're using the pre-rendercore level editor
			//			with the post-rendercore shaders and data.
			//			In rendercore, some the uniform names for these lights changed. So the new shaders no longer work
			//			with the old level editor. As a temporary fix, let's substitute in hard coded defaults. 
			topLight = vec3(1.0, 1.0, 1.0);
			sideLight = vec3(1.0, 1.0, 1.0);
			bottomLight = vec3(1.0, 1.0, 1.0);
		#endif

        vec3 light = topLight * light_weights.x + sideLight * light_weights.y + bottomLight * light_weights.z;

        // vec3 light_dir = normalize(vec3(0.0, 1.0, 1.0));
        // vec3 light_diffuse = max(0.0, dot(N, light_dir)) * vec3(1.3, 1.0, 0.7);
        // light += light_diffuse;

        #ifdef HEIGHT_BLEND
            light = mix(light, vec3(1.0), layer_emissive);

            #if defined(PRELIT)//need to integrate with ao bake on volcano?
                //vec4 illumination_map = texture2D(s_cc3Texture2Ds[4], v_lightMapCoord);
                //gl_FragColor.rgb *= illumination_map.rgb;  
            #else
                gl_FragColor.rgb *= light;  
            #endif
        #else
            
            float NdotV = max(0.0, dot(N, V));
            float gloss = pow(2.0, 1.0 + 9.0 * EnvironmentConfig.SpecularGlossStrength.x);
            



            #if defined(REFLECT)

                float light_specular = pow(NdotV, SpecPower);
            
                float spec_mask = reflMask * reflMask;

                gl_FragColor.rgb = mix(vec3(1.0 + (light_specular * SpecMult * spec_mask * v_vertexColor.r) * (1.0 - reflMask)) * gl_FragColor.rgb, gl_FragColor.rgb * (0.5 + (reflection * ReflectionValue) * reflMask), reflMask);



                #if defined(OVERLAY)
                    float overlay1Blend = clamp(v_vertexColor.g * v_vertexColor.g * overlay1Mask * 4.0, 0.0, 1.0);
                    float overlay2Blend = clamp(v_vertexColor.b * v_vertexColor.b * overlay2Mask * 4.0, 0.0, 1.0);
                    
                    gl_FragColor.rgb = mix(gl_FragColor.rgb, Overlay2Color.rgb, overlay2Blend);
                    gl_FragColor.rgb = mix(gl_FragColor.rgb, Overlay1Color.rgb, overlay1Blend);
                    gl_FragColor.rgb += mix(vec3(0.0), vec3(light_specular * 0.5 * spec_mask), clamp(overlay1Blend + overlay1Blend, 0.0, 1.0));

                #else

                #endif


                


                #if defined(PRELIT)
                    //light = mix(light, SelfIllumination + diffuseColor.rgb, clamp(SelfIllumination, 0.0, 1.0));

                #else
                    light = mix(light, SelfIllumination + diffuseColor.rgb, clamp(SelfIllumination, 0.0, 1.0));
                    gl_FragColor.rgb *= light;   
                #endif

            #elif (defined(BORDER) || defined(NO_BORDER)) && defined(USE_LIGHT)
                vec3 VPAdj = (getCameraPositionWorld() - v_position_world);

                vec3 SPEC_TUNING_TEMP = vec3(-0.25, 0.5, 0.0); 

                VPAdj = normalize(VPAdj) + (SPEC_TUNING_TEMP * mat3(getWorldToCamera())); 

                float default_SpecPower = 100.0;
                float default_spec_intensity = 1.5;
                vec3 default_spec_color = vec3(1.0, 1.0, 1.0);

                float NdotVAdj = max(0.0, dot(N, normalize(VPAdj)));

                float SpecPower = SpecularPower;
                SpecPower = mix(default_SpecPower, SpecPower, SpecularCustomUsage); 

                float spec_intensity = SpecularIntensity;
                spec_intensity = mix(default_spec_intensity, spec_intensity, SpecularCustomUsage); 

                vec3 spec_color = SpecularColor.rgb;
                spec_color = mix(default_spec_color, spec_color, SpecularCustomUsage); 

                
                #if defined(SPEC_LIGHT)
                    vec3 spec_reflect = reflect(normalize(v_world_light_dir), N);
                #else
                    vec3 spec_reflect = reflect(normalize(GlobalUniform_SpecularLightDir), N);
                #endif

                float NdotL = clamp(dot(-spec_reflect, V), 0.0, 1.0);
                NdotL = mix(NdotVAdj, NdotL, SpecularLightAlign);

                float light_specular = pow(NdotL, SpecPower);

                float spec_mask = diffuseColor.a;
                spec_mask = mix(1.0, spec_mask, SpecularCustomUsage); 

                gl_FragColor.rgb *= light;
                //gl_FragColor.rgb = mix(gl_FragColor.rgb, spec_color, light_specular);//testing for spec shape and pos tuning 
                gl_FragColor.rgb += vec3((light_specular * spec_intensity * spec_mask * spec_color));


            #elif !defined(REFLECT_CUBE) && !defined(PRELIT)
                float light_specular = pow(NdotV, gloss) * EnvironmentConfig.SpecularGlossStrength.y;
                gl_FragColor.rgb *= light;   
                gl_FragColor.rgb *= 1.0 + light_specular;
            #endif
        #endif
    #endif

    #if defined(LIGHTMAP_ENABLED) && !defined(REFLECT_CUBE) && !defined(PRELIT) && !defined(LAVA_BLEND)
        #if defined(USE_NORMAL_MAP) && !defined(WORLD_LAVA_BLEND)
            const int lightmapTextureIndex = 2;
        #elif defined(COMPOSITE)
            #if defined(OVERLAY)
                const int lightmapTextureIndex = 1;
            #else
                const int lightmapTextureIndex = 4;
            #endif
        #elif defined(WORLD_LAVA_BLEND)
            const int lightmapTextureIndex = 6;
        #else
            const int lightmapTextureIndex = 1;
        #endif
        vec4 lightMapTexel = texture2D(s_cc3Texture2Ds[lightmapTextureIndex], v_lightMapCoord);
        gl_FragColor = gl_FragColor * lightMapTexel;
    #endif

    #ifdef USE_VERTEX_COLOR
        gl_FragColor *= v_vertexColor;
    #elif defined(WORLD_REGION)
        gl_FragColor.a *= v_vertexColor.a;
    #endif

    #if defined(ENVIRONMENT) && !defined(MANTIS_PBR)          // (note -- MANTIS_PBR already has environmental reflections built in)
        vec3 R = reflect(-V, N);
        vec3 environment = textureColor.a * textureCube(u_reflectionMap, R).rgb;
        gl_FragColor.rgb += environment;
    #endif
        
    #if defined(FRESNEL_ENABLED) && !defined(MANTIS_PBR)      // (note -- MANTIS_PBR already has fresnel effect built in)
        float nDotVInv = 1.0 - max(0.0, dot(N, V));
        float fresnelLevel = nDotVInv;
        fresnelLevel = pow(fresnelLevel, fresnelFalloff);
        float fresnelLerp = fresnelLevel * fresnelColor.a;
        // add fresnel
        gl_FragColor.rgb = mix(gl_FragColor.rgb, fresnelColor.rgb, fresnelLerp);
    #endif

    #ifdef SHIELD2
        vec4 shield_tex = texture2D(s_cc3Texture2Ds[1], v_texCoord);
        float fresnel = pow(1.0 - max(0.0, dot(N, V)), fresnelFalloff);
        gl_FragColor.rgb *= 1.0 + shield_tex.r * ShieldBorderAmount;
        gl_FragColor.rgb = mix(gl_FragColor.rgb * (1.0 - fresnel * 0.5), gl_FragColor.rgb + fresnelColor.rgb * fresnel * 1.4 - 0.1, gl_FragColor.a);
    
        float radius = 1.0 - min(1.0, length(v_position_local.xy - vec2(0.0, 600.0)) / 5500.0);
        gl_FragColor.rgb *= vec3(radius);
    
        vec2 shield_uv = v_position_world.zy * ShieldScale;
        float shield_cloud = texture2D(s_cc3Texture2Ds[2], shield_uv + getSceneTime() * ShieldScroll).x - texture2D(s_cc3Texture2Ds[2], shield_uv - getSceneTime() * ShieldScroll + vec2(0.5, 0.0)).x;
        shield_cloud = 1.0 - abs(fract(shield_cloud * 2.0) - 0.5) * 2.0;
        shield_cloud = pow(shield_cloud, ShieldPower);
        gl_FragColor.rgb *= 1.0 + shield_cloud * 4.0 * (1.0 - gl_FragColor.a * 0.9);

        float edge = clamp((radius - 1.0 + ShieldFade) * 50.0, 0.0, 1.0);
        gl_FragColor.rgb *= 1.0 + edge * (1.0 - edge) * 20.0;
        gl_FragColor.rgb *= edge;

    #endif

    #if defined(SHIELD)
        vec4 shield_multi_tex = texture2D(s_cc3Texture2Ds[0], v_texCoord);
        float shield_diffuse = shield_multi_tex.r;
        float shield_mask = shield_multi_tex.g;
        float shield_alpha = shield_multi_tex.b;


        float fresnel = pow(1.0 - max(0.0, dot(N, V)), fresnelFalloff);
        vec3 shield_color = mix(getMaterialColor().rgb, ColorFadeOverride.rgb, clamp(ColorFadeOverride.a, 0.0, 1.0));
        //shield_color = vec3(1.0, 0.5, 1.0);
        gl_FragColor.rgb = shield_color * vec3(shield_diffuse);

        gl_FragColor.rgb *= 1.0 + vec3(shield_mask) * ShieldBorderAmount;
        gl_FragColor.rgb += fresnelColor.rgb * fresnel * 1.4 - 0.1;
    
        float radius = 1.0 - min(1.0, length(v_position_local.xy - vec2(0.0, 600.0)) / 5500.0);
        gl_FragColor.rgb *= vec3(radius);
    
        vec2 shield_uv = v_position_world.zy * ShieldScale;
        float shield_cloud = texture2D(s_cc3Texture2Ds[1], shield_uv + getSceneTime() * ShieldScroll).x - texture2D(s_cc3Texture2Ds[1], shield_uv - getSceneTime() * ShieldScroll + vec2(0.5, 0.0)).x;
        shield_cloud = 1.0 - abs(fract(shield_cloud * 2.0) - 0.5) * 2.0;
        shield_cloud = pow(shield_cloud, ShieldPower);
        gl_FragColor.rgb *= 1.0 + shield_cloud * 4.0 * (1.0 - shield_alpha * 0.9);

        float edge = clamp((radius - 1.0 + ShieldFade) * 50.0, 0.0, 1.0);
        float alpha = 1.0 + edge * (1.0 - edge) * 20.0;
        gl_FragColor.a = shield_alpha * alpha * edge;
    #endif

    #ifdef GRID
        // Wave Tick //
        float wave_phase = BattlePlacement_WaveTimePercentage;
        float wave_cooldown = 1.0 - max(0.0, wave_phase - 0.975) * 40.0;
        float wave_blast = 1.0 - abs(max(0.0, wave_phase - 0.95) * 20.0 - 0.5) * 2.0;
        wave_phase = pow(wave_phase, 4.0);
        wave_phase = wave_phase * wave_cooldown * 0.8 + 0.2;

        vec3 grid_tex = gl_FragColor.rgb;
        gl_FragColor.rgb = gl_FragColor.r * vec3(0.3, 0.9, 1.0);
    
        // Cells //
        float grid_cells = 0.0;
        for(int i=0; i < 8; i++)
        {
            float grid_cell = 1.0 - min(1.0, length(BattlePlacement_FactoryLocations[i] - v_position_world.xz) / 1200.0);
            grid_cells += grid_cell * grid_cell;
        }

        grid_cells *= grid_cells;
        gl_FragColor.rgb = mix(gl_FragColor.rgb, vec3(0.3, 0.9, 1.0) * 1.5, grid_cells * wave_phase);

        // Beams //
        vec2 beam_uv = v_position_world.xz * 0.00005;
        vec4 beam_tex = texture2D(s_cc3Texture2Ds[2], beam_uv * 2.0);
        vec4 beam = vec4(1.0) - abs(fract(beam_tex * 2.0 + grid_cells * -4.0 * wave_phase + getSceneTime() * vec4(1.2, 1.5, 1.9, 0.0)) - 0.5) * 2.0;
        beam.x = pow(beam.x, 8.0);
        beam.y = pow(beam.y, 8.0);
        beam.z = pow(beam.z, 8.0);

        float beams = min(1.0, beam.x + beam.y + beam.z);
        beams = pow(beams, 5.0);
        gl_FragColor.rgb *= 1.0 + beams * 3.0 * wave_phase * (0.5 + grid_cells + grid_tex.g);
        gl_FragColor.rgb += vec3(0.3, 0.9, 1.0) * wave_phase * wave_phase * (0.3 + 0.5 * wave_blast);
        gl_FragColor.a = 1.0;
    #endif

    #ifdef BLINK
        float blink = cos(getSceneTime() * 2.0 * M_PI * currentBlinkFrequency()) * 0.5 + 0.5;
        gl_FragColor.rgb = mix(gl_FragColor.rgb, gl_FragColor.rgb * currentBlinkIntensity() * currentBlinkColor(), blink);
        gl_FragColor.rgb = mix(currentBlinkColor() * blink, gl_FragColor.rgb, getMaterialColor().r);
    #endif

    #if defined(ANIMATED_TEX)
        vec3 multiTex = texture2D(s_cc3Texture2Ds[0], v_texCoord).rgb; //tex 0
        vec3 multiTexAnim = texture2D(s_cc3Texture2Ds[0], vec2(v_texCoord.x + (v_animCoord.x * Base2ScrollSpeed.x), v_texCoord.y + (v_animCoord.y * Base2ScrollSpeed.y))).rgb;

        float BaseOffsetY = BaseOffsets.y;
        float AlphaOffsetY = AlphaOffsets.y;



        #if defined(USE_PROGRESS)
            float thisProgress = (ProgressAB.x * GlobalUniform_ProgressA) + (ProgressAB.y * GlobalUniform_ProgressB);
            BaseOffsetY = BaseOffsetY + thisProgress;
            AlphaOffsetY = AlphaOffsetY + thisProgress;
        #endif

        vec3 animTexBase = texture2D(s_cc3Texture2Ds[1], vec2(v_texCoord2.x + BaseOffsets.x, v_texCoord2.y + BaseOffsetY)).rgb; //tex 1
        vec3 animTexAlpha = texture2D(s_cc3Texture2Ds[1], vec2(v_texCoord2.x + AlphaOffsets.x, v_texCoord2.y + AlphaOffsetY)).rgb;

        vec4 TintColor = getMaterialColor();

        vec3 finalColor = multiTex.r * TintColor.rgb * animTexBase * mix(1.0, multiTexAnim.g, Base2Blend) * Intensity;
        float finalAlpha = multiTex.b * animTexAlpha.r * TintColor.a;


        gl_FragColor = vec4(finalColor.r, finalColor.g, finalColor.b, finalAlpha);
    #endif

    #if defined(FOGOFWAR)

            vec4 u_war_battle_placement_properties_0XX = vec4( //top - dual lane
                -12800.0, //start
                1900.0, //length (X)
                -3900.0, //outer edge (Z)
                15.0 //center (Z)
                );

            vec4 u_war_battle_placement_properties_1XX = vec4( //bottom - dual lane
                -12800.0, //start
                -500.0, //length (X)
                3900.0, //outer edge (Z)
                -15.0 //center (Z)
                );

            float grid = 0.0;
            
            //to display a very large grid
            vec4 u_war_battle_placement_properties_XX = vec4( 
                -10000.0, //start
                20000.0, //length (X)
                -10000.0, //outer edge (Z)
                10000.0 //center (Z)
                );




            #if defined(GRID_ALWAYS_ON)
                grid += grid_render(v_N, u_war_battle_placement_properties_XX, 1.0, v_position_world);
            #else
                if(BattlePlacement_GridVisibility.x > 0.1)
                //if(1.0 > 0.0)
                {
                    grid += grid_render(v_N, BattlePlacement_WarBattlePlacementProperties0, BattlePlacement_BorderPulse0 + 1.0, v_position_world);
                }

                if(BattlePlacement_GridVisibility.y > 0.1)
                //if(1.0 > 0.0)
                {
                    grid += grid_render(v_N, BattlePlacement_WarBattlePlacementProperties1, BattlePlacement_BorderPulse1 + 1.0, v_position_world);
                }            
            #endif


            vec3 color_tint = BattlePlacement_ZoneColor.rgb;
            #if defined(USE_LOCAL_COLOR)
                color_tint = getMaterialColor().rgb;
            //#else
            //    color_tint = vec3(1.0, 0.0, 0.0);
            #endif

            gl_FragColor.rgb = mix(gl_FragColor.rgb, vec3(BattlePlacement_ZoneColor.a) * (color_tint), grid);

    #endif
    
    #if defined(PRELIT)
        #if defined(WORLD)
            world_shadow_receive();
        #else
            shadow_receive();
        #endif
    #else
        #if defined(WORLD)
            world_shadow_receive();
        #else
            shadow_receive();
        #endif
    #endif
  
    #ifdef FOG
        // Cloud Shadows //
        vec2 clouds_uv = vec2(v_position_world.xz) / vec2(50000.0) + vec2(-0.005, -0.015) * getSceneTime();
        vec4 clouds_tex = texture2D(cloudTexture, clouds_uv);
        float cloud_cycle = abs(fract(clouds_tex.r + getSceneTime() * 0.02) - 0.5) * 2.0;
        //gl_FragColor.rgb *= cloud_cycle * 0.3 + 0.85; 
    #endif
    
    #ifdef EMISSIVE
        vec3 emissive_color = mix(textureColor.rgb, EmissiveColor.rgb, EmissiveColor.a);
        // DavidJ --    when using MANTIS_PBR mode, the "emissive" parameter is stored in the alpha
        //              channel of the properties textures, rather than the base color texture. This
        //              is because the parameters texture is less sensitive to texture compression
        //              artefacts (meaning the quality loss that comes as a result of adding a alpha
        //              channel is less of a problem for the parameters texture, relative to the
        //              base color texture)
        #if defined(MANTIS_PBR)
            float emissiveParameter = properties.a;//move to mantis lighting section
        #else
            float emissiveParameter = textureColor.a;
        #endif
        gl_FragColor.rgb *= 1.0 - emissiveParameter;
        gl_FragColor.rgb += emissiveParameter * emissive_color;
        gl_FragColor.a = 1.0;
    #endif

    #ifdef VERTPUSH_BORDER
        #ifdef FREEZE
            float freeze_gray = dot(gl_FragColor.rgb, M_GRAYSCALE);
            gl_FragColor.rgb = mix(vec3(0.05, 0.25, 0.5), vec3(0.5, 0.9, 1.0) * 2.0, freeze_gray);
        #else
            gl_FragColor.rgb *= getMaterialColor().r;           
        #endif
    #endif

    #ifdef FOG
        float fog_far = EnvironmentConfig.FogNearFarMax.x + EnvironmentConfig.FogNearFarMax.y;
        float fog_blend = clamp((V_length - EnvironmentConfig.FogNearFarMax.x) / fog_far, 0.0, EnvironmentConfig.FogNearFarMax.z);
        gl_FragColor.rgb = mix(gl_FragColor.rgb, EnvironmentConfig.FogColor, fog_blend);
    #endif

    #ifdef COLOR_FADE
        gl_FragColor.rgb = mix(gl_FragColor.rgb, FadeColor.rgb, FadeColor.a);
    #endif
    
    #if defined(FREEZE2)

        float sheen_speed = 3.0;
        float x = gl_FragCoord.x;
        //x = gl_FragCoord.x / getViewport().z * 2.0;//hologram
        float y = gl_FragCoord.y / getViewport().w * 2.0;
        y *= 1.5;

        float adjust_scale = 0.01;

        //y = (v_position_world.x + v_position_world.z) * adjust_scale;
        //x = (v_position_world.y + v_position_world.y) * adjust_scale;

        y = (v_position_world.z) * adjust_scale;
        x = (v_position_world.x) * adjust_scale;

 
        //adjust_scale = 100.0;

        //to avoid swimming with viewport panning
        //x = v_texCoord.x * adjust_scale;
        //y = v_texCoord.y * adjust_scale;

        float pixel_size = 200.0;

        float scene_time = getSceneTime();
        sheen_speed *= 1.0/pixel_size;

        vec2 uv_a = vec2(x * 0.05 + scene_time * sheen_speed, -y - scene_time * sheen_speed);
        vec2 uv_b = vec2(x * 0.04 - scene_time * sheen_speed, -y + scene_time * sheen_speed);
        sheen_speed *= 1.5;
        vec2 uv_c = vec2(gl_FragCoord.x / getViewport().z * 43.0 - scene_time * sheen_speed, gl_FragCoord.y / getViewport().w * 33.0 + scene_time * sheen_speed);
        vec2 uv_d = vec2(gl_FragCoord.x / -getViewport().z * 31.0 + scene_time * sheen_speed, gl_FragCoord.y / -getViewport().w * 42.0 - scene_time * sheen_speed);

        vec4 noise_a = texture2D(dissolveMask, uv_a);
        vec4 noise_b = texture2D(dissolveMask, uv_b);

        vec4 cloud_a = texture2D(cloudTexture, uv_c);
        vec4 cloud_b = texture2D(cloudTexture, uv_d);
        float cloud = cloud_a.r * cloud_b.g * 2.0;
        if(cloud < 0.5)
        {
            cloud = 1.0;
        }
        else
        {
            cloud = 0.1;
        }
        
        float sheen_effect = noise_a.r * noise_b.g * 80.0;
        
        vec3 _V = V;
        vec3 _N = N;
        //_V = vec3(0.0, -1.0, 0.0);

        float nDotVInv = 1.0 - max(0.0, dot(_N, _V));
        float fresnelLevel = nDotVInv;
        
        fresnelLevel = pow(fresnelLevel, 2.0);

        //option to fresnel the "icicles"
        sheen_effect *= fresnelLevel;

        float gray = 1.0 - clamp(dot(gl_FragColor.rgb, M_GRAYSCALE), 0.0, 1.0);
        gray += sheen_effect;

        vec3 freeze = vec3(90.0/225.0, 160.0/255.0, 230.0/255.0);//add sheen here
        float freezeAmt = 140.0/255.0;
        gl_FragColor.rgb = mix(gl_FragColor.rgb * freeze, freeze, freezeAmt * gray);


        gl_FragColor.rgb += fresnelLevel * 2.0 * cloud;
    
    #endif

    #ifdef UNIT_HIGHLIGHT
        gl_FragColor.rgb = mix(gl_FragColor.rgb, HighlightColor.rgb, HighlightColor.a);
    #endif

    #if defined(EMISSIVE_RIM_FADE)
        gl_FragColor.rgb += emissiveParameter * emissive_color;
    #endif

    #if defined(DISSOLVE) || defined(GACHA) || defined(HOLOGRAM)
        float dissolve_strength = currentDissolveStrengthDurationDirectionFade().x;
        float dissolve_duration = currentDissolveStrengthDurationDirectionFade().y;
        float dissolve_direction = currentDissolveStrengthDurationDirectionFade().z;
        float dissolve_fade = currentDissolveStrengthDurationDirectionFade().w;
        float dissolve_width = DissolveWidthTexwidthTexscale.x;
        float dissolve_texwidth = DissolveWidthTexwidthTexscale.y;
        float dissolve_texscale = DissolveWidthTexwidthTexscale.z;
        float dissolve_start_pos = 3600.0;
        float dissolve_end_pos = -500.0;
    #endif

    #if defined(DISSOLVE) || defined(GACHA)
        float dissolve_delta = clamp(u_cc3ObjectTime[int(INSTANCE_ID)] / dissolve_duration, 0.0, 1.0);
        dissolve_delta = mix(dissolve_delta, 1.0 - dissolve_delta, dissolve_direction);

        vec4 dissolve_tex = texture2D(dissolveMask, v_position_local.xz / dissolve_texscale);
        float dissolve_blend = (mix(dissolve_start_pos, dissolve_end_pos, dissolve_delta) - v_position_local.y - mix(dissolve_tex.x, dissolve_tex.y, dissolve_delta) * dissolve_texwidth) / dissolve_width;    
        float dissolve_edge = clamp(dissolve_blend - 0.4, 0.0, 1.0);
        gl_FragColor.rgb = mix((1.0 - getMaterialColor().r) * currentDissolveColor().rgb + gl_FragColor.rgb * currentDissolveColor().rgb * dissolve_strength, gl_FragColor.rgb, dissolve_edge);
        gl_FragColor.a = dissolve_blend;       
    #endif

    #if defined(HOLOGRAM)
        float hologram_speed = 20.0;
        // Note: the 2.0 scale below comes from a legacy bug where cocos3d used to return the viewport *not* in pixels; we now compensate manually to keep existing art looking like it always did.
        float y = gl_FragCoord.y / getViewport().w * 2.0;
        #if defined(HOLOGRAM_SCALE)
            y *= HologramScale;
        #endif

        vec4 noise_a = texture2D(dissolveMask, vec2(0.0, y + getSceneTime() * hologram_speed / dissolve_texscale));
        vec4 noise_b = texture2D(dissolveMask, vec2(0.5, y - getSceneTime() * hologram_speed / dissolve_texscale));
        
        float lines = abs(fract(y * 128.0 + getSceneTime()) * 2.0 - 1.0);
        lines *= mix(0.3, 1.0, lines);
        float scanlines = clamp((noise_a.x + noise_b.x) * 0.5 * lines, 0.0, 1.0);
        float gray = dot(gl_FragColor.rgb, M_GRAYSCALE);
        vec3 hologram = gray * currentDissolveColor().rgb * (1.0 + scanlines * 3.0);      
    #endif

    #ifdef BLUEPRINT
        float blueprint_gray = dot(textureColor.rgb, M_GRAYSCALE);      
        float blueprint_fresnel = max(0.0, dot(N, V));
        vec3 blueprint = vec3(blueprint_fresnel * blueprint_gray * BlueprintColor.rgb * 1.0);
        gl_FragColor.rgb = blueprint;
    #endif

    #ifdef HOLOGRAM_BUILD
        dissolve_start_pos = u_dissolveHighestHeight;
        dissolve_end_pos = u_dissolveLowestHeight;
        float delta = clamp(u_cc3ObjectTime[int(INSTANCE_ID)] / dissolve_duration, 0.0, 1.0);
        delta = mix(delta, 1.0 - delta, dissolve_direction);
        float dissolve_blend = (mix(dissolve_start_pos, dissolve_end_pos, delta) - v_position_local.y) / dissolve_width;
        float dissolve_edge = clamp(1.0 - dissolve_blend, 0.0, 1.0);          
        vec4 hologram_build = mix(vec4(gl_FragColor.rgb, currentDissolveColor().a), vec4(hologram * (dissolve_edge * 8.0 + 1.0), 1.0), clamp(dissolve_blend, 0.0, 1.0));                     
        gl_FragColor = hologram_build;
    #elif defined(HOLOGRAM_FADE)
        float delta = clamp(u_cc3ObjectTime[int(INSTANCE_ID)] / dissolve_duration, 0.0, 1.0);
        delta = mix(delta, 1.0 - delta, dissolve_direction);
        gl_FragColor.rgb = mix(gl_FragColor.rgb, hologram, delta);            
        gl_FragColor.a = 1.0;
    #elif defined(HOLOGRAM)

        gl_FragColor.rgb = hologram;   
        #if defined(HOLOGRAM_SCALE)
            gl_FragColor.a = HologramAlpha;
        #else 
            gl_FragColor.a = 1.0;
        #endif
    #endif    

    #ifdef GLASS
        vec3 R = reflect(-V, N);
        vec3 environment = textureCube(u_reflectionMap, R).rgb;
        gl_FragColor = vec4(0.0);

        float fresnel = pow(1.0 - max(0.0, dot(N, V)), GlassFresnel);
        gl_FragColor += mix(GlassFrontColor, GlassSideColor + environment.r, fresnel);
    #endif

    #if defined(HOLOGRAM2)

        float hologram_speed = 10.0;
        float y = gl_FragCoord.y / getViewport().w * 2.0;

        float dissolve_texscale2 = 500.0;
        y *= 0.25;


        vec4 noise_a = texture2D(dissolveMask, vec2(0.0, y + getSceneTime() * hologram_speed / dissolve_texscale2));
        vec4 noise_b = texture2D(dissolveMask, vec2(0.5, y - getSceneTime() * hologram_speed / dissolve_texscale2));

        vec3 multiTex = texture2D(s_cc3Texture2Ds[0], v_texCoord).rgb;
        
        float lines = abs(fract(y * 128.0 + getSceneTime()) * 2.0 - 1.0);
        lines *= mix(0.3, 1.0, lines);
        float scanlines = clamp((noise_a.x + noise_b.x) * 0.5 * lines, 0.0, 1.0);

        gl_FragColor.rgb *= mix(vec3(2.0 * gl_FragColor.a), vec3(4.0), scanlines);
        gl_FragColor.a *= scanlines * 2.0;
    #endif

    #if defined(FINAL_TINT)
        gl_FragColor.rgb *= FinalTint.rgb * FinalTint.a;
    #endif
    #if !defined(SHIELD)
        float colorFadeAnimationWeight = 1.0;
        float colorFadeDuration = ColorFadeStartAndDuration.y;
        if (colorFadeDuration > 0.0) {
            float colorFadeStartTime = ColorFadeStartAndDuration.x;
            colorFadeAnimationWeight = (u_cc3ObjectTime[int(INSTANCE_ID)] - colorFadeStartTime) / colorFadeDuration;
            colorFadeAnimationWeight = 1.0 - clamp(colorFadeAnimationWeight, 0.0, 1.0);
        }
        gl_FragColor.rgb = mix(gl_FragColor.rgb, ColorFadeOverride.rgb, clamp(colorFadeAnimationWeight * ColorFadeOverride.a, 0.0, 1.0));
    #endif

    #ifdef SHADER_DEBUG
        // Diffuse //
        if(DebugIndex == 1)
            gl_FragColor = vec4(textureColor.rgb, 1.0);
 
        // UV1 //
        else if(DebugIndex == 2)
            gl_FragColor = vec4(v_texCoord, 0.0, 0.0);

        // Vertex Color //
        else if(DebugIndex == 3)
            #if defined(USE_VERTEX_COLOR) || defined(HEIGHT_BLEND)
                gl_FragColor = v_vertexColor;
            #elif defined(VERTPUSH_BORDER)
                gl_FragColor = getMaterialColor();
            #else
                gl_FragColor = vec4(1.0);
            #endif
 
        // Vertex Normals //
        else if(DebugIndex == 4)
            #if defined(USE_LIGHT) || defined(FRESNEL_ENABLED) || defined(ENVIRONMENT) || defined(SHIELD) || defined(GLASS)
                gl_FragColor = vec4(v_N * 0.5 + 0.5, 1.0);
            #else
                gl_FragColor = vec4(1.0);
            #endif

        // Alpha //
        else if(DebugIndex == 5)
            gl_FragColor = vec4(gl_FragColor.aaa, 1.0);

        // Lighting //
        else if(DebugIndex == 6)
        {
            //#if defined(BORDER) || defined(VERTPUSH_BORDER)
                vec3 topLight = EnvironmentConfig.LightTop;
                vec3 sideLight = EnvironmentConfig.LightSide;
                vec3 bottomLight = EnvironmentConfig.LightBottom;
            //#else
                //vec3 topLight = LightEnvTop;
                //vec3 sideLight = LightEnvSide;
                //vec3 bottomLight = LightEnvBottom;
            //#endif

            #if defined(USE_LIGHT)
                vec3 debug_weights = vec3(max(0.0, N.y), 1.0 - abs(N.y), max(0.0, -N.y));
                vec3 debug_light = topLight * debug_weights.x + sideLight * debug_weights.y + bottomLight * debug_weights.z;
                gl_FragColor = vec4(debug_light, 1.0);
            #else
                gl_FragColor = vec4(1.0);
            #endif
        }
    
        #ifdef MANTIS_PBR
            //if (DebugIndex >= MantisLightingDebug_Begin && DebugIndex < MantisLightingDebug_End) 
            //{
                //gl_FragColor.rgb = CalculateMantisLightingDebug(DebugIndex, properties, diffuseAlbedoSRGB, v_position_local, N, V);
                //gl_FragColor.a = 1.0f;
            //}
        #endif
    #endif

    #if defined(BORDER) && (BORDER_METHOD == BORDER_METHOD_PIXEL_BASED_ALPHA_CHANNEL) && !(defined(DISSOLVE) || defined(GACHA) || defined(HOLOGRAM))
        gl_FragColor.a = 0.0;       // write zero alpha to flag that we want this object to have borders
    #endif

    #if (__VERSION__ >= 300) && defined(BORDER) && (BORDER_METHOD == BORDER_METHOD_PIXEL_BASED_MRT)
        // DavidJ --    adding in a "sqrt" here as a cheap approximation of a linearize function.
        //              This helps us better distinquish between tiny changes in depth that might happen
        //              within a single character, and large changes in depth that might happen from
        //              one character to another.
        secondaryDepth = int(sqrt(gl_FragCoord.z) * 65535.0 - 32768.0);
    #endif
#endif
}
