#define SUPPORT_GLSL300
//hardware and platform support
#include "shader_setup.fsh"


//===================================================================================================================//
// SHADER DEFINES //
//===================================================================================================================//

//define features here - move to a separate .fsh file later if it gets cluttered here

#if !defined(SHADOW_GEN)

    //world directional lighting with normal map - single uv set - no vert color - multi map usage for spec mask (multi0)
    #if defined(WORLD_NORMAL_MULTI0_LEGACY)
        #define DIFFUSE_MAP_0_INDEX 0
        #define NORMAL_MAP_0_INDEX 1
        #define MATERIAL_MAP_0_INDEX 2
        #define USE_NORMAL_MAP 1
        #define USE_WORLD_LIGHTING 1
        #define USE_REFLECT 1
        #define USE_SPEC 1
        #define USE_DEFAULT_ENV_MAP 1
        #define USE_FRESNEL 1
        #define USE_MULTI0_MASK 1
        #define USE_FOG 1
        #define USE_LEGACY_WORLD_LIGHTING 1
    #endif

    //fixed world directional lighting with normal map - single uv set - no vert color - multi map usage for spec mask (multi0) - light is set in material not the scene
    #if defined(FIXEDWORLD_NORMAL_MULTI0_LEGACY)
        #define DIFFUSE_MAP_0_INDEX 0
        #define NORMAL_MAP_0_INDEX 1
        #define MATERIAL_MAP_0_INDEX 2
        #define USE_NORMAL_MAP 1
        #define USE_WORLD_LIGHTING 1
        #define USE_FIXEDWORLD_LIGHTING 1
        #define USE_REFLECT 1
        #define USE_SPEC 1
        #define USE_DEFAULT_ENV_MAP 1
        #define USE_FRESNEL 1
        #define USE_MULTI0_MASK 1
        #define USE_FOG 1
        #define USE_LEGACY_WORLD_LIGHTING 1
    #endif

    //world directional lighting with normal map - single uv set - no vert color - multi map usage for spec mask (multi1) - alpha fade - loading shader lerp
    #if defined(WORLD_NORMAL_MULTI0_LEGACY_ALPHAFADE_LOADING)
        #define DIFFUSE_MAP_0_INDEX 0
        #define NORMAL_MAP_0_INDEX 1
        #define MATERIAL_MAP_0_INDEX 2
        #define AUX_MAP_0_INDEX 3
        #define USE_SPHERE_COORD 1
        #define USE_NORMAL_MAP 1
        #define USE_WORLD_LIGHTING 1
        #define USE_REFLECT 1
        #define USE_SPEC 1
        #define USE_DEFAULT_ENV_MAP 1
        #define USE_FRESNEL 1
        #define USE_MULTI0_MASK 1
        #define USE_FOG 1
        #define USE_LEGACY_WORLD_LIGHTING 1
        #define USE_ALPHAFADE 1
        #define USE_LOADING_FADE 1
        #define USE_FADE_COLOR 1
        //#define USE_TIME_VALUE 1
    #endif

    //scene lighting with normal map - single uv set - no vert color - material - r: alpha g: spec/refl mask b: emissive
    #if defined(NORMAL_REFL_ALPHA)
        #define DIFFUSE_MAP_0_INDEX 0
        #define NORMAL_MAP_0_INDEX 1
        #define MATERIAL_MAP_0_INDEX 2
        #define REFL_MAP_0_INDEX 3
        #define USE_NORMAL_MAP 1
        #define USE_DEFAULT_LIGHTING 1
        #define USE_REFLECT 1
        #define USE_MATERIAL_SPEC_REFL 1
        #define USE_REFL_MAP 1
        #define USE_SPHERE_COORD 1
        #define USE_DEFAULT_SPEC 1
        #define USE_MATERIAL_ALPHA 1
        #define USE_MATERIAL_FRESNEL 1
        #define ALPHA_CHANNEL 0
        #define SPEC_MASK_CHANNEL 1
        #define FRESNEL_CHANNEL 2
        #define USE_FOG 1
    #endif

#endif

//shader features
#include "shader_features.sh"

//attributes, vertex streams in/out, and uniforms
#include "shader_frag.fsh"

//shader debug
#include "shader_debug.fsh"



//===================================================================================================================//
// SHADER //
//===================================================================================================================//

void main() 
{
 
    setup_textures();
    setup_normals();
    setup_reflect();
    setup_sphere_coords();
    setup_fresnel_texture();
    setup_world_lighting();
    
    apply_lighting();
    apply_reflection();
    apply_specular();
    apply_fresnel_light();
    apply_shadows();
    apply_fog();
    apply_fades();
    apply_alpha();

    shader_debug();
}
