
#if __VERSION__ >= 300

uniform mediump isampler2D GridLookup;
uniform mediump isampler2D LightPositionTable;
uniform sampler2D LightColorTable;

uniform vec2 u_tiledLightingLookupMin;
uniform vec2 u_tiledLightingLookupReciprocalScale;
uniform vec2 u_tiledLightingLookupTableDims;

vec3 CalculateSphereLight(vec3 samplePosition, vec3 sampleNormal, vec3 lightPosition, vec3 lightColor, float lightPower)
{
    // this is just temporary math for testing!
    vec3 o = samplePosition - lightPosition;
    float scale = lightPower / (dot(o, o) + 1.0f);

    // extreme falloff between .1f and 0.05f brightness -- to ensure that the falloff radius is finite and manageable
    scale *= clamp(20.f * (scale - 0.05f), 0.0f, 1.0f);

    return lightColor * scale;
}

vec3 CalculateTiledLighting(vec3 worldSpacePosition, vec3 worldSpaceNormal)
{
    // For a given world space position, lookup any nearby dynamic lights
    // This is a simplified and customized version of "tiled lighting" or "clustered lighting"

    // We just use a 2D lookup -- which is convenient for Iceland, wherein the world is mostly
    // just a small 2D plane

    vec2 uv = (worldSpacePosition.xz - u_tiledLightingLookupMin) * u_tiledLightingLookupReciprocalScale;
    ivec2 xy = clamp(int2(uv * u_tiledLightingLookupTableDims), ivec2(0), ivec2(u_tiledLightingLookupTableDims) - ivec2(1,1));
    // if (((xy.x & 1) ^ (xy.y & 1)) == 0) return vec3(0,1,0); else return vec3(1,0,0);
    int lookup = texelFetch(GridLookup, xy, 0).r;

    float positionReconstructionScalar = 1.0f/(16384.0f*u_tiledLightingLookupReciprocalScale.x);

    vec3 result = vec3(0.0f);
    if (lookup != 0) {
        for (;;) {
            int4 A = texelFetch(LightPositionTable, int2(lookup, 0), 0).rgba;
            vec4 lightColor = texelFetch(LightColorTable, int2(lookup, 0), 0).rgba;
            if (lightColor.a == 0.0f) break;        // this is the terminator -- note that the final texel in the image is guaranteed to be a terminator -- to prevent infinite loops

            vec3 lightPosition = vec3(A.rgb) * positionReconstructionScalar + vec3(u_tiledLightingLookupMin.x);
            float lightPower = float(A.a);
            result += CalculateSphereLight(worldSpacePosition, worldSpaceNormal, lightPosition.xyz, lightColor.rgb, lightPower);
            ++lookup;
        }
    }

    return result;
}

#endif
