local HAS_INIT = 'hasInitKey'
local CENTRAL_OBJECTIVE_MARKER_NAME = 'GroundCircle'
local MINE_MAX_MINERS_KEY = 'maxMinersKey'
local MINE_MINERS_PER_MINE = 1
local MINE_CURRENT_CRYSTALS_KEY = 'currentCrystalsKey'
local MINER_WAYPOINT = 'MinerWaypoint'
local BARRICADE_NUMERATOR = 3
local BARRICADE_DENOMINATOR = 2
local MINE_INDEX_KEY = 'mineIndexKey'

twoLanesMapNoObjective_tickTrigger = function(entity)
    local hasInit = objc.get_local_value(entity, HAS_INIT)
    if hasInit == 0 then
        objc.set_global_value(IS_TWO_LANE_MAP_GLOBAL_KEY, 1)

        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end

        local p1Mines = objc.location_marker_node_names_with_prefix("P1_Mine")
        for k,v in pairs(p1Mines) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalMineAtPosition(x, y, 1, v, true)
        end

        if is2vE == 1 then
            local p3Mines = objc.location_marker_node_names_with_prefix("P3_Mine")
            for k,v in pairs(p3Mines) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalMineAtPosition(x, y, 3, v, true)
            end
        end

        local x, y = objc.location_of_marker_node_named("P1_DropOff")
        spawnMinerWaypointAtPosition(x, y, 1)

        local x, y = objc.location_of_marker_node_named("P2_DropOff")
        spawnMinerWaypointAtPosition(x, y, 2)

        local player1Entities = objc.list_entities(1)
        for _, player1Entity in pairs(player1Entities) do
            if objc.is_barricade(player1Entity) or objc.is_base(player1Entity) then
                local barricadeHealth = objc.get_entity_value_for_key(player1Entity, ENTITY_VALUE_KEY_MAX_HEALTH)
                barricadeHealth = objc.divide_with_floor(barricadeHealth * BARRICADE_NUMERATOR, BARRICADE_DENOMINATOR)
                objc.set_entity_value_for_key(player1Entity, ENTITY_VALUE_KEY_MAX_HEALTH, barricadeHealth)
                objc.set_entity_value_for_key(player1Entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, barricadeHealth)
            end
        end

        local player2Entities = objc.list_entities(2)
        for _, player2Entity in pairs(player2Entities) do
            if objc.is_barricade(player2Entity) or objc.is_base(player2Entity) then
                local barricadeHealth = objc.get_entity_value_for_key(player2Entity, ENTITY_VALUE_KEY_MAX_HEALTH)
                barricadeHealth = objc.divide_with_floor(barricadeHealth * BARRICADE_NUMERATOR, BARRICADE_DENOMINATOR)
                objc.set_entity_value_for_key(player2Entity, ENTITY_VALUE_KEY_MAX_HEALTH, barricadeHealth)
                objc.set_entity_value_for_key(player2Entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, barricadeHealth)
            end
        end

        local playerNumbers = objc.get_player_numbers_for_team_number(1) -- if this is a 2v2 or a 2ve, set base income to 75%
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                objc.set_entity_value_for_key(objc.get_base_entity(1), ENTITY_VALUE_KEY_INCOME_TIER, 1)
                objc.set_entity_value_for_key(objc.get_base_entity(2), ENTITY_VALUE_KEY_INCOME_TIER, 1)
            end
        end

        objc.spawn('gameRulesEntity', 1, 1)
        objc.set_local_value(entity, HAS_INIT, 1)
    elseif hasInit == 1 then
        --[[
        local is2vE = 0
        local playerNumbers = objc.get_player_numbers_for_team_number(1)
        for _, number in pairs(playerNumbers) do
            if number > 2 then
                is2vE = 1
            end
        end

        local p1Miners = objc.location_marker_node_names_with_prefix("P1_MinePP")
        for k,v in pairs(p1Miners) do
            local x, y = objc.location_of_marker_node_named(v)
            spawnCrystalWorker(x, y, 1)
        end

        if is2vE == 1 then
            local p3Miners = objc.location_marker_node_names_with_prefix("P3_MinePP")
            for k,v in pairs(p3Miners) do
                local x, y = objc.location_of_marker_node_named(v)
                spawnCrystalWorker(x, y, 3)
            end
        end]]
        objc.set_local_value(entity, HAS_INIT, 2)
    end
end

spawnCrystalMineAtPosition = function(xPos, yPos, playerNumber, name, should_set_view)
    local spawnedMine = objc.spawn_skinned_entity_with_key_at_point(SKINNED_ENTITY_KEY_CRYSTAL_MINE, 1, playerNumber, xPos, yPos, 1, 0)
    objc.set_local_value(spawnedMine[1], MINE_INDEX_KEY, string.sub(name, -1))
    objc.set_placeable_objective_marker_node_name(spawnedMine[1], name)

    if should_set_view then
        objc.set_placeable_objective_view(spawnedMine[1], name)
    end
end

spawnMinerWaypointAtPosition = function(xPos, yPos, playerNumber)
    objc.spawn_at_point(MINER_WAYPOINT, 1, playerNumber, xPos, yPos, 1, 0)
end

spawnCrystalWorker = function(xPos, yPos, side)
    local faction = objc.get_commander_factions(side)
    if faction[1] == FACTION_BLUE then
        objc.spawn_at_point(ASCENSION_MINER, 1, side, xPos, yPos, 1, 0)
    elseif faction[1] == FACTION_GREEN then
        objc.spawn_at_point(LEGION_MINER, 1, side, xPos, yPos, 1, 0)
    elseif faction[1] == FACTION_RED then
        objc.spawn_at_point(ROGUE_MINER, 1, side, xPos, yPos, 1, 0)
    else
        objc.spawn_at_point(ROGUE_MINER, 1, side, xPos, yPos, 1, 0)
    end
end
