local UNIT_LUA = 'unitTutorial1.lua'

-- Variables
local COMMANDER_RUN_SPEED = 250
local HP_LOAD_IN_PER_TICK = 500

-- Narrative steps
local PLACEMENT_INIT = 'placementInit'
local INIT_CRASHED_SHIP = 'initCrashedShip'
local KURO_INIT = 'kuroInit'
local ANT_INIT = 'antInit'
local VAL_RUN_IN = 'valRunIn'
local SUMMON_BARRICADE_ANIMIATION = 'summonBarricadeAnimation'
local SUMMON_BARRICADES = 'summonBarricades'
local LOAD_IN_BARRICADES = 'loadInBarricades'
local ENEMY_BARRICADE_KILLABLE = 'enemyBarricadeKillable'
local CHECK_BARRICADE_LOW_HP = 'checkBarricadeLowHp'
local ENEMY_BARRICADE_LOW_HP = 'enemyBarricadeLowHp'
local VAL_DEATH = 'valDeath'
local SHIP_INIT = 'shipInit'

unitTutorial1_tickTrigger = function(entity)

    -- healthbar
    local shipInit = objc.get_local_value(entity, SHIP_INIT)
    if shipInit == 0 then
        local base = objc.get_base_entity(2)
        objc.add_trigger(base, TRIGGER_NAME_ON_DAMAGE_TRIGGER, UNIT_LUA, 'tutorialShipHealthVisible_takeTripleDamage')
        objc.set_placement_transition_lerp_visibility(false)
        objc.set_local_value(entity, SHIP_INIT, 1)
    end

    -- Play kuro's spawn animation
    local kuroInit = objc.get_local_value(entity, KURO_INIT)
    if kuroInit == 0 then
        local team1Entities = objc.list_entities(1)
        for _,unit in pairs(team1Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'C_Ascension2Tutorial' then
                objc.add_trigger(unit, TRIGGER_NAME_ON_DAMAGE_TRIGGER, UNIT_LUA, 'unitTutorial1_invulnerable')
                objc.play_override_animation(unit, "W_Kuro_Battle_Combat_StoryTutorial_Spawn_01", objc.convert_frames_to_subrounds(100))
                objc.idle_entity_indefinitely(unit)
                objc.set_local_value(entity, KURO_INIT, 1)
            end
        end
    end



    -- Introduce badguy
    local valRunIn = objc.get_global_value(VAL_RUN_IN)
    if valRunIn == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.get_entity_value_for_key(unit, ENTITY_VALUE_KEY_IDENTIFIER) == 'C_DuchessTutorial' then
                objc.set_entity_facing_direction(unit, -1, 0)
                objc.add_trigger(unit, TRIGGER_NAME_ON_DAMAGE_TRIGGER, UNIT_LUA, 'unitTutorial1_invulnerable')
                objc.remove_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, 'C_Duchess_tickTrigger')
                objc.remove_trigger(unit, TRIGGER_NAME_ON_ATTACK_TRIGGER, 'C_Duchess_onAttackTrigger')
                objc.remove_trigger(unit, TRIGGER_NAME_DEATH_TRIGGER, 'C_Duchess_deathTrigger')
                objc.idle_entity_indefinitely(unit)
                objc.set_global_value(VAL_RUN_IN, 2)
            end
        end
    end



    -- Load in the barricades' healthbars
    local loadInBarricades = objc.get_global_value(LOAD_IN_BARRICADES)
    if loadInBarricades == 1 then
        local team2Entities = objc.list_entities(2)
        for _,unit in pairs(team2Entities) do
            if objc.is_base(unit) then
                objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'unitTutorial1_loadInHealthbar')
            end
        end

        local team1Entities = objc.list_entities(1)
        for _,unit in pairs(team1Entities) do
            if objc.is_base(unit) then
                objc.add_trigger(unit, TRIGGER_NAME_TICK_TRIGGER, UNIT_LUA, 'unitTutorial1_loadInHealthbar')
            end
        end

        objc.set_global_value(LOAD_IN_BARRICADES, 0)
    end
end

tutorialShipHealthVisible_takeTripleDamage = function(damage, targetEntity, sourceEntity)
    return damage * 3
end

unitTutorial1_invulnerable = function(damage, targetEntity, sourceEntity)
    return 0
end

unitTutorial1_doubleDamage = function(damage, targetEntity, sourceEntity)
    local doubleDamage = damage * 2
    return doubleDamage
end

unitTutorial1_unkillableDoubleDamage = function(damage, targetEntity, sourceEntity)
    local doubleDamage = damage * 2
    if doubleDamage >= objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH) then
        return 0
    end
    return doubleDamage
end

unitTutorial1_unkillable = function(damage, targetEntity, sourceEntity)
    if damage >= objc.get_entity_value_for_key(targetEntity, ENTITY_VALUE_KEY_CURRENT_HEALTH) then
        return 0
    end
    return damage
end

-- Tick trigger that heals you quickly to full
unitTutorial1_loadInHealthbar = function(entity)
    local currHp = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH)
    local maxHp = objc.get_entity_value_for_key(entity, ENTITY_VALUE_KEY_MAX_HEALTH)
    local newHp = currHp + HP_LOAD_IN_PER_TICK
    if newHp >= maxHp then
        newHp = maxHp
        objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, newHp)
        objc.remove_trigger(entity, TRIGGER_NAME_TICK_TRIGGER, 'unitTutorial1_loadInHealthbar')
        return
    end
    objc.set_entity_value_for_key(entity, ENTITY_VALUE_KEY_CURRENT_HEALTH, newHp)
end