#if defined(GL_ES)
    precision highp float;
#endif

#define NUM_SAMPLES 10.0
#define NUM_LIGHTS 5

uniform sampler2D s_cc3Texture2Ds[1];

uniform int shouldUseLights[NUM_LIGHTS];
uniform float lightsX[NUM_LIGHTS];
uniform float lightsY[NUM_LIGHTS];
uniform float lightsIntensity[NUM_LIGHTS];

varying vec2 v_texCoord;

void main() {
    vec4 color = texture2D(s_cc3Texture2Ds[0], v_texCoord);
    
    for (int light = 0; light < NUM_LIGHTS; light++) {
        if (shouldUseLights[light] == 1) {
            vec2 lightPosition = vec2(lightsX[light], lightsY[light]);
            vec2 delta = v_texCoord - lightPosition;
            vec2 texCoord = v_texCoord;
            delta *= 1.0 / NUM_SAMPLES * 0.7;
        
            float decay = 1.0 * lightsIntensity[light];
            float weight = 1.0 / NUM_SAMPLES;
        
            for (int i = 0; i < int(NUM_SAMPLES); i++) {
                texCoord -= delta;
                vec4 sample = texture2D(s_cc3Texture2Ds[0], texCoord);
                sample *= decay * weight;
                color += sample;
                decay *= exp((float(-i) + 2.0)/10.0);
            }
        }
    }
    
    gl_FragColor = clamp(color, 0.0, 1.0);
}