////////////////////////////////////////////////////////////////////////////////////////////////////
// wm_region_cb.sh
////////////////////////////////////////////////////////////////////////////////////////////////////

#if !defined(WM_DEPENDENCIES_SH)
#define WM_DEPENDENCIES_SH

uniform struct
{

/* KenD -- note on Metal
 * The order of these elements within the struct do not match those in the pipeline layout.
 * The order must match so that the offset of each element matches.
 * Additionally, this struct should have all the elements that the pipeline layout contains.
 * Lastly, because the pipeline layout is packed, these elements should be packed too.
 * Following the below non-conforming set of elements is a conforming set of elements.
 */
#if 0

    float _UVScale1;

#ifdef DETAIL
    float _DetailScale;
    float _DetailRoadInfluence;
    float _DetailAmount1;
    float _DetailAmount2;
#endif

#ifdef ROADS
    float4 _RoadColor;
    float4 _RoadEdgeColor;
#endif

#if defined(GRADIENT) && defined(COMPLEX_GRADIENT)
    float _GradientDistance;
    float3 _GradientOuterColor;
    float3 _GradientInnerColor;
#endif

#ifdef LAYERED
    float _UVScale2;

    float _Influence1;
    float _Influence2;
    float _MaskInfluence;
    float _HeightInfluence;
    float _DetailBlendInfluence;
    float _Shift;

    float4 _EdgeColor;
#endif

#if defined(COLOR_MODULATE)
    float3 _TopColor1;
    float3 _BottomColor1;
    float _DiffuseAmount1;
    float3 _TopColor2;
    float3 _BottomColor2;
    float _DiffuseAmount2;
#endif

#if !defined(MANTIS_LIGHTING)
    float _SpecularAmount1;
    float _Smoothness1;
    float _SpecularAmount2;
    float _Smoothness2;
#endif

#ifdef WATER
    float _WaterSpeed;
    float _NormalStrength;
    float3 _WaterColor;
#endif

#else // end non-conforming elements

float _Influence1;
packed_float3 _TopColor1;
packed_float3 _BottomColor1;
float _DiffuseAmount1;
float _SpecularAmount1;
float _Smoothness1;
float _UVScale1;

// DETAIL
float _DetailScale;
float _DetailNear;
float _DetailFar;
float _DetailBlendInfluence;
float _DetailRoadInfluence;
float _DetailAmount1;
float _DetailAmount2;

// ROADS
float4 _RoadColor;
float4 _RoadEdgeColor;
float _RoadNoiseScale;
float _RoadNoiseStrength;

// GRID
float _GridAmount;

// GRADIENT
float _GradientDistance;
packed_float3 _GradientOuterColor;
packed_float3 _GradientInnerColor;

// LAYERED
float _Influence2;
packed_float3 _TopColor2;
packed_float3 _BottomColor2;
float _DiffuseAmount2;
float _SpecularAmount2;
float _Smoothness2;
float _UVScale2;
float _MaskInfluence;
float _HeightInfluence;
float _Shift;
float4 _EdgeColor;

//WATER
float _WaterSpeed;
float _NormalStrength;
packed_float3 _WaterColor;
#endif
} material;

#if defined(CLOUDS) || defined(FOG)

uniform struct
{
    float3 _Inscatter;
    float _MonochromeOpticalThickness;
} globalLighting;

#endif

uniform struct
{
    vec2 _insetDequant;
    vec4 _teamColors[3];
} RegionBlock;


/* Uniforms that do not change per frame */
uniform float3 ParticleFixedMaterial_nonBillboardOrientationInitialRotation;
uniform float3 ParticleFixedMaterial_nonBillboardOrientationInitialSpeed;
uniform float3 ParticleFixedMaterial_nonBillboardOrientationFinalSpeed;
uniform float  ParticleFixedMaterial_xzAngularSpeed;
uniform float  ParticleFixedMaterial_xzAngularAcceleration;
uniform float3 ParticleFixedMaterial_radialSpeed;
uniform float  ParticleFixedMaterial_numberExtraRows;
uniform float3 ParticleFixedMaterial_radialAcceleration;
uniform float  ParticleFixedMaterial_numberExtraColumns;
uniform float3 ParticleFixedMaterial_linearAcceleration;
uniform float3 ParticleFixedMaterial_anchor;
uniform float3 ParticleFixedMaterial_billboardAxis;
uniform float  ParticleFixedMaterial_textureRotationAngle;
uniform float3 ParticleFixedMaterial_colorTimings;
uniform float  ParticleFixedMaterial_textureAnimSpeed;
uniform float4 ParticleFixedMaterial_scaleTimings;
uniform float4 ParticleFixedMaterial_uvOffset;
uniform float  ParticleFixedMaterial_stretchFactor;
uniform float  ParticleFixedMaterial_stretchYOnly;
uniform float  ParticleFixedMaterial_overmultiplyAlpha;
uniform float2 ParticleFixedMaterial_frameRange;
// fragment shader
uniform float4 ParticleFixedMaterial_clampValues;
uniform float  ParticleFixedMaterial_useColor;
uniform float  ParticleFixedMaterial_multiplyColorWithRGB;
// additional
uniform float  ParticleFixedMaterial_erdFaceAxis;
uniform float  ParticleFixedMaterial_erdStartAngle;
uniform float  ParticleFixedMaterial_erdAnglePerInstance;
uniform float  ParticleFixedMaterial_erdSpinRate;
//#if defined(GENERIC_PROGRAMMABLE_BLENDING) && defined(GL_EXT_shader_framebuffer_fetch)
uniform float3 ParticleFixedMaterial_programmableBlendingValues;
//#endif


/* Uniforms that do change per frame */
uniform float4 ParticleFrameMaterial_packed[4];
/*
Layout:
float3 _cameraPosition;     // 0
float _opacity;

float2 _fadeRange;          // 1
float _alphaTestValue;
float _hideShadedWireframe;

float3 _targetLocation;     // 2
float _emitterDuration;

float4 _diffuseColor;       // 3
*/


uniform mat4 ParticleMatricesMaterial_viewMatrix;
uniform mat4 ParticleMatricesMaterial_projMatrix;


#if defined(INSTANCING_ENABLED)

    #if CC3_PLATFORM_ANDROID
        #define MAX_PARTICLE_INSTANCES 16
    #else
        #define MAX_PARTICLE_INSTANCES 25
    #endif

    uniform mat4 ParticleInstanceMaterial_modelMatrices[MAX_PARTICLE_INSTANCES];
    uniform mat4 ParticleInstanceMaterial_orientationMatrices[MAX_PARTICLE_INSTANCES];
    uniform float ParticleInstanceMaterial_times[MAX_PARTICLE_INSTANCES];

#else

    uniform mat4 ParticleIndividual_modelMatrix;
    uniform mat4 ParticleIndividual_orientationMatrix;
    uniform float ParticleIndividual_time;

#endif

#endif

